@@ -18,7 +18,7 @@
 #define myx_admin_library_h
 
 #if defined(__WIN__) || defined(_WIN32) || defined(_WIN64)
-#define __LCC__
+#include <winsock2.h>
 #endif
 
 #include <mysql.h>
