@@ -23,7 +23,7 @@
 #include <stdio.h>
 #include <mysql.h>
 #include <pcre.h>
-#include <myx_util_functions.h>
+#include <myx_util_public_interface.h>
 
 #include <myx_public_interface.h>
 
@@ -118,7 +118,8 @@ typedef enum myx_backup_error
   MYX_BACKUP_SQL_ERROR,
   MYX_BACKUP_STOPPED,
   MYX_BACKUP_CHARSET_CONVERSION,
-  MYX_BACKUP_WRONG_CHARSET
+  MYX_BACKUP_WRONG_CHARSET,
+  MYX_BACKUP_FILE_IS_MYSQLDUMP
 } MYX_BACKUP_ERROR;
 
 typedef enum myx_expression_error
@@ -166,6 +167,15 @@ typedef enum
   MYX_RHS_AVAILABLE, MYX_RHS_NOT_AVAILABLE, MYX_RHS_NEW_HOST
 } MYX_REPL_HOST_STATUS;
 
+typedef enum
+{
+  MYX_UOP_GLOBAL,
+  MYX_UOP_SCHEMA,
+  MYX_UOP_TABLE, 
+  MYX_UOP_COLUMN,
+  MYX_UOP_ROUTINE
+} MYX_USER_OBJECT_PRIVILEGE_TYPE;
+
 /*
  * Structs
  */
@@ -176,26 +186,26 @@ typedef enum
 
 /// [SCRIPT::LibInterfaceMapper] -generate_classes MYX_USER_OBJECT_PRIVILEGES
 typedef struct {
-    unsigned char *host;
-    unsigned char *object_name;
+    char *host;
+    char *object_name;
 
     unsigned int user_privileges_num;
-    unsigned char **user_privileges;
+    char **user_privileges;
 } MYX_USER_OBJECT_PRIVILEGES;
 
 /// [SCRIPT::LibInterfaceMapper] -generate_classes MYX_USER
 typedef struct {
-    unsigned char *user_name;
-    unsigned char *password;
-    unsigned char *full_name;
-    unsigned char *description;
-    unsigned char *email;
-    unsigned char *contact_information;
+    char *user_name;
+    char *password;
+    char *full_name;
+    char *description;
+    char *email;
+    char *contact_information;
     unsigned int icon_length;
-    unsigned char *icon;
+    char *icon;
 
     unsigned int hosts_num;
-    unsigned char **hosts;
+    char **hosts;
 
     unsigned int user_object_privileges_num;
     MYX_USER_OBJECT_PRIVILEGES *user_object_privileges;
@@ -576,7 +586,7 @@ MYX_PUBLIC_FUNC MYX_USER * myx_get_user_with_privileges(MYSQL *mysql, const char
 MYX_PUBLIC_FUNC int myx_set_user(MYSQL *mysql, MYX_USER *user, const char *previous_user_name, int new_user);
 MYX_PUBLIC_FUNC int myx_del_user(MYSQL *mysql, const char *user_name);
 MYX_PUBLIC_FUNC int myx_free_user(MYX_USER *user);
-MYX_PUBLIC_FUNC MYX_USER_OBJECT_PRIVILEGES * myx_get_privilege_struct(MYSQL *mysql, const char *object_name);
+MYX_PUBLIC_FUNC MYX_USER_OBJECT_PRIVILEGES * myx_get_privilege_struct(MYSQL *mysql, const char *object_name, MYX_USER_OBJECT_PRIVILEGE_TYPE privilege_type);
 MYX_PUBLIC_FUNC int myx_free_user_priv(MYX_USER_OBJECT_PRIVILEGES *user_priv);
 
 MYX_PUBLIC_FUNC char * myx_resize_vector_block(char * ptr, size_t size_of_block, size_t count_of_blocks);
