@@ -15,7 +15,7 @@
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 
 #include "myx_admin_library.h"
-#include "myx_aux_functions.h"
+#include "myx_util_functions.h"
 #include "myx_library.h"
 #include <stdio.h>
 #include <string.h>
@@ -26,7 +26,7 @@
 #include <sys/stat.h>
 #include <glib.h>
 #include <pcre.h>
-#include "myx_xml_aux_functions.h"
+#include "myx_xml_util_functions.h"
 
 #define LOG_FILE_LINE_LEN 4096
 
