@@ -26,7 +26,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 #include <glib.h>
 
 #include "myx_admin_library.h"
-#include "myx_xml_aux_functions.h"
+#include "myx_xml_util_functions.h"
 
 #define MAX_LINE_LENGTH 4096        /*the same as in mysys/default.c*/
 
@@ -91,7 +91,7 @@ static INI_LINE_TYPE process_ini_line(PROCESS_LINE_STATUS *plt,
                                       const char *line,
                                       const char **groups,
                                       MYX_ADMIN_LIB_ERROR *error_code);
-static MYX_GUI_WIDGET *find_widget(char *mysql_identifier,
+static MYX_GUI_WIDGET *find_widget(xmlChar *mysql_identifier,
                                    MYX_GUI_DESCRIPTION *gui_desc);
 //unused static PROCESS_LINE_STATUS* new_pls();
 static void init_line_status(PROCESS_LINE_STATUS * pls);
@@ -276,7 +276,7 @@ MYX_GUI_DESCRIPTION *
           widget->active= 1;
 
         if (widget->default_value)
-          widget->value= xmlStrdup(widget->default_value);
+          widget->value= (char*)xmlStrdup((xmlChar*)widget->default_value);
       }
     }
   }
@@ -361,7 +361,7 @@ MYX_GUI_DESCRIPTION* myx_process_mysql_cnf_file(MYX_GUI_DESCRIPTION *desc,
         widget->loose_option= normalized_option->loose_option;
 
         widget->value= !normalized_option->widget->multiple 
-                          ? (char*)xmlStrdup(normalized_option->value)
+                          ? (char*)xmlStrdup((xmlChar*)normalized_option->value)
                           : xstr_append(xstr_append(widget->value, ","),
                                         normalized_option->value);
       }
@@ -788,7 +788,7 @@ static NORMALIZED_OPTION* normalize_option(const char *name,
 //  while ((tmp= strchr(tmp, '_')))
 //    *tmp= '-';
 
-  if ((widget= find_widget(noption->name, desc)))
+  if ((widget= find_widget((xmlChar*)noption->name, desc)))
   {
     if (widget->widget_type == MYX_CHECKBOX)
     {
@@ -845,7 +845,7 @@ static NORMALIZED_OPTION* normalize_option(const char *name,
       return NULL;
     }
 
-    if ( (widget= find_widget(noption->name, desc)))
+    if ( (widget= find_widget((xmlChar*)noption->name, desc)))
     {
       assert(widget->widget_type == MYX_CHECKBOX);
       assert(widget->checkbox->is_boolean);
@@ -1071,7 +1071,7 @@ static void write_description(FILE *file, const MYX_GUI_WIDGET *widget)
 }
 
 /*a mysql-identifier is a mysql option*/
-static MYX_GUI_WIDGET *find_widget(char *mysql_identifier, 
+static MYX_GUI_WIDGET *find_widget(xmlChar *mysql_identifier, 
                                    MYX_GUI_DESCRIPTION *gui_desc)
 {
   unsigned int i,j,k,l;
@@ -1085,14 +1085,14 @@ static MYX_GUI_WIDGET *find_widget(char *mysql_identifier,
         MYX_GUI_WIDGET * widget= gui_desc->pages[i].groups[j].widgets+k;
 
         /* option is an alias for option_size */
-        if (! xmlStrncmp(widget->id,mysql_identifier,(int)strlen(mysql_identifier))
-            && !xmlStrcmp(widget->id+strlen(mysql_identifier),"-size"))
+        if (! xmlStrncmp((xmlChar*)widget->id,mysql_identifier,(int)strlen((char*)mysql_identifier))
+            && !xmlStrcmp((xmlChar*)widget->id+strlen((char*)mysql_identifier),(xmlChar*)"-size"))
           return widget;
-        if (! xmlStrcmp(widget->id,mysql_identifier))
+        if (! xmlStrcmp((xmlChar*)widget->id,mysql_identifier))
           return widget; /* we have found it */
         for (l=0; l< widget->alt_names_num; l++)
         {
-          if (! xmlStrcmp(widget->alt_names[l], mysql_identifier) )
+          if (! xmlStrcmp((xmlChar*)widget->alt_names[l], mysql_identifier) )
             return widget;
         }
       }
