@@ -20,7 +20,7 @@
 #include <assert.h>
 
 #include <myx_admin_library.h>
-#include <myx_aux_functions.h>
+#include <myx_util_functions.h>
 
 /*
  * public functions definitions
@@ -67,7 +67,7 @@ MYX_PROCESS_LIST * myx_get_process_list(MYSQL *mysql)
       MYSQL_FIELD * fields= mysql_fetch_fields(res);
       int fi[8];
       build_field_subst(show_processes_columns,show_processes_columns_end,
-                        fields,fields+num_fields,fi);
+                        fields,fields+num_fields,(unsigned int*)fi);
 
 #define GET_STR_FIELD(n) fi[n]==-1 ? NULL : g_strdup(row[fi[n]]?row[fi[n]]:"")
       process_info->id=       GET_STR_FIELD(0);
