@@ -23,6 +23,15 @@
 
 #define USER_INFO_TABLENAME "user_info"
 
+/*
+ * public functions declarations
+ */
+
+
+int myx_is_table(MYSQL *mysql, const char *schema, const char *table);
+// myx_is_procedure will return false/0 for functions
+int myx_is_procedure(MYSQL *mysql, const char *schema, const char *proc);
+
 /*
  * public functions definitions
  */
@@ -226,6 +235,18 @@ char * q_select_user_info[q_v_count]=
           "_utf8'Y',_utf8'N') as pv "\
     "from mysql.tables_priv"
 
+#define SELECT_PROC_PRIV_50(name)\
+ "select "\
+    "cast(cast(host AS BINARY) AS CHAR CHARACTER SET utf8) as h, "\
+    "concat(cast(cast(db AS BINARY) AS CHAR CHARACTER SET utf8),"\
+            "_utf8'.',"\
+         "cast(cast(routine_name AS BINARY) AS CHAR CHARACTER SET utf8)) as o, "\
+    "_utf8'Proc_priv_" name "' as pn, "\
+    "if(find_in_set(_utf8'" name "', "\
+        "cast(cast(Proc_priv AS BINARY) AS CHAR CHARACTER SET utf8)),"\
+          "_utf8'Y',_utf8'N') as pv "\
+    "from mysql.procs_priv"
+
 #define SELECT_TABLE_COLUMNS_PRIV_40(name)\
  "select host as h, concat(db,'.',table_name) as o, "\
                 "'Column_priv_" name "' as pn, "\
@@ -530,7 +551,29 @@ Query_privs q_select_columns_privs[q_v_count]=
   }
 };
 
+char * q_select_procs_50[]=
+{
+  SELECT_PROC_PRIV_50( "Execute"       )    WHERE_V_41(),
+  SELECT_PROC_PRIV_50( "Alter routine" )    WHERE_V_41(),
+  SELECT_PROC_PRIV_50( "Grant"         )    WHERE_V_41()
+};
 
+Query_privs q_select_procs_privs[q_v_count]=
+{
+  {
+    q_select_procs_50,
+    q_select_procs_50
+  },
+  {
+    q_select_procs_50,
+    q_select_procs_50
+  },
+  {
+    q_select_procs_50,
+    q_select_procs_50 +
+                      sizeof(q_select_procs_50)/sizeof(*q_select_procs_50)
+  }
+};
 
 static char *replace_escaped(MYSQL *mysql, const char *query, 
                              const char *var, const char *value)
@@ -649,12 +692,12 @@ MYX_USER_OBJECT_PRIVILEGES * look_for_obj_name(MYX_USER * user,
 
 char ** look_for_host(MYX_USER * user, char * host_name)
 {
-  unsigned char ** host= user->hosts;
-  unsigned char ** hosts_end= host + user->hosts_num;
+  char ** host= user->hosts;
+  char ** hosts_end= host + user->hosts_num;
   for ( ; host!=hosts_end; host++)
   {
     if (!strcmp(*host,host_name))
-      return (char**)host;
+      return host;
   }
   return 0;
 }
@@ -682,7 +725,7 @@ MYX_USER_OBJECT_PRIVILEGES *
     o_priv->user_privileges=      0;
     if (!look_for_host(user,host) && get_hosts)
     {
-      unsigned char ** added_host;
+      char ** added_host;
       MYX_VECTOR_PUSH_BACK(user->hosts,user->hosts_num,added_host);
       *added_host= g_strdup(o_priv->host);
     }
@@ -734,7 +777,7 @@ int get_privileges_from_query(MYSQL *mysql, MYX_USER * user,
       MYX_USER_OBJECT_PRIVILEGES *o_priv= safe_add_object(mysql,user,
                                                           object_name,
                                                           host,get_hosts);
-      unsigned char ** priv;
+      char ** priv;
       MYX_VECTOR_PUSH_BACK(o_priv->user_privileges,
                            o_priv->user_privileges_num,priv);
       *priv= g_strdup_printf("%s=%s", priv_name ? priv_name : "", priv_value ? priv_value : "");
@@ -833,7 +876,8 @@ MYX_USER * myx_get_user_opt_privileges(MYSQL *mysql,
       get_privileges_from_queries(mysql,user,  q_select_db_privs+v,0)>=0 &&
       (!get_privileges || 
        (get_privileges_from_queries(mysql,user,q_select_tables_privs+v,0)>=0 &&
-        get_privileges_from_queries(mysql,user,q_select_columns_privs+v,0)>=0))
+        get_privileges_from_queries(mysql,user,q_select_columns_privs+v,0)>=0 &&
+        get_privileges_from_queries(mysql,user,q_select_procs_privs+v,0)>=0))
      )
   {
     return user;
@@ -1009,22 +1053,26 @@ int myx_set_user(MYSQL *mysql, MYX_USER *user,
                  const char *previous_user_name, int is_new_user)
 {
   MYX_USER *current_user;
-  unsigned char sqlcmd[500];
-  unsigned char field_buf[30];
-  unsigned char value_buf[30];
-  unsigned char *schema_name, *table_name, *column_name;
+  char sqlcmd[500];
+  char field_buf[30];
+  char value_buf[30];
+  char *schema_name, *table_name, *column_name;
   const char * begin;
-  unsigned char *s;
+  char *s;
   unsigned int i, j;
   int dot_count, u_variant, num_host;
   int icon_changed = 0;
-  unsigned char *long_sqlcmd, *sqlcmd_end;
+  char *long_sqlcmd, *sqlcmd_end;
   MYSQL_RES* res;
   unsigned long cur_name_len, name_len= (unsigned long)strlen(user->user_name);
   int utf8_bin= 0;
   const char *new_user= user->user_name;
   const char *cur_user= previous_user_name;
+  int is_tbl, is_proc, is_col;
   
+  // ensure user info table exists before we start
+  myx_check_mysql_user_info_table(mysql);
+
   if (mysql_full_version_is_later_or_equal_than(mysql,4,1,11)
       && (!mysql_version_is_later_or_equal_than(mysql,5,0)
           || mysql_full_version_is_later_or_equal_than(mysql,5,0,3)))
@@ -1033,13 +1081,19 @@ int myx_set_user(MYSQL *mysql, MYX_USER *user,
   if (utf8_bin)
   {
     if (myx_mysql_query(mysql, "SET collation_connection = utf8_bin"))
+    {
+      g_message("save user: can't set collation_connection to utf8_bin");
       goto error;
+    }
   }
 
   if (!is_new_user) // Update existing User
   {
     if (!(current_user= myx_get_user(mysql, previous_user_name)))
+    {
+      g_message("save user: can't retrieve user information");
       goto error;
+    }
     cur_user= current_user->user_name;
     cur_name_len= (unsigned long)strlen(cur_user);
 
@@ -1065,6 +1119,7 @@ int myx_set_user(MYSQL *mysql, MYX_USER *user,
                           " WHERE User=@cur_user",
                           new_user, cur_user))
       {
+        g_message("save user: error updating user name or privilege");
         goto error;
       }
     }
@@ -1083,8 +1138,8 @@ int myx_set_user(MYSQL *mysql, MYX_USER *user,
         char *cmd, *cmd2;
         
         cmd= replace_escaped(mysql, 
-                             "INSERT INTO mysql.user(User, Host, Password)"
-                             " SELECT @new_user, @host_name, Password"
+                             "INSERT INTO mysql.user(User, Host, Password, ssl_cipher, x509_issuer, x509_subject)"
+                             " SELECT @new_user, @host_name, Password, ssl_cipher, x509_issuer, x509_subject"
                              " FROM mysql.user"
                              " WHERE User=@new_user ORDER BY Host LIMIT 1",
                              "@new_user", new_user);
@@ -1095,6 +1150,7 @@ int myx_set_user(MYSQL *mysql, MYX_USER *user,
         if (myx_mysql_query(mysql, cmd2))
         {
           g_free(cmd2);
+          g_message("save user: error adding new user entry");
           goto error;
         }
         g_free(cmd2);
@@ -1124,6 +1180,7 @@ int myx_set_user(MYSQL *mysql, MYX_USER *user,
 
         if (myx_mysql_query(mysql, cmd2))
         {
+          g_message("save user: error deleting old user entry");
           g_free(cmd2);
           goto error;
         }
@@ -1159,7 +1216,7 @@ int myx_set_user(MYSQL *mysql, MYX_USER *user,
                            " WHERE User=@cur_user",
                            "@cur_user", cur_user);
 
-      if (myx_mysql_query(mysql, cmd)
+      if (!myx_mysql_query(mysql, cmd)
           && (res= mysql_store_result(mysql)))
       {
         g_free(cmd);
@@ -1169,7 +1226,7 @@ int myx_set_user(MYSQL *mysql, MYX_USER *user,
       else
       {
         g_free(cmd);
-        g_message("Error executing query %s:\n%s",sqlcmd,
+        g_message("Error executing query %s:\n%s",cmd,
                   myx_mysql_error(mysql));
         goto error;
       }
@@ -1177,7 +1234,7 @@ int myx_set_user(MYSQL *mysql, MYX_USER *user,
       if (replace_user_info(mysql,user,name_len,
                             user_info_replace_parts[u_variant],
                             new_user, cur_user))
-      {
+      {      
         goto error;
       }
     }
@@ -1195,6 +1252,7 @@ int myx_set_user(MYSQL *mysql, MYX_USER *user,
       
       if (myx_mysql_query(mysql, cmd2))
       {
+        g_message("save user: error updating password");
         g_free(cmd2);
         goto error;
       }
@@ -1210,7 +1268,7 @@ int myx_set_user(MYSQL *mysql, MYX_USER *user,
       unsigned long host_len= (unsigned long)strlen(user->hosts[i]);
       
       sqlcmd_end= strmov(sqlcmd, 
-                         "INSERT INTO mysql.user(User, Host, Password)"
+                         "INSERT INTO mysql.user(User, Host, Password, ssl_cipher, x509_issuer, x509_subject)"
                          " VALUES('");
       sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end,
                                             new_user, (gulong)strlen(new_user));
@@ -1220,11 +1278,12 @@ int myx_set_user(MYSQL *mysql, MYX_USER *user,
       sqlcmd_end= strmov(sqlcmd_end, "', Password('");
       sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end, user->password,
                                             pass_len);
-      sqlcmd_end= strmov(sqlcmd_end, "'))");
+      sqlcmd_end= strmov(sqlcmd_end, "'), '', '', '')");
 
       if (myx_mysql_real_query(mysql, sqlcmd,
                                (unsigned int)(sqlcmd_end-sqlcmd)))
       {
+        g_message("save user: error adding new user entry");
         goto error;
       }
     }
@@ -1237,6 +1296,7 @@ int myx_set_user(MYSQL *mysql, MYX_USER *user,
                             user_info_insert_parts,
                             new_user, cur_user))
       {
+        g_message("save user: error adding user information entry");
         goto error;
       }
     }
@@ -1257,6 +1317,7 @@ int myx_set_user(MYSQL *mysql, MYX_USER *user,
 //      char *error= myx_mysql_error(mysql);
 //      int error_num= myx_mysql_errno(mysql);
 
+      g_message("save user: error deleting old user privileges");
       goto error;
     }
     g_free(cmd);
@@ -1265,6 +1326,7 @@ int myx_set_user(MYSQL *mysql, MYX_USER *user,
                          "@new_user", new_user);
     if (myx_mysql_query(mysql, cmd))
     {
+      g_message("save user: error deleting old table privileges");
       g_free(cmd);
       goto error;
     }
@@ -1274,11 +1336,25 @@ int myx_set_user(MYSQL *mysql, MYX_USER *user,
                          "@new_user", new_user); 
     if (myx_mysql_query(mysql, cmd))
     {
+      g_message("save user: error deleting old column privileges");
       g_free(cmd);
       goto error;
     }
     g_free(cmd);
 
+    if(mysql_full_version_is_later_or_equal_than(mysql,5,0,3))
+    {
+      cmd= replace_escaped(mysql, "DELETE FROM mysql.procs_priv WHERE User=@new_user",
+                          "@new_user", new_user); 
+      if (myx_mysql_query(mysql, cmd))
+      {
+        g_message("save user: error deleting old SP privileges");
+        g_free(cmd);
+        goto error;
+      }
+      g_free(cmd);
+    }
+
     long_sqlcmd= g_malloc0(sizeof(char)*700);
 
     for(i= 0; i<user->user_object_privileges_num; i++)
@@ -1313,6 +1389,7 @@ int myx_set_user(MYSQL *mysql, MYX_USER *user,
         if (myx_mysql_real_query(mysql, long_sqlcmd,
                                  (unsigned int)(sqlcmd_end-long_sqlcmd)))
         {
+          g_message("save user: error updating user information");
           g_free(long_sqlcmd);
           goto error;
         }
@@ -1353,6 +1430,7 @@ int myx_set_user(MYSQL *mysql, MYX_USER *user,
         if (myx_mysql_real_query(mysql, long_sqlcmd,
                                  (unsigned int)(sqlcmd_end-long_sqlcmd)))
         {
+          g_message("save user: error adding schema privileges");
           g_free(long_sqlcmd);
           goto error;
         }
@@ -1369,13 +1447,28 @@ int myx_set_user(MYSQL *mysql, MYX_USER *user,
 
         if (dot_count==1)
         {
-          sqlcmd_end= strmov(long_sqlcmd,
-                             "INSERT INTO mysql.tables_priv"
-                             "(Host, Db, User, Table_name, Timestamp, Grantor,"
-                              "Table_priv, Column_priv) VALUES('");
+          if(myx_is_table(mysql, schema_name, table_name)) {
+            is_tbl= 1;
+            is_col= is_proc= 0;
+            sqlcmd_end= strmov(long_sqlcmd,
+                              "INSERT INTO mysql.tables_priv"
+                              "(Host, Db, User, Table_name, Timestamp, Grantor,"
+                                "Table_priv, Column_priv) VALUES('");
+          }
+          else
+          {
+            is_col= is_tbl= 0;
+            is_proc= myx_is_procedure(mysql, schema_name, table_name);
+            sqlcmd_end= strmov(long_sqlcmd,
+                              "INSERT INTO mysql.procs_priv"
+                              "(Host, Db, User, Routine_name, Routine_type, Grantor, Proc_priv, Timestamp) "
+                              "VALUES('");
+          }
         }
         else
         {
+          is_col= 1;
+          is_proc= is_tbl= 0;
           column_name= strtok(NULL, ".");
           column_len= (unsigned long)strlen(column_name);
           sqlcmd_end= strmov(long_sqlcmd,
@@ -1395,7 +1488,19 @@ int myx_set_user(MYSQL *mysql, MYX_USER *user,
         sqlcmd_end= strmov(sqlcmd_end, "', '");
         sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end, table_name,
                                               table_len);
-        sqlcmd_end= strmov(sqlcmd_end, "', now(), '");
+        
+        if(is_tbl || is_col)
+        {
+          sqlcmd_end= strmov(sqlcmd_end, "', now(), '");
+        }
+        else if(is_proc)
+        {
+          sqlcmd_end= strmov(sqlcmd_end, "', 'PROCEDURE', '");
+        }
+        else
+        {
+          sqlcmd_end= strmov(sqlcmd_end, "', 'FUNCTION', '");
+        }
 
         if (dot_count!=1)
         {
@@ -1407,43 +1512,70 @@ int myx_set_user(MYSQL *mysql, MYX_USER *user,
           // No grantor at this time
           begin= sqlcmd_end= strmov(sqlcmd_end, "', '");
 
-          // Get selected Table_privs
-          for(j= 0; j<privs->user_privileges_num; j++)
+          if(is_tbl)
           {
-            if ( strstr(privs->user_privileges[j], "Table_priv") &&
-                !strcmp(value_of_str(value_buf, privs->user_privileges[j]),"Y"))
+            // Get selected Table_privs
+            for(j= 0; j<privs->user_privileges_num; j++)
+            {
+              if ( strstr(privs->user_privileges[j], "Table_priv") &&
+                  !strcmp(value_of_str(value_buf, privs->user_privileges[j]),"Y"))
+              {
+                sqlcmd_end= strmov(sqlcmd_end,
+                                  strstr(name_of_str(field_buf,
+                                                      privs->user_privileges[j]),
+                                          "_priv_")+6);
+                sqlcmd_end= strmov(sqlcmd_end, ",");
+              }
+            }
+          }
+          else
+          {
+            // Get selected Proc_privs
+            for(j= 0; j<privs->user_privileges_num; j++)
+            {
+              if ( strstr(privs->user_privileges[j], "Proc_priv") &&
+                  !strcmp(value_of_str(value_buf, privs->user_privileges[j]),"Y"))
+              {
+                sqlcmd_end= strmov(sqlcmd_end,
+                                  strstr(name_of_str(field_buf,
+                                                      privs->user_privileges[j]),
+                                          "_priv_")+6);
+                sqlcmd_end= strmov(sqlcmd_end, ",");
+              }
+            }
+          }
+          if (sqlcmd_end!=begin) // Remove last ,
+            sqlcmd_end--;
+        }
+
+        // Get selected Column_privs
+        if(is_tbl || is_col)
+        {
+          begin= sqlcmd_end= strmov(sqlcmd_end, "', '");
+          for(j=0;j<privs->user_privileges_num;j++)
+          {
+            if( strstr(privs->user_privileges[j], "Column_priv")&&
+              !strcmp(value_of_str(value_buf, privs->user_privileges[j]),"Y"))
             {
               sqlcmd_end= strmov(sqlcmd_end,
-                                 strstr(name_of_str(field_buf,
+                                strstr(name_of_str(field_buf,
                                                     privs->user_privileges[j]),
                                         "_priv_")+6);
               sqlcmd_end= strmov(sqlcmd_end, ",");
             }
           }
-          if (sqlcmd_end!=(unsigned char*)begin) // Remove last ,
+          if(sqlcmd_end!=begin) // Remove last ,
             sqlcmd_end--;
         }
 
-        begin= sqlcmd_end= strmov(sqlcmd_end, "', '");
-
-        // Get selected Column_privs
-        for(j=0;j<privs->user_privileges_num;j++)
+        if(is_tbl || is_col)
         {
-          if( strstr(privs->user_privileges[j], "Column_priv")&&
-             !strcmp(value_of_str(value_buf, privs->user_privileges[j]),"Y"))
-          {
-            sqlcmd_end= strmov(sqlcmd_end,
-                               strstr(name_of_str(field_buf,
-                                                  privs->user_privileges[j]),
-                                      "_priv_")+6);
-            sqlcmd_end= strmov(sqlcmd_end, ",");
-          }
+          sqlcmd_end= strmov(sqlcmd_end, "')");
+        }
+        else
+        {
+          sqlcmd_end= strmov(sqlcmd_end, "', now())");
         }
-
-        if(sqlcmd_end!=(unsigned char*)begin) // Remove last ,
-          sqlcmd_end--;
-
-        sqlcmd_end= strmov(sqlcmd_end, "')");
 
         g_free(s);
 
@@ -1480,8 +1612,8 @@ error:
   return -1;
 }
 
-MYX_USER_OBJECT_PRIVILEGES * myx_get_privilege_struct(MYSQL *mysql, 
-                                                      const char *object_name)
+MYX_USER_OBJECT_PRIVILEGES * myx_get_privilege_struct(MYSQL *mysql, const char *object_name, MYX_USER_OBJECT_PRIVILEGE_TYPE object_type)
+                                                      
 {
   MYSQL_RES* res;
   MYSQL_FIELD *fields;
@@ -1490,7 +1622,6 @@ MYX_USER_OBJECT_PRIVILEGES * myx_get_privilege_struct(MYSQL *mysql,
 
   MYX_USER_OBJECT_PRIVILEGES *user_priv=
                                  g_malloc0(sizeof(MYX_USER_OBJECT_PRIVILEGES));
-  unsigned int point_count;
   char * sqlcmd;
   unsigned char s[255];
   unsigned int j;
@@ -1504,13 +1635,24 @@ MYX_USER_OBJECT_PRIVILEGES * myx_get_privilege_struct(MYSQL *mysql,
   user_priv->user_privileges_num= 0;
   user_priv->user_privileges= NULL;
 
-  point_count= sub_str_count(".", object_name);
-
-  sqlcmd= 
-    !*object_name  ? "SELECT * FROM mysql.user WHERE 1=2"  : // Global
-    point_count==0 ? "SELECT * FROM mysql.db   WHERE 1=2"  : // db
-    point_count==1 ? "SHOW COLUMNS FROM mysql.tables_priv" : // db.Table
-                     "SHOW COLUMNS FROM mysql.columns_priv"; // db.table.column
+  switch (object_type)
+  { 
+    case MYX_UOP_GLOBAL:
+      sqlcmd= "SELECT * FROM mysql.user WHERE 1=2"; // Global
+      break;
+    case MYX_UOP_SCHEMA:
+      sqlcmd= "SELECT * FROM mysql.db   WHERE 1=2"; // db
+      break;
+    case MYX_UOP_TABLE:
+      sqlcmd= "SHOW COLUMNS FROM mysql.tables_priv";
+      break;
+    case MYX_UOP_ROUTINE:
+      sqlcmd= "SHOW COLUMNS FROM mysql.procs_priv";
+      break;
+    case MYX_UOP_COLUMN:
+      sqlcmd= "SHOW COLUMNS FROM mysql.columns_priv"; // db.table.column
+      break;
+  }
 
   if (myx_mysql_query(mysql, sqlcmd) || !(res= mysql_store_result(mysql)))
   {
@@ -1519,7 +1661,7 @@ MYX_USER_OBJECT_PRIVILEGES * myx_get_privilege_struct(MYSQL *mysql,
   }
   else
   {
-    if (!point_count)
+    if (object_type == MYX_UOP_GLOBAL || object_type == MYX_UOP_SCHEMA)
     {
       // Depending on the version of the server there might be
       num_fields= mysql_num_fields(res); // different columns
@@ -1543,7 +1685,7 @@ MYX_USER_OBJECT_PRIVILEGES * myx_get_privilege_struct(MYSQL *mysql,
         g_free(field_name);
       }
     }
-    else if (point_count==1 || point_count==2)
+    else if (object_type == MYX_UOP_TABLE || object_type == MYX_UOP_ROUTINE || object_type == MYX_UOP_COLUMN)
     {
       MYX_VECTOR_RESERVE(user_priv->user_privileges,mysql_num_rows(res)-4);
       while ((row= mysql_fetch_row(res)))
@@ -1609,3 +1751,48 @@ int myx_del_user(MYSQL *mysql, const char *user_name)
 
   return 0;
 }
+
+int myx_is_table(MYSQL *mysql, const char *schema, const char *table)
+{
+  MYSQL_RES *res;
+  char *q= g_strconcat("SHOW TABLE STATUS FROM ", schema, " LIKE \"", table, "\"", NULL);
+  myx_mysql_query(mysql, q);
+  g_free(q);
+  res= mysql_store_result(mysql);
+  if(res)
+  {
+    int ret= (mysql_num_rows(res) > 0);
+    mysql_free_result(res);
+    return ret;
+  }
+  return 0;
+}
+
+int myx_is_procedure(MYSQL *mysql, const char *schema, const char *proc)
+{
+  MYSQL_RES *res;
+  char *q= g_strconcat("SHOW PROCEDURE STATUS LIKE \"", proc, "\"", NULL);
+  myx_mysql_query(mysql, q);
+  g_free(q);
+  res= mysql_store_result(mysql);
+  if(res)
+  {
+    int ret= (mysql_num_rows(res) > 0);
+    mysql_free_result(res);
+    return ret;
+  }
+  return 0;
+}
+
+void myx_compact_privs(MYSQL *mysql)
+{
+  char *q = "DELETE FROM mysql.db WHERE Select_priv='N' AND Insert_priv='N' AND Update_priv='N'"
+    " AND Delete_priv='N' AND Create_priv='N' AND Drop_priv='N' AND Grant_priv='N'";
+    " AND References_priv='N' AND Index_priv='N' AND Alter_priv='N' AND Create_tmp_table_priv='N'"
+    " AND Lock_tables_priv='N' AND Create_view_priv='N' AND Show_view_priv='N' AND Create_routine_priv='N'"
+    " AND Alter_routine_priv='N' AND Execute_priv='N'";
+  myx_mysql_query(mysql, q);
+
+  q = "FLUSH PRIVILEGES";
+  myx_mysql_query(mysql, q);
+}
\ No newline at end of file
