@@ -14,7 +14,7 @@ along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 
 #include "myx_admin_library.h"
-#include "myx_xml_aux_functions.h"
+#include "myx_xml_util_functions.h"
 #include <libxml/xmlmemory.h>
 #include <libxml/parser.h>
 #include <libxml/tree.h>
@@ -146,9 +146,15 @@ MYX_HEALTH_PAGES* myx_read_in_health_pages(const char* filename, MYX_ADMIN_LIB_E
   xmlNodePtr root;
   MYX_HEALTH_PAGES *health_pages;
 
+  char *local_filename;
   *error_code = MYX_ADMIN_NO_ERROR;
 
-  if (! file_exists(filename) )
+  if (!(local_filename= g_filename_from_utf8(filename, -1, NULL, NULL, NULL)))
+  {
+    *error_code= MYX_CHARSET_CONVERSION_ERROR;
+    return NULL;
+  };  
+  if (! file_exists(local_filename) )
   {
     *error_code = MYX_ADMIN_ERROR_CANT_OPEN_FILE;
     return NULL;
