@@ -944,7 +944,7 @@ MYX_BACKUP_PROFILE *MABackupPanel::gather_profile()
   else if (get_toggle("binlogpos_radio")->get_active())
     flags |= MYX_B_POINT_IN_TIME_BACKUP;
   
-  if (get_toggle("backup_all_check"))
+  if (get_toggle("backup_all_check")->get_active())
     flags |= MYX_B_COMPLETE_SCHEMATAS;
 
 
