@@ -67,7 +67,7 @@ class CellRendererProgress : public Gtk::CellRendererText
     }
     
   protected:    
-    void CellRendererProgress::render_vfunc(const Glib::RefPtr<Gdk::Window>& window,
+    void render_vfunc(const Glib::RefPtr<Gdk::Window>& window,
                                               Gtk::Widget& widget,
                                               const Gdk::Rectangle& bg_area,
                                               const Gdk::Rectangle& cell_area,
