@@ -20,6 +20,7 @@
 
 #include "MAPanel.h"
 #include <MGTableBrowserList.h>
+
 #include <myx_admin_public_interface.h>
 
 class MDataInterface;
@@ -91,13 +92,12 @@ class MACatalogsPanel : public MAPanel {
     class ViewColumns : public Gtk::TreeModel::ColumnRecord {
       public:
         ViewColumns() {
-          add(_icon); add(_name); add(_type); add(_tables); add(_time);
+          add(_icon); add(_name); add(_comment); add(_updatable);
         };
         Gtk::TreeModelColumn<Glib::RefPtr<Gdk::Pixbuf> > _icon;
         Gtk::TreeModelColumn<Glib::ustring> _name;
-        Gtk::TreeModelColumn<Glib::ustring> _type;
-        Gtk::TreeModelColumn<Glib::ustring> _tables;
-        Gtk::TreeModelColumn<Glib::ustring> _time;
+        Gtk::TreeModelColumn<Glib::ustring> _comment;
+        Gtk::TreeModelColumn<Glib::ustring> _updatable;
     } _view_columns;
 
     class UserColumns : public Gtk::TreeModel::ColumnRecord {
@@ -141,13 +141,16 @@ class MACatalogsPanel : public MAPanel {
     class ProcedureColumns : public Gtk::TreeModel::ColumnRecord {
       public:
         ProcedureColumns() {
-          add(_icon); add(_name); add(_parameter); add(_ctime); add(_utime);
+          add(_icon); add(_name); add(_definer); add(_created); 
+          add(_modified); add(_return_datatype); add(_comment);
         };
         Gtk::TreeModelColumn<Glib::RefPtr<Gdk::Pixbuf> > _icon;
         Gtk::TreeModelColumn<Glib::ustring> _name;
-        Gtk::TreeModelColumn<Glib::ustring> _parameter;
-        Gtk::TreeModelColumn<Glib::ustring> _ctime;
-        Gtk::TreeModelColumn<Glib::ustring> _utime;
+        Gtk::TreeModelColumn<Glib::ustring> _definer;
+        Gtk::TreeModelColumn<Glib::ustring> _created;
+        Gtk::TreeModelColumn<Glib::ustring> _modified;
+        Gtk::TreeModelColumn<Glib::ustring> _return_datatype;
+        Gtk::TreeModelColumn<Glib::ustring> _comment;
     } _procedure_columns;
     
     class MaintTableColumns : public Gtk::TreeModel::ColumnRecord {
@@ -163,9 +166,10 @@ class MACatalogsPanel : public MAPanel {
     Gtk::Menu _schemata_menu;
 
     MYX_SCHEMA_TABLES *_tables;
-    MYX_SCHEMA_TABLE_STATUS *_table_status;
-
-    MGTableEditor *_editor;
+    //MYX_SCHEMA_TABLE_STATUS *_table_status;
+    MYX_SCHEMA_ENTITY_STATUS *_entity_status;
+    
+    int _open_editors;
     
     bool _details_shown;
     
@@ -174,6 +178,8 @@ class MACatalogsPanel : public MAPanel {
     Glib::RefPtr<Gdk::Pixbuf> _schema_icon;
     Glib::RefPtr<Gdk::Pixbuf> _sys_schema_icon;
     Glib::RefPtr<Gdk::Pixbuf> _table_icon;
+    Glib::RefPtr<Gdk::Pixbuf> _proc_icon;
+    Glib::RefPtr<Gdk::Pixbuf> _view_icon;
     Glib::RefPtr<Gdk::Pixbuf> _column_icon;
     Glib::RefPtr<Gdk::Pixbuf> _key_column_icon;
 
@@ -232,14 +238,14 @@ class MACatalogsPanel : public MAPanel {
     void create_table();
     void show_create_table();
     void edit_table();
-		void edit_table_data();
-		void drop_table();
+    void edit_table_data();
+    void drop_table();
     
     
     void close_create_table_window(GdkEventAny *ev);
     void save_create_table(const char *script);
 
-    void table_editor_closed();
+    void table_editor_closed(MGTableEditor *editor);
     void table_edit_mode(bool flag);
     
     void table_maintenance(MaintenanceCommand command= Select);
@@ -249,6 +255,19 @@ class MACatalogsPanel : public MAPanel {
     void schema_selected(MGBrowserList *sender,
                          const Gtk::TreeIter &node);
 
+    void create_procedure();
+    void create_function();
+    void create_procedure_impl(MYX_SCHEMA_STORED_PROCEDURE_TYPE t);
+    void edit_procedure();
+    void drop_procedure();
+
+    void create_view();
+    void edit_view();
+    void drop_view();
+
+    char *run_script_editor(const char *title, const char *sql);
+    char *input_string_dialog(const char *title, const char *prompt);
+
   public:
     MACatalogsPanel(MAdministrator *app, MDataInterface *info);
     ~MACatalogsPanel();
