@@ -169,10 +169,11 @@ class TimeGraph : public MAHealthPanel::BaseGraph {
       {
         if (_meter)
           delete _meter;
+        _meter= 0;
       }
 
       if (_meter)
-        _meter->set_text_format(Glib::ustring((char*)graph->value_caption)+" %i%%");
+        _meter->set_text_format(Glib::ustring((char*)graph->value_caption?:"")+" %i%%");
 
       if (graph->graph_caption && graph->display_graph_caption)
         _graph->set_caption(graph->graph_caption);
@@ -186,7 +187,7 @@ class TimeGraph : public MAHealthPanel::BaseGraph {
       if (graph->max_formula)
         _max_formula= graph->max_formula;
 
-      _caption= graph->graph_caption;
+      _caption= graph->graph_caption?:"";
       _expr= graph->value_formula;
       if (_cexpr)
         myx_free_expression(_cexpr);
@@ -281,7 +282,7 @@ class BarGraph : public MAHealthPanel::BaseGraph {
       _hbox= Gtk::manage(new Gtk::HBox(false, 8));
       _hbox->show();
 
-      _label= Gtk::manage(new Gtk::Label((char*)graph->graph_caption, 0.0, 0.5));
+      _label= Gtk::manage(new Gtk::Label((char*)graph->graph_caption?:"", 0.0, 0.5));
       _label->show();
       _hbox->pack_start(*_label, false, true);
 
@@ -297,7 +298,7 @@ class BarGraph : public MAHealthPanel::BaseGraph {
     virtual void modify(MYX_HEALTH_GRAPH *graph)
     {
       if (graph->value_caption)
-        _graph->set_current_format(Glib::ustring((char*)graph->value_caption));
+        _graph->set_current_format(Glib::ustring((char*)graph->value_caption?:""));
       else
         _graph->set_current_format("");
 
@@ -408,13 +409,13 @@ MAHealthPanel::MAHealthPanel(MAdministrator *app, MDataInterface *data)
   _graph_menu.show_all();
 
   // setup group menu
-  item= Gtk::manage(new Gtk::MenuItem(_("Add Group")));
+  item= Gtk::manage(new Gtk::MenuItem(_("Add a Group")));
   item->signal_activate().connect(sigc::mem_fun(*this,&MAHealthPanel::add_group_mi));
   _group_menu.append(*item);
   item= Gtk::manage(new Gtk::MenuItem(_("Rename Group")));
   item->signal_activate().connect(sigc::mem_fun(*this,&MAHealthPanel::rename_group_mi));
   _group_menu.append(*item);
-  item= Gtk::manage(new Gtk::MenuItem(_("Add Graph")));
+  item= Gtk::manage(new Gtk::MenuItem(_("Add a Graph")));
   item->signal_activate().connect(sigc::mem_fun(*this,&MAHealthPanel::add_graph_mi));
   _group_menu.append(*item);
   _group_menu.append(*Gtk::manage(new Gtk::SeparatorMenuItem()));
@@ -425,7 +426,7 @@ MAHealthPanel::MAHealthPanel(MAdministrator *app, MDataInterface *data)
   _group_menu.show_all();
   
   // setup page menu
-  item= Gtk::manage(new Gtk::MenuItem(_("Add Page")));
+  item= Gtk::manage(new Gtk::MenuItem(_("Add a Page")));
   item->signal_activate().connect(sigc::mem_fun(*this,&MAHealthPanel::add_page_mi));
   _page_menu.append(*item);
   item= Gtk::manage(new Gtk::MenuItem(_("Edit Page")));
@@ -1252,7 +1253,7 @@ bool MAHealthPanel::commit_graph_edit(MYX_HEALTH_GRAPH *graph)
   graph->value_formula= g_strdup(_graph_dlg->get_text("formula_text")->get_buffer()->get_text().c_str());
   str= _graph_dlg->get_entry("max_formula_entry")->get_text();
   graph->max_formula= str.empty()?0:g_strdup(str.c_str());
-  str= _graph_dlg->get_entry("max_caption_entry")->get_text();
+  str= _graph_dlg->get_entry("value_caption_entry")->get_text();
   graph->value_caption= str.empty()?0:g_strdup(str.c_str());
   str= _graph_dlg->get_entry("max_caption_entry")->get_text();
   graph->max_caption= str.empty()?0:g_strdup(str.c_str());
