@@ -20,8 +20,8 @@
 #include "MGPreferences.h"
 
 class MAPreferences : public MGPreferences {
-    virtual void process_options(MYX_APPLICATION_OPTIONS *options);
-    virtual MYX_APPLICATION_OPTIONS *prepare_options();
+    virtual void process_options(MYX_APPLICATION_OPTIONS *options, bool shared);
+    virtual MYX_APPLICATION_OPTIONS *prepare_options(bool shared);
     
   public:
     MAPreferences();
