@@ -38,16 +38,16 @@ Glib::ustring MAPreferenceGroup::get_title()
 void MAPreferenceGroup::init()
 {
   _xml->get_toggle("admin_show_global_check")->
-    signal_toggled().connect(SigC::slot(*this,&MAPreferenceGroup::set_dirty));
+    signal_toggled().connect(sigc::mem_fun(*this,&MAPreferenceGroup::set_dirty));
 
   _xml->get_toggle("admin_show_table_check")->
-    signal_toggled().connect(SigC::slot(*this,&MAPreferenceGroup::set_dirty));
+    signal_toggled().connect(sigc::mem_fun(*this,&MAPreferenceGroup::set_dirty));
  
   _xml->get_toggle("timestamp_backups_check")->
-    signal_toggled().connect(SigC::slot(*this,&MAPreferenceGroup::set_dirty));
+    signal_toggled().connect(sigc::mem_fun(*this,&MAPreferenceGroup::set_dirty));
   
   _xml->get_spin("connlist_autorefresh")->
-    signal_changed().connect(SigC::slot(*this,&MAPreferenceGroup::set_dirty));
+    signal_changed().connect(sigc::mem_fun(*this,&MAPreferenceGroup::set_dirty));
 }
 
 
