@@ -42,6 +42,7 @@ struct LoadThreadArgument {
   std::string path;
   std::string charset;
   ProgressData pdata;
+  MYX_BACKUP_ERROR error;
 };
 
 
@@ -177,7 +178,7 @@ void MARestorePanel::show_content_info(const std::string &file,
     Gtk::TreeIter sit, tit;
     Gtk::TreeModel::Row row, trow;
     bool found= false;
-
+    
     // check if this schema is already in the list
     for (sit= _selection_store->children().begin();
          sit != _selection_store->children().end(); ++sit)
@@ -345,6 +346,7 @@ void MARestorePanel::file_selected(MGBrowserList *sender, const Gtk::TreeIter &n
     std::string item;
     std::string path;
     std::string charset;
+    MYX_BACKUP_ERROR error;
 
     _browser->get_row_object(node, item);
 
@@ -370,20 +372,23 @@ void MARestorePanel::file_selected(MGBrowserList *sender, const Gtk::TreeIter &n
     }
 
     _app->push_status(_("Processing backup file..."));
-    _cur_content= perform_load(path, charset);
+    _cur_content= perform_load(path, charset, error);
     _cur_content_charset= charset;
     
     if (!_cur_content)
     {
       if (!op_aborted)
       {
+        char *ptr= myx_get_backup_error_string(error);
         Gtk::MessageDialog dlg(*_app->window(),
-                               ufmt(_("Error reading backup file\n'%s'"),
-                                    path.c_str()),
-                               false,
+                               ufmt(_("<b>Error Reading Backup File</b>\n\nFile: %s\nError: %s"),
+                                    path.c_str(),
+                                    ptr),
+                               true,
                                Gtk::MESSAGE_ERROR,
                                Gtk::BUTTONS_OK,
                                true);
+        g_free(ptr);
         dlg.run();
       }
       _app->pop_status();
@@ -456,13 +461,13 @@ bool MARestorePanel::update_status(void *data)
 
   if (ldata->ready)
   {
-		double d = (double)ldata->bytes/(double)ldata->bytes_total;
-		if(d < 0.01)	// workaround for a gtkmm (2.2.12) bug (?)
-									// if 0.004 <= d < 0.007 it displays progress fraction == 100%
-		{
-			d = 0.01;
-		}
-		_progress->set_fraction(d);
+    double d = (double)ldata->bytes/(double)ldata->bytes_total;
+    if(d < 0.01)	// workaround for a gtkmm (2.2.12) bug (?)
+      // if 0.004 <= d < 0.007 it displays progress fraction == 100%
+    {
+      d = 0.01;
+    }
+    _progress->set_fraction(d);
     _progress->set_text(ufmt(_("%'lli of %'lli bytes"), ldata->bytes, ldata->bytes_total));
     ldata->ready= false;
   }
@@ -484,7 +489,6 @@ void MARestorePanel::close_progress()
 void *MARestorePanel::load_thread(void *data)
 {
   LoadThreadArgument *arg;
-  MYX_BACKUP_ERROR err;
   MYX_BACKUP_CONTENT *content;
   
   mysql_thread_init();
@@ -494,9 +498,7 @@ void *MARestorePanel::load_thread(void *data)
   content= myx_get_backup_content(arg->path.c_str(), arg->charset.c_str(),
                                 MYX_BT_SQL_SCRIPT,
                                 1024, // update every 1024 bytes
-                                progress_cb, &arg->pdata, &err);
-  if (err != MYX_BACKUP_NO_ERROR)
-    g_message("Error reading backup content file: %i", err);
+                                progress_cb, &arg->pdata, &arg->error);
   mysql_thread_end();
   return content;
 }
@@ -549,7 +551,8 @@ void MARestorePanel::restore_warning_cb(const char *msg, void *data)
 
 
 MYX_BACKUP_CONTENT *MARestorePanel::perform_load(const std::string &file,
-                                                 const std::string &charset)
+                                                 const std::string &charset,
+                                                 MYX_BACKUP_ERROR &error)
 {
   LoadThreadArgument arg;
   MDispatcher disp;
@@ -593,6 +596,8 @@ MYX_BACKUP_CONTENT *MARestorePanel::perform_load(const std::string &file,
 
   get_toggle("option1_check")->set_active(false);
   get_toggle("option2_check")->set_active(true);
+  
+  error= arg.error;
 
   return content;
 }
@@ -609,7 +614,7 @@ bool MARestorePanel::check_tables_to_be_dropped(MYX_BACKUP_CONTENT *content)
     return true;
   }
   
-  Glib::ustring msg;
+  Glib::ustring msg, text;
 
   // check for tables that will be restored
   for (unsigned int i= 0; i < drop_list->tables_num; i++)
@@ -622,9 +627,9 @@ bool MARestorePanel::check_tables_to_be_dropped(MYX_BACKUP_CONTENT *content)
   
   myx_free_backup_content(drop_list);
   
-  msg= _("The following tables in the backup file already exist and will be dropped before restoring from backup.\nWould you like to drop these tables and proceed with restoration?");
-  msg= "\n"+msg;  
-  Gtk::MessageDialog dlg(msg, false, Gtk::MESSAGE_WARNING, Gtk::BUTTONS_YES_NO);
+  text= _("The following tables in the backup file already exist and will be dropped before restoring from backup.\nWould you like to drop these tables and proceed with restoration?");
+  text+= "\n"+msg;
+  Gtk::MessageDialog dlg(text, false, Gtk::MESSAGE_WARNING, Gtk::BUTTONS_YES_NO);
 
   if (dlg.run()==Gtk::RESPONSE_NO)
   {
