@@ -77,8 +77,8 @@ void MAServerConnectionsPanel::setup_thread_list(Gtk::TreeView *tree)
     column= tree->get_column(i);
     column->set_clickable();
     column->set_resizable(true);
-    column->signal_clicked().connect(SigC::bind<Gtk::TreeView*,int>
-                                     (SigC::slot(*this,&MAServerConnectionsPanel::list_column_clicked),
+    column->signal_clicked().connect(sigc::bind<Gtk::TreeView*,int>
+                                     (sigc::mem_fun(*this,&MAServerConnectionsPanel::list_column_clicked),
                                       tree, i));
   }
 }
@@ -102,8 +102,8 @@ void MAServerConnectionsPanel::setup_user_list(Gtk::TreeView *tree)
     column= tree->get_column(i);
     column->set_clickable();
     column->set_resizable(true);
-    column->signal_clicked().connect(SigC::bind<Gtk::TreeView*,int>
-                                     (SigC::slot(*this,&MAServerConnectionsPanel::list_column_clicked),
+    column->signal_clicked().connect(sigc::bind<Gtk::TreeView*,int>
+                                     (sigc::mem_fun(*this,&MAServerConnectionsPanel::list_column_clicked),
                                       tree, i));
   }
 }
@@ -122,44 +122,44 @@ bool MAServerConnectionsPanel::init()
   _thread_icon= PIXCACHE->load("16x16_Thread.png");
   
   
-  ((Gtk::Notebook*)get_widget("notebook"))->signal_switch_page().connect(SigC::slot(*this,&MAServerConnectionsPanel::notebook_switched_page));
-  get_button("kill_user_button")->signal_clicked().connect(SigC::slot(*this,&MAServerConnectionsPanel::kill_user_clicked));
-  get_button("kill_thread_button")->signal_clicked().connect(SigC::slot(*this,&MAServerConnectionsPanel::kill_thread_clicked));
-  get_button("refresh_button")->signal_clicked().connect(SigC::slot(*this,&MAServerConnectionsPanel::refresh_thread_list));
-  get_button("copy_button")->signal_clicked().connect(SigC::slot(*this,&MAServerConnectionsPanel::copy_info_clicked));
+  ((Gtk::Notebook*)get_widget("notebook"))->signal_switch_page().connect(sigc::mem_fun(*this,&MAServerConnectionsPanel::notebook_switched_page));
+  get_button("kill_user_button")->signal_clicked().connect(sigc::mem_fun(*this,&MAServerConnectionsPanel::kill_user_clicked));
+  get_button("kill_thread_button")->signal_clicked().connect(sigc::mem_fun(*this,&MAServerConnectionsPanel::kill_thread_clicked));
+  get_button("refresh_button")->signal_clicked().connect(sigc::mem_fun(*this,&MAServerConnectionsPanel::refresh_thread_list));
+  get_button("copy_button")->signal_clicked().connect(sigc::mem_fun(*this,&MAServerConnectionsPanel::copy_info_clicked));
   
   _thread_list= Gtk::ListStore::create(_thr_columns);
   _user_list= Gtk::ListStore::create(_user_columns);
   _user_thread_list= Gtk::ListStore::create(_thr_columns);
 
-  _thread_list->set_default_sort_func(SigC::bind<Glib::RefPtr<Gtk::ListStore>*>
-                                      (SigC::slot(*this, &MAServerConnectionsPanel::compare_thr_row),
+  _thread_list->set_default_sort_func(sigc::bind<Glib::RefPtr<Gtk::ListStore>*>
+                                      (sigc::mem_fun(*this, &MAServerConnectionsPanel::compare_thr_row),
                                        &_thread_list));
-  _user_thread_list->set_default_sort_func(SigC::bind<Glib::RefPtr<Gtk::ListStore>*>
-                                           (SigC::slot(*this, &MAServerConnectionsPanel::compare_thr_row),
+  _user_thread_list->set_default_sort_func(sigc::bind<Glib::RefPtr<Gtk::ListStore>*>
+                                           (sigc::mem_fun(*this, &MAServerConnectionsPanel::compare_thr_row),
                                             &_user_thread_list));
 
   _thread_tree= tree= (Gtk::TreeView*)get_widget("threads_tree");
   tree->set_model(_thread_list);
   setup_thread_list(tree);
-  tree->get_selection()->signal_changed().connect(SigC::slot(*this,&MAServerConnectionsPanel::update_sensitivity));
+  tree->get_selection()->signal_changed().connect(sigc::mem_fun(*this,&MAServerConnectionsPanel::update_sensitivity));
 
   _user_tree= tree= (Gtk::TreeView*)get_widget("users_tree");
   tree->set_model(_user_list);
   setup_user_list(tree);
-  tree->get_selection()->signal_changed().connect(SigC::slot(*this,&MAServerConnectionsPanel::user_list_clicked));
+  tree->get_selection()->signal_changed().connect(sigc::mem_fun(*this,&MAServerConnectionsPanel::user_list_clicked));
 
   _user_thread_tree= tree= (Gtk::TreeView*)get_widget("user_threads_tree");
   tree->set_model(_user_thread_list);
   setup_thread_list(tree);
-  tree->get_selection()->signal_changed().connect(SigC::slot(*this,&MAServerConnectionsPanel::update_sensitivity));
+  tree->get_selection()->signal_changed().connect(sigc::mem_fun(*this,&MAServerConnectionsPanel::update_sensitivity));
 
   _default_user_icon= PIXCACHE->load("16x16_User.png");
   
   if (prefs.connection_list_auto_refresh > 0)
-    _timer= Glib::signal_timeout().connect(SigC::slot(*this,&MAServerConnectionsPanel::refresh_thread_list_b), prefs.connection_list_auto_refresh*1000);
+    _timer= Glib::signal_timeout().connect(sigc::mem_fun(*this,&MAServerConnectionsPanel::refresh_thread_list_b), prefs.connection_list_auto_refresh*1000);
 
-  _app->signal_prefs_changed().connect(SigC::slot(*this,&MAServerConnectionsPanel::prefs_changed));
+  _app->signal_prefs_changed().connect(sigc::mem_fun(*this,&MAServerConnectionsPanel::prefs_changed));
 
   return true;
 }
@@ -171,7 +171,7 @@ void MAServerConnectionsPanel::prefs_changed()
     _timer.disconnect();
 
   if (prefs.connection_list_auto_refresh > 0)
-    _timer= Glib::signal_timeout().connect(SigC::slot(*this,&MAServerConnectionsPanel::refresh_thread_list_b), prefs.connection_list_auto_refresh*1000);
+    _timer= Glib::signal_timeout().connect(sigc::mem_fun(*this,&MAServerConnectionsPanel::refresh_thread_list_b), prefs.connection_list_auto_refresh*1000);
 }
 
 
@@ -287,7 +287,7 @@ void MAServerConnectionsPanel::refresh_thread_list()
 
       row[_thr_columns._col_icon]= _thread_icon;
       row[_thr_columns._col_pid]= (char*)plist->process_infos[i].id;
-      row[_thr_columns._col_user_icon]= !info.icon.is_null() ? info.icon: _default_user_icon;
+      row[_thr_columns._col_user_icon]= info.icon ? info.icon: _default_user_icon;
       row[_thr_columns._col_user]= (char*)plist->process_infos[i].user;
       row[_thr_columns._col_host]= (char*)plist->process_infos[i].host;
       row[_thr_columns._col_db]= (char*)plist->process_infos[i].db?:"";
@@ -304,7 +304,7 @@ void MAServerConnectionsPanel::refresh_thread_list()
         
         users[(char*)plist->process_infos[i].user]= jter;
 
-        jrow[_user_columns._col_user_icon]= !info.icon.is_null() ?info.icon: _default_user_icon;
+        jrow[_user_columns._col_user_icon]= info.icon ?info.icon: _default_user_icon;
         jrow[_user_columns._col_user]= (char*)plist->process_infos[i].user;
         jrow[_user_columns._col_full_name]= info.fullname;
         jrow[_user_columns._col_nthreads]= 1;
@@ -467,7 +467,7 @@ void MAServerConnectionsPanel::user_list_clicked()
         
         row[_thr_columns._col_icon]= _thread_icon;
         row[_thr_columns._col_pid]= (char*)plist->process_infos[i].id;
-        row[_thr_columns._col_user_icon]= !info.icon.is_null()?info.icon:_default_user_icon;
+        row[_thr_columns._col_user_icon]= info.icon?info.icon:_default_user_icon;
         row[_thr_columns._col_user]= (char*)plist->process_infos[i].user;
         row[_thr_columns._col_host]= (char*)plist->process_infos[i].host;
         row[_thr_columns._col_db]= (char*)plist->process_infos[i].db;
