@@ -32,12 +32,140 @@
 #define PAGE_BLOCK_SIZE (1024*5)
 
 
+class MGFindDialog : public Gtk::Window {
+    Gtk::VBox vbox;
+    Gtk::VBox cbox;
+    Gtk::HBox bbox;
+    Gtk::Entry entry;
+    Gtk::Button find_button;
+    Gtk::Button cancel_button;
+    Gtk::CheckButton case_check;
+    Gtk::Label status;
+    Glib::RefPtr<Gtk::SizeGroup> sgroup;
+    
+    //Gtk::TextView *text;
+    Gtk::TreeView *tree;
+    int column;
+    
+    void event_delete(GdkEventAny *ev);
+    void pressed_find();
+    void pressed_cancel();
+    
+  public:
+    MGFindDialog(const Glib::ustring &title, Gtk::TreeView *tree, int column);
+};
+
+
+void MGFindDialog::event_delete(GdkEventAny *ev)
+{
+  pressed_cancel();
+}
+
+
+void MGFindDialog::pressed_find()
+{  
+  Glib::ustring text= entry.get_text();
+  bool case_sens= case_check.get_active();
+  Gtk::TreeIter iter, end;
+  Gtk::TreeRow row;
+
+  iter= tree->get_selection()->get_selected();
+  if (!iter)
+  {
+    iter= tree->get_model()->children().begin();
+    status.set_text("");
+  }
+  else
+    ++iter;
+  end= tree->get_model()->children().end();
+
+  if (!case_sens)
+    text= text.casefold();
+  
+  while (iter != end)
+  {
+    Glib::ustring line;
+    
+    row= *iter;
+    row.get_value(column, line);
+
+    if (!case_sens)
+      line= line.casefold();
+
+    if (line.find(text) != Glib::ustring::npos)
+    {
+      tree->get_selection()->select(iter);
+      status.set_text("");
+      break;
+    }
+    ++iter;
+  }
+  if (iter == end)
+  {
+    status.set_text("Text not found");
+  }
+}
+
+
+void MGFindDialog::pressed_cancel()
+{
+  hide();
+  delete this;
+}
+
+
+
+MGFindDialog::MGFindDialog(const Glib::ustring &title, Gtk::TreeView *aTree, int aColumn)
+  : Gtk::Window(Gtk::WINDOW_TOPLEVEL), vbox(false, 12), cbox(false, 8), bbox(false, 12),
+   find_button(Gtk::Stock::FIND), cancel_button(Gtk::Stock::CANCEL),
+   case_check(_("Case-sensitive"))
+{
+  tree= aTree;
+  column= aColumn;
+
+  set_position(Gtk::WIN_POS_MOUSE);
+  set_title(_("Find Text in Page"));
+  add(vbox);
+  vbox.set_border_width(12);
+  
+  sgroup= Gtk::SizeGroup::create(Gtk::SIZE_GROUP_HORIZONTAL);
+  
+  set_transient_for(*(Gtk::Window*)aTree->get_toplevel());
+  set_default_size(300, 100);
+
+  vbox.pack_start(entry, false, false);
+  vbox.pack_start(cbox, true, true);
+  vbox.pack_start(bbox, false, false);
+
+  cbox.pack_start(case_check, false, false);
+  cbox.pack_start(status, false, false);
+
+  bbox.pack_end(find_button, false, true);
+  bbox.pack_end(cancel_button, false, true);
+
+  sgroup->add_widget(find_button);
+  sgroup->add_widget(cancel_button);
+  
+  find_button.signal_clicked().connect(sigc::mem_fun(*this,&MGFindDialog::pressed_find));
+  cancel_button.signal_clicked().connect(sigc::mem_fun(*this,&MGFindDialog::pressed_cancel));
+  
+  show_all();
+}
+
+
+
 MAServerLogsPanel::MAServerLogsPanel(MAdministrator *app, MDataInterface *data)
   : MAPanel(app, data)
 {
 }
 
 
+void MAServerLogsPanel::search_log(LogType type)
+{
+  MGFindDialog *dlg= new MGFindDialog(_("Find Text in Page"), _list[type], 0);
+}
+
+
 void MAServerLogsPanel::setup_panel(LogType type)
 {
   Gtk::TreeView *tree= _tree[type];
@@ -101,6 +229,9 @@ bool MAServerLogsPanel::init()
     sprintf(buffer, "%s_select", types[i]);
     get_button(buffer)->signal_clicked().connect(sigc::bind<LogType>(sigc::mem_fun(*this,&MAServerLogsPanel::select_log),(LogType)i));
 
+    sprintf(buffer, "%s_search", types[i]);
+    get_button(buffer)->signal_clicked().connect(sigc::bind<LogType>(sigc::mem_fun(*this,&MAServerLogsPanel::search_log),(LogType)i));
+
     sprintf(buffer, "%s_save", types[i]);
     _saveb[i]= get_button(buffer);
     _saveb[i]->signal_clicked().connect(sigc::bind<LogType>(sigc::mem_fun(*this,&MAServerLogsPanel::save_log),(LogType)i));
