@@ -416,7 +416,7 @@ bool MAUserAdministrationPanel::init()
     Gtk::MenuItem *item;
     
     _schemas_popup_menu= new Gtk::Menu();
-    item= make_item(_menu, _("_Remove Schema from Priv List"), Gtk::Stock::NEW);
+    item= make_item(_schemas_popup_menu, _("_Remove Schema from Priv List"), Gtk::Stock::NEW);
     //_user_menu_items[i++]= item;
     //item->signal_activate().connect(sigc::mem_fun(*this, &MAUserAdministrationPanel::add_user_activate));
     _schema_browser->set_popup_menu(_schemas_popup_menu);    
@@ -449,7 +449,6 @@ void MAUserAdministrationPanel::show()
       myg_show_error(_("Could not retrieve user privilege information."));
       return;
     }
-    _first_show= false;
   }
 
   _user_names= _data->get_user_names();
@@ -466,6 +465,8 @@ void MAUserAdministrationPanel::show()
   }
   */
   reload_catalogs();
+  
+  _first_show= false;
 }
 
 
@@ -536,9 +537,9 @@ bool MAUserAdministrationPanel::add_host_to_user(MYX_USER *user, const Glib::ust
   if (!host.empty() && !dupe)
   {
     user->hosts_num++;
-    user->hosts= (unsigned char**)myx_resize_vector_block((char*)user->hosts,
+    user->hosts= (char**)myx_resize_vector_block((char*)user->hosts,
                                                           sizeof(char*), user->hosts_num);
-    user->hosts[user->hosts_num-1]= (unsigned char*)g_strdup(host.c_str());
+    user->hosts[user->hosts_num-1]= (char*)g_strdup(host.c_str());
     
     return true;
   }
@@ -555,10 +556,10 @@ MYX_USER_OBJECT_PRIVILEGES *MAUserAdministrationPanel::clone_privilege_struct(MY
   new_priv->host= NULL;
   new_priv->object_name= NULL;
   new_priv->user_privileges_num= priv->user_privileges_num;
-  new_priv->user_privileges= (unsigned char**)myx_resize_vector_block(NULL, sizeof(char*), priv->user_privileges_num);
+  new_priv->user_privileges= (char**)myx_resize_vector_block(NULL, sizeof(char*), priv->user_privileges_num);
   for (unsigned int i= 0; i < priv->user_privileges_num; i++)
   {
-    new_priv->user_privileges[i]= (unsigned char*)g_strdup((gchar*)priv->user_privileges[i]);
+    new_priv->user_privileges[i]= (char*)g_strdup((gchar*)priv->user_privileges[i]);
   }
 
   return new_priv;
@@ -578,22 +579,22 @@ static void* fetch_privilege_templates_worker(MYSQL *mysql)
 {
   WantedPrivileges *privs= g_new0(WantedPrivileges, 1);
   
-  if (!(privs->global= myx_get_privilege_struct(mysql, "")))
+  if (!(privs->global= myx_get_privilege_struct(mysql, "", MYX_UOP_GLOBAL)))
     goto end;
-  if (!(privs->schema= myx_get_privilege_struct(mysql, "x")))
+  if (!(privs->schema= myx_get_privilege_struct(mysql, "x", MYX_UOP_SCHEMA)))
     goto end;
   if (mysql_version_is_later_or_equal_than(mysql,5,0))
   {
-    if (!(privs->proc= myx_get_privilege_struct(mysql, "x.x")))
+    if (!(privs->proc= myx_get_privilege_struct(mysql, "x.x", MYX_UOP_ROUTINE)))
       goto end;
   }
   else
   {
     privs->proc= NULL;
   }    
-  if (!(privs->table= myx_get_privilege_struct(mysql, "mysql.user")))
+  if (!(privs->table= myx_get_privilege_struct(mysql, "mysql.user", MYX_UOP_TABLE)))
     goto end;
-  if (!(privs->column= myx_get_privilege_struct(mysql, "x.x.x")))
+  if (!(privs->column= myx_get_privilege_struct(mysql, "x.x.x", MYX_UOP_COLUMN)))
     goto end;
   return privs;
 end:
@@ -694,8 +695,8 @@ MYX_USER_OBJECT_PRIVILEGES *
         g_assert_not_reached();
       }
 
-      priv->host= (unsigned char*)g_strdup(host.c_str());
-      priv->object_name= (unsigned char*)g_strdup(object.c_str());
+      priv->host= g_strdup(host.c_str());
+      priv->object_name= g_strdup(object.c_str());
 
       _object_privilege[key] = priv;
     }
@@ -889,7 +890,7 @@ void MAUserAdministrationPanel::apply_shown_user()
 {
   Gtk::Entry *entry;
 
-#define REPLACE(s, us) if (s) g_free(s); s= (unsigned char*)g_strdup(us.c_str())
+#define REPLACE(s, us) if (s) g_free(s); s= g_strdup(us.c_str())
 
   entry= (Gtk::Entry*)get_widget("user_entry");
   REPLACE(_current_user->user_name, entry->get_text());
@@ -916,7 +917,7 @@ void MAUserAdministrationPanel::apply_shown_user()
   {
     if (_current_user->icon)
       free(_current_user->icon);
-    _current_user->icon= _current_icon_data;
+    _current_user->icon= (char*)_current_icon_data;
     _current_user->icon_length= _current_icon_data_length;
   }
 }
@@ -1268,10 +1269,10 @@ void MAUserAdministrationPanel::limit_changed(const char *w)
                   (char*)name, (char*)value.c_str()) < 0)
   {
     privs->user_privileges_num++;
-    privs->user_privileges= (unsigned char**)myx_resize_vector_block((char*)privs->user_privileges,
+    privs->user_privileges= (char**)myx_resize_vector_block((char*)privs->user_privileges,
                                                                      sizeof(char*), privs->user_privileges_num);
     privs->user_privileges[privs->user_privileges_num-1]=
-      (unsigned char*)g_strdup_printf("%s=%s", name, value.c_str());
+      (char*)g_strdup_printf("%s=%s", name, value.c_str());
   }
 
   set_dirty();
@@ -1580,13 +1581,13 @@ void MAUserAdministrationPanel::create_new_user()
       }
     } while (dupe);
 
-    user->user_name= (unsigned char*)g_strdup(new_name);
-    user->full_name= (unsigned char*)g_strdup("");
-    user->password= (unsigned char*)g_strdup("");
-    user->full_name= (unsigned char*)g_strdup("");
-    user->description= (unsigned char*)g_strdup("");
-    user->email= (unsigned char*)g_strdup("");
-    user->contact_information= (unsigned char*)g_strdup("");
+    user->user_name= g_strdup(new_name);
+    user->full_name= g_strdup("");
+    user->password= g_strdup("");
+    user->full_name= g_strdup("");
+    user->description= g_strdup("");
+    user->email= g_strdup("");
+    user->contact_information= g_strdup("");
 
     add_host_to_user(user, "%");
     
@@ -1654,6 +1655,18 @@ void MAUserAdministrationPanel::clone_user()
 
 void MAUserAdministrationPanel::remove_user()
 {
+  if (strcmp(_current_user->user_name, "root")==0)
+  {
+    Gtk::MessageDialog dlg(*_app->window(),
+                           _("You cannot delete the root user."),
+                           false,
+                           Gtk::MESSAGE_WARNING,
+                           Gtk::BUTTONS_OK,
+                           true);
+    dlg.run();
+    return;
+  }
+
   Gtk::MessageDialog dlg(*_app->window(),
                          ufmt(_("Permanently remove account '%s' from MySQL server at %s:%i?"),
                               _current_user->user_name,//_current_user_name.c_str(),
@@ -1996,7 +2009,23 @@ void MAUserAdministrationPanel::remove_host()
 {
   if (_current_host.empty())
     return;
-  
+
+  if (strcmp(_current_user->user_name, "root")==0)
+  {
+    if (_current_user->hosts_num == 1)
+    {
+      Gtk::MessageDialog dlg(*_app->window(),
+                             ufmt(_("Cannot totally remove access from root user.\nPlease remove manually if you are sure about that."),
+                                  _current_host.c_str()),
+                             false,
+                             Gtk::MESSAGE_WARNING,
+                             Gtk::BUTTONS_OK,
+                             true);
+      dlg.run();
+      return;
+    }
+  }
+
   if (_current_user->hosts_num == 1)
   {
     Gtk::MessageDialog dlg(*_app->window(),
