@@ -237,14 +237,17 @@ bool MAdministrator::init(MInstanceInfo *inst)
   _win->signal_delete_event().connect(sigc::mem_fun(*this,&MAdministrator::close_cb));
   _xml->get_menu_item("new_connection1")->signal_activate().connect(sigc::mem_fun(*this,&MAdministrator::new_connection_cb));
   _xml->get_menu_item("close_connection1")->signal_activate().connect(sigc::mem_fun(*this,&MAdministrator::close_connection_cb));
+  _xml->get_menu_item("reconnect1")->signal_activate().connect(sigc::mem_fun(*this,&MAdministrator::reconnect_cb));
   _xml->get_menu_item("quit1")->signal_activate().connect(sigc::mem_fun(*this,&MAdministrator::menu_quit_cb));
 
 //  _xml->get_menu_item("sidebar1")->signal_activate().connect(sigc::mem_fun(*this,&MAdministrator::toggle_sidebar_cb));
   _xml->get_menu_item("contents1")->signal_activate().connect(sigc::mem_fun(*this,&MAdministrator::open_help));
   _xml->get_menu_item("about1")->signal_activate().connect(sigc::mem_fun(*this,&MAdministrator::menu_about_cb));
 
-  _xml->get_menu_item("preferences1")->signal_activate().connect(sigc::mem_fun(*this,&MAdministrator::menu_preferences_cb));
-  
+  _xml->get_menu_item("manage_connections1")->signal_activate().connect(sigc::bind<bool>(sigc::mem_fun(*this,&MAdministrator::menu_preferences_cb),true));
+  _xml->get_menu_item("preferences1")->signal_activate().connect(sigc::bind<bool>(sigc::mem_fun(*this,&MAdministrator::menu_preferences_cb),false));
+  _xml->get_menu_item("mysql_text_console1")->signal_activate().connect(sigc::mem_fun(*this,&MAdministrator::start_console));
+
   _progressbar= (Gtk::ProgressBar*)get_widget("status_progress");
 
   _dummy_panel= new MADummyPanel(this, _dataif);
@@ -732,15 +735,16 @@ void MAdministrator::menu_about_cb()
 }
 
 
-void MAdministrator::menu_preferences_cb()
+void MAdministrator::menu_preferences_cb(bool conn_only)
 {
   if (!_prefs_editor)
   {
     _prefs_editor= MGPreferencesEditor::instance();
+    _prefs_editor->set_transient_for(*_win);
     _prefs_editor->signal_changed().connect(sigc::mem_fun(*this,&MAdministrator::prefs_changed));
   }
 
-  _prefs_editor->show();
+  _prefs_editor->show(conn_only);
 }
 
 
@@ -930,3 +934,32 @@ void MAdministrator::paste_cb()
     tview->get_buffer()->paste_clipboard(Gtk::Clipboard::get());
   }
 }
+
+
+
+void MAdministrator::start_console()
+{
+  MYX::UserConnection conn= _instance->get_connection_data();
+
+  if (!Glib::find_program_in_path("gnome-terminal").empty())
+    Glib::spawn_command_line_async(ufmt("gnome-terminal -e \"mysql -u%s -h%s -P%i %s\"",
+                                        conn.username.c_str(), conn.hostname.c_str(), conn.port,
+                                        conn.password.empty()?"":std::string("-p"+conn.password).c_str()));
+  else
+    Glib::spawn_command_line_async(ufmt("xterm -e \"mysql -u%s -h%s -P%i %s\"",
+                                        conn.username.c_str(), conn.hostname.c_str(), conn.port,
+                                        conn.password.empty()?"":std::string("-p"+conn.password).c_str()));
+}
+
+
+void MAdministrator::reconnect_cb()
+{
+  if (!_instance->reconnect())
+  {
+    set_status(_("Reconnection failed."));
+  }
+  else
+    set_status(_("Reconnected."));
+}
+
+
