@@ -84,6 +84,7 @@ class MAdministrator : public Glib::ObjectBase {
     
     void new_connection_cb();
     void close_connection_cb();
+    void reconnect_cb();
     
     void update_edit_menu();
     void cut_cb();
@@ -91,9 +92,11 @@ class MAdministrator : public Glib::ObjectBase {
     void paste_cb();
 
     void menu_quit_cb();
-    void menu_preferences_cb();
+    void menu_preferences_cb(bool conn_only);
     void menu_about_cb();
 
+    void start_console();
+
     void disconnect_db_handler();
     void connect_db_handler();
     
