@@ -367,7 +367,7 @@ Gtk::Widget *MDynamicInterface::process_group(MYX_GUI_GROUP *group)
       
       check->set_active(group->widgets[i].active!=0);
       
-      check->signal_toggled().connect(SigC::bind<const char*>(SigC::slot(*this,&MDynamicInterface::option_enable_toggled),(char*)group->widgets[i].id));
+      check->signal_toggled().connect(sigc::bind<const char*>(sigc::mem_fun(*this,&MDynamicInterface::option_enable_toggled),(char*)group->widgets[i].id));
       
       table->attach(*check, 0, 1, y, y+1, Gtk::FILL, Gtk::FILL);
 
@@ -450,8 +450,8 @@ void MDynamicInterface::add_enabled_by_handler(MYX_GUI_ENABLED_BY_CONTROL *info,
   switch (obj.type)
   {
   case MYX_CHECKBOX:
-    ((Gtk::CheckButton*)obj.w)->signal_toggled().connect(SigC::bind<WSensControl*>
-                                        (SigC::slot(*this,&MDynamicInterface::update_sensitivity),
+    ((Gtk::CheckButton*)obj.w)->signal_toggled().connect(sigc::bind<WSensControl*>
+                                        (sigc::mem_fun(*this,&MDynamicInterface::update_sensitivity),
                                          data));
     break;
   default:
@@ -857,7 +857,7 @@ void MInnoDBFilePathEditor::update_fs_list()
   _fs_list->clear();
   if (_instance->get_fs_info(fsinfo))
   {  
-    _fs_list->set_sort_func(0, SigC::slot(*this, &MInnoDBFilePathEditor::fscompare));
+    _fs_list->set_sort_func(0, sigc::mem_fun(*this, &MInnoDBFilePathEditor::fscompare));
     _fs_list->set_sort_column_id(0, Gtk::SORT_DESCENDING);
 
     for (std::list<MInstanceInfo::FilesystemInfo>::iterator it= fsinfo.begin();
@@ -969,7 +969,7 @@ long long MInnoDBFilePathEditor::parse_size(const Glib::ustring &size)
 {
   long long value= 1;
 
-  sscanf(size.substr(0, size.size()-1).c_str(), "%lf", &value);
+  sscanf(size.substr(0, size.size()-1).c_str(), "%lli", &value);
 
   switch (size.uppercase()[size.size()-1])
   {
@@ -1117,7 +1117,7 @@ MFilePathEditor::MFilePathEditor(MDynamicInterface *owner, bool dir_only)
   pack_start(_entry);
   pack_start(_button, false, false);
   
-  _button.signal_clicked().connect(SigC::slot(*this,&MFilePathEditor::clicked));
+  _button.signal_clicked().connect(sigc::mem_fun(*this,&MFilePathEditor::clicked));
   show_all();
 }
 
@@ -1235,13 +1235,13 @@ MInnoDBFilePathEditor::MInnoDBFilePathEditor(MDynamicInterface *owner, MInstance
 
   attach(_add_button, 1, 2, 0, 1, Gtk::FILL, Gtk::SHRINK);
   _add_button.show();
-  _add_button.signal_clicked().connect(SigC::slot(*this,&MInnoDBFilePathEditor::add_file));
+  _add_button.signal_clicked().connect(sigc::mem_fun(*this,&MInnoDBFilePathEditor::add_file));
 
   attach(_remove_button, 1, 2, 1, 2, Gtk::FILL, Gtk::SHRINK);
   _remove_button.show();
-  _remove_button.signal_clicked().connect(SigC::slot(*this,&MInnoDBFilePathEditor::remove_file));
+  _remove_button.signal_clicked().connect(sigc::mem_fun(*this,&MInnoDBFilePathEditor::remove_file));
 
-  _tree.get_selection()->signal_changed().connect(SigC::slot(*this,&MInnoDBFilePathEditor::update_sensitivity));
+  _tree.get_selection()->signal_changed().connect(sigc::mem_fun(*this,&MInnoDBFilePathEditor::update_sensitivity));
 
   _tree.set_size_request(180, 100);
 
@@ -1256,7 +1256,7 @@ MInnoDBFilePathEditor::MInnoDBFilePathEditor(MDynamicInterface *owner, MInstance
     Gtk::TreeView *tree= (Gtk::TreeView*)_dlg_xml->get_widget("fs_tree");
 
     tree->get_selection()->set_mode(Gtk::SELECTION_BROWSE);
-    tree->get_selection()->signal_changed().connect(SigC::slot(*this,&MInnoDBFilePathEditor::selected_fs));
+    tree->get_selection()->signal_changed().connect(sigc::mem_fun(*this,&MInnoDBFilePathEditor::selected_fs));
 
     tree->append_column(_("Path"), _fs_columns._path);
     tree->append_column(_("Type"), _fs_columns._type);
@@ -1267,11 +1267,11 @@ MInnoDBFilePathEditor::MInnoDBFilePathEditor(MDynamicInterface *owner, MInstance
     _fs_list= Gtk::ListStore::create(_fs_columns);
     tree->set_model(_fs_list);
 
-    _dlg_xml->get_widget("graph")->signal_expose_event().connect(SigC::slot(*this,&MInnoDBFilePathEditor::expose_graph));;
+    _dlg_xml->get_widget("graph")->signal_expose_event().connect(sigc::mem_fun(*this,&MInnoDBFilePathEditor::expose_graph));;
 
-    ((Gtk::Entry*)_dlg_xml->get_widget("name_entry"))->signal_changed().connect(SigC::slot(*this,&MInnoDBFilePathEditor::update_dlg_sensitivity));
-    ((Gtk::SpinButton*)_dlg_xml->get_widget("size_spin"))->signal_changed().connect(SigC::slot(*this,&MInnoDBFilePathEditor::selected_fs));
-    ((Gtk::OptionMenu*)_dlg_xml->get_widget("unit_menu"))->signal_changed().connect(SigC::slot(*this,&MInnoDBFilePathEditor::selected_fs));
+    ((Gtk::Entry*)_dlg_xml->get_widget("name_entry"))->signal_changed().connect(sigc::mem_fun(*this,&MInnoDBFilePathEditor::update_dlg_sensitivity));
+    ((Gtk::SpinButton*)_dlg_xml->get_widget("size_spin"))->signal_changed().connect(sigc::mem_fun(*this,&MInnoDBFilePathEditor::selected_fs));
+    ((Gtk::OptionMenu*)_dlg_xml->get_widget("unit_menu"))->signal_changed().connect(sigc::mem_fun(*this,&MInnoDBFilePathEditor::selected_fs));
   }
 
   {
