@@ -125,10 +125,13 @@ MInstanceInfo::SavedInfo::SavedInfo(const MYX::UserConnection &user_conn)
 }
 
 
-void MInstanceInfo::SavedInfo::process_options(MYX_APPLICATION_OPTIONS *options)
+void MInstanceInfo::SavedInfo::process_options(MYX_APPLICATION_OPTIONS *options, bool shared)
 {
   const char *value;
 
+  if (shared)
+    return;
+  
   for (unsigned int i= 0; i < options->option_groups_num; i++)
   {
     MYX_OPTION_GROUP *group= options->option_groups+i;
@@ -152,23 +155,27 @@ void MInstanceInfo::SavedInfo::process_options(MYX_APPLICATION_OPTIONS *options)
 
 
 
-MYX_APPLICATION_OPTIONS *MInstanceInfo::SavedInfo::prepare_options()
+MYX_APPLICATION_OPTIONS *MInstanceInfo::SavedInfo::prepare_options(bool shared)
 {
-  MYX_APPLICATION_OPTIONS *options= (MYX_APPLICATION_OPTIONS*)g_malloc0(sizeof(MYX_APPLICATION_OPTIONS));
-  std::list<ValuePair> l;
+  if (!shared)
+  {
+    MYX_APPLICATION_OPTIONS *options= (MYX_APPLICATION_OPTIONS*)g_malloc0(sizeof(MYX_APPLICATION_OPTIONS));
+    std::list<ValuePair> l;
 
-  l.clear();
-  l.push_back(ValuePair("error_log",error_log_path));
-  l.push_back(ValuePair("general_log",general_log_path));
-  l.push_back(ValuePair("slow_log",slow_log_path));
+    l.clear();
+    l.push_back(ValuePair("error_log",error_log_path));
+    l.push_back(ValuePair("general_log",general_log_path));
+    l.push_back(ValuePair("slow_log",slow_log_path));
 
-  add_group(options, "log_files", l);
+    add_group(options, "log_files", l);
   
-  l.clear();
-  l.push_back(ValuePair("mycnf_path",mycnf_path));
-  add_group(options, "general", l);
-
-  return options;
+    l.clear();
+    l.push_back(ValuePair("mycnf_path",mycnf_path));
+    add_group(options, "general", l);
+    
+    return options;
+  }
+  return 0;
 }
 
 
@@ -205,6 +212,25 @@ void MInstanceInfo::disconnect()
 }
 
 
+bool MInstanceInfo::reconnect()
+{
+  MYSQL *mysql;
+  MYX_USER_CONNECTION conn;
+  
+  _user_conn.fill(&conn);
+  
+  mysql= myx_mysql_init();
+  if (myx_connect_to_instance(&conn, mysql) < 0)
+    return false;
+
+  if (_mysql)
+    myx_mysql_close(_mysql);
+  _mysql= mysql;
+
+  return true;
+}
+
+
 void MInstanceInfo::set_connection(MYSQL *mysql, 
                                    const MYX::UserConnection &user_conn)
 {
