@@ -558,18 +558,21 @@ again:
   
   if (row < 0) return;
   
-  //myx_use_schema([_owner mysql], [self selectedSchema]->schema_name);
+  myx_push_schema([_owner mysql], [self selectedSchema]->schema_name);
   
   editor= [[[MSQLEditor alloc] initWithMySQL:[_owner mysql]] autorelease];
   
   viewS= [self objectOfType:MYX_ENTITY_VIEW atIndex:row];
-  
+retry:  
   view= [[_owner dispatcher] performCallback: (void*(*)(MYSQL*,void*,void*,void*))myx_dbm_get_view_data
                                     argument: [self selectedSchema]->catalog_name    
                                     argument: [self selectedSchema]->schema_name
                                     argument: viewS->view_name
                                waitForWindow: [topBox window]
                                      message: @"Getting view definition..."];    
+  if (!view && ![[_owner dispatcher] checkConnection] && [[_owner dispatcher] reconnect])
+    goto retry;
+  
   if (!view)
   {
     if (mysql_error([_owner mysql]) != 0)
@@ -578,6 +581,7 @@ again:
                                     [NSString stringWithFormat:@"Could not retrieve definition of view %s",viewS->view_name],
                                     [_owner mysql]);
     }
+    myx_pop_schema([_owner mysql]);
     return;
   }
   [editor setScript:NSStr(view->definition)];
@@ -611,6 +615,7 @@ again:
     
     [self refreshContents:nil];
   }
+  myx_pop_schema([_owner mysql]);
 }
 
 
@@ -631,11 +636,15 @@ again:
     return;
   }
   
+retry:
   error= (MYX_LIB_ERROR)[[_owner dispatcher] performQuery: [NSString stringWithFormat:@"DROP VIEW `%s`.`%s`", [self selectedSchema]->schema_name, view]
                                             waitForWindow: [topBox window]
                                                   message: @"Dropping view..."];
   if (error != MYX_NO_ERROR)
   {
+    if (![[_owner dispatcher] checkConnection] && [[_owner dispatcher] reconnect])
+      goto retry;
+    
     NSRunAlertPanel(@"Error",
                     [NSString stringWithFormat: @"Error dropping view %s: %s", 
                       view, mysql_error([_owner mysql])], @"OK", nil, nil);
@@ -650,7 +659,7 @@ again:
 - (IBAction)dropView:(id)sender
 {
   MYX_VIEW_STATUS *view;
-  
+    
   view= [self objectOfType:MYX_ENTITY_VIEW atIndex:[viewTable selectedRow]];
   
   NSAlert *alert = [NSAlert alertWithMessageText:@"Drop View?"
@@ -704,7 +713,12 @@ again:
                                waitForWindow: [topBox window]
                                      message: [NSString stringWithFormat:@"Executing %@...",what]];
     if (error == MYX_SQL_ERROR)
+    {
+      if (![[_owner dispatcher] checkConnection] && [[_owner dispatcher] reconnect])
+        goto again;
+      
       MXRunAlertPanelWithMySQLError([NSString stringWithFormat:@"Error executing %@.",what], @"", [_owner mysql]);
+    }
     else if (error != MYX_NO_ERROR)
       MXRunAlertPanelWithError(@"Error", [NSString stringWithFormat:@"Error executing %@.",what], error);
 
@@ -735,6 +749,7 @@ again:
  
   editor= [[[MSQLEditor alloc] initWithMySQL:[_owner mysql]] autorelease];
     
+retry:
   spdata= [[_owner dispatcher] performCallback: (void*(*)(MYSQL*,void*,void*,void*,void*))myx_dbm_get_sp_data
                                        argument: [self selectedSchema]->catalog_name
                                        argument: [self selectedSchema]->schema_name
@@ -744,6 +759,9 @@ again:
                                         message: @"Getting routine definition..."];    
   if (!spdata)
   {
+    if (![[_owner dispatcher] checkConnection] && [[_owner dispatcher] reconnect])
+      goto retry;
+    
     if (mysql_error([_owner mysql]) != 0)
     {
       MXRunAlertPanelWithMySQLError(@"Error",
@@ -765,7 +783,9 @@ again:
   {
     MYX_LIB_ERROR error;
     
-    error= [[_owner dispatcher] performQuery: [NSString stringWithFormat:@"DROP %@ `%s`.`%s`",sptype,[self selectedSchema]->schema_name,spdata->name]
+    myx_use_schema([_owner mysql], [self selectedSchema]->schema_name);
+    
+    error= [[_owner dispatcher] performQuery: [NSString stringWithFormat:@"DROP %@ IF EXISTS `%s`.`%s`",sptype,[self selectedSchema]->schema_name,spdata->name]
                                waitForWindow: [topBox window]
                                      message: @"Dropping Old Routine..."];
     
@@ -870,20 +890,24 @@ again:
   if (schema)
   {
     void *res;
+retry:
     res=[[_owner dispatcher] performCallback:(void*(*)(MYSQL*,void*,void*))myx_get_schema_entity_status
                                     argument:schema->catalog_name
                                     argument:schema->schema_name
                                waitForWindow:[_owner window]
                                      message:[NSString stringWithFormat:@"Fetching objects from '%s'...",schema->schema_name]];
+    if (!res && ![[_owner dispatcher] checkConnection] && [[_owner dispatcher] reconnect])
+      goto retry;
+    
     [self objectStatusArrived:nil result:res];
     
     [tabView setHidden:NO];
-    [[topBox viewWithTag:50] setHidden:YES];
+    [emptyText setHidden:YES];
   }
   else
   {
     [tabView setHidden:YES];
-    [[topBox viewWithTag:50] setHidden:NO];
+    [emptyText setHidden:NO];
 
     [self objectStatusArrived:nil result:NULL];
   }
@@ -1016,6 +1040,14 @@ again:
   [delSchemaButton setAlternateImage:[NSImage imageNamed:@"del_pressed.png"]];
   [delSchemaButton setEnabled:NO];
   
+  [tablesTable setDoubleAction:@selector(editTable:)];
+  [viewTable setDoubleAction:@selector(editView:)];
+  [viewTable setTarget:self];
+  [spTable setDoubleAction:@selector(editProcedure:)];
+  [spTable setTarget:self];
+  [funcTable setDoubleAction:@selector(editProcedure:)];
+  [funcTable setTarget:self];
+  
   [[topBox viewWithTag:200] setToolTip:@"Create a new schema (database)."];
   [[topBox viewWithTag:201] setToolTip:@"Drop the selected schema and all its tables and other assets."];
   
