@@ -3,6 +3,7 @@
 #import <MySQLToolsCommon/MTextImageCell.h>
 #import <MySQLToolsCommon/MMySQLDispatcher.h>
 #import <MySQLToolsCommon/MPreferences.h>
+#import <MySQLToolsCommon/MConnectionInfo.h>
 #import <MySQLToolsCommon/mxUtils.h>
 #import <MySQLToolsCommon/MDialogs.h>
 #include <sys/time.h>
@@ -313,11 +314,14 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
 {
   unsigned int j;
   [_items release];
-  _items= [[NSMutableArray alloc] initWithCapacity:sg->variables_num];
-  for (j= 0; j < sg->variables_num; j++)
+  _items= [[NSMutableArray alloc] initWithCapacity:sg ? sg->variables_num : 0];
+  if (sg)
   {
-    Item *vitem= [Item itemWithPtr:sg->variables+j type:'V'];
-    [_items addObject:vitem];
+    for (j= 0; j < sg->variables_num; j++)
+    {
+      Item *vitem= [Item itemWithPtr:sg->variables+j type:'V'];
+      [_items addObject:vitem];
+    }
   }
 }
 
@@ -325,12 +329,15 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
 {
   unsigned int j;
   [_items release];
-  _items= [[NSMutableArray alloc] initWithCapacity:g->variables_num];
-  for (j= 0; j < g->variables_num; j++)
+  _items= [[NSMutableArray alloc] initWithCapacity:g ? g->variables_num : 0];
+  if (g)
   {
-    Item *vitem= [Item itemWithPtr:g->variables+j type:'V'];
-    [_items addObject:vitem];
-  }  
+    for (j= 0; j < g->variables_num; j++)
+    {
+      Item *vitem= [Item itemWithPtr:g->variables+j type:'V'];
+      [_items addObject:vitem];
+    }
+  }
 }
 
 - (NSArray*)items
@@ -432,8 +439,10 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
   if (myx_connect_to_instance(conn,mysql) < 0)
   {
     NSLog(@"Could not connect to MySQL instance");
+    myx_free_user_connection_content(conn);
+    g_free(conn);
     myx_mysql_close(mysql);
-	mysql= NULL;
+    mysql= NULL;
     return;
   }
   
@@ -471,7 +480,10 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
     }
   }
   myx_mysql_close(mysql);
-  
+
+  myx_free_user_connection_content(conn);
+  g_free(conn);
+
   [self release];
 }
 
@@ -563,20 +575,31 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
 {
   id sender= [notification object];
   Item *item= (Item*)[sender itemAtRow:[sender selectedRow]];
+  
   if (sender == statusIndex)
   {
-    if (item->type == 'S')
-      [[statusTable dataSource] setFromSubgroup:item->ptr];
+    if (item)
+    {
+      if (item->type == 'S')
+        [[statusTable dataSource] setFromSubgroup:item->ptr];
+      else
+        [[statusTable dataSource] setFromGroup:item->ptr];
+    }
     else
-      [[statusTable dataSource] setFromGroup:item->ptr];
+      [[statusTable dataSource] setFromGroup:NULL];
     [statusTable reloadData];
   }
   else
   {
-    if (item->type == 'S')
-      [[serverTable dataSource] setFromSubgroup:item->ptr];
+    if (item)
+    {
+      if (item->type == 'S')
+        [[serverTable dataSource] setFromSubgroup:item->ptr];
+      else
+        [[serverTable dataSource] setFromGroup:item->ptr];    
+    }
     else
-      [[serverTable dataSource] setFromGroup:item->ptr];    
+      [[serverTable dataSource] setFromGroup:NULL];
     [serverTable reloadData];
   }
 }
@@ -612,6 +635,13 @@ shouldEditTableColumn:(NSTableColumn *)aTableColumn
 
 - (void)variablesArrived:(id)arg result:(void*)result
 {
+  if (!result && ![[_owner dispatcher] checkConnection] && [[_owner dispatcher] reconnect])
+  {
+    [self refreshServerVariables:nil];
+    [self refreshStatusVariables:nil];
+    return;
+  }
+  
   if ([arg isEqualToString:@"status"])
   {
     if (_statusVariables)
@@ -636,6 +666,10 @@ shouldEditTableColumn:(NSTableColumn *)aTableColumn
 
 - (void)valueSaved:(id)argument result:(void*)result
 {
+  if (![[_owner dispatcher] checkConnection] && [[_owner dispatcher] reconnect])
+    ;
+    //[self setValue:
+    
   g_free((char*)argument);
   [self refreshServerVariables:self];
 }
@@ -874,9 +908,11 @@ shouldEditTableColumn:(NSTableColumn *)aTableColumn
     [self update:nil];
     
     _cancelThread= NO;
-    [NSThread detachNewThreadSelector:@selector(updateThread:)
-                             toTarget:self
-                           withObject:[NSValue valueWithPointer:[_owner serverInfo]]];
+    
+    if ([_owner serverInfo])
+      [NSThread detachNewThreadSelector:@selector(updateThread:)
+                               toTarget:self
+                             withObject:[NSValue valueWithPointer:[[_owner serverInfo] createUserConnection]]];
   }
   return self;
 }
