@@ -20,10 +20,13 @@
 + (NSImage*)icon;
 + (NSString*)label;
 + (NSString*)toolTip;
++ (BOOL)needsConnection;
 
 - (id)initWithNibFile:(NSString*)file
            panelOwner:(id<MAdministratorProtocol>)owner;
 
+- (id)initWithMessage:(NSString*)message owner:(id)owner;
+
 // must be overriden
 - (id)initWithOwner:(id<MAdministratorProtocol>)owner;
 
