@@ -3,7 +3,7 @@
 #import <Cocoa/Cocoa.h>
 #include "myx_admin_public_interface.h"
 
-typedef MYX_LOGFILE* (*MALogParserFunction)(const char*, int, int, int*);
+typedef MYX_LOGFILE* (*MALogParserFunction)(FILE*, int, int, int*);
 
 
 @interface MAServerLogView : NSObject
@@ -16,6 +16,8 @@ typedef MYX_LOGFILE* (*MALogParserFunction)(const char*, int, int, int*);
     IBOutlet id topView;
     
     NSString *_path;
+	NSString *_error;
+	FILE *_file;
     MYX_LOGFILE *_log;
     int *_lineIndex;
     
@@ -25,8 +27,11 @@ typedef MYX_LOGFILE* (*MALogParserFunction)(const char*, int, int, int*);
 - (IBAction)saveLog:(id)sender;
 - (IBAction)selectLog:(id)sender;
 
-- (id)initWithPath:(NSString*)path
-     parseFunction:(MALogParserFunction)func;
+- (id)initWithFile:(FILE*)file
+			  path:(NSString*)path
+			 error:(NSString*)error
+			parser:(MALogParserFunction)parser;
+
 - (NSView*)topView;
 
 @end
\ No newline at end of file
