@@ -7,14 +7,14 @@
 //
 
 #import <AppKit/AppKit.h>
-
+#import "MAMenuContextView.h"
 
 typedef struct {
   double value;
   time_t ts;
 } MATimeGraphSample;
 
-@interface MATimeGraph : NSView {
+@interface MATimeGraph : MAMenuContextView {
   MATimeGraphSample *_samples;
   int _maxSamples;
   int _sampleCount;
