@@ -53,11 +53,12 @@
     PrivilegeDataSource *_schemaPrivDS;
     PrivilegeDataSource *_tablePrivDS;
     PrivilegeDataSource *_columnPrivDS;
+    PrivilegeDataSource *_routinePrivDS;
 	
-	MFilteredSchemaDataSource *_schemaDS;
-	MFilteredSchemaDataSource *_tableDS;
-
-	id globalPageView;
+    MFilteredSchemaDataSource *_schemaDS;
+    MFilteredSchemaDataSource *_objectDS;
+    
+    id globalPageView;
     BOOL globalVisible;
     id tablePageView;
     BOOL tableVisible;
@@ -67,6 +68,7 @@
     MYX_USER_OBJECT_PRIVILEGES *_schemaPrivs;
     MYX_USER_OBJECT_PRIVILEGES *_tablePrivs;    
     MYX_USER_OBJECT_PRIVILEGES *_columnPrivs;
+    MYX_USER_OBJECT_PRIVILEGES *_routinePrivs;
     
     NSImage *_privilegeIcon;
 	
