@@ -1,5 +1,6 @@
 
 #include <glib.h>
+#include <mysqld_error.h>
 
 #import "MAUserAdministration.h"
 #import "MUserListDataSource.h"
@@ -11,7 +12,6 @@
 
 #import <MySQLToolsCommon/MOutlineView.h>
 #import <MySQLToolsCommon/MTextImageCell.h>
-#import <MySQLToolsCommon/MPtrWrap.h>
 
 #import <MySQLToolsCommon/mxUtils.h>
 
@@ -139,8 +139,12 @@ extern NSString *MASchemaDataSourceChanged;
 
   if (!names)
   {
-    MXRunAlertPanelWithMySQLError(nil, @"Could not retrieve user list:", 
-                                  [_owner mysql]);
+    if (myx_mysql_errno([_owner mysql]) == ER_TABLEACCESS_DENIED_ERROR)
+      NSRunAlertPanel(@"Error", @"Insufficient privileges for user account administration. Try reconnecting with the 'root' account.", 
+                      @"OK", nil, nil);
+    else
+      MXRunAlertPanelWithMySQLError(@"Error", @"Could not retrieve user list:", 
+                                    [_owner mysql]);
     return;
   }
   array= [[[NSMutableArray alloc] init] autorelease];
@@ -169,17 +173,33 @@ extern NSString *MASchemaDataSourceChanged;
 {
   MYX_USER *info= (MYX_USER*)infoptr;
   MUserItem *item;
+  
+  if (!infoptr)
+  {
+    if (![[_owner dispatcher] checkConnection])
+      [[_owner dispatcher] reconnect];
+    return;
+  }
 
   item= [_userDS itemForUser:[NSString stringWithUTF8String:(char*)arg]];
   if (item)
   {
     if (info)
     {
+      NSString *oldHost= [self selectedHost];
+      MUserHostItem *hitem;
+
       [item setUserInfo:info];
       [userOutline reloadItem:item reloadChildren:YES];
+      
+      hitem= [item itemForHost:oldHost];
+      if (hitem)
+        [userOutline selectRow:[userOutline rowForItem:hitem]
+          byExtendingSelection:NO];
+      
       // if the item is selected, display it
       if ([self selectedUser] == item)
-        [self showUser:item host: nil];
+        [self showUser:item host:[self selectedHost]];
     }
     else
     {
@@ -236,8 +256,9 @@ extern NSString *MASchemaDataSourceChanged;
   info->contact_information= g_strdup([[contactText string] UTF8String]);
   
   // save it in a thread and wait
+retry:
   username= g_strdup([[item name] UTF8String]);
-  result= (int)[[_owner dispatcher] performCallback:(void*(*)(MYSQL*,void*,void*,void*))myx_set_user
+  result= (long)[[_owner dispatcher] performCallback:(void*(*)(MYSQL*,void*,void*,void*))myx_set_user
                                            argument:info
                                            argument:username
                                            argument:(void*)([item isNewUser]?1:0)
@@ -246,10 +267,15 @@ extern NSString *MASchemaDataSourceChanged;
   g_free(username);
   if (result < 0)
   {
+    if (![[_owner dispatcher] checkConnection] && [[_owner dispatcher] reconnect])
+      goto retry;
+    
     [self showMessageSheet:@"Error"
                       info:@"Could not save changes to the user."];
     NSLog(@"Error: %s", myx_mysql_error([_owner mysql]));
   }
+  
+  [item setSaved];
 
   [self setNeedsSave:NO];
   return YES;
@@ -304,45 +330,13 @@ extern NSString *MASchemaDataSourceChanged;
   return nil;
 }
 
-- (void)addHostSheetDidEnd:(NSWindow *)sheet 
-         returnCode:(int)returnCode
-        contextInfo:(void *)contextInfo
-{
-  [sheet orderOut:self];
-  
-  if (returnCode == 1)
-  {
-    NSString *host;
-    MUserItem *user= [self selectedUser];
-    MUserHostItem *item;
-    
-    if ([[addHostMatrix cellWithTag:0] state] == NSOnState)
-      host= @"localhost";
-    else if ([[addHostMatrix cellWithTag:1] state] == NSOnState)
-      host= @"%";
-    else
-      host= [addHostText stringValue];
-
-    if (!(item= [user addHost:host]))
-      NSRunAlertPanel(@"Error", @"'%@' is already in the list of allowed hosts for the user.",
-                      @"OK", nil, nil, host);
-    else
-    {
-      [userOutline reloadItem:user reloadChildren:YES];
-      [userOutline selectRow:[userOutline rowForItem:item] byExtendingSelection:NO];
-      [self setNeedsSave:YES];
-    }
-  }
-  
-}
-
 @end
 
 
 @implementation MAUserAdministration
 
 
-static void grantSelectedRows(NSTableView *table, PrivilegeDataSource *ds)
+static BOOL grantSelectedRows(NSTableView *table, PrivilegeDataSource *ds)
 {
   NSIndexSet *rows= [table selectedRowIndexes];
   int i, c= [table numberOfRows];
@@ -357,14 +351,16 @@ static void grantSelectedRows(NSTableView *table, PrivilegeDataSource *ds)
       [tmparr addObject:[ds tableView:table objectValueForTableColumn:nil row:i]];
     }
   }
-  c= [rows count];
+  c= [tmparr count];
   for (i= 0; i < c; i++)
   {
     [ds grant: [tmparr objectAtIndex:i]];
   }
+  return c>0;
 }
 
-static void revokeSelectedRows(NSTableView *table, PrivilegeDataSource *ds)
+
+static BOOL revokeSelectedRows(NSTableView *table, PrivilegeDataSource *ds)
 {
   NSIndexSet *rows= [table selectedRowIndexes];
   int i, c= [table numberOfRows];
@@ -377,6 +373,7 @@ static void revokeSelectedRows(NSTableView *table, PrivilegeDataSource *ds)
       [ds revoke: [ds tableView:table objectValueForTableColumn:nil row:i]];
     }
   }
+  return c>0;
 }
 
 
@@ -406,6 +403,12 @@ static void revokeSelectedRows(NSTableView *table, PrivilegeDataSource *ds)
   {
     if ([item originalName])
     {
+      if ([[item originalName] isEqualTo:@"root"])
+      {
+        NSRunAlertPanel(@"Warning",@"You cannot remove the root user.",@"OK",nil,nil);
+        return;
+      }
+      
       NSAlert *alert = [NSAlert alertWithMessageText:@"Remove Account?"
                                        defaultButton:@"Remove" alternateButton:@"Cancel" otherButton:nil        
                            informativeTextWithFormat:@"The account '%@' will be permanently removed.", 
@@ -426,19 +429,19 @@ static void revokeSelectedRows(NSTableView *table, PrivilegeDataSource *ds)
         contextInfo:(void *)contextInfo
 {
   MUserItem *item= (MUserItem*)contextInfo;
-  
+
   [[alert window] orderOut:self];
-  
+
   if (returnCode == NSAlertDefaultReturn)
   {
     [self setNeedsSave:NO];
-  
+
     if ([item originalName])
       [[_owner dispatcher] performCallback:(void*(*)(MYSQL*,void*))myx_del_user
                                   argument:(char*)[[item originalName] UTF8String]
                              waitForWindow:[topBox window]
                                    message:@"Removing user..."];
-  
+
     [_userDS removeUser:item];
     [userOutline reloadData];
   }
@@ -447,6 +450,23 @@ static void revokeSelectedRows(NSTableView *table, PrivilegeDataSource *ds)
 
 - (IBAction)addHost:(id)sender
 {
+  if ([self needsSave])
+  {
+    int rc;
+    rc= NSRunAlertPanel(@"Add Host",@"There are changes that need to be saved or discarded before adding a new host.",
+                        @"Save Changes",@"Cancel",@"Discard Changes");
+    if (rc == NSAlertDefaultReturn)
+    {
+      [self saveChanges:nil];
+    }
+    else if (rc == NSAlertAlternateReturn)
+      return;
+    else
+    {
+      [self discardChanges:nil];
+    }
+  }
+
   [NSApp beginSheet:addHostPanel
      modalForWindow:[topBox window] 
                     modalDelegate:self
@@ -454,20 +474,61 @@ static void revokeSelectedRows(NSTableView *table, PrivilegeDataSource *ds)
                       contextInfo:NULL];
 }
 
+
+- (void)addHostSheetDidEnd:(NSWindow *)sheet 
+                returnCode:(int)returnCode
+               contextInfo:(void *)contextInfo
+{
+  [sheet orderOut:self];
+  
+  if (returnCode == 1)
+  {
+    NSString *host;
+    MUserItem *user= [self selectedUser];
+    MUserHostItem *item;
+
+    if ([[addHostMatrix cellWithTag:0] state] == NSOnState)
+      host= @"localhost";
+    else if ([[addHostMatrix cellWithTag:1] state] == NSOnState)
+      host= @"%";
+    else
+      host= [addHostText stringValue];
+
+    if (!(item= [user addHost:host]))
+      NSRunAlertPanel(@"Error", @"'%@' is already in the list of allowed hosts for the user.",
+                      @"OK", nil, nil, host);
+    else
+    {
+      [userOutline reloadItem:user reloadChildren:YES];
+      [userOutline selectRow:[userOutline rowForItem:item] byExtendingSelection:NO];
+      [self setNeedsSave:YES];
+    }
+  }
+}
+
+
 - (IBAction)removeHost:(id)sender
 {
   MUserItem *user= [self selectedUser];
   if ([self selectedHost])
   {
-	int userRow= [userOutline selectedRow];
+    int userRow= [userOutline selectedRow];
 
-	while (userRow >= 0 && [userOutline levelForRow:userRow]!=0)
-	  userRow--;
+    if ([[user name] isEqualTo:@"root"] && [[user hosts] count] == 1)
+    {
+      NSRunAlertPanel(@"Warning",
+                      @"You cannot totally remove access from the root user.\nPlease remove manually if you are sure about that.",
+                      @"OK",nil,nil);
+      return;
+    }
+
+    while (userRow >= 0 && [userOutline levelForRow:userRow]!=0)
+      userRow--;
     [user removeHost:[self selectedHost]];
     [self setNeedsSave:YES];
-	[userOutline selectRow:userRow
-	  byExtendingSelection:NO];
-	[userOutline reloadData];
+    [userOutline selectRow:userRow
+      byExtendingSelection:NO];
+    [userOutline reloadData];
   }
 }
 
@@ -482,7 +543,7 @@ static void revokeSelectedRows(NSTableView *table, PrivilegeDataSource *ds)
   MYX_USER *user= [[self selectedUser] userInfo];
 
   [panel setTitle:@"Open Icon"];
-  
+
   if ([panel runModalForTypes:[NSArray arrayWithObject:@"png"]] == NSOKButton)
   {
     NSData *data= [NSData dataWithContentsOfFile:[panel filename]];
@@ -513,57 +574,68 @@ static void revokeSelectedRows(NSTableView *table, PrivilegeDataSource *ds)
 
 - (IBAction)globalAddPrivilege:(id)sender
 {
-  grantSelectedRows(globalAvailableTable, _globalPrivDS);
-  [globalAvailableTable reloadData];
-  [globalAssignedTable reloadData];
-  [globalAvailableTable deselectAll: self];
-  [self setNeedsSave:YES];
+  if (grantSelectedRows(globalAvailableTable, _globalPrivDS))
+  {
+    [globalAvailableTable reloadData];
+    [globalAssignedTable reloadData];
+    [globalAvailableTable deselectAll: self];
+    [self setNeedsSave:YES];
+  }
 }
 
 - (IBAction)globalRemovePrivilege:(id)sender
 {
-  revokeSelectedRows(globalAssignedTable, _globalPrivDS);
-  [globalAvailableTable reloadData];
-  [globalAssignedTable reloadData];
-  [globalAssignedTable deselectAll: self];
-  [self setNeedsSave:YES];
+  if (revokeSelectedRows(globalAssignedTable, _globalPrivDS))
+  {
+    [globalAvailableTable reloadData];
+    [globalAssignedTable reloadData];
+    [globalAssignedTable deselectAll: self];
+    [self setNeedsSave:YES];
+  }
 }
 
 - (IBAction)schemaAddPrivilege:(id)sender
 {
-  grantSelectedRows(schemaAvailableTable, _schemaPrivDS);
-  [schemaAvailableTable reloadData];
-  [schemaAssignedTable reloadData];
-  [schemaAvailableTable deselectAll: self];  
-  
-  [self setNeedsSave:YES];
+  if (grantSelectedRows(schemaAvailableTable, _schemaPrivDS))
+  {
+    [schemaAvailableTable reloadData];
+    [schemaAssignedTable reloadData];
+    [schemaAvailableTable deselectAll: self];  
+    [self setNeedsSave:YES];
+  }
 }
 
 - (IBAction)schemaRemovePrivilege:(id)sender
 {
-  revokeSelectedRows(schemaAssignedTable, _schemaPrivDS);
-  [schemaAvailableTable reloadData];
-  [schemaAssignedTable reloadData];
-  [schemaAssignedTable deselectAll: self];  
-  [self setNeedsSave:YES];
+  if (revokeSelectedRows(schemaAssignedTable, _schemaPrivDS))
+  {
+    [schemaAvailableTable reloadData];
+    [schemaAssignedTable reloadData];
+    [schemaAssignedTable deselectAll: self];  
+    [self setNeedsSave:YES];
+  }
 }
 
 - (IBAction)tableAddPrivilege:(id)sender
 {
-  grantSelectedRows(tableAvailableTable, [tableAvailableTable dataSource]);
-  [tableAvailableTable reloadData];
-  [tableAssignedTable reloadData];
-  [tableAvailableTable deselectAll: self];  
-  [self setNeedsSave:YES];
+  if (grantSelectedRows(tableAvailableTable, [tableAvailableTable dataSource]))
+  {
+    [tableAvailableTable reloadData];
+    [tableAssignedTable reloadData];
+    [tableAvailableTable deselectAll: self];  
+    [self setNeedsSave:YES];
+  }
 }
 
 - (IBAction)tableRemovePrivilege:(id)sender
 {
-  revokeSelectedRows(tableAssignedTable, [tableAvailableTable dataSource]);
-  [tableAvailableTable reloadData];
-  [tableAssignedTable reloadData];
-  [tableAssignedTable deselectAll: self];
-  [self setNeedsSave:YES];
+  if (revokeSelectedRows(tableAssignedTable, [tableAvailableTable dataSource]))
+  {
+    [tableAvailableTable reloadData];
+    [tableAssignedTable reloadData];
+    [tableAssignedTable deselectAll: self];
+    [self setNeedsSave:YES];
+  }
 }
 
 - (IBAction)addHostAdd:(id)sender
@@ -602,15 +674,19 @@ static void revokeSelectedRows(NSTableView *table, PrivilegeDataSource *ds)
   [_tablePrivDS release];
   [_columnPrivDS release];
   [_schemaDS release];
-  [_tableDS release];
+  [_objectDS release];
 
   [globalPageView release];
   [tablePageView release];
   
-  myx_free_user_priv(_globalPrivs);
-  myx_free_user_priv(_schemaPrivs);
-  myx_free_user_priv(_tablePrivs);
-  myx_free_user_priv(_columnPrivs);
+  if (_globalPrivs)
+    myx_free_user_priv(_globalPrivs);
+  if (_schemaPrivs)
+    myx_free_user_priv(_schemaPrivs);
+  if (_tablePrivs)
+    myx_free_user_priv(_tablePrivs);
+  if (_columnPrivs)
+    myx_free_user_priv(_columnPrivs);
   
   [_privilegeIcon release];
   
@@ -623,16 +699,17 @@ typedef struct {
   MYX_USER_OBJECT_PRIVILEGES *schema;
   MYX_USER_OBJECT_PRIVILEGES *table;
   MYX_USER_OBJECT_PRIVILEGES *column;
+  MYX_USER_OBJECT_PRIVILEGES *routine;
 } ALL_PRIVILEGES;
 
 static ALL_PRIVILEGES *get_all_privileges(MYSQL *mysql)
 {
   ALL_PRIVILEGES *privs= g_new0(ALL_PRIVILEGES, 1);
-  
-  privs->global= myx_get_privilege_struct(mysql, "");
-  privs->schema= myx_get_privilege_struct(mysql, "x");
-  privs->table= myx_get_privilege_struct(mysql, "x.x");
-  privs->column= myx_get_privilege_struct(mysql, "x.x.x");
+  privs->global= myx_get_privilege_struct(mysql, "", MYX_UOP_GLOBAL);
+  privs->schema= myx_get_privilege_struct(mysql, "x", MYX_UOP_SCHEMA);
+  privs->table= myx_get_privilege_struct(mysql, "x.x", MYX_UOP_TABLE);
+  privs->column= myx_get_privilege_struct(mysql, "x.x.x", MYX_UOP_COLUMN);
+  privs->routine= myx_get_privilege_struct(mysql, "x.x", MYX_UOP_ROUTINE);
   return privs;
 }
 
@@ -662,11 +739,9 @@ static ALL_PRIVILEGES *get_all_privileges(MYSQL *mysql)
   }
 }
 
-- (id)initWithOwner: (id<MAdministratorProtocol>)owner
-{
-  self= [super initWithNibFile: @"UserAdministration" panelOwner: owner];
-  _defaultFrame= [[self topView] frame];
 
+- (void)setup
+{
   globalPageView= [[tabView tabViewItemAtIndex:1] retain];
   tablePageView= [[tabView tabViewItemAtIndex:3] retain];
   
@@ -694,8 +769,8 @@ static ALL_PRIVILEGES *get_all_privileges(MYSQL *mysql)
   [schemaOutline setDataSource: _schemaDS];
   MXExpandOutline(schemaOutline, NO);
 
-  _tableDS= [[MFilteredSchemaDataSource alloc] initWithDataSource:[_owner sharedColumnDS]];
-  [tableOutline setDataSource: _tableDS];
+  _objectDS= [[MFilteredSchemaDataSource alloc] initWithDataSource:[_owner sharedObjectDS]];
+  [tableOutline setDataSource: _objectDS];
   MXExpandOutline(tableOutline, NO);
   {
     NSMutableArray *array;
@@ -709,9 +784,9 @@ static ALL_PRIVILEGES *get_all_privileges(MYSQL *mysql)
     {
       for (i= 0; i < privs->global->user_privileges_num; i++)
       {
-        char *end= strstr(privs->global->user_privileges[i],"_priv");
+        char *end= strstr((char*)privs->global->user_privileges[i],"_priv");
         if (end)
-          [array addObject: [NSString stringWithCString: privs->global->user_privileges[i]
+          [array addObject: [NSString stringWithCString: (char*)privs->global->user_privileges[i]
                                                  length: end-(char*)privs->global->user_privileges[i]]];
       }
     }
@@ -729,9 +804,9 @@ static ALL_PRIVILEGES *get_all_privileges(MYSQL *mysql)
     {
       for (i= 0; i < privs->schema->user_privileges_num; i++)
       {
-        char *end= strstr(privs->schema->user_privileges[i],"_priv");
+        char *end= strstr((char*)privs->schema->user_privileges[i],"_priv");
         if (end)
-          [array addObject: [NSString stringWithCString: privs->schema->user_privileges[i]
+          [array addObject: [NSString stringWithCString: (char*)privs->schema->user_privileges[i]
                                                  length: end-(char*)privs->schema->user_privileges[i]]];
       }
     }
@@ -749,10 +824,10 @@ static ALL_PRIVILEGES *get_all_privileges(MYSQL *mysql)
     {
       for (i= 0; i < privs->table->user_privileges_num; i++)
       {
-        char *end= strchr(privs->table->user_privileges[i], '=');
-        if (end && g_str_has_prefix(privs->table->user_privileges[i], "Table_priv_"))
+        char *end= strchr((char*)privs->table->user_privileges[i], '=');
+        if (end && g_str_has_prefix((char*)privs->table->user_privileges[i], "Table_priv_"))
         {
-          char *begin= privs->table->user_privileges[i]+sizeof("Table_priv_")-1;
+          char *begin= (char*)privs->table->user_privileges[i]+sizeof("Table_priv_")-1;
           [array addObject: [NSString stringWithCString: begin
                                                  length: end - begin]];
         }
@@ -765,15 +840,34 @@ static ALL_PRIVILEGES *get_all_privileges(MYSQL *mysql)
     [tableAssignedTable setDataSource: _tablePrivDS];
     [tableAvailableTable setDataSource: _tablePrivDS];
 
+    array= [NSMutableArray array];
+    if (privs->routine)
+    {
+      for (i= 0; i < privs->routine->user_privileges_num; i++)
+      {
+        char *end= strchr((char*)privs->routine->user_privileges[i], '=');
+        if (end && g_str_has_prefix((char*)privs->routine->user_privileges[i], "Proc_priv_"))
+        {
+          char *begin= (char*)privs->routine->user_privileges[i]+sizeof("Proc_priv_")-1;
+          [array addObject: [NSString stringWithCString: begin
+                                                 length: end - begin]];
+        }
+      }
+    }
+    _routinePrivs= privs->routine;
+    _routinePrivDS= [[PrivilegeDataSource alloc] initWithPrivileges: array
+	                                                      sourceTable: tableAvailableTable
+                                                               type: PRoutine];
+    
     array= [NSMutableArray array];
     if (privs->column)
     {
       for (i= 0; i < privs->column->user_privileges_num; i++)
       {
-        char *end= strchr(privs->column->user_privileges[i], '=');
-        if (end && g_str_has_prefix(privs->column->user_privileges[i], "Column_priv_"))
+        char *end= strchr((char*)privs->column->user_privileges[i], '=');
+        if (end && g_str_has_prefix((char*)privs->column->user_privileges[i], "Column_priv_"))
         {
-          char *begin= privs->column->user_privileges[i]+sizeof("Column_priv_")-1;
+          char *begin= (char*)privs->column->user_privileges[i]+sizeof("Column_priv_")-1;
           [array addObject: [NSString stringWithCString: begin
                                                  length: end - begin]];
         }
@@ -790,7 +884,18 @@ static ALL_PRIVILEGES *get_all_privileges(MYSQL *mysql)
   _privilegeIcon= [[NSImage imageNamed:@"16x16_Right.png"] retain];
 
   [self showUser:nil host:nil];
+}
 
+
+- (id)initWithOwner: (id<MAdministratorProtocol>)owner
+{
+  self= [super initWithNibFile: @"UserAdministration" panelOwner: owner];
+  _defaultFrame= [[self topView] frame];
+  
+  if ([owner mysql])
+    [self setup];
+  else
+    [[topBox viewWithTag:1000] setStringValue:@"Not Connected"];
   return self;
 }
 
@@ -819,25 +924,9 @@ static ALL_PRIVILEGES *get_all_privileges(MYSQL *mysql)
   [self setButton:TAG_UserRemove enabled:NO image:@"deluser"];
   [self setButton:TAG_HostAdd enabled:NO image:@"addhost"];
   [self setButton:TAG_HostRemove enabled:NO image:@"delhost"];
-
-  {
-	NSView *view;
-	view= [[tabView tabViewItemAtIndex:1] view];
-	[[view viewWithTag:50] setToolTip:@"Grant selected privileges to the user."];
-	[[view viewWithTag:51] setToolTip:@"Revoke selected privileges from the user."];
-	view= [[tabView tabViewItemAtIndex:2] view];  
-	[[view viewWithTag:52] setToolTip:@"Grant selected privileges in the selected schema to the user."];
-	[[view viewWithTag:53] setToolTip:@"Revoke selected privileges in the selected schema from the user."];
-	view= [[tabView tabViewItemAtIndex:3] view];
-	[[view viewWithTag:54] setToolTip:@"Grant selected privileges in the selected object to the user."];
-	[[view viewWithTag:55] setToolTip:@"Revoke selected privileges in the selected object from the user."];
-	
-	[[topBox viewWithTag:200] setToolTip:@"Add a new user account. The account will only be valid once you add a host from where it can connect."];
-	[[topBox viewWithTag:201] setToolTip:@"Remove selected user account and all its registered hosts."];
-	
-	[[topBox viewWithTag:202] setToolTip:@"Add a host from where the selected user can connect."];
-	[[topBox viewWithTag:203] setToolTip:@"Remove the selected host from the user's allowed host list."];
-  }
+  
+  [[topBox viewWithTag:1000] setHidden:NO];
+  [tabView setHidden:YES];
 }
 
 
@@ -851,11 +940,11 @@ static ALL_PRIVILEGES *get_all_privileges(MYSQL *mysql)
   
   flag= anUser!=nil;
   
-  [[topBox viewWithTag:300] setEnabled:flag];
-  [[topBox viewWithTag:301] setEnabled:flag];
   [tabView setEnabledRecursive:flag];
+//  [[topBox viewWithTag:300] setEnabled:flag && [self needsSave]];
+//  [[topBox viewWithTag:301] setEnabled:flag && [self needsSave]];  
   
-#define STRVALUE(v) (user ? (user->v ?[NSString stringWithUTF8String:user->v]:@"") : @"")
+#define STRVALUE(v) (user ? (user->v ?[NSString stringWithUTF8String:(char*)(user->v)]:@"") : @"")
   [usernameText setStringValue: STRVALUE(user_name)];
   [passwordText setStringValue:@"________"]; // this is for telling the password didn't change
   [confirmText setStringValue:@"________"];
@@ -888,9 +977,13 @@ static ALL_PRIVILEGES *get_all_privileges(MYSQL *mysql)
   else
     tmp= @"No user selected";
   [userLabel1 setStringValue: tmp];
+  [userLabel1 setTextColor: user&&host ? [NSColor blackColor] : [NSColor redColor]];
   [userLabel2 setStringValue: tmp];
+  [userLabel2 setTextColor: user&&host ? [NSColor blackColor] : [NSColor redColor]];
   [userLabel3 setStringValue: tmp];
+  [userLabel3 setTextColor: user&&host ? [NSColor blackColor] : [NSColor redColor]];
   [userLabel4 setStringValue: tmp];
+  [userLabel4 setTextColor: user&&host ? [NSColor blackColor] : [NSColor redColor]];
 #undef VALUE
 
   if (user && host)
@@ -903,14 +996,17 @@ static ALL_PRIVILEGES *get_all_privileges(MYSQL *mysql)
     [globalAvailableTable reloadData];
     [globalAssignedTable deselectAll:self];
     [globalAvailableTable deselectAll:self];
-    
+
     [_schemaPrivDS setPrivileges: NULL];
+    [schemaOutline deselectAll:nil];
     [schemaAssignedTable reloadData];
     [schemaAvailableTable reloadData];
     [schemaAssignedTable deselectAll:self];
     [schemaAvailableTable deselectAll:self];
     [_tablePrivDS setPrivileges: NULL];
     [_columnPrivDS setPrivileges: NULL];
+    [_routinePrivDS setPrivileges: NULL];
+    [tableOutline deselectAll:nil];
     [tableAssignedTable reloadData];
     [tableAvailableTable reloadData];
     [tableAssignedTable deselectAll:self];
@@ -971,7 +1067,7 @@ static ALL_PRIVILEGES *get_all_privileges(MYSQL *mysql)
 
 - (IBAction)filterTableList:(id)sender
 {
-  [_tableDS performSearch:[sender stringValue]];
+  [_objectDS performSearch:[sender stringValue]];
   [tableOutline reloadData];
 }
 
@@ -983,6 +1079,13 @@ static ALL_PRIVILEGES *get_all_privileges(MYSQL *mysql)
 }
 
 
+- (void)setNeedsSave:(BOOL)flag
+{
+  [super setNeedsSave:flag];
+  [[topBox viewWithTag:300] setEnabled:flag];
+  [[topBox viewWithTag:301] setEnabled:flag];
+}
+
 
 //===================== Delegate for Outlines =======================
 
@@ -1039,6 +1142,9 @@ static ALL_PRIVILEGES *get_all_privileges(MYSQL *mysql)
       [self setButton:TAG_UserRemove enabled:NO image:@"deluser"];
       [self setButton:TAG_HostAdd enabled:NO image:@"addhost"];
       [self setButton:TAG_HostRemove enabled:NO image:@"delhost"];
+      
+      [[topBox viewWithTag:1000] setHidden:NO];
+      [tabView setHidden:YES];
     }
     else
     {
@@ -1064,6 +1170,9 @@ static ALL_PRIVILEGES *get_all_privileges(MYSQL *mysql)
     }
     [schemaOutline deselectAll:nil];
     [tableOutline deselectAll:nil];
+
+    [[topBox viewWithTag:1000] setHidden:YES];
+    [tabView setHidden:NO];
   }
   else if ([notification object] == schemaOutline)
   {
@@ -1154,6 +1263,28 @@ static ALL_PRIVILEGES *get_all_privileges(MYSQL *mysql)
       [tableAssignedTable reloadData];
       [tableAvailableTable reloadData];
     }
+    else if (item && item->type == MSPItemType)
+    {
+      MYX_SCHEMA *schema= [item schema];
+      MYX_SCHEMA_STORED_PROCEDURE *proc= [item sp];
+      if (proc && host && user)
+      {
+        NSString *object=[NSString stringWithFormat:@"%s.%s",schema->schema_name,proc->name];
+        privs= [user findPrivilegesForObject:object host:host];
+        if (!privs)
+          privs= [user addPrivileges:_routinePrivs forObject:object host:host];
+        [_routinePrivDS setPrivileges:privs];
+        editable= YES;
+      }
+      else
+      {
+        [_routinePrivDS setPrivileges:NULL];
+      }
+      [tableAssignedTable setDataSource:_routinePrivDS];
+      [tableAvailableTable setDataSource:_routinePrivDS];
+      [tableAssignedTable reloadData];
+      [tableAvailableTable reloadData];
+    }
     // refresh enabled controls
     [self setPage:3 enabled:editable];
   }
