@@ -52,7 +52,7 @@ static NSView *makeWidget(MYX_GUI_WIDGET *widget)
     {
       NSButton *btn= [[[NSButton alloc] init] autorelease];
       [btn setButtonType:NSSwitchButton];
-      [btn setTitle:[NSString stringWithUTF8String:widget->caption]];
+      [btn setTitle:[NSString stringWithUTF8String:(char*)widget->caption]];
       [btn sizeToFit];
       wid= btn;
       break;
@@ -138,8 +138,8 @@ static NSView *makeWidget(MYX_GUI_WIDGET *widget)
       
       for (i= 0; i < widget->dropdownbox->items_num; i++)
       {
-        char *ptr= strchr(widget->dropdownbox->items[i],'=');
-        [pop addItemWithTitle:[NSString stringWithCString:widget->dropdownbox->items[i] length:ptr-(char*)widget->dropdownbox->items[i]]];
+        char *ptr= strchr((char*)widget->dropdownbox->items[i],'=');
+        [pop addItemWithTitle:[NSString stringWithCString:(char*)widget->dropdownbox->items[i] length:ptr-(char*)widget->dropdownbox->items[i]]];
         [[pop lastItem] setRepresentedObject:[NSString stringWithCString:ptr+1]];
       }
       [pop sizeToFit];
@@ -184,7 +184,7 @@ static NSView *makeWidget(MYX_GUI_WIDGET *widget)
   {
     if (group->widgets[w].caption)
     {        
-      label_width= MAX(label_width, [font widthOfString:[NSString stringWithUTF8String:group->widgets[w].caption]]);
+      label_width= MAX(label_width, [font widthOfString:[NSString stringWithUTF8String:(char*)group->widgets[w].caption]]);
       if (group->widgets[w].widget_type == MYX_CHECKBOX)
         label_width /= 2;
     }
@@ -201,7 +201,7 @@ static NSView *makeWidget(MYX_GUI_WIDGET *widget)
     float h= 0.0;
     float dh= 0.0;
     float left_width= 0.0;
-    NSString *optionId= [NSString stringWithCString:group->widgets[w].id];
+    NSString *optionId= [NSString stringWithCString:(char*)group->widgets[w].id];
 
     if (group->widgets[w].widget_type == MYX_CHECKBOX)
     {
@@ -378,17 +378,21 @@ static NSView *makeWidget(MYX_GUI_WIDGET *widget)
   [topv scrollPoint:NSMakePoint(0,y)];
 
   // the description
-  textf= makeLabel(page->description);
+  textf= makeLabel((char*)page->description);
   [pagev addSubview:textf];
   [textf setAlignment:NSCenterTextAlignment];
   [textf setFrame:NSMakeRect(17.0,tsize.height-4.0-[textf frame].size.height,
                              [pagev frame].size.width-17.0*2, [textf frame].size.height)];
   [textf setAutoresizingMask:NSViewMinYMargin|NSViewWidthSizable];
   
-  [item setLabel:[NSString stringWithUTF8String:page->caption]];
+  [item setLabel:[NSString stringWithUTF8String:(char*)page->caption]];
     
   [tab addTabViewItem:item];
   [item release];
+
+  [topv scrollPoint:NSMakePoint(0,y)];
+  
+  [sview setNeedsDisplay:YES];
 }
 
 
@@ -402,10 +406,10 @@ static NSView *makeWidget(MYX_GUI_WIDGET *widget)
     for (w= 0; w < group->widgets_num; w++)
     {
       MYX_GUI_WIDGET *widget= group->widgets+w;
-      NSString *optionId= [NSString stringWithUTF8String:widget->id];
+      NSString *optionId= [NSString stringWithUTF8String:(char*)widget->id];
       id view= [_optionControls objectForKey:optionId];
-      char *value= widget->value;
-      if (!value) value= widget->default_value;
+      char *value= (char*)widget->value;
+      if (!value) value= (char*)widget->default_value;
       if (!view)
       {
         NSLog(@"view for option %s not found", widget->id);
@@ -499,7 +503,7 @@ static NSView *makeWidget(MYX_GUI_WIDGET *widget)
     for (w= 0; w < group->widgets_num; w++)
     {
       MYX_GUI_WIDGET *widget= group->widgets+w;
-      NSString *optionId= [NSString stringWithUTF8String:widget->id];
+      NSString *optionId= [NSString stringWithUTF8String:(char*)widget->id];
       id view= [_optionControls objectForKey:optionId];
       if (!view)
       {
@@ -521,7 +525,7 @@ static NSView *makeWidget(MYX_GUI_WIDGET *widget)
           if (widget->checkbox->invert)
             v= !v;
           g_free(widget->value);
-          widget->value= g_strdup(v?"Checked":"Unchecked");
+          widget->value= (unsigned char*)g_strdup(v?"Checked":"Unchecked");
           break;
         }
         case MYX_TEXTEDIT:
@@ -529,22 +533,22 @@ static NSView *makeWidget(MYX_GUI_WIDGET *widget)
           if (strcmp2((char*)widget->textedit->edit_type,"innodbfilepath")==0)
           {
             g_free(widget->value);
-            widget->value= g_strdup([[view stringValue] UTF8String]);
+            widget->value= (unsigned char*)g_strdup([[view stringValue] UTF8String]);
           }
           else if (strcmp2((char*)widget->textedit->edit_type,"directory")==0)
           {
             g_free(widget->value);
-            widget->value= g_strdup([[(MAFileTextField*)view path] UTF8String]);
+            widget->value= (unsigned char*)g_strdup([[(MAFileTextField*)view path] UTF8String]);
           }
           else if (strcmp2((char*)widget->textedit->edit_type,"file")==0)
           {
             g_free(widget->value);
-            widget->value= g_strdup([[(MAFileTextField*)view path] UTF8String]);
+            widget->value= (unsigned char*)g_strdup([[(MAFileTextField*)view path] UTF8String]);
           }
           else
           {
             g_free(widget->value);
-            widget->value= g_strdup([[view stringValue] UTF8String]);
+            widget->value= (unsigned char*)g_strdup([[view stringValue] UTF8String]);
           }
           break;
         }
@@ -556,13 +560,13 @@ static NSView *makeWidget(MYX_GUI_WIDGET *widget)
 						[[pop selectedItem] title]?:@""];
           
           g_free(widget->value);
-          widget->value= g_strdup([str UTF8String]);
+          widget->value= (unsigned char*)g_strdup([str UTF8String]);
           break;
         }
         case MYX_DROPDOWNBOX:
         {
           g_free(widget->value);
-          widget->value= g_strdup([[[view selectedItem] representedObject] UTF8String]);
+          widget->value= (unsigned char*)g_strdup([[[view selectedItem] representedObject] UTF8String]);
           break;
         }
       }
@@ -658,9 +662,9 @@ static NSView *makeWidget(MYX_GUI_WIDGET *widget)
   file= fdopen(fd, "w");
   if (!file)
   {
-	NSRunAlertPanel(nil, [NSString stringWithFormat:@"Error updating configuration file:%s.", strerror(errno)],
-					@"OK", nil, nil);
-	return NO;
+    NSRunAlertPanel(nil, [NSString stringWithFormat:@"Error updating configuration file:%s.", strerror(errno)],
+                  @"OK", nil, nil);
+    return NO;
   }
   
   rc= myx_update_mysql_cnf_filef(_descr, [_file fileSystemRepresentation],
