@@ -10,6 +10,7 @@
 
 @class MMySQLDispatcher;
 @class MSchemaDataSource;
+@class MConnectionInfo;
 
 @protocol MAdministratorProtocol
 
@@ -18,7 +19,7 @@
 - (MMySQLDispatcher*)dispatcher;
 
 - (MYSQL*)mysql;
-- (MYX_USER_CONNECTION*)serverInfo;
+- (MConnectionInfo*)serverInfo;
 
 - (int)serverStatus;
 - (BOOL)isLocal;
@@ -28,7 +29,7 @@
 - (void)refreshSchemata;
 
 - (MSchemaDataSource*)sharedSchemaDS;
-- (MSchemaDataSource*)sharedColumnDS;
+- (MSchemaDataSource*)sharedObjectDS;
 
 - (MYX_CATALOGS*)catalogList;
 
