@@ -29,10 +29,12 @@
 - (MYX_USER*)userInfo;
 - (NSArray*)hosts;
 - (MUserHostItem*)addHost:(NSString*)host;
+- (MUserHostItem*)itemForHost:(NSString*)host;
 - (void)removeHost:(NSString*)host;
 - (void)setOriginalName:(NSString*)name;
 - (NSString*)originalName;
 - (BOOL)isNewUser;
+- (void)setSaved;
 
 - (void)setName:(NSString*)name;
 - (NSString*)name;
