@@ -18,12 +18,13 @@
 #define myx_library_h
 
 #if defined(__WIN__) || defined(_WIN32) || defined(_WIN64)
-#define __LCC__
+  #define __LCC__
+  #define _WINDOWS
 #endif
 
 #include "myx_public_interface.h"
-#include "myx_aux_functions.h"
-#include "myx_shared_aux_functions.h"
+#include "myx_util_functions.h"
+#include "myx_shared_util_functions.h"
 
 #define DBSTR(mysql, str) myx_convert_dbstr_utf8(mysql, str)
 
