@@ -1,4 +1,4 @@
-/* Copyright (C) 2003,2004 MySQL AB
+/* Copyright (C) 2004 MySQL AB
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
 #endif
 
 #include <mysql.h>
-#include <myx_aux_functions.h>
+#include <myx_util_public_interface.h>
 #include <stdio.h>
 
 
@@ -44,18 +44,13 @@ extern "C" {
 /// [SCRIPT::LibInterfaceMapper] -public_interface "libmysqlx"
 #define libmysqlx_PUBLIC_INTERFACE_VERSION 10004
 
+/// [SCRIPT::LibInterfaceMapper] -add_to_uses "myx_util_public_interface"
+/// [SCRIPT::LibInterfaceMapper] -add_datatypes_from "..\..\library_util\include\myx_util_public_interface.h"
 
 /*
  * Enums
  */
 
-typedef enum myx_lib_error
-{
-    MYX_NO_ERROR = 0, MYX_ERROR_CANT_OPEN_FILE, MYX_ERROR_CANT_CONNECT_TO_INSTANCE, MYX_XML_PARSE_ERROR,
-    MYX_XML_NO_VALID_DOCUMENT, MYX_XML_EMPTY_DOCUMENT, MYX_SQL_ERROR, MYX_STOP_EXECUTION, MYX_ALLOC_CHANGE_ERROR,
-    MYX_OBJECT_NOT_FOUND, MYX_CANT_READ_FROM_FILE, MYX_CHARSET_CONVERSION_ERROR,MYX_CHARSET_WRONG_CHARSET_SPECIFIED
-} MYX_LIB_ERROR;
-
 typedef enum myx_user_connection_type
 {
     MYX_MYSQL_CONN = 0, MYX_ODBC_CONN, MYX_ORACLE_CONN, MYX_MSSQL_CONN, MYX_SQLITE_CONN
@@ -110,18 +105,6 @@ typedef enum
  */
 
 typedef void (*MYX_QUERY_CALLBACK)(MYSQL *mysql, void *cdata, const char *query, unsigned int length);
-  
-/// [SCRIPT::LibInterfaceMapper] -generate_classes MYX_STRINGLIST
-typedef struct {
-    unsigned int strings_num;
-    char **strings;
-} MYX_STRINGLIST;
-
-/// [SCRIPT::LibInterfaceMapper] -generate_classes MYX_NAME_VALUE_PAIR
-typedef struct {
-  char *name;
-  char *value;
-} MYX_NAME_VALUE_PAIR;
 
 /// [SCRIPT::LibInterfaceMapper] -generate_classes MYX_MYSQL_ERROR_MSG
 typedef struct {
@@ -161,6 +144,32 @@ typedef struct {
   MYX_USER_CONNECTION *user_connections;
 } MYX_USER_CONNECTIONS;
 
+/// [SCRIPT::LibInterfaceMapper] -generate_classes MYX_MYSQLD_CONNECTION
+typedef struct {
+  MYX_USER_CONNECTION *connection;
+} MYX_MYSQLD_CONNECTION;
+
+/// [SCRIPT::LibInterfaceMapper] -generate_classes MYX_MYSQLD_CONNECTIONS
+typedef struct {
+  unsigned int mysqld_connections_num;
+  int last_connection;
+  MYX_MYSQLD_CONNECTION *mysqld_connections;
+} MYX_MYSQLD_CONNECTIONS;
+
+/// [SCRIPT::LibInterfaceMapper] -generate_classes MYX_IM_CONNECTION
+typedef struct {
+  MYX_USER_CONNECTION *connection;
+  MYX_MYSQLD_CONNECTIONS *mysqlds;
+} MYX_IM_CONNECTION;
+
+/// [SCRIPT::LibInterfaceMapper] -generate_classes MYX_IM_CONNECTIONS
+typedef struct {
+  int last_im_connection;
+
+  unsigned int im_connections_num;
+  MYX_IM_CONNECTION *im_connections;
+} MYX_IM_CONNECTIONS;
+
 // Definition of program options
 
 /// [SCRIPT::LibInterfaceMapper] -generate_classes MYX_OPTION_GROUP
@@ -205,6 +214,8 @@ typedef struct {
   unsigned int not_null;
 } MYX_SCHEMA_TABLE_COLUMN;
 
+typedef MYX_SCHEMA_TABLE_COLUMN MYX_SCHEMA_VIEW_COLUMN;
+
 /// [SCRIPT::LibInterfaceMapper] -generate_classes MYX_TABLE_INDEX_COLUMN
 typedef struct {
   char *column_name;
@@ -226,21 +237,21 @@ typedef struct {
 
 /// [SCRIPT::LibInterfaceMapper] -generate_classes MYX_TABLE_STATUS
 typedef struct {
-  unsigned char *table_name;
-  unsigned char *table_type;
-  unsigned char *row_format;
-  unsigned char *rows;
-  unsigned char *avg_row_length;
-  unsigned char *data_length;
-  unsigned char *max_data_length;
-  unsigned char *index_length;
-  unsigned char *data_free;
-  unsigned char *auto_increment;
-  unsigned char *create_time;
-  unsigned char *update_time;
-  unsigned char *check_time;
-  unsigned char *create_options;
-  unsigned char *comment;
+  char *table_name;
+  char *table_type;
+  char *row_format;
+  char *rows;
+  char *avg_row_length;
+  char *data_length;
+  char *max_data_length;
+  char *index_length;
+  char *data_free;
+  char *auto_increment;
+  char *create_time;
+  char *update_time;
+  char *check_time;
+  char *create_options;
+  char *comment;
 
   unsigned int indexes_num;
   MYX_TABLE_INDEX *indexes;
@@ -249,17 +260,62 @@ typedef struct {
   MYX_SCHEMA_TABLE_COLUMN *columns;
 } MYX_TABLE_STATUS;
 
+//typedef MYX_TABLE_STATUS MYX_VIEW_STATUS;
+
 /// [SCRIPT::LibInterfaceMapper] -generate_classes MYX_SCHEMA_TABLE_STATUS
 typedef struct {
   unsigned int schema_tables_num;
   MYX_TABLE_STATUS *schema_tables;
 } MYX_SCHEMA_TABLE_STATUS;
 
-  
+//typedef MYX_SCHEMA_TABLE_STATUS MYX_SCHEMA_VIEW_STATUS;
+
+// -----------------------------------------------------------------------------------------------------------
+// View Status
+
+/// [SCRIPT::LibInterfaceMapper] -generate_classes MYX_VIEW_STATUS
+typedef struct 
+{
+  char *view_name;
+  char *comment;
+  unsigned int flags;
+} MYX_VIEW_STATUS;
+
+/// [SCRIPT::LibInterfaceMapper] -generate_classes MYX_SCHEMA_VIEW_STATUS
+typedef struct 
+{
+  unsigned int schema_views_num;
+  MYX_VIEW_STATUS *schema_views;
+} MYX_SCHEMA_VIEW_STATUS;
+
+// -----------------------------------------------------------------------------------------------------------
+// Schema entiry Status
+
+typedef enum
+{
+  MYX_ENTITY_TABLE = 1,
+  MYX_ENTITY_VIEW,
+  MYX_ENTITY_PROC,
+  MYX_ENTITY_FUNC
+} MYX_ENTITY_TYPE;
+
+/// [SCRIPT::LibInterfaceMapper] -generate_classes MYX_ENTITY_STATUS
+typedef struct 
+{
+  MYX_ENTITY_TYPE entity_type;
+  void *entity;
+} MYX_ENTITY_STATUS;
+
+/// [SCRIPT::LibInterfaceMapper] -generate_classes MYX_SCHEMA_ENTITY_STATUS
+typedef struct 
+{
+  unsigned int schema_entities_num;
+  MYX_ENTITY_STATUS *schema_entities;
+} MYX_SCHEMA_ENTITY_STATUS;
+
 // -----------------------------------------------------------------------------------------------------------
 // Catalog / Schema Info
 
-
 /// [SCRIPT::LibInterfaceMapper] -generate_classes MYX_SCHEMA_INDICES
 typedef struct {
   unsigned int indices_num;
@@ -360,13 +416,13 @@ typedef struct {
 
 typedef struct myx_trans_text
 {
-    char *id;
+    unsigned char *id;
     char *trans;
 } MYX_TRANS_TEXT;
 
 typedef struct myx_trans_textgroup
 {
-    char *id;
+    unsigned char *id;
     unsigned int texts_num;
     MYX_TRANS_TEXT *texts;
 } MYX_TRANS_TEXTGROUP;
@@ -436,6 +492,7 @@ typedef struct
 typedef enum
 {
   MYX_QT_SELECT,
+  MYX_QT_SELECT_INTO_OUTFILE,
   MYX_QT_UPDATE,
   MYX_QT_INSERT,
   MYX_QT_DELETE,
@@ -500,7 +557,8 @@ typedef enum
 {
   MYX_RSCT_INTEGER= 0, MYX_RSCT_FLOAT, MYX_RSCT_STRING, MYX_RSCT_DATE, MYX_RSCT_TIME, MYX_RSCT_DATETIME,
   MYX_RSCT_BLOB, MYX_RSCT_TEXT,
-  MYX_RSCT_ENUM, MYX_RSCT_SET
+  MYX_RSCT_ENUM, MYX_RSCT_SET,
+  MYX_RSCT_DECIMAL
 } MYX_RS_COLUMN_TYPE;
 
 #define MYX_RSCT_NEEDS_QUOTE(t) (t != MYX_RSCT_INTEGER && t != MYX_RSCT_FLOAT)
@@ -649,6 +707,8 @@ typedef struct
 
   struct MYX_RS_ACTIONS *actions;
 
+  bigint memory_used;
+  
   double query_time;
   double fetch_time;
 } MYX_RESULTSET;
@@ -706,7 +766,8 @@ typedef enum {
 
 typedef enum {
   MYX_DBM_TE_MYISAM = 0, MYX_DBM_TE_INNODB, MYX_DBM_TE_MEMORY, MYX_DBM_TE_MERGE, MYX_DBM_TE_BDB,
-  MYX_DBM_TE_ISAM, MYX_DBM_TE_NDB
+  MYX_DBM_TE_ISAM, MYX_DBM_TE_NDB, MYX_DBM_TE_FALCON, MYX_DBM_TE_ARCHIVE, MYX_DBM_TE_CSV, 
+  MYX_DBM_TE_BLACKHOLE, MYX_DBM_TE_FEDERATED
 } MYX_DBM_TABLE_ENGINE;
 
 typedef enum {
@@ -722,7 +783,8 @@ typedef enum {
 } MYX_DBM_TABLE_RAID_TYPE;
 
 typedef enum {
-  MYX_DBM_TRF_DEFAULT = 0, MYX_DBM_TRF_DYNAMIC, MYX_DBM_TRF_FIXED, MYX_DBM_TRF_COMPRESSED
+  MYX_DBM_TRF_DEFAULT = 0, MYX_DBM_TRF_DYNAMIC, MYX_DBM_TRF_FIXED, MYX_DBM_TRF_COMPRESSED,
+  MYX_DBM_TRF_REDUNDANT, MYX_DBM_TRF_COMPACT
 } MYX_DBM_TABLE_ROW_FORMAT;
 
 typedef enum {
@@ -921,6 +983,8 @@ typedef struct {
 
   unsigned int indices_num;
   MYX_DBM_INDEX_DATA *indices;
+
+  char *federated_connection;
 } MYX_DBM_TABLE_DATA;
 
 /// [SCRIPT::LibInterfaceMapper] -generate_classes MYX_DBM_STORED_PROCEDURE_DATA
@@ -940,13 +1004,6 @@ typedef struct {
   char *definition;
 } MYX_DBM_VIEW_DATA;
 
-  
-
-// ------------------------------
-// International File
-
-typedef struct myx_intl_file MYX_INTL_FILE;
-
 
 // ------------------------------
 // Table Data Exporting
@@ -965,7 +1022,8 @@ typedef enum {
   MYX_TE_COL_STRING,
   MYX_TE_COL_INTEGER,
   MYX_TE_COL_FLOAT,
-  MYX_TE_COL_BOOL
+  MYX_TE_COL_BOOL,
+  MYX_TE_COL_DATE
 } MYX_TE_COLUMN_TYPE;
 
 
@@ -1041,7 +1099,7 @@ MYX_PUBLIC_FUNC int myx_get_public_interface_version();
 
 MYX_PUBLIC_FUNC void myx_free_lib_str(char *str);
 MYX_PUBLIC_FUNC int myx_str_in_stringlist(MYX_STRINGLIST *stringlist, const char *src);
-MYX_PUBLIC_FUNC int myx_free_stringlist(MYX_STRINGLIST *stringlist);
+MYX_PUBLIC_FUNC int myx_free_lib_stringlist(MYX_STRINGLIST *stringlist);
 
 //Helper functions
 MYX_PUBLIC_FUNC char * get_value_from_text_ex(const char *txt, int txt_length, const char *regexpr, unsigned int substring_nr);
@@ -1054,6 +1112,8 @@ MYX_PUBLIC_FUNC MYSQL * myx_mysql_embedded_init();
 
 MYX_PUBLIC_FUNC MYSQL * myx_mysql_init();
 
+MYX_PUBLIC_FUNC int myx_init_library(const char *datadir);
+
 MYX_PUBLIC_FUNC void myx_mysql_set_query_hooks(MYSQL *mysql, 
                                                void (*pre_hook)(MYSQL *mysql, void *cdata, const char *query, unsigned int length),
                                                void (*post_hook)(MYSQL *mysql, void *cdata, const char *query, unsigned int length),
@@ -1063,8 +1123,11 @@ MYX_PUBLIC_FUNC const char *myx_get_my_cnf_path();
 MYX_PUBLIC_FUNC int myx_connect_to_instance(MYX_USER_CONNECTION *user_connection, MYSQL *mysql);
 MYX_PUBLIC_FUNC int myx_mysql_close(MYSQL *mysql);
 MYX_PUBLIC_FUNC int myx_use_schema(MYSQL *mysql, const char *schema);
+MYX_PUBLIC_FUNC int myx_push_schema(MYSQL *mysql, const char *schema);
+MYX_PUBLIC_FUNC int myx_pop_schema(MYSQL *mysql);
 MYX_PUBLIC_FUNC char * myx_get_default_schema(MYSQL *mysql);
 MYX_PUBLIC_FUNC unsigned long myx_get_thread_id(MYSQL *mysql);
+MYX_PUBLIC_FUNC int myx_ping_server(MYSQL *mysql);
 MYX_PUBLIC_FUNC int myx_get_mysql_quote_char(MYSQL *mysql, int detect);
 
 MYX_PUBLIC_FUNC char * myx_mysql_esc(MYSQL *mysql, const char *cmd, ...);
@@ -1076,6 +1139,7 @@ MYX_PUBLIC_FUNC bigint myx_mysql_affected_rows(MYSQL *mysql);
 MYX_PUBLIC_FUNC char * myx_get_mysql_full_version(MYSQL *mysql);
 MYX_PUBLIC_FUNC int myx_get_mysql_major_version(MYSQL *mysql);
 MYX_PUBLIC_FUNC int myx_get_mysql_minor_version(MYSQL *mysql);
+MYX_PUBLIC_FUNC int myx_get_mysql_release(MYSQL *mysql);
 MYX_PUBLIC_FUNC int mysql_version_is_later_or_equal_than(MYSQL *mysql,
                                                          int major_version, int minor_version);
 MYX_PUBLIC_FUNC int mysql_full_version_is_later_or_equal_than(MYSQL *mysql,
@@ -1094,19 +1158,41 @@ MYX_PUBLIC_FUNC void myx_set_mysql_error(char *error, int errn);
 
 MYX_PUBLIC_FUNC MYX_MYSQL_ERROR_MSGS * myx_mysql_error_msgs_fetch(MYSQL *mysql);
 MYX_PUBLIC_FUNC void myx_mysql_error_msgs_free(MYX_MYSQL_ERROR_MSGS *errors);
+
+MYX_PUBLIC_FUNC void build_field_subst(const char ** column_name,
+                                       const char ** column_name_end,
+                                       MYSQL_FIELD * res_fields,
+                                       MYSQL_FIELD * res_fields_end,
+                                       int * i_field);
+
+MYX_PUBLIC_FUNC int use_schema_store_old_one(MYSQL * mysql,
+                                             const char * schema_name,
+                                             char ** old_schema_name);
+MYX_PUBLIC_FUNC void restore_old_schema(MYSQL * mysql, char * old_schema_name);
+
   
 //Network functions
 MYX_PUBLIC_FUNC unsigned long myx_resolve_network_name(const char *hostname);
 MYX_PUBLIC_FUNC int myx_ping_host(unsigned long ip, int ping_timeout, MYX_PING_RESULT *ping_result);
 MYX_PUBLIC_FUNC int myx_is_localhost(const char *hostname);
+MYX_PUBLIC_FUNC int myx_get_ip_as_string(const char *hostname, char *ip);
 
 //User connection functions
-MYX_PUBLIC_FUNC MYX_USER_CONNECTIONS* myx_load_user_connections(const char *filename, MYX_LIB_ERROR *error_code);
+MYX_PUBLIC_FUNC MYX_USER_CONNECTIONS * myx_load_user_connections(const char *filename, MYX_LIB_ERROR *error_code);
 MYX_PUBLIC_FUNC int myx_store_user_connections(MYX_USER_CONNECTIONS *user_connections, MYX_PASSWORD_STORAGE_TYPE pwd_storage_type, const char *filename);
 MYX_PUBLIC_FUNC int myx_free_user_connections(MYX_USER_CONNECTIONS *user_connections);
 MYX_PUBLIC_FUNC int myx_free_user_connection_content(MYX_USER_CONNECTION *user_connection);
 MYX_PUBLIC_FUNC int myx_add_user_connection(MYX_USER_CONNECTIONS **user_connections, MYX_USER_CONNECTION *new_connection);
 
+//IM/MYSQLD connection functions
+MYX_PUBLIC_FUNC MYX_IM_CONNECTIONS * myx_load_im_connections(const char *filename, MYX_LIB_ERROR *error_code);
+MYX_PUBLIC_FUNC int myx_store_im_connections(MYX_IM_CONNECTIONS *im_connections, MYX_PASSWORD_STORAGE_TYPE pwd_storage_type, const char *filename);
+MYX_PUBLIC_FUNC int myx_free_im_connections(MYX_IM_CONNECTIONS *im_connections);
+MYX_PUBLIC_FUNC int myx_add_im_connection(MYX_IM_CONNECTIONS **im_connections, MYX_IM_CONNECTION *new_connection);
+MYX_PUBLIC_FUNC int myx_add_mysqld_connection(MYX_IM_CONNECTION **im_connection, MYX_MYSQLD_CONNECTION *new_connection);
+MYX_PUBLIC_FUNC MYX_MYSQLD_CONNECTIONS *myx_new_mysqld_connections(int nconnections);
+MYX_PUBLIC_FUNC void myx_free_mysqld_connections(MYX_MYSQLD_CONNECTIONS *connections);
+
 //client/server info functions
 MYX_PUBLIC_FUNC MYX_MACHINE_INFO * myx_get_server_info(MYX_USER_CONNECTION *user_conn, MYSQL *mysql);
 MYX_PUBLIC_FUNC MYX_MACHINE_INFO * myx_get_client_info(MYSQL *mysql);
@@ -1119,13 +1205,17 @@ MYX_PUBLIC_FUNC int myx_get_available_server_features(MYSQL *mysql, MYX_AVAILABL
 MYX_PUBLIC_FUNC MYX_CATALOGS * myx_get_catalogs(MYSQL *mysql);
 MYX_PUBLIC_FUNC MYX_SCHEMA_TABLES * myx_get_schema_tables(MYSQL *mysql, const char *catalog_name, const char *schema_name);
 MYX_PUBLIC_FUNC MYX_SCHEMA_INDICES * myx_get_schema_indices(MYSQL *mysql, const char *catalog_name, const char *schema_name);
-MYX_PUBLIC_FUNC MYX_SCHEMA_STORED_PROCEDURES * myx_get_schema_sps(MYSQL *mysql, const char *catalog_name, const char *schema_name);
 MYX_PUBLIC_FUNC MYX_SCHEMA_TABLE_STATUS * myx_get_schema_table_status(MYSQL *mysql, const char *catalog_name, const char *schema_name);
+MYX_PUBLIC_FUNC MYX_SCHEMA_VIEW_STATUS * myx_get_schema_view_status(MYSQL *mysql, const char *catalog_name, const char *schema_name);
+MYX_PUBLIC_FUNC MYX_SCHEMA_STORED_PROCEDURES * myx_get_schema_sps(MYSQL *mysql, const char *catalog_name, const char *schema_name);
+MYX_PUBLIC_FUNC MYX_SCHEMA_ENTITY_STATUS * myx_get_schema_entity_status(MYSQL *mysql, const char *catalog_name, const char *schema_name);
 MYX_PUBLIC_FUNC int myx_free_catalogs(MYX_CATALOGS *catalogs);
 MYX_PUBLIC_FUNC int myx_free_schema_tables(MYX_SCHEMA_TABLES *schema_tables);
 MYX_PUBLIC_FUNC int myx_free_schema_indices(MYX_SCHEMA_INDICES *schema_indices);
-MYX_PUBLIC_FUNC int myx_free_schema_sps(MYX_SCHEMA_STORED_PROCEDURES *schema_sps);
+MYX_PUBLIC_FUNC int myx_free_schema_entity_status(MYX_SCHEMA_ENTITY_STATUS *schema_tables);
 MYX_PUBLIC_FUNC int myx_free_schema_table_status(MYX_SCHEMA_TABLE_STATUS *schema_tables);
+MYX_PUBLIC_FUNC int myx_free_schema_view_status(MYX_SCHEMA_VIEW_STATUS *schema_tables);
+MYX_PUBLIC_FUNC int myx_free_schema_sps(MYX_SCHEMA_STORED_PROCEDURES *schema_sps);
 
 //Application options functions
 MYX_PUBLIC_FUNC int myx_store_application_options(MYX_APPLICATION_OPTIONS *options, char *filename);
@@ -1145,7 +1235,7 @@ MYX_PUBLIC_FUNC const char * myx_tg(MYX_TRANS *trans, const char *id_textgroup,
 
 //Various
 //Check if a string matches a given search string
-MYX_PUBLIC_FUNC int myx_match_pattern(const char *text, const char *search, int case_sensitive);
+MYX_PUBLIC_FUNC int myx_match_pattern(const char *text, const char *search, int case_insensitive, int strict);
 
 //Check whether a given string can be properly converted to an encoding
 MYX_PUBLIC_FUNC int myx_check_utf8_convertible(const char *str, const char *encoding);
@@ -1166,6 +1256,9 @@ MYX_PUBLIC_FUNC MYX_SYN_SUGGESTIONS* myx_lookup_word(MYX_SYN* syn,const char *wo
 MYX_PUBLIC_FUNC int myx_free_syn_suggestions(MYX_SYN_SUGGESTIONS *s);
 
 // query functions
+MYX_PUBLIC_FUNC void myx_mysql_limit_resultset_size(MYSQL *mysql, unsigned int limitMB);
+MYX_PUBLIC_FUNC unsigned int myx_mysql_get_resultset_size_limit(MYSQL *mysql);
+
 MYX_PUBLIC_FUNC int myx_identifier_needs_quotes(const char *name);
 MYX_PUBLIC_FUNC MYX_RESULTSET * myx_query_execute(MYSQL *mysql, const char *sql, int enforce_editable,
   MYX_STRINGLIST *params, MYX_LIB_ERROR *error_code, void *user_data,
@@ -1186,6 +1279,8 @@ MYX_PUBLIC_FUNC int myx_query_pack_resultset(MYX_RESULTSET *resultset);
 MYX_PUBLIC_FUNC int myx_query_discard_actions(MYX_RESULTSET *resultset);
 MYX_PUBLIC_FUNC int myx_query_free_action_errors(MYX_RS_ACTION_ERRORS *errors);
 
+MYX_PUBLIC_FUNC int myx_get_unix_newline_count(const char *str);
+
 MYX_PUBLIC_FUNC MYX_Q_TYPE myx_query_type(const char *sql);
 MYX_PUBLIC_FUNC char * myx_query_add_column_to_sql(MYSQL *mysql, const char *catalog, const char *schema, 
                                                    const char *table, const char *column, char *sqlcmd, 
@@ -1232,6 +1327,17 @@ MYX_PUBLIC_FUNC char * myx_dbm_make_script_from_sps(MYSQL *mysql, const char *ca
 MYX_PUBLIC_FUNC MYX_LIB_ERROR myx_dbm_create_view(MYSQL *mysql, const char *catalog, const char *schema,
                                                   const char *view_name, const char *select_sql);
 
+MYX_PUBLIC_FUNC MYX_LIB_ERROR myx_dbm_drop_sp(MYSQL *mysql, 
+                                              const char *catalog_name, 
+                                              const char *schema_name, 
+                                              const char *sp_name, 
+                                              MYX_SCHEMA_STORED_PROCEDURE_TYPE sp_type);
+
+MYX_PUBLIC_FUNC MYX_LIB_ERROR myx_dbm_drop_view(MYSQL *mysql, 
+                                                const char *catalog_name, 
+                                                const char *schema_name, 
+                                                const char *view_name);
+
 MYX_PUBLIC_FUNC MYX_DBM_VIEW_DATA * myx_dbm_get_view_data(MYSQL *mysql, const char *catalog, const char *schema,
                                                          const char *view);
   
@@ -1256,12 +1362,6 @@ MYX_PUBLIC_FUNC int myx_charsets_store(const char *filename, MYX_DBM_CHARSETS *o
 MYX_PUBLIC_FUNC MYX_DBM_CHARSETS * myx_charsets_load(const char *filename, MYX_LIB_ERROR *error_code);
 MYX_PUBLIC_FUNC int myx_free_charsets(MYX_DBM_CHARSETS *obj);
 
-/* International file functions */
-MYX_PUBLIC_FUNC MYX_INTL_FILE * myx_new_intl_file(const char *filename, const char *charset, MYX_LIB_ERROR *error);
-MYX_PUBLIC_FUNC void myx_free_intl_file(MYX_INTL_FILE *intl_file);
-MYX_PUBLIC_FUNC bigint myx_read_char_from_intl_file(MYX_INTL_FILE *intl_file, int *bytes_read, MYX_LIB_ERROR *error);
-MYX_PUBLIC_FUNC char * myx_detect_charset_from_sql_file(const char * filename);
-
 /* Others */
 MYX_PUBLIC_FUNC MYX_IMAGE_FORMAT myx_guess_image_format(const char *data, unsigned int length);
 
