@@ -15,20 +15,14 @@
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 
 #include <stdio.h>
-#include <myx_aux_functions.h>
-
-#if defined(__WIN__) || defined(_WIN32) || defined(_WIN64)
-#define MYX_PUBLIC_FUNC __declspec(dllexport)
-#else
-#define MYX_PUBLIC_FUNC
-#endif
-
+#include <myx_util_functions.h>
 
 typedef struct {
  
   int state;
 
   bigint bytes_read;
+  bigint chars_read;
   int line_num;
 
   bigint stmt_begin_char; // stmt_begin in chars
@@ -60,26 +54,37 @@ MYX_PUBLIC_FUNC
 MYX_PUBLIC_FUNC void myx_done_string_wrapper(MYX_STRING_WRAPPER *w);
 
 MYX_PUBLIC_FUNC bigint myx_read_char_from_string(void *w,
-                                                 int *bytes_read, int *error);
+                                                 int *bytes_read, MYX_LIB_ERROR *error);
 
 MYX_PUBLIC_FUNC
         int myx_get_next_sql_statement_file(MYX_SQL_PARSE_ENVIRONMENT *we,
                                             MYX_INTL_FILE *file, char **buffer,
                                             int *buffer_len, int resize_buffer,
-                                            int *error);
+                                            int (*progress_report)(
+                                                           bigint bytes_read,
+                                                           bigint bytes_total,
+                                                           void *user_data),
+                                            bigint file_size,
+                                            void *user_data,
+                                            MYX_LIB_ERROR *error);
 
 MYX_PUBLIC_FUNC int myx_get_next_sql_statement(MYX_SQL_PARSE_ENVIRONMENT * we,
                                                char **buffer, int *buffer_len,
-                                               int resize_buffer, int *error,
+                                               int resize_buffer, MYX_LIB_ERROR *error,
                                                bigint (*read_char)
                                                   (
-                                                    void *user_data,
+                                                    MYX_INTL_FILE *user_data,
                                                     int *bytes_read,
-                                                    int *error
+                                                    MYX_LIB_ERROR *error
                                                   ),
-                                               void *arg);
+                                                int (*progress_report)(
+                                                           bigint bytes_read,
+                                                           bigint bytes_total,
+                                                           void *user_data),
+                                                bigint file_size,
+                                                void* user_data,
+                                              void *arg);
 
 MYX_PUBLIC_FUNC void kill_comments(char * masked_sql);
 MYX_PUBLIC_FUNC void mask_quotas(char * masked_sql);
 MYX_PUBLIC_FUNC void mask_quotas_and_brackets(char * masked_sql);
-
