@@ -16,7 +16,7 @@
 
 
 #include "myx_public_interface.h"
-#include "myx_aux_functions.h"
+#include "myx_util_functions.h"
 #include "myx_library.h"
 
 #define STR_EMPTY(s)  (!(s) || !*(s))
@@ -370,6 +370,16 @@ MYX_DBM_TABLE_DATA * myx_dbm_retrieve_table_data(MYSQL *mysql, MYX_DBM_DATATYPES
               table_data->table_engine= MYX_DBM_TE_ISAM;
             else if(strcmp2(row[i], "NDB")==0)
               table_data->table_engine= MYX_DBM_TE_NDB;
+            else if(strcasecmp(row[i], "JSTAR")==0||strcasecmp(row[i], "FALCON")==0)
+              table_data->table_engine= MYX_DBM_TE_FALCON;
+            else if(strcmp2(row[i], "ARCHIVE")==0)
+              table_data->table_engine= MYX_DBM_TE_ARCHIVE;
+            else if(strcmp2(row[i], "CSV")==0)
+              table_data->table_engine= MYX_DBM_TE_CSV;
+            else if(strcmp2(row[i], "BLACKHOLE")==0)
+              table_data->table_engine= MYX_DBM_TE_BLACKHOLE;
+            else if(strcmp2(row[i], "FEDERATED")==0)
+              table_data->table_engine= MYX_DBM_TE_FEDERATED;
             else
               table_data->table_engine= MYX_DBM_TE_MYISAM;
           }
@@ -406,6 +416,10 @@ MYX_DBM_TABLE_DATA * myx_dbm_retrieve_table_data(MYSQL *mysql, MYX_DBM_DATATYPES
               table_data->row_format= MYX_DBM_TRF_FIXED;
             else if(strcasecmp(row[i], "COMPRESSED") == 0)
               table_data->row_format= MYX_DBM_TRF_COMPRESSED;
+            else if(strcasecmp(row[i], "COMPACT") == 0)
+              table_data->row_format= MYX_DBM_TRF_COMPACT;
+            else if(strcasecmp(row[i], "REDUNDANT") == 0)
+              table_data->row_format= MYX_DBM_TRF_REDUNDANT;
             else
               table_data->row_format= MYX_DBM_TRF_DEFAULT;
           }
@@ -516,6 +530,9 @@ MYX_DBM_TABLE_DATA * myx_dbm_retrieve_table_data(MYSQL *mysql, MYX_DBM_DATATYPES
     table_data->raid_chunks= get_value_from_text(tbl_options, len, "RAID_CHUNKS\\s*=\\s*(\\w+)");
     table_data->raid_chunk_size= get_value_from_text(tbl_options, len, "RAID_CHUNKSIZE\\s*=\\s*(\\w+)");
 
+    // connection for federated
+    table_data->federated_connection= get_value_from_text(tbl_options, len, "CONNECTION\\s*=\\s*'(.*)'");
+
     //Foreign key constraints
     parse_foreign_keys(mysql, tbl_options, table_data);
     
@@ -843,6 +860,8 @@ void myx_dbm_free_table_data(MYX_DBM_TABLE_DATA *table_data)
   }
   g_free(table_data->fks);
 
+  g_free(table_data->federated_connection);
+
   g_free(table_data);
 }
 
@@ -990,6 +1009,7 @@ MYX_DBM_CHARSETS * myx_dbm_retrieve_charsets(MYSQL *mysql, MYX_LIB_ERROR *error_
   return charsets;
 }
 
+#ifdef notused
 static int string_contains_only(const char* str, char c)
 {
 	if(*str == '\0')
@@ -999,6 +1019,7 @@ static int string_contains_only(const char* str, char c)
 	while(*str++ == c);
 	return *str == '\0';
 }
+#endif
 
 char * dbm_get_sql_column_create_code(MYX_DBM_COLUMN_DATA *column, MYX_DBM_SERVER_VERSION *version, int pk_clause)
 {
@@ -1033,16 +1054,36 @@ char * dbm_get_sql_column_create_code(MYX_DBM_COLUMN_DATA *column, MYX_DBM_SERVE
   if(column->not_null)
     col_def= str_g_append(col_def, " NOT NULL");
 
-  if(!column->default_value_is_null && column->default_value && column->default_value[0])
+  if(!column->default_value_is_null && column->default_value /*&& column->default_value[0]*/)
+                                                             // this caused problems settings empty string def val
   {
-    if(column->datatype_pointer->group == MYX_DBM_DTG_NUMERIC) 
+    tmp = col_def;
+    if((column->datatype_pointer) && (column->datatype_pointer->group == MYX_DBM_DTG_NUMERIC))
     {
       if (strcmp(column->default_value, "") != 0)
         col_def= g_strdup_printf("%s DEFAULT %s", col_def, column->default_value);
+      else
+        col_def= g_strdup_printf("%s DEFAULT 0", col_def);
+    }
+    else if((column->datatype_pointer) && (column->datatype_pointer->group == MYX_DBM_DTG_DATETIME))
+    {
+      // starting with 5.x MySQL server doesn't accept '' as default datetime value
+      if (strcmp(column->default_value, "") != 0)
+      {
+        col_def= g_strdup_printf("%s DEFAULT '%s'", col_def, column->default_value);
+      }
+      else
+      {
+        col_def= g_strdup_printf("%s DEFAULT 0", col_def);
+      }
     }
     else
       col_def= g_strdup_printf("%s DEFAULT '%s'", col_def, column->default_value);
+
+    g_free(tmp);
   }
+  else if (column->default_value_is_null)
+    col_def= str_g_append(col_def, " DEFAULT NULL");
 
   if(column->auto_inc)
     col_def= str_g_append(col_def, " AUTO_INCREMENT");
@@ -1218,17 +1259,40 @@ char * myx_dbm_get_sql_option_create_code(MYX_DBM_TABLE_DATA *table, MYX_DBM_SER
   char *sql= NULL;
   char *tmp;
 
-  //Engine
-  switch (table->table_engine)
+  if((version->major_version >= 5) || ((version->major_version >= 4) && (version->minor_version >= 1)))
   {
-    case MYX_DBM_TE_INNODB: sql= str_g_append(sql, "ENGINE = InnoDB" _br); break;
-    case MYX_DBM_TE_MEMORY: sql= str_g_append(sql, "ENGINE = MEMORY" _br); break;
-    case MYX_DBM_TE_MERGE:  sql= str_g_append(sql, "ENGINE = MERGE" _br);  break;
-    case MYX_DBM_TE_BDB:    sql= str_g_append(sql, "ENGINE = BDB" _br);    break;
-    case MYX_DBM_TE_ISAM:   sql= str_g_append(sql, "ENGINE = ISAM" _br);   break;
-    case MYX_DBM_TE_NDB:    sql= str_g_append(sql, "ENGINE = NDB" _br);    break;
-    //default: sql= g_strdup(""); break;
-    default: sql= str_g_append(sql, "ENGINE = MYISAM" _br); break;
+    //Engine
+    switch (table->table_engine)
+    {
+      case MYX_DBM_TE_INNODB: sql= str_g_append(sql, "ENGINE = InnoDB" _br); break;
+      case MYX_DBM_TE_MEMORY: sql= str_g_append(sql, "ENGINE = MEMORY" _br); break;
+      case MYX_DBM_TE_MERGE:  sql= str_g_append(sql, "ENGINE = MERGE" _br);  break;
+      case MYX_DBM_TE_BDB:    sql= str_g_append(sql, "ENGINE = BDB" _br);    break;
+      case MYX_DBM_TE_ISAM:   sql= str_g_append(sql, "ENGINE = ISAM" _br);   break;
+      case MYX_DBM_TE_NDB:    sql= str_g_append(sql, "ENGINE = NDB" _br);    break;
+      case MYX_DBM_TE_FALCON: sql= str_g_append(sql, "ENGINE = JSTAR" _br);  break;
+      case MYX_DBM_TE_ARCHIVE:sql= str_g_append(sql, "ENGINE = ARCHIVE" _br);break;
+      case MYX_DBM_TE_CSV:    sql= str_g_append(sql, "ENGINE = CSV" _br);    break;
+      case MYX_DBM_TE_BLACKHOLE:    sql= str_g_append(sql, "ENGINE = BLACKHOLE" _br);    break;
+      case MYX_DBM_TE_FEDERATED:    sql= str_g_append(sql, "ENGINE = FEDERATED" _br);    break;
+      //default: sql= g_strdup(""); break;
+      default: sql= str_g_append(sql, "ENGINE = MYISAM" _br); break;
+    }
+  }
+  else  // 4.0 or earlier
+  {
+    //Engine
+    switch (table->table_engine)
+    {
+      case MYX_DBM_TE_INNODB: sql= str_g_append(sql, "TYPE = InnoDB" _br); break;
+      case MYX_DBM_TE_MEMORY: sql= str_g_append(sql, "TYPE = MEMORY" _br); break;
+      case MYX_DBM_TE_MERGE:  sql= str_g_append(sql, "TYPE = MERGE" _br);  break;
+      case MYX_DBM_TE_BDB:    sql= str_g_append(sql, "TYPE = BDB" _br);    break;
+      case MYX_DBM_TE_ISAM:   sql= str_g_append(sql, "TYPE = ISAM" _br);   break;
+      case MYX_DBM_TE_NDB:    sql= str_g_append(sql, "TYPE = NDB" _br);    break;
+      //default: sql= g_strdup(""); break;
+      default: sql= str_g_append(sql, "ENGINE = MYISAM" _br); break;
+    }
   }
 
   //auto_inc
@@ -1324,6 +1388,10 @@ char * myx_dbm_get_sql_option_create_code(MYX_DBM_TABLE_DATA *table, MYX_DBM_SER
     sql= str_g_append(sql, "ROW_FORMAT = FIXED" _br);
   else if(table->row_format == MYX_DBM_TRF_COMPRESSED)
     sql= str_g_append(sql, "ROW_FORMAT = COMPRESSED" _br);
+  else if(table->row_format == MYX_DBM_TRF_COMPACT)
+    sql= str_g_append(sql, "ROW_FORMAT = COMPACT" _br);
+  else if(table->row_format == MYX_DBM_TRF_REDUNDANT)
+    sql= str_g_append(sql, "ROW_FORMAT = REDUNDANT" _br);
   /*else
     if(for_alter)
       sql= str_g_append(sql, "ROW_FORMAT = DEFAULT" _br);*/
@@ -1340,6 +1408,10 @@ char * myx_dbm_get_sql_option_create_code(MYX_DBM_TABLE_DATA *table, MYX_DBM_SER
   if(!STR_EMPTY(table->max_rows))
       sql= str_g_append_and_free(sql, g_strdup_printf("MAX_ROWS = %s" _br, table->max_rows));
 
+  //connection for federated
+  if(!STR_EMPTY(table->federated_connection))
+      sql= str_g_append_and_free(sql, g_strdup_printf("CONNECTION = %s" _br, table->federated_connection));
+
   //remove last _br
   sql[strlen(sql)-strlen(_br)]= 0;
 
@@ -1449,7 +1521,7 @@ MYX_PUBLIC_FUNC char * myx_dbm_get_table_sql_diff(MYX_DBM_TABLE_DATA *existing_t
           ex_column= existing_table->columns+i;
           break;
         }
-      }
+      };
 
       //if column is not found anymore, drop it
       if(!column)
@@ -1486,14 +1558,19 @@ MYX_PUBLIC_FUNC char * myx_dbm_get_table_sql_diff(MYX_DBM_TABLE_DATA *existing_t
 
         //Check DefaultValue
         //if there is no default value
-        if (STR_EMPTY(column->default_value) || column->default_value_is_null)
+        if (!column->default_value_is_null != !ex_column->default_value_is_null)
+        {
+          column_changed= 1;
+        }
+        else if (STR_EMPTY(column->default_value) || column->default_value_is_null)
         {
           //and the default value is different than 0, "", 0-datetime and 0-date
-          if((ex_column->default_value || !column->default_value_is_null) &&
+          if((ex_column->default_value || column->default_value_is_null) &&
             (strcmp2(ex_column->default_value, "0") != 0) &&
             (strcmp2(ex_column->default_value, "") != 0) &&
             (strcmp2(ex_column->default_value, "0000-00-00 00:00:00") != 0) &&
-            (strcmp2(ex_column->default_value, "0000-00-00") != 0))
+            (strcmp2(ex_column->default_value, "0000-00-00") != 0) &&
+             !ex_column->default_value_is_null)
             column_changed= 1;
         }
         else
@@ -1615,10 +1692,14 @@ MYX_PUBLIC_FUNC char * myx_dbm_get_table_sql_diff(MYX_DBM_TABLE_DATA *existing_t
         {
           tmp2= myx_dbm_get_sql_column_create_code_no_pk(column, version);
 
-          if(name_changed)
+          if(name_changed) 
+          {
             tmp= g_strdup_printf("," _br " CHANGE COLUMN `%s` %s", column->original_name, tmp2);
+          }
           else
+          {
             tmp= g_strdup_printf("," _br " MODIFY COLUMN %s", tmp2);
+          }
 
           alter_spec= str_g_append(alter_spec, tmp);
 
@@ -1633,8 +1714,31 @@ MYX_PUBLIC_FUNC char * myx_dbm_get_table_sql_diff(MYX_DBM_TABLE_DATA *existing_t
       char previous_column_name[256];
       strcpy(previous_column_name, "");
 
-      if(existing_table->columns_num > 0)
-        strcpy(previous_column_name, existing_table->columns[existing_table->columns_num-1].name);
+      /*
+      // check whether the name of the last existing column was changed
+      // and use the new name in the AFTER clause if so
+      for(j= existing_table->columns_num - 1; j >= 0; j--)
+      {
+        //j= existing_table->columns_num-1;
+        for(i= 0; i < altered_table->columns_num; i++)
+        {
+          if(NAMECMP(altered_table->columns[i].original_name, existing_table->columns[j].name) == 0)
+          {
+            strcpy(previous_column_name, altered_table->columns[i].name);
+            j= -1;
+            break;
+          }
+        }
+        if((int)j == -1) 
+        {
+          break;
+        }
+      }
+//      if(j != -1)
+//      {
+//        strcpy(previous_column_name, existing_table->columns[j].name);
+//      }
+    */
 
       //Add new columns
       for(i= 0; i < altered_table->columns_num; i++)
@@ -1655,42 +1759,66 @@ MYX_PUBLIC_FUNC char * myx_dbm_get_table_sql_diff(MYX_DBM_TABLE_DATA *existing_t
         {
           tmp2= myx_dbm_get_sql_column_create_code(column, version);
 
-          if(existing_table->columns_num == 0)
+          //if(existing_table->columns_num == 0)
+          if(existing_table->columns_num == 0 || strcmp(previous_column_name,"") == 0)
             tmp= g_strdup_printf("," _br " ADD COLUMN %s FIRST", tmp2);
           else
             tmp= g_strdup_printf("," _br " ADD COLUMN %s AFTER `%s`", tmp2, previous_column_name);
 
-          strcpy(previous_column_name, column->name);
+          //strcpy(previous_column_name, column->name);
 
           alter_spec= str_g_append(alter_spec, tmp);
           g_free(tmp);
           g_free(tmp2);
         }
+        //++
+        strcpy(previous_column_name, column->name);
       }
     }
 
     //Check count of primary key cols
-    for(i= 0; i<altered_table->indices_num; i++)
+    if(altered_table->indices_num > 0)
     {
-      if(NAMECMP(altered_table->indices[i].name, "PRIMARY") == 0)
+      for(i= 0; i<altered_table->indices_num; i++)
       {
-        found= 0;
-        for(j= 0; j<existing_table->indices_num; j++)
+        if(NAMECMP(altered_table->indices[i].name, "PRIMARY") == 0)
         {
-          if(NAMECMP(existing_table->indices[j].name, "PRIMARY") == 0)
+          found= 0;
+          for(j= 0; j<existing_table->indices_num; j++)
           {
-            found= 1;
-            if(altered_table->indices[i].columns_num != existing_table->indices[j].columns_num)
-              new_pk= 1;
+            if(NAMECMP(existing_table->indices[j].name, "PRIMARY") == 0)
+            {
+              found= 1;
+              if(altered_table->indices[i].columns_num != existing_table->indices[j].columns_num)
+                new_pk= 1;
 
-            break;
+              break;
+            }
           }
-        }
 
-        if(!found)
+          if(!found)
+            new_pk= 1;
+            
+          break;
+        }
+      }
+    }
+    else  // altered table has no pks
+    {
+      for(i= 0; i<existing_table->indices_num; i++)
+      {
+        if(NAMECMP(existing_table->indices[i].name, "PRIMARY") == 0)
+        {
+          // existing table had PKs - need to DROP PRIMARY KEY (below)
           new_pk= 1;
-          
-        break;
+          // remote column attrs
+          for(i= 0; i<altered_table->columns_num; i++)
+          {
+            altered_table->columns[i].primary_key= 0;
+            //altered_table->columns[i].auto_inc= 0;
+          }
+          break;
+        }
       }
     }
 
@@ -1699,8 +1827,33 @@ MYX_PUBLIC_FUNC char * myx_dbm_get_table_sql_diff(MYX_DBM_TABLE_DATA *existing_t
       if(ex_pk)
         alter_spec= str_g_append(alter_spec, "," _br " DROP PRIMARY KEY");
 
+      // use index instead of PK flag (bugfix for #7136)
       tmp3= NULL;
-      for(i= 0; i<altered_table->columns_num; i++)
+      for(i= 0; i<altered_table->indices_num; i++)
+      {
+        if(NAMECMP(altered_table->indices[i].name, "PRIMARY") == 0)
+        {
+          for(j= 0; j<altered_table->indices[i].columns_num; j++)
+          {
+            if(!tmp3)
+              tmp2= g_strdup_printf("`%s`", altered_table->indices[i].columns[j].name);
+            else
+              tmp2= g_strdup_printf(", `%s`", altered_table->indices[i].columns[j].name);
+
+            if (altered_table->indices[i].columns[j].len && altered_table->indices[i].columns[j].len[0])
+              tmp2= str_g_append_and_free(tmp2, g_strdup_printf("(%s)", altered_table->indices[i].columns[j].len));
+
+            tmp3= str_g_append(tmp3, tmp2);
+            g_free(tmp2);
+          }
+
+          if(!found)
+            new_pk= 1;
+            
+          break;
+        }
+      }
+      /*for(i= 0; i<altered_table->columns_num; i++)
       {
         if(altered_table->columns[i].primary_key)
         {
@@ -1711,7 +1864,7 @@ MYX_PUBLIC_FUNC char * myx_dbm_get_table_sql_diff(MYX_DBM_TABLE_DATA *existing_t
           tmp3= str_g_append(tmp3, tmp2);
           g_free(tmp2);
         }
-      }
+      }*/
 
       if (tmp3)
       {
@@ -1906,13 +2059,18 @@ MYX_PUBLIC_FUNC char * myx_dbm_get_table_sql_diff(MYX_DBM_TABLE_DATA *existing_t
     if(existing_table->table_engine != altered_table->table_engine)
       switch (altered_table->table_engine)
       {
-      case MYX_DBM_TE_INNODB: alter_options= str_g_append(alter_options, _br "TYPE = InnoDB"); break;
-      case MYX_DBM_TE_MEMORY: alter_options= str_g_append(alter_options, _br "TYPE = MEMORY"); break;
-      case MYX_DBM_TE_MERGE:  alter_options= str_g_append(alter_options, _br "TYPE = MERGE");  break;
-      case MYX_DBM_TE_BDB:    alter_options= str_g_append(alter_options, _br "TYPE = BDB");    break;
-      case MYX_DBM_TE_ISAM:   alter_options= str_g_append(alter_options, _br "TYPE = ISAM");   break;
-      case MYX_DBM_TE_NDB:    alter_options= str_g_append(alter_options, _br "TYPE = NDB");    break;
-      default:                alter_options= str_g_append(alter_options, _br "TYPE = MYISAM"); break;
+      case MYX_DBM_TE_INNODB: alter_options= str_g_append(alter_options, _br "ENGINE = InnoDB"); break;
+      case MYX_DBM_TE_MEMORY: alter_options= str_g_append(alter_options, _br "ENGINE = MEMORY"); break;
+      case MYX_DBM_TE_MERGE:  alter_options= str_g_append(alter_options, _br "ENGINE = MERGE");  break;
+      case MYX_DBM_TE_BDB:    alter_options= str_g_append(alter_options, _br "ENGINE = BDB");    break;
+      case MYX_DBM_TE_ISAM:   alter_options= str_g_append(alter_options, _br "ENGINE = ISAM");   break;
+      case MYX_DBM_TE_NDB:    alter_options= str_g_append(alter_options, _br "ENGINE = NDB");    break;
+      case MYX_DBM_TE_FALCON: alter_options= str_g_append(alter_options, _br "ENGINE = JSTAR");  break;
+      case MYX_DBM_TE_ARCHIVE:alter_options= str_g_append(alter_options, _br "ENGINE = ARCHIVE");break;
+      case MYX_DBM_TE_CSV:    alter_options= str_g_append(alter_options, _br "ENGINE = CSV");    break;
+      case MYX_DBM_TE_BLACKHOLE:    alter_options= str_g_append(alter_options, _br "ENGINE = BLACKHOLE");    break;
+      case MYX_DBM_TE_FEDERATED:    alter_options= str_g_append(alter_options, _br "ENGINE = FEDERATED");    break;
+      default:                alter_options= str_g_append(alter_options, _br "ENGINE = MYISAM"); break;
       }
 
     //auto_inc
@@ -2058,6 +2216,12 @@ MYX_PUBLIC_FUNC char * myx_dbm_get_table_sql_diff(MYX_DBM_TABLE_DATA *existing_t
       case MYX_DBM_TRF_COMPRESSED:
         alter_options= str_g_append(alter_options, _br "ROW_FORMAT = COMPRESSED");
         break;
+      case MYX_DBM_TRF_REDUNDANT:
+        alter_options= str_g_append(alter_options, _br "ROW_FORMAT = REDUNDANT");
+        break;
+      case MYX_DBM_TRF_COMPACT:
+        alter_options= str_g_append(alter_options, _br "ROW_FORMAT = COMPACT");
+        break;        
       case MYX_DBM_TRF_DEFAULT:
         alter_options= str_g_append(alter_options, _br "ROW_FORMAT = DEFAULT");
         break;
@@ -2217,6 +2381,53 @@ MYX_DBM_STORED_PROCEDURE_DATA *myx_dbm_get_sp_data(MYSQL *mysql, const char *cat
   return spdata;
 }
 
+// return 0 when OK
+MYX_LIB_ERROR myx_dbm_drop_sp(MYSQL *mysql, const char *catalog_name, const char *schema_name, const char *sp_name, MYX_SCHEMA_STORED_PROCEDURE_TYPE sp_type)
+{
+  const char *q;
+  char *query;
+  
+  MYX_LIB_ERROR error;
+
+  if (check_exact_mysql_version(mysql, 5, 0, 0))
+  {
+    if (sp_type == MSPT_PROCEDURE)
+      q= "DROP PROCEDURE `%s`";
+    else 
+      q= "DROP FUNCTION `%s`";
+    
+    query= g_strdup_printf(q, sp_name);
+  }
+  else
+  {
+    if (sp_type == MSPT_PROCEDURE)
+      q= "DROP PROCEDURE `%s`.`%s`";
+    else 
+      q= "DROP FUNCTION `%s`.`%s`";
+    
+    query= g_strdup_printf(q, schema_name, sp_name);
+  }
+
+  myx_query_execute_direct(mysql, query, &error);
+  g_free(query);
+  
+  return error;
+}
+
+// return 0 when OK
+MYX_LIB_ERROR myx_dbm_drop_view(MYSQL *mysql, const char *catalog_name, const char *schema_name, const char *view_name)
+{
+  char *query;
+  MYX_LIB_ERROR error;
+
+  query= g_strdup_printf("DROP VIEW IF EXISTS `%s`.`%s`", schema_name, view_name);
+
+  myx_query_execute_direct(mysql, query, &error);
+  g_free(query);
+  
+  return error;
+}
+
 
 void myx_dbm_free_sp_data(MYX_DBM_STORED_PROCEDURE_DATA *sp_data)
 {
@@ -2367,7 +2578,7 @@ char *myx_dbm_make_script_from_sps(MYSQL *mysql, const char *catalog_name,
 
 
   script= g_strdup("DELIMITER $$\n\n"
-    "// -----------------------------------------------------------------------------\n\n");
+    "-- -----------------------------------------------------------------------------\n\n");
   // create a big script from the sorted list of SPs
   for (i= 0; i < spcount; i++)
   {
@@ -2376,10 +2587,10 @@ char *myx_dbm_make_script_from_sps(MYSQL *mysql, const char *catalog_name,
     sptype= spdata[i]->sp_type==MSPT_FUNCTION?"FUNCTION":"PROCEDURE";
     
     script= str_g_append_and_free(script,
-                                  g_strdup_printf("DROP %s IF EXISTS `%s`.`%s`$$\n", sptype, schema_name,
+                                  g_strdup_printf("DROP %s IF EXISTS `%s`.`%s` $$\n", sptype, schema_name,
                                                   spdata[i]->name));
-    script= str_g_append_and_free(script, g_strdup_printf("%s$$\n\n", spdata[i]->definition));
-    script= str_g_append(script, "// -----------------------------------------------------------------------------\n\n");
+    script= str_g_append_and_free(script, g_strdup_printf("%s $$\n\n", spdata[i]->definition));
+    script= str_g_append(script, "-- -----------------------------------------------------------------------------\n\n");
   }
 
   script= str_g_append(script, "DELIMITER ;");
@@ -2394,6 +2605,39 @@ char *myx_dbm_make_script_from_sps(MYSQL *mysql, const char *catalog_name,
   return script;
 }
 
+// create table t1 ...
+// will become
+// create table `schema`.`t1` 
+static char *add_schema_name_to_stmt(const char *unquoted_schema_name, 
+                                     const char *name, // entity name
+                                     const char *sql)  // sql stmt
+{
+  char *name_start= strstr(sql, name);
+  char *stmt_start;
+  char *retval;
+  char quote_char= '`';
+  
+  if(!name_start)
+  {
+    return g_strdup(sql);
+  }
+  if((name_start[-1] == '`') || (name_start[-1] == '"')) // quoted name
+  {
+    name_start -= 1;
+	quote_char= name_start[0];
+  }
+  if(name_start[-1] == '.') // already have schema name added?
+  {
+    return g_strdup(sql);
+  }
+  stmt_start= g_malloc((gulong) (name_start - sql) + 1);
+  strncpy(stmt_start, sql, name_start - sql);
+  stmt_start[name_start - sql]= '\0';
+  retval= g_strdup_printf("%s %c%s%c.%s", stmt_start, quote_char, unquoted_schema_name, quote_char, name_start);
+  g_free(stmt_start);
+  return retval;
+}
+
 static const char * show_create_table[]=
 {
   "Database",        // 0 (col 0)
@@ -2414,6 +2658,7 @@ char * myx_dbm_get_create_sql(MYSQL *mysql, const char *catalog_name, const char
   MYSQL_FIELD *fields;
   unsigned int num_fields;
   char *sql= NULL;
+  char *sql2= NULL;
 
   if ((object_type == MYX_DBM_OT_SCHEMA) && (!mysql_version_is_later_or_equal_than(mysql, 4, 1)))
     return g_strdup_printf("CREATE DATABASE `%s`", schema_name);
@@ -2472,6 +2717,7 @@ char * myx_dbm_get_create_sql(MYSQL *mysql, const char *catalog_name, const char
     myx_dbm_free_sp_data(sp_data);
   }
 
-  return sql;
+  sql2= add_schema_name_to_stmt(schema_name, name, sql);
+  g_free(sql);
+  return sql2;
 }
-
