@@ -1,4 +1,4 @@
-/* Copyright (C) 2003 MySQL AB
+/* Copyright (C) 2003, 2004, 2005 MySQL AB
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -20,11 +20,17 @@
 #include <assert.h>
 #ifdef __GNUC__
 #include <unistd.h>
+#include <ctype.h>
+#include "config.h"
+#ifdef ENABLE_NLS
+#include <libintl.h>
+#endif
 #endif
 
 #include "myx_library.h"
 #include "myx_network.h"
-#include "myx_aux_functions.h"
+#include "myx_util_functions.h"
+#include "myx_shared_util_functions.h"
 
 // needed from my_global.h, cant include it because of conflicts
 #ifdef __GNUC__
@@ -48,6 +54,8 @@ typedef struct {
   MYX_QUERY_CALLBACK pre_query_hook;
   MYX_QUERY_CALLBACK post_query_hook;
   void *client_data;
+  unsigned int resultset_memory_limit; // 0 for no limit, > 0 for xMB of limit
+  char **schema_stack;
   short minor_version;
   short major_version;
   short patchlevel;
@@ -185,7 +193,17 @@ void oi_ansi_quotes(ADVANCED_OPTION_DESCRIPTION * opt,
                     const char * value,
                     MYSQL_REAL_CONNECT_PARAMETERS * options)
 {
-  options->ansi_quotes= value ? toupper(*value) == 'Y' : -1;
+  if (value)
+  {
+    switch (toupper(*value))
+    {
+      case 'Y': options->ansi_quotes= 1; break;
+      case 'N': options->ansi_quotes= 0; break;
+      default:  options->ansi_quotes= -1; break;
+    }
+  }
+  else
+    options->ansi_quotes= -1;
 }
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -245,6 +263,20 @@ __declspec(dllexport) BOOL WINAPI DllMain(
 }
 #endif 
 
+
+int myx_init_library(const char *datadir)
+{
+#ifdef ENABLE_NLS
+  char *path= g_strdup_printf("%s/locale", datadir);
+  bindtextdomain(GETTEXT_PACKAGE, path);
+  g_free(path);
+  bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
+  textdomain(GETTEXT_PACKAGE);
+#endif
+  return 0;
+}
+
+
 ///////////////////////////////////////////////////////////////////////////////
 /** @brief get version of library
     @ingroup Common_functions
@@ -349,6 +381,23 @@ MYX_MYSQL *myx_mysql_get_private(MYSQL *mysql)
 }
 
 
+unsigned int myx_mysql_get_resultset_size_limit(MYSQL *mysql)
+{
+  MYX_MYSQL *data= myx_mysql_get_private(mysql);
+  if (data)
+    return data->resultset_memory_limit;
+  return 0;
+}
+
+
+void myx_mysql_limit_resultset_size(MYSQL *mysql, unsigned int limitMB)
+{
+  MYX_MYSQL *data= myx_mysql_get_private(mysql);
+  if (data)
+    data->resultset_memory_limit= limitMB;
+}
+
+
 const char *server_args[] =
 {
   "app_dummy",
@@ -457,19 +506,19 @@ static void modify_sql_mode(MYSQL *mysql, int ansi_quotes)
   MYSQL_ROW row;
   char *current_mode= NULL;
 
-  if (!mysql_query(mysql, "SELECT @@sql_mode"))
-  {
-    res= mysql_store_result(mysql);
-    row= mysql_fetch_row(res);
-    current_mode= g_strdup(row[0]);
-    mysql_free_result(res);
-  }
-
   // make sure we're not in ansi_quotes mode, unless asked to
   if (mysql_version_is_later_or_equal_than(mysql,4,1))
   {
     char *query;
 
+    if (!mysql_query(mysql, "SELECT @@sql_mode"))
+    {
+      res= mysql_store_result(mysql);
+      row= mysql_fetch_row(res);
+      current_mode= g_strdup(row[0]);
+      mysql_free_result(res);
+    }
+        
     if (ansi_quotes < 0) // not set, keep current value
       ;
     else if (ansi_quotes == 0) // OFF
@@ -479,19 +528,21 @@ static void modify_sql_mode(MYSQL *mysql, int ansi_quotes)
       {
         char **tokens= g_strsplit(current_mode, ",", 0);
         unsigned int i, j;
-        for (i= j= 0; tokens[i]; i++)
+        for (i= j= 0; tokens[j]; )
         {
-          if (strcasecmp(tokens[i], "ANSI_QUOTES")==0)
-            ;
-          else
+          if (strcasecmp(tokens[j], "ANSI_QUOTES")==0)
+          {
+            g_free(tokens[j]);
             j++;
-          tokens[i]= tokens[j];
+            continue;
+          }
+          tokens[i++]= tokens[j++];
         }
-        tokens[j]= NULL;
+        tokens[i]= NULL;
         g_free(current_mode);
         if (tokens[0])
         {
-          current_mode= g_strjoin(",", tokens);
+          current_mode= g_strjoinv(",", tokens);
           g_strfreev(tokens);
         }
         else
@@ -510,13 +561,12 @@ static void modify_sql_mode(MYSQL *mysql, int ansi_quotes)
       }
     }
 
-    query= g_strdup_printf("SET SESSION sql_mode='%s'", current_mode);
+    query= g_strdup_printf("SET SESSION sql_mode='%s'", current_mode ? current_mode : "");
     mysql_query(mysql, query);
     g_free(query);
   }
 }
 
-
 ///////////////////////////////////////////////////////////////////////////////
 /** @brief prepare a connection to a MySQL server
     @ingroup bridge_to_MySQL_API
@@ -558,22 +608,10 @@ int myx_connect_to_instance(MYX_USER_CONNECTION *user_conn, MYSQL *mysql)
 #if !(defined(__WIN__) || defined(_WIN32) || defined(_WIN64))
   if (!connect_options.unix_socket)
   {
-    gchar *out= NULL, *err= NULL;
-//    gint est; can't request exist status because when it's being called
-//    in the GRT, it will have SIGCHLD caught by the JVM and g_spawn needs it
-    if (g_spawn_command_line_sync("mysql_config --socket",
-                                  &out, &err, NULL,
-                                  NULL) && out)
-    {
-      char *ptr= strchr(out, '\n');
-      if (ptr)
-      {
-        *ptr= 0;
-        unix_socket= g_strdup(out);
-      }
-    }
-    g_free(out);
-    g_free(err);
+    if (access("/tmp/mysql.sock", F_OK) == 0)
+      unix_socket= g_strdup("/tmp/mysql.sock");
+    else if (access("/var/lib/mysql/mysql.sock", F_OK) == 0)
+      unix_socket= g_strdup("/var/lib/mysql/mysql.sock");
   }
 #endif
   
@@ -581,11 +619,14 @@ int myx_connect_to_instance(MYX_USER_CONNECTION *user_conn, MYSQL *mysql)
   //of mysql_real_connect are in utf8
   //mysql_options(mysql, MYSQL_SET_CHARSET_NAME, "utf8");
 
+#if MYSQL_VERSION_ID >= 40103
   if (myx_mysql_get_private(mysql)->embedded)
     mysql_options(mysql, MYSQL_OPT_USE_EMBEDDED_CONNECTION, (char *)1);
   else
     mysql_options(mysql, MYSQL_OPT_USE_REMOTE_CONNECTION, (char *)1);
+#endif
 
+  mysql_options(mysql, MYSQL_SET_CHARSET_NAME, "utf8");
   if (!mysql_real_connect(mysql, user_conn->hostname, user_conn->username,
                           user_conn->password, user_conn->schema,
                           user_conn->port, 
@@ -709,6 +750,54 @@ int myx_use_schema(MYSQL *mysql, const char *schema)
   return res;
 }
 
+
+int myx_push_schema(MYSQL *mysql, const char *schema)
+{
+  char *schema_name= myx_convert_utf8_dbstr(mysql, schema);
+  char *old_schema= myx_get_default_schema(mysql);
+  int res= mysql_select_db(mysql, schema_name);
+  MYX_MYSQL *priv= myx_mysql_get_private(mysql);
+  int i= 0;
+  
+  if (priv->schema_stack)
+    while (priv->schema_stack[i]) i++;
+  
+  priv->schema_stack= g_realloc(priv->schema_stack, sizeof(char*)*(i+2));
+  
+  priv->schema_stack[i]= old_schema;
+  priv->schema_stack[i+1]= NULL;
+  
+  g_free(schema_name);
+  
+  return res;
+}
+
+int myx_pop_schema(MYSQL *mysql)
+{
+  char *schema_name;
+  MYX_MYSQL *priv= myx_mysql_get_private(mysql);
+  int i;
+  
+  if (!priv->schema_stack)
+    return -1;
+  
+  for (i= 0; priv->schema_stack[i]; i++);
+  
+  if (i == 0)
+    return -1;
+  
+  schema_name= myx_convert_utf8_dbstr(mysql, priv->schema_stack[i-1]);
+  g_free(priv->schema_stack[i-1]);
+  priv->schema_stack[i-1]= NULL;
+
+  i= mysql_select_db(mysql, schema_name);
+  
+  g_free(schema_name);
+  
+  return i;
+}
+
+
 ///////////////////////////////////////////////////////////////////////////////
 /** @brief return the connections default schema (database)
     @ingroup bridge_to_MySQL_API
@@ -736,6 +825,19 @@ unsigned long myx_get_thread_id(MYSQL *mysql)
   return mysql_thread_id(mysql);
 }
 
+///////////////////////////////////////////////////////////////////////////////
+/** @brief Checks availability of the connection.
+    @ingroup bridge_to_MySQL_API
+
+    @param mysql connection handler
+
+    @return @see mysql_ping
+*//////////////////////////////////////////////////////////////////////////////
+MYX_PUBLIC_FUNC int myx_ping_server(MYSQL *mysql)
+{
+  return mysql_ping(mysql);
+}
+
 ///////////////////////////////////////////////////////////////////////////////
 /** @brief get mysql_errno() for connection
     @ingroup bridge_to_MySQL_API
@@ -872,7 +974,7 @@ void myx_get_mysql_version(MYSQL *mysql)
     const char *version;
     int major= 0, minor= 0, pl= 0;
     version= mysql_get_server_info(mysql);
-    if ( sscanf(version, "%i.%i.%i%*s", &major, &minor, &pl) != 3)
+    if ( version && sscanf(version, "%i.%i.%i%*s", &major, &minor, &pl) < 3)
     {
     }
     priv->major_version= major;
@@ -919,6 +1021,25 @@ int myx_get_mysql_minor_version(MYSQL *mysql)
   return priv->minor_version;
 }
 
+///////////////////////////////////////////////////////////////////////////////
+/** @brief get release number of mysql server
+    @ingroup bridge_to_MySQL_API
+
+    @param mysql connection handler
+    @return release number of connection
+
+    calls myx_get_mysql_version() and returns minor_version
+*//////////////////////////////////////////////////////////////////////////////
+MYX_PUBLIC_FUNC int myx_get_mysql_release(MYSQL *mysql)
+{
+  MYX_MYSQL *priv;
+  myx_get_mysql_version(mysql);
+  
+  priv= myx_mysql_get_private(mysql);
+
+  return priv->patchlevel;
+}
+
 ///////////////////////////////////////////////////////////////////////////////
 /** @brief get information about the server machine
     @ingroup Host_information_management
@@ -1041,7 +1162,7 @@ int myx_free_pc_info(MYX_MACHINE_INFO *machine_info)
  * DESCRIPTION
  *   Checks whether a text string matches a given search string,
  *   interpreting certain wild characters appropriately.
- * 
+ *
  * RETURN VALUE
  * 
  * -1 if the search string is empty
@@ -1061,7 +1182,7 @@ static gunichar dummy(gunichar c)
   return c;
 }
 
-int myx_match_pattern(const char *text, const char *search, int case_sensitive)
+int myx_match_pattern(const char *text, const char *search, int case_sensitive, int strict)
 {
 #ifdef __GNUC__
   const char *stack[strlen(text)+2];
@@ -1069,8 +1190,10 @@ int myx_match_pattern(const char *text, const char *search, int case_sensitive)
   const char **stack= g_malloc(sizeof(char)*((unsigned int)strlen(text)+2));
 #endif
   int stackp= 0;
+  const char *search_begin= search;
   const char *tend= text+g_utf8_strlen(text, (gssize)strlen(text));
   const char *send= search+g_utf8_strlen(text, (gssize)strlen(search));
+
   gunichar (*normcase)(gunichar)= case_sensitive ? dummy : g_unichar_toupper;
 
   if (!search || !*search)
@@ -1123,7 +1246,20 @@ int myx_match_pattern(const char *text, const char *search, int case_sensitive)
 #ifndef __GNUC__
   g_free((void*)stack);
 #endif
-  return !*search;
+
+  if (strict)
+  {
+    if (!*search)
+    {
+      if (search > search_begin && g_utf8_prev_char(search) && *g_utf8_prev_char(search) == '*')
+        return 1;
+      return *text ? 0 : 1;
+    }
+    else
+      return 0;
+  }
+  else
+    return !*search;
 }
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -1615,4 +1751,9 @@ int myx_check_utf8_convertible(const char *str, const char *encoding)
   return 1;
 }
 
+int myx_free_lib_stringlist(MYX_STRINGLIST *stringlist)
+{
+  return myx_free_stringlist(stringlist);
+}
+
 /** @} */ // end of bridge_to_MySQL_API
