@@ -16,7 +16,7 @@
 
 #include <myx_public_interface.h>
 #include <myx_library.h>
-#include <myx_aux_functions.h>
+#include <myx_util_functions.h>
 #include <myx_recordset.h>
 #include <myx_query.h>
 #include <string.h>
@@ -57,6 +57,9 @@
 #endif
 
 
+extern unsigned int myx_mysql_get_resultset_size_limit(MYSQL *mysql);
+
+
 static int int_compare(const char*, const char*);
 static int dummy_compare(const char*, const char*);
 
@@ -128,17 +131,30 @@ MYX_RESULTSET * myx_query_execute(MYSQL *mysql, const char *sql, int enforce_edi
   MYX_RS_FIELD *current_field;
 
   MYX_TIMER_VALUE timer;
+  
+  bigint memory_limit= myx_mysql_get_resultset_size_limit(mysql) * 1024LL * 1024LL;
 
+  if (!resultset)
+  {
+    *error_code= MYX_OUT_OF_MEMORY;
+    return NULL;
+  }
   *error_code= MYX_NO_ERROR;
 
   resultset->mysql= mysql;
   if (sql)
   {
     resultset->query= query_analyze(mysql, sql);
+    if (!resultset->query)
+    {
+      *error_code= MYX_OUT_OF_MEMORY;
+      g_free(resultset);
+      return NULL;
+    }
 
     //Set parameters
     query_set_params(resultset->query, parameters);
-
+    
     if ((resultset->query->query_type != MYX_QT_SELECT) &&
         (resultset->query->query_type != MYX_QT_SHOW) &&
         (resultset->query->query_type != MYX_QT_DESCRIBE) &&
@@ -193,6 +209,14 @@ MYX_RESULTSET * myx_query_execute(MYSQL *mysql, const char *sql, int enforce_edi
       else
         resultset->columns_num_to_display= resultset->columns_num;
       resultset->columns= g_malloc0(sizeof(MYX_RS_COLUMN) * resultset->columns_num);
+      
+      if (!resultset->columns)
+      {
+        *error_code= MYX_OUT_OF_MEMORY;
+        mysql_free_result(res);
+        return NULL;
+      }
+      
       fields = mysql_fetch_fields(res);
 
       for (i= 0;i<resultset->columns_num;i++)
@@ -201,6 +225,13 @@ MYX_RESULTSET * myx_query_execute(MYSQL *mysql, const char *sql, int enforce_edi
 
         //Set column infos
         current_column->name= myx_convert_dbstr_utf8(mysql, fields[i].name);
+        if (!current_column->name)
+        {
+          *error_code= MYX_OUT_OF_MEMORY;
+          mysql_free_result(res);
+          myx_query_free_resultset(resultset);
+          return NULL;
+        }
         switch(fields[i].type)
         {
         //To be completed
@@ -216,6 +247,9 @@ MYX_RESULTSET * myx_query_execute(MYSQL *mysql, const char *sql, int enforce_edi
         case MYSQL_TYPE_DOUBLE:
           current_column->column_type= MYX_RSCT_FLOAT;
           break;
+        case MYSQL_TYPE_DECIMAL:
+          current_column->column_type= MYX_RSCT_DECIMAL;
+          break;
         case MYSQL_TYPE_BLOB:
         case MYSQL_TYPE_TINY_BLOB:
         case MYSQL_TYPE_MEDIUM_BLOB:
@@ -254,19 +288,40 @@ MYX_RESULTSET * myx_query_execute(MYSQL *mysql, const char *sql, int enforce_edi
       resultset->rows= g_malloc0(sizeof(MYX_RS_ROW)*ROWS_ALLOCATE_BLOCK_COUNT);
       rows_allocated= ROWS_ALLOCATE_BLOCK_COUNT;
 
+      if (!resultset->rows)
+      {
+        *error_code= MYX_OUT_OF_MEMORY;
+        mysql_free_result(res);
+        myx_query_free_resultset(resultset);
+        return NULL;
+      }
+      
       row_count= 0;
       while ((row= mysql_fetch_row(res)))
       {
         if (row_count>=rows_allocated)
         {
+          void *new_buffer;
+          
           if(resultset_realloc_before)
             (* resultset_realloc_before)(user_data);
 
-          resultset->rows= g_realloc(resultset->rows, sizeof(MYX_RS_ROW) * (rows_allocated+ROWS_ALLOCATE_BLOCK_COUNT));
+          new_buffer= g_realloc(resultset->rows, sizeof(MYX_RS_ROW) * (rows_allocated+ROWS_ALLOCATE_BLOCK_COUNT));
           rows_allocated+= ROWS_ALLOCATE_BLOCK_COUNT;
 
           if(resultset_realloc_after)
             (* resultset_realloc_after)(user_data);
+
+          if (!new_buffer)
+          {
+            // out of mem, return with what we've got so far
+            *error_code= MYX_OUT_OF_MEMORY;
+            resultset->rows_num= row_count;
+            mysql_free_result(res);
+            return resultset;
+          }
+          else
+            resultset->rows= new_buffer;
         }
 
         //Get field lengths
@@ -278,14 +333,25 @@ MYX_RESULTSET * myx_query_execute(MYSQL *mysql, const char *sql, int enforce_edi
         
         //Alloc current row
         current_row->fields= g_malloc0(sizeof(MYX_RS_FIELD) * resultset->columns_num);
+        if (!current_row->fields)
+        {
+          // out of mem, return with what we've got so far
+          *error_code= MYX_OUT_OF_MEMORY;
+          resultset->rows_num= row_count;
+          mysql_free_result(res);
+          return resultset;
+        }
 
-        for (k= 0;k<resultset->columns_num;k++)
+        for (k= 0; k < resultset->columns_num; k++)
         {
           current_column= resultset->columns + k;
           current_field= current_row->fields + k;
 
           current_field->value_length= field_lengths[k];
-          if (current_field->value_length>0)
+          
+          resultset->memory_used+= current_field->value_length;
+          
+          if (current_field->value_length > 0)
           {
             if (current_column->column_type==MYX_RSCT_STRING
                 || current_column->column_type==MYX_RSCT_TEXT)
@@ -299,20 +365,45 @@ MYX_RESULTSET * myx_query_execute(MYSQL *mysql, const char *sql, int enforce_edi
               current_field->value[current_field->value_length]= 0;
             }
 
-            bytes_loaded+=current_field->value_length;
+            bytes_loaded+= current_field->value_length;
           }
           else
           {
             if ((current_column->column_type == MYX_RSCT_STRING
                  || current_column->column_type == MYX_RSCT_TEXT)
                 && current_field->value_length == 0 && row[k]!=NULL)
-            {
               current_field->value= g_strdup("");
-            }
+            else
+              continue; // NULL blob, don't check whether malloc failed
+          }
+          
+          // checks whether the value copy failed 
+          // (if the value is NULL then either the field is NULL or the
+          // malloc failed)
+          if (current_field->value==NULL && row[k]!=NULL)
+          {
+            int ii= k;
+            while (ii >= 0)
+              g_free(current_row->fields[ii--].value);
+            g_free(current_row->fields);
+            // out of mem, return with what we've got so far
+            *error_code= MYX_OUT_OF_MEMORY;
+            resultset->rows_num= row_count;
+            mysql_free_result(res);
+            return resultset;
           }
         }
         row_count++;
 
+        // check if we've reached the memory usage limit
+        if (memory_limit > 0 && resultset->memory_used > memory_limit)
+        {
+          *error_code= MYX_MEMORY_LIMIT_EXCEEDED;
+          resultset->fetch_time= timer_stop(&timer);
+          resultset->rows_num= row_count;
+          return resultset;
+        }
+        
         //Call the callback-function after a fixed number of bytes have been transfered
         if (bytes_loaded>=CALLBACK_AFTER_BYTES)
         {
@@ -326,6 +417,7 @@ MYX_RESULTSET * myx_query_execute(MYSQL *mysql, const char *sql, int enforce_edi
             //return NULL;
 
             *error_code= MYX_STOP_EXECUTION;
+            resultset->rows_num= row_count;
             resultset->fetch_time= timer_stop(&timer);
             return resultset;
           }
@@ -342,6 +434,8 @@ MYX_RESULTSET * myx_query_execute(MYSQL *mysql, const char *sql, int enforce_edi
 
       resultset->rows_num= row_count; //(unsigned int)mysql_num_rows(res);
       resultset->rows= g_realloc(resultset->rows, sizeof(MYX_RS_ROW)*resultset->rows_num);
+      
+      resultset->memory_used+= sizeof(MYX_RS_ROW)*resultset->rows_num;
 
       //Call the callback-function one last time to let the program fetch the last records
       if (*error_code != MYX_STOP_EXECUTION && progress_row_fetch)
@@ -431,9 +525,13 @@ static int do_rs_action_add(MYX_RESULTSET *resultset, MYX_RS_ACTION *action)
   return 0;
 }
 
-
+// a and b are NOT values but pointers to the values!!!
 static int row_compare(const void *a, const void *b)
 {
+  int aa = *(const int *)(a);
+  int bb = *(const int *)(b);
+  return bb - aa;
+/*
 #if defined(__WIN__) || defined(_WIN32) || defined(_WIN64)
   if (a < b)
     return 1;
@@ -449,6 +547,7 @@ static int row_compare(const void *a, const void *b)
   else
     return 0;
 #endif
+*/
 }
 
 
@@ -589,6 +688,9 @@ int myx_query_update_resultset(MYX_RESULTSET *resultset)
     unsigned int c;
     unsigned int j;
 
+    if (row >= resultset->rows_num)
+      continue;
+
     if (resultset->actions)
     {
       // Move actions up if the are below a deleted row
@@ -623,6 +725,26 @@ int myx_query_update_resultset(MYX_RESULTSET *resultset)
 }
 
 
+int myx_query_pack_resultset(MYX_RESULTSET *resultset)
+{
+  unsigned int row;
+  for (row= 0; row < resultset->rows_num; row++)
+  {
+    if (resultset->rows[row].fields == NULL)
+    {
+      if (row < resultset->rows_num-1)
+      {
+        memmove(&resultset->rows[row],
+                &resultset->rows[row+1],
+                (resultset->rows_num-row-1)*sizeof(MYX_RS_ROW));
+      }
+      resultset->rows_num--;
+    }
+  }
+  return 0;
+}
+
+
 int myx_query_free_resultset(MYX_RESULTSET *resultset)
 {
   MYX_RS_COLUMN *current_column;
@@ -631,13 +753,15 @@ int myx_query_free_resultset(MYX_RESULTSET *resultset)
 
   unsigned int i, j;
 
-  for(i=0;i<resultset->columns_num;i++)
+  if (resultset->columns)
   {
-    current_column= resultset->columns+i;
-    g_free(current_column->name);
+    for(i=0;i<resultset->columns_num;i++)
+    {
+      current_column= resultset->columns+i;
+      g_free(current_column->name);
+    }
+    g_free(resultset->columns);
   }
-  g_free(resultset->columns);
-
   for(i=0;i<resultset->rows_num;i++)
   {
     current_row= resultset->rows+i;
@@ -758,6 +882,7 @@ static void prepare_value(MYSQL *mysql, const char *value, unsigned int value_le
     {
     case MYX_RSCT_INTEGER:
     case MYX_RSCT_FLOAT:
+    case MYX_RSCT_DECIMAL:
       memcpy(buffer, value, value_length);
       buffer[value_length]= 0;
       break;
@@ -768,8 +893,9 @@ static void prepare_value(MYSQL *mysql, const char *value, unsigned int value_le
     case MYX_RSCT_TIME:
     case MYX_RSCT_DATE:
     case MYX_RSCT_DATETIME:
-      g_snprintf(buffer, value_length*2, "'%.*s'", 
-                 value_length, value);
+//      g_snprintf(buffer, value_length*2, "'%.*s'", 
+//                 value_length, value);
+      g_snprintf(buffer, value_length*2 + 2 + 1, "'%s'", value);
       break;
 
     case MYX_RSCT_STRING:
@@ -821,7 +947,7 @@ static char *prepare_pks(MYX_RESULTSET *rs, MYX_RS_ACTION *action)
   g_assert(rs->columns_num == query->columns_num);
 
   //if this is not an added row, take value from RS
-  if(action->row <= rs->rows_num)
+  if(action->row < rs->rows_num)
   {
     // iterate over list of columns
     for (i= 0; i < query->columns_num; i++)
@@ -884,11 +1010,52 @@ static void append_error(MYX_RS_ACTION_ERRORS **errors, MYSQL *mysql,
   
   error= (*errors)->errors + (*errors)->errors_num-1;
   
+  error->level= MYX_QEL_ERROR;
   error->error= myx_mysql_errno(mysql);
-  error->error_text= g_strdup(myx_mysql_error(mysql));
+  error->error_text= myx_mysql_error(mysql); // myx_mysql_error() already strdup()s
   error->action= action;
 }
 
+#if MYSQL_VERSION_ID >= 50000
+static void append_warnings(MYX_RS_ACTION_ERRORS **errors, MYSQL *mysql, 
+                            MYX_RS_ACTION *action)
+{
+  MYX_RS_ACTION_ERROR *error;
+  MYSQL_RES *res;
+  MYSQL_ROW row;
+  int c;
+
+  c= mysql_warning_count(mysql);
+  if (c > 0 && !myx_mysql_query(mysql, "SHOW WARNINGS") &&
+      (res= mysql_use_result(mysql)))
+  {
+    if (!*errors)
+      *errors= g_malloc0(sizeof(MYX_RS_ACTION_ERRORS));
+
+    (*errors)->errors= g_realloc((*errors)->errors,
+                                 sizeof(MYX_RS_ACTION_ERROR)*((*errors)->errors_num+c));
+
+    while ((row= mysql_fetch_row(res)))
+    {
+      error= (*errors)->errors + (*errors)->errors_num++;
+
+      if (strcasecmp(row[0], "Note")==0)
+        error->level= MYX_QEL_NOTE;
+      else
+        error->level= MYX_QEL_WARNING;
+      error->error= atoi(row[1]);
+      error->error_text= g_strdup(row[2]);
+      error->action= action;
+    }
+
+    mysql_free_result(res);
+  }
+}
+#else // mysql < 5.0
+// nothing
+# define append_warnings(a,b,c) do {} while (0)
+#endif // mysql < 5.0
+
 #if 0
 static void append_charset_error(MYX_RS_ACTION_ERRORS **errors,
                                  const char *charset,
@@ -966,6 +1133,7 @@ static int add_same_row_actions_of_type(MYX_RESULTSET *resultset, int i, int add
   MYX_RS_ACTION *action= resultset->actions->actions + i;
   char **columns_arr= NULL;
   char **values_arr= NULL;
+  char *t;
   unsigned int j;
   int count= 1;
 
@@ -999,14 +1167,19 @@ static int add_same_row_actions_of_type(MYX_RESULTSET *resultset, int i, int add
     {
       int q;
       // look up the column in the list
-      for (q= 0; columns_arr[q]; q++)
-        if (strcmp(action2->column->name, columns_arr[q])==0)
+      for (q= 0; columns_arr[q]; q++) 
+      {
+        t= quote_identifier(action2->column->name);
+        if (strcmp(t, columns_arr[q])==0)
         { // value is already in the list, replace it with the newer one
+          g_free(t);
           g_free(values_arr[q]);
           prepare_action_value(resultset->mysql, action2, buffer);
           values_arr[q]= g_strdup(buffer);
           break;
         }
+        g_free(t);
+      }
       if (!columns_arr[q])
       {
 //        if (myx_identifier_needs_quotes(action->column->name))
@@ -1120,6 +1293,7 @@ static int do_update_value(MYX_RESULTSET *resultset, const char *pks,
                            MYX_RS_ACTION_ERRORS **errors, char *buffer)
 {
   char *query;
+  int mysql_is_50= mysql_version_is_later_or_equal_than(resultset->mysql,5,0);
 
   prepare_action_value(resultset->mysql, action, buffer);
 
@@ -1148,10 +1322,16 @@ static int do_update_value(MYX_RESULTSET *resultset, const char *pks,
   {
     action->status= MYX_RSAS_FAILED;
     append_error(errors, resultset->mysql, action);
+    
+    if (mysql_is_50)
+      append_warnings(errors, resultset->mysql, action);
   }
   else
   {
     action->status= MYX_RSAS_APPLIED;
+    
+    if (mysql_is_50)
+      append_warnings(errors, resultset->mysql, action);    
   }
   g_free(query);
         
@@ -1174,6 +1354,7 @@ MYX_RS_ACTION_ERRORS *myx_query_apply_actions(MYX_RESULTSET *resultset)
   char *buffer= NULL;
   char already_deleted;
   int mysql_is_41= mysql_version_is_later_or_equal_than(resultset->mysql,4,1);
+  int mysql_is_50= mysql_version_is_later_or_equal_than(resultset->mysql,5,0);
 
   buffer_size= 100;
   // go through all values to make a big enough buffer
@@ -1260,12 +1441,17 @@ MYX_RS_ACTION_ERRORS *myx_query_apply_actions(MYX_RESULTSET *resultset)
         {
           result= MYX_RSAS_FAILED;
           append_error(&errors, resultset->mysql, action);
+          if (mysql_is_50)
+            append_warnings(&errors, resultset->mysql, action);
         }
         else
         {
           unsigned int c;
           MYX_RS_COLUMN *auto_inc_column= NULL;
           
+          if (mysql_is_50)
+            append_warnings(&errors, resultset->mysql, action);
+          
           /* check if the table has an auto-inc column that wasn't mentioned
            in the INSERT. in that case, we will get the value assigned to that
            column and add an action to the resultset with that value to signal
@@ -1407,9 +1593,16 @@ MYX_RS_ACTION_ERRORS *myx_query_apply_actions(MYX_RESULTSET *resultset)
           {
             action->status= MYX_RSAS_FAILED;
             append_error(&errors, resultset->mysql, action);
+            
+            if (mysql_is_50)
+              append_warnings(&errors, resultset->mysql, action);
           }
           else
+          {
             action->status= MYX_RSAS_APPLIED;
+            if (mysql_is_50)
+              append_warnings(&errors, resultset->mysql, action);
+          }
           g_free(query);
         }
         else
@@ -1451,7 +1644,7 @@ MYX_RS_ACTION_ERRORS *myx_query_apply_actions(MYX_RESULTSET *resultset)
       str= NULL;
       for (j= 0; keys_arr[j]; j++)
       {
-        char *tmp= g_strdup_printf("%s`%s`=%s", j>0?", ":"",keys_arr[j], values_arr[j]);
+        char *tmp= g_strdup_printf("%s%s=%s", j>0?", ":"", keys_arr[j], values_arr[j]);
 
         str= str_g_append_and_free(str, tmp);
       }
@@ -1475,10 +1668,16 @@ MYX_RS_ACTION_ERRORS *myx_query_apply_actions(MYX_RESULTSET *resultset)
       {
         action->status= MYX_RSAS_FAILED;
         append_error(&errors, resultset->mysql, action);
+        
+        if (mysql_is_50)
+          append_warnings(&errors, resultset->mysql, action);
       }
       else
       {
         action->status= MYX_RSAS_APPLIED;
+        
+        if (mysql_is_50)
+          append_warnings(&errors, resultset->mysql, action);
       }
       
       // go over all the involved records and update their status
@@ -1495,6 +1694,7 @@ MYX_RS_ACTION_ERRORS *myx_query_apply_actions(MYX_RESULTSET *resultset)
   return errors;
 }
 
+
 int myx_query_discard_actions(MYX_RESULTSET *resultset)
 {
   unsigned int i;
@@ -1646,6 +1846,12 @@ int myx_query_compare_possible(MYX_RESULTSET *ra, MYX_RESULTSET *rb)
 {
   unsigned int i;
   
+  // fix of #12165
+  if(!ra || !rb)
+  {
+    return 0;
+  }
+  
   if (ra->columns_num != rb->columns_num)
     return 0;
   
@@ -1907,8 +2113,15 @@ static char *replace_variables(char *str, MYX_RESULTSET *resultset)
   time_t t;
   char *tmp;
 
-  tmp= str_g_replace(g_strdup(resultset->query->sql), "\r\n", " ");
-  tmp= str_g_replace(tmp, "\n", " ");
+  if(resultset->query && resultset->query->sql)
+  {
+    tmp= str_g_replace(g_strdup(resultset->query->sql), "\r\n", " ");
+    tmp= str_g_replace(tmp, "\n", " ");
+  }
+  else
+  {
+    tmp= g_strdup("");
+  }
 
   time(&t);
   str= str_g_replace(str, "$DATE$", ctime(&t));
@@ -2007,6 +2220,7 @@ static MYX_TE_COLUMN *setup_columns(MYX_RESULTSET *resultset)
     columns[c].name= resultset->columns[c].name;
     switch (resultset->columns[c].column_type)
     {
+    case MYX_RSCT_DECIMAL:
     case MYX_RSCT_INTEGER:
       columns[c].ctype= MYX_TE_COL_INTEGER;
       break;
@@ -2203,5 +2417,3 @@ int myx_export_resultset(MYSQL *mysql, MYX_TABLE_EXPORTER_INFO *info,
 
   return 0;
 }
-
-
