@@ -28,7 +28,6 @@
  */
 
 static int make_state_transition(int state, bigint c, MYX_SQL_PARSE_ENVIRONMENT *we, int *delim_i);
-static char *check_delimiter(const char *buffer, unsigned int buflen);
                                     
 /* (note: this diagram is missing several transitions)
 
@@ -164,23 +163,20 @@ int myx_get_next_sql_statement(MYX_SQL_PARSE_ENVIRONMENT *we,
 
   h= 0;
   delim_i= 0;
-  while ( state!=1 && (resize_buffer || h< *buffer_len-6) &&
+  while ( state!=1 && (resize_buffer || h< (*buffer_len)-6) &&
          (c= read_char(arg, &bytes_read, error)) != -1 )
   {
     we->chars_read++;
     we->bytes_read+= bytes_read;
 
-    if((we->bytes_read % 1024) == 0)
+    if((progress_report != NULL) && ((we->bytes_read % 8192) == 0))
+    {
+      if(progress_report(we->bytes_read, file_size, user_data))
       {
-      if(progress_report)
-      {
-        if(progress_report(we->bytes_read, file_size, user_data))
-        {
-          return -1;
-        }
+        return -1;
       }
     }
-    if (resize_buffer && h == *buffer_len - 7)
+    if (resize_buffer && !(h < (*buffer_len) - 7))
     {
       gchar *tmp;
 
@@ -196,8 +192,35 @@ int myx_get_next_sql_statement(MYX_SQL_PARSE_ENVIRONMENT *we,
     }
 
     if (c == '\n')
+    {
       we->line_num++;
 
+      // Check for the special delimiter command.
+      if (h > sizeof("delimiter "))
+      {
+        if (g_strncasecmp(*buffer, "delimiter ", sizeof("delimiter ") - 1) == 0)
+        {
+          // DELIMITER keyword found. Extract the new delimiter.
+          char *delim = *buffer + sizeof("delimiter ") - 1;
+          delim = str_trim(g_strndup(delim, (gsize) h - sizeof("delimiter ") + 1));
+          if (delim)
+          {
+            g_free(we->alt_delimiter);
+            if (strcmp(delim, ";") == 0)
+            {
+              g_free(delim);
+              we->alt_delimiter= NULL;
+            }
+            else
+              we->alt_delimiter= delim;
+            delim_i= 0;
+            state= 1; // force final state
+            break;
+          };
+        };
+      };
+    };
+
     /* Record the character if we are in a "no comment"-section */
     if (state != 6 && state != 5 && state != 9 && state != 12)
     {
@@ -207,29 +230,12 @@ int myx_get_next_sql_statement(MYX_SQL_PARSE_ENVIRONMENT *we,
           * Windows uses a byte order mark (BOM) for example to mark a 
           *  file as UTF8
           */
-        char *delim;
         if (h == 0)
         {
           we->stmt_begin_char= we->chars_read-1;//we->bytes_read - 1;
           we->stmt_begin_line= we->line_num;
         }
         h+= g_unichar_to_utf8( (gunichar) c, (*buffer)+h );
-        
-        delim= check_delimiter(*buffer, h);
-        if (delim)
-        {
-          g_free(we->alt_delimiter);
-          if (strcmp(delim, ";")==0)
-          {
-            g_free(delim);
-            we->alt_delimiter= NULL;
-          }
-          else
-            we->alt_delimiter= delim;
-          delim_i= 0;
-          state= 1; // force final state
-          break;
-        }
       }
     }
 
@@ -248,7 +254,7 @@ int myx_get_next_sql_statement(MYX_SQL_PARSE_ENVIRONMENT *we,
       h-= 1;
       if (h == 0)
       {
-        we->stmt_begin_char= we->chars_read-1;//we->bytes_read - 1;
+        we->stmt_begin_char= we->chars_read-1;
         we->stmt_begin_line= we->line_num;
       }
     }
@@ -257,7 +263,7 @@ int myx_get_next_sql_statement(MYX_SQL_PARSE_ENVIRONMENT *we,
       h-= 3;
       if (h == 0)
       {
-        we->stmt_begin_char= we->chars_read-1;//we->bytes_read - 1;
+        we->stmt_begin_char= we->chars_read-1;
         we->stmt_begin_line= we->line_num;
       }
     }
@@ -269,10 +275,10 @@ int myx_get_next_sql_statement(MYX_SQL_PARSE_ENVIRONMENT *we,
   if (*error) return 0;
 
   if (we->alt_delimiter)
-    we->stmt_end_char= we->chars_read//we->bytes_read 
+    we->stmt_end_char= we->chars_read
       - strlen(we->alt_delimiter);
   else
-    we->stmt_end_char= we->chars_read-1;//we->bytes_read - 1;
+    we->stmt_end_char= we->chars_read-1;
   we->stmt_end_line= we->line_num;
 
   (*buffer)[h]= '\0';
@@ -434,7 +440,6 @@ void kill_comments(char * sql_to_mask)
 {
   MYX_SQL_PARSE_ENVIRONMENT we;
   int state= 0, new_state;
-//  int delim_i= 0;
   char * pos, * next_pos;
   myx_init_sql_parse_environment_fill_zero(&we);
   for (pos= sql_to_mask; *pos; pos= next_pos)
@@ -472,7 +477,6 @@ void mask_quotas(char * sql_to_mask)
 {
   MYX_SQL_PARSE_ENVIRONMENT we;
   int state= 0;
-//  int delim_i= 0;
   char * pos, * next_pos;
   myx_init_sql_parse_environment_fill_zero(&we);
   for (pos= sql_to_mask; *pos; pos= next_pos)
@@ -507,11 +511,11 @@ void mask_quotas_and_brackets(char * sql_to_mask)
   mask_out_string(sql_to_mask,'(', ')','x');
 }
 
-static char *check_delimiter(const char *buffer, unsigned int buflen)
+static char *check_delimiterX(const char *buffer, unsigned int buflen)
 {
   unsigned int i;
   
-  if (buflen < sizeof("DELIMITER X")-1)
+  if (buflen < sizeof("DELIMITER X"))
     return NULL;
   
   for (i= 0; i < buflen; i++)
