@@ -167,8 +167,8 @@ MYX_SYN* myx_refresh_dbinfo(MYSQL *mysql, MYX_SYN *_syn)
   stmt= "SHOW TABLES";
   if ( mysql_query(mysql, stmt) || !(result= mysql_store_result(mysql)) )
   {
-    myx_free_syn(syn);
-    return NULL;
+    // probably means there's no DB selected, return with keywords only
+    return syn;
   }
 
   while ( (row= mysql_fetch_row(result)) )
@@ -355,6 +355,11 @@ int myx_highlight(MYX_SQL_HIGHLIGHTING *sql_highlighting, const char *text, unsi
   }
   else
   {
+    if (line_no >= sql_highlighting->line_states_size)
+    {
+      sql_highlighting->line_states_size+= ARRAY_INC_SIZE;
+      sql_highlighting->line_states= g_realloc(sql_highlighting->line_states, sizeof(MYX_LINE_STATE)*sql_highlighting->line_states_size);
+    }
     assert((line_no-1) >= 0 && (line_no-1) < sql_highlighting->line_states_size);
     state= sql_highlighting->line_states[line_no-1].state; 
     token_begin= (sql_highlighting->line_states[line_no-1].token_begin!=-1)? 0: -1;
@@ -449,7 +454,7 @@ int myx_highlight(MYX_SQL_HIGHLIGHTING *sql_highlighting, const char *text, unsi
 
       if (token_begin != -1)
       {
-	make_new_word(sql_highlighting, token_begin, counter-1, text, last_state);
+        make_new_word(sql_highlighting, token_begin, counter-1, text, last_state);
       }
 
       token_begin= counter;
@@ -460,7 +465,7 @@ int myx_highlight(MYX_SQL_HIGHLIGHTING *sql_highlighting, const char *text, unsi
     }
     else if (state == 5 && c == '-')
     {
-      assert(token_begin == counter-1);
+//      assert(token_begin == counter-1);
     }
     
     counter+= g_utf8_next_char(p)-p;
@@ -626,7 +631,10 @@ static void make_new_word(MYX_SQL_HIGHLIGHTING *sql_highlighting, bigint begin,
 
   if (!word->word_type)
   {
-    type= g_tree_lookup(sql_highlighting->syn->tree,token_text);
+    if (sql_highlighting->syn)
+      type= g_tree_lookup(sql_highlighting->syn->tree,token_text);
+    else
+      type= NULL;
     word->word_type= (type)?(*type):MYX_SYN_NORMAL;    
   }
 
