@@ -33,6 +33,8 @@ int main()
 	}
 #endif
       }
+// charset autodetection was remmoved from library
+#if 0
       const char * charset= myx_detect_charset_from_sql_file(filename);
       for (char * pos= filename; *pos; pos++)
       {
@@ -41,8 +43,8 @@ int main()
       }
       printf("charset in file \"%s\" is \"%s\"\n",filename,charset);
       g_free((gpointer)charset);
+#endif
     }
   }
   return 0;
 }
-
