@@ -24,15 +24,17 @@
 
 #if defined(__WIN__) || defined(_WIN32) // _WIN32 also includes WIN64
   #ifndef _WINDOWS
-    // Just to ease life. This way the symbol has to be given during build.
+    // Just to ease life. This way the symbol has not to be given during build.
     // __WIN__ and _WIN32 are implicitely defined on Windows, however _WINDOWS is not.
     #define _WINDOWS
   #endif
 
-#define  WIN32_LEAN_AND_MEAN
+  #define  WIN32_LEAN_AND_MEAN
   #include <windows.h>
   #include <direct.h>
   #include <shlobj.h> // Font folder location
+  #include <wingdi.h> // wglGetProcAddress
+  #include "mmsystem.h" // OutputDebugStr
 
   // The export/import template macro is needed if any STL template is used across DLL boundaries.
   // The only exceptions are string and wstring, as they are already exported.
@@ -50,53 +52,48 @@
 
   #include <GL/gl.h>
   #include <GL/glu.h>
-#else
+  #include "gc_glext.h"
+#else // !WINDOWS
   #define GENERIC_CANVAS_API 
   #define __cdecl
 
   #include <stdlib.h>
   #define MAX_PATH PATH_MAX
 
-  #include <X11/Xlib.h>
-
   #ifdef __APPLE__
+    #define GL_GLEXT_LEGACY
     #include <OpenGL/gl.h>
+    #include "gc_glext.h"
     #include <OpenGL/glu.h>
-    #include <OpenGL/glx.h>
-  #else
+    #include <OpenGL/OpenGL.h>
+  #else // !__APPLE__
+    #include <X11/Xlib.h>
+
+    #define GL_GLEXT_LEGACY
     #include <GL/gl.h>
+    #include "gc_glext.h"
     #include <GL/glu.h>
+    #define GLX_GLXEXT_LEGACY
     #include <GL/glx.h>
+    #include <GL/glxext.h>
   #endif
-#endif // #fidef _WINDOWS
+#endif // !_WINDOWS
 
 #define _USE_MATH_DEFINES
 #include <math.h>
 
+#include <time.h>
+
 // STL namespaces and templates.
 #include <string>
 #include <map>
 #include <set>
 #include <vector>
+#include <algorithm>
 
 using namespace std;
 
-#ifdef __GNUC__
-  #include <ext/hash_map>
-
-  #define stdext __gnu_cxx
-
-  #ifndef __myhash
-    #define __myhash
-    namespace __gnu_cxx {
-      template<class A> struct hash<A*>
-      { size_t operator()(A *__x) const { return (unsigned long)__x; } };
-    };
-  #endif
-  using namespace __gnu_cxx;
-#else
-  #include <hash_map>
-  using namespace stdext;
-#endif
+#include <errno.h>
+#include <glib.h>
 
 #endif // #ifndef __GC_H__
