@@ -1,23 +1,25 @@
-/* Copyright (C) 2004 MySQL AB
-
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-
-   This program is distributed in the hope that it will be useful,
+/* Copyright (c) 2004, 2005 MySQL AB
+  
+   This library is free software; you can redistribute it and/or
+   modify it under the terms of the GNU Lesser General Public
+   License as published by the Free Software Foundation; either
+   version 2 of the License, or (at your option) any later version.
+  
+   This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
-
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+   Lesser General Public License for more details.
+  
+   You should have received a copy of the GNU Lesser General Public
+   License along with this library; if not, write to the
+   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+   Boston, MA 02111-1307, USA.
+ */
 
 #ifndef __MYX_GRT_H__
 #define __MYX_GRT_H__
 
-#define GRT_VERSION "2.0.6 beta"
+#define GRT_VERSION "2.0.12"
 
 #if defined(__WIN__) || defined(_WIN32) || defined(_WIN64)
 # define __LCC__
@@ -26,13 +28,19 @@
 # define MYX_SOCKET int
 #endif
 
-#include <myx_public_interface.h>
-#ifdef ENABLE_LUA_MODULES
-#include <lua.h>
-#endif
+#include <myx_util_public_interface.h>
 #ifdef ENABLE_JAVA_MODULES
-#include <jni.h>
-#include <com_mysql_grt_GrtCallbackNative.h>
+# ifdef __APPLE__
+#  include <JavaVM/jni.h>
+#  ifdef FROM_XCODE
+#   include <MySQLGRT/com_mysql_grt_GrtCallbackNative.h>
+#  else
+#   include <com_mysql_grt_GrtCallbackNative.h>
+#  endif
+# else
+#  include <jni.h>
+#  include <com_mysql_grt_GrtCallbackNative.h>
+# endif
 #endif
 
 #ifdef __cplusplus
@@ -53,7 +61,8 @@ extern "C" {
 /// [SCRIPT::LibInterfaceMapper] -public_interface "libmysqlgrt"
 #define libmysqlgrt_PUBLIC_INTERFACE_VERSION 10000
 
-/// [SCRIPT::LibInterfaceMapper] -add_to_uses "myx_public_interface"
+/// [SCRIPT::LibInterfaceMapper] -add_to_uses "myx_util_public_interface, myx_public_interface"
+/// [SCRIPT::LibInterfaceMapper] -add_datatypes_from "..\..\..\mysql-gui-common\library_util\include\myx_util_public_interface.h"
 /// [SCRIPT::LibInterfaceMapper] -add_datatypes_from "..\..\..\mysql-gui-common\library\include\myx_public_interface.h"
 
 
@@ -87,7 +96,11 @@ typedef enum
 
   MYX_GRT_JAVA_NOT_FOUND,
   MYX_GRT_JAVA_REGISTRY_CORRUPTED,
-  MYX_GRT_JAVA_JRE_CANNOT_BE_LOADED
+  MYX_GRT_JAVA_JRE_CANNOT_BE_LOADED,
+
+  MYX_GRT_SHELL_UNSUPPORTED,
+  MYX_GRT_SHELL_ALREADY_LOADED,
+  MYX_GRT_SHELL_INIT_ERROR
 
 } MYX_GRT_ERROR;
 
@@ -136,6 +149,14 @@ typedef enum
   MYX_IT_MANY_STANDARD
 } MYX_ICON_TYPE;
 
+typedef enum
+{
+  MYX_GVCR_DELETE,
+  MYX_GVCR_DICT_ITEM_CHANGE,
+  MYX_GVCR_LIST_CHANGE,
+  MYX_GVCR_LIST_ITEM_CHANGE
+} MYX_GRT_VALUE_CALLBACK_REASON;
+
 /*
  * Structs
  */
@@ -143,13 +164,19 @@ typedef enum
 typedef struct MYX_GRT_MODULE_LOADER MYX_GRT_MODULE_LOADER;
 typedef struct MYX_GRT MYX_GRT;
 
+typedef enum MYX_GRT_SHELL_INTERFACE
+{
+  MYX_GRT_SHELL_LUA,
+  MYX_GRT_SHELL_PYTHON
+} MYX_GRT_SHELL_INTERFACE;
+  
 typedef enum MYX_GRT_SHELL_COMMAND
 {
   MYX_GRT_SHELL_COMMAND_UNKNOWN = -1,
   MYX_GRT_SHELL_COMMAND_EXIT = 0,
   MYX_GRT_SHELL_COMMAND_ALL,
   MYX_GRT_SHELL_COMMAND_ERROR,
-  MYX_GRT_SHELL_COMMAND_LUA,
+  MYX_GRT_SHELL_COMMAND_STATEMENT, // was _LUA
   MYX_GRT_SHELL_COMMAND_HELP,
   MYX_GRT_SHELL_COMMAND_LS,
   MYX_GRT_SHELL_COMMAND_CD,
@@ -180,6 +207,7 @@ typedef struct MYX_GRT_MODULE
   MYX_GRT_FUNCTION *functions;
   char *extends;
   struct MYX_GRT_MODULE_PRIVATE *priv; // private data for each loader
+  char dont_bridge_loaded_values;
 } MYX_GRT_MODULE;
 
 
@@ -258,6 +286,17 @@ typedef struct
   MYX_GRT_DICT_ITEM *items; // this must be in strcmp() order of name
 } MYX_GRT_DICT;
 
+typedef struct MYX_GRT_VALUE_EXTENDED
+{
+  MYX_GRT_MODULE *bridge_module; // if set, callbacks to the bridge are made if the value is changed
+  void *bridge_data_object; // data set by the bridge so it is possible to associate bridged objects with GRT values
+  void *bridge_data_owner; // data set by the bridge so it is possible to associate bridged objects with GRT values
+  char *bridge_dict_key; // if this is a dict item, store the key name so the bridge can identify the property associated with the value
+  unsigned int bridge_list_index; // if this is a list item, store the index so the bridge can identify the item associated with the value
+
+  unsigned int listeners_num;
+  struct MYX_GRT_VALUE_LISTENER **listeners;
+} MYX_GRT_VALUE_EXTENDED;
 
 typedef struct MYX_GRT_VALUE 
 {
@@ -271,14 +310,15 @@ typedef struct MYX_GRT_VALUE
     MYX_GRT_DICT *d;
   } value;
 
-  MYX_GRT_MODULE *bridge_module; // if set, callbacks to the bridge are made if the value is changed
-  void *bridge_data_object; // data set by the bridge so it is possible to associate bridged objects with GRT values
-  void *bridge_data_owner; // data set by the bridge so it is possible to associate bridged objects with GRT values
-  char *bridge_dict_key; // if this is a dict item, store the key name so the bridge can identify the property associated with the value
-  unsigned int bridge_list_index; // if this is a list item, store the index so the bridge can identify the item associated with the value
+  MYX_GRT_VALUE_EXTENDED *extended;
+} MYX_GRT_VALUE;  
 
-} MYX_GRT_VALUE;
-  
+typedef struct MYX_GRT_VALUE_LISTENER
+{
+  MYX_GRT *listeners_grt;
+  int (*function)(MYX_GRT *, MYX_GRT_VALUE *, MYX_GRT_VALUE_CALLBACK_REASON, void *);
+  void *user_data;
+} MYX_GRT_VALUE_LISTENER;
 
 /// [SCRIPT::LibInterfaceMapper] -generate_classes MYX_GRT_MSG
 typedef struct MYX_GRT_MSG {
@@ -318,7 +358,9 @@ typedef enum
     MYX_LUA_MODULE_TYPE,
     MYX_PYTHON_MODULE_TYPE,
     MYX_PHP_MODULE_TYPE,
-    MYX_DELPHI_MODULE_TYPE
+    MYX_DELPHI_MODULE_TYPE,
+    MYX_OBJC_MODULE_TYPE,
+    MYX_CPP_MODULE_TYPE
 //    MYX_PERL_MODULE_TYPE,
 } MYX_GRT_MODULE_TYPE;
 
@@ -336,15 +378,19 @@ typedef struct MYX_GRT_AGENT_SESSION MYX_GRT_AGENT_SESSION;
  */
 
 typedef void (*MYX_GRT_PRINT_CALLBACK)(const char *, void*);
-typedef int (*MYX_GRT_MESSAGE_CALLBACK)(MYX_GRT_MSGS *msgs, void *user_data);
+typedef void (*MYX_GRT_MESSAGE_CALLBACK)(MYX_GRT_MSGS *msgs, void *user_data);
 typedef int (*MYX_GRT_INPUT_CALLBACK)(const char *caption, MYX_GRT_INPUT_OPTIONS options, const char **text, void *user_data);
+typedef int (*MYX_GRT_STATUS_QUERY_CALLBACK)(void *user_data);
 typedef void (*MYX_GRT_LOG_CALLBACK)(MYX_GRT *, int, const char *, const char*);
+typedef int (*MYX_GRT_VALUE_LISTENER_CALLBACK)(MYX_GRT *grt, MYX_GRT_VALUE *value, MYX_GRT_VALUE_CALLBACK_REASON reason, void *user_data);
 
-MYX_PUBLIC_FUNC int myx_free_stringlist(MYX_STRINGLIST *stringlist);
+MYX_PUBLIC_FUNC int myx_free_lib_stringlist(MYX_STRINGLIST *stringlist);
 
 MYX_PUBLIC_FUNC MYX_GRT * myx_grt_initialize(int options);
 MYX_PUBLIC_FUNC void myx_grt_finalize(MYX_GRT *grt);
 
+MYX_PUBLIC_FUNC void myx_grt_catch_glib_messages(MYX_GRT *grt, int do_catch);
+
 #ifdef ENABLE_JAVA_MODULES
 JNIEXPORT jstring JNICALL Java_Grt_callGrtFunction
   (JNIEnv *, jobject, jlong, jstring, jstring, jstring);
@@ -358,9 +404,11 @@ const char *myx_grt_error_string(MYX_GRT_ERROR error);
 MYX_PUBLIC_FUNC void myx_grt_set_output_callback(MYX_GRT *grt, void *user_data,
                                  void (*process_output_func)(const char *text, void *user_data));
 MYX_PUBLIC_FUNC void myx_grt_set_message_callback(MYX_GRT *grt, void *user_data,
-                                 int (*process_message_func)(MYX_GRT_MSGS *msgs, void *user_data));
+                                 void (*process_message_func)(MYX_GRT_MSGS *msgs, void *user_data));
 MYX_PUBLIC_FUNC void myx_grt_set_input_callback(MYX_GRT *grt, void *user_data,
                                  int (*process_input_func)(const char *caption, MYX_GRT_INPUT_OPTIONS options, const char **text, void *user_data));
+MYX_PUBLIC_FUNC void myx_grt_set_status_query_callback(MYX_GRT *grt, void *user_data,
+                                 int (*process_status_query_func)(void *user_data));
 
 
 // ----------------------------------------------------------------------------------------------------------------------
@@ -368,10 +416,13 @@ MYX_PUBLIC_FUNC void myx_grt_set_input_callback(MYX_GRT *grt, void *user_data,
 
 MYX_PUBLIC_FUNC const char * myx_grt_version();
 
+MYX_PUBLIC_FUNC void myx_grt_init_threads();
+
 MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_get_root(MYX_GRT *grt);
 MYX_PUBLIC_FUNC MYX_GRT_ERROR myx_grt_set_root(MYX_GRT *grt, MYX_GRT_VALUE *new_root);
 
 MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_reference_cache_lookup(MYX_GRT *grt, const char *ref_id);
+MYX_PUBLIC_FUNC void myx_grt_reference_cache_add(MYX_GRT *grt, MYX_GRT_VALUE *obj);
 MYX_PUBLIC_FUNC void myx_grt_reference_cache_clear(MYX_GRT *grt);
 MYX_PUBLIC_FUNC void myx_grt_reference_cache_rescan(MYX_GRT *grt);
 
@@ -382,7 +433,7 @@ MYX_PUBLIC_FUNC void myx_grt_messages_free(MYX_GRT_MSGS *msgs);
 
 MYX_PUBLIC_FUNC void myx_grt_messages_stack_add(MYX_GRT *grt, int msg_type, const char *message,
                                           MYX_STRINGLIST *details, int copy_details, int progress);
-MYX_PUBLIC_FUNC void myx_grt_messages_stack_add_message(MYX_GRT *grt, const char *message, 
+MYX_PUBLIC_FUNC void myx_grt_messages_stack_add_message(MYX_GRT *grt, const char *message,
                                           MYX_STRINGLIST *details, int copy_details, ...);
 MYX_PUBLIC_FUNC void myx_grt_messages_stack_add_error(MYX_GRT *grt, const char *message, 
                                           MYX_STRINGLIST *details, int copy_details, ...);
@@ -420,6 +471,7 @@ MYX_PUBLIC_FUNC MYX_GRT_MODULE_LOADER * myx_grt_module_loader_create(MYX_GRT *gr
                   MYX_GRT_ERROR (*init_module)(MYX_GRT_MODULE_LOADER *loader, const char *file_name, MYX_GRT_MODULE **retmodule),
                   MYX_GRT_ERROR (*call_function)(MYX_GRT_FUNCTION *func, MYX_GRT_VALUE *argument, MYX_GRT_VALUE **retval),
                   void *priv);
+MYX_PUBLIC_FUNC void * myx_grt_module_get_private_data(MYX_GRT_MODULE *module);
 MYX_PUBLIC_FUNC void * myx_grt_module_loader_get_private_data(MYX_GRT_MODULE_LOADER *loader);
 MYX_PUBLIC_FUNC MYX_GRT_MODULE * myx_grt_module_create(MYX_GRT_MODULE_LOADER *loader, const char *name, 
                                                        const char *path, const char *extends, void *priv);
@@ -508,8 +560,10 @@ MYX_PUBLIC_FUNC const char * myx_grt_struct_get_desc(MYX_GRT *grt, MYX_GRT_STRUC
 MYX_PUBLIC_FUNC const char * myx_grt_struct_get_bridge_name(MYX_GRT_STRUCT *gstruct);
 MYX_PUBLIC_FUNC MYX_GRT_MODULE * myx_grt_struct_get_bridge(MYX_GRT *grt, MYX_GRT_STRUCT *gstruct);
 
-MYX_PUBLIC_FUNC int myx_grt_struct_check_inherits_from(MYX_GRT *grt, const char *struct_name, const char *parent_name);
-MYX_PUBLIC_FUNC int myx_grt_struct_check_is_or_inherits_from(MYX_GRT *grt, const char *struct_name, const char *parent_name);
+MYX_PUBLIC_FUNC int myx_grt_struct_inherits_from(MYX_GRT *grt, const char *struct_name, const char *parent_name);
+MYX_PUBLIC_FUNC int myx_grt_struct_is_or_inherits_from(MYX_GRT *grt, const char *struct_name, const char *parent_name);
+
+MYX_PUBLIC_FUNC char * myx_grt_struct_get_icon_path(MYX_GRT *grt, MYX_GRT_STRUCT *gstruct, MYX_ICON_TYPE icon_type);
 
 MYX_PUBLIC_FUNC const char * myx_grt_struct_get_icon(MYX_GRT *grt, const char *source_path, 
                  MYX_GRT_STRUCT *gstruct, MYX_ICON_TYPE icon_type, unsigned int *length);
@@ -543,6 +597,7 @@ MYX_PUBLIC_FUNC MYX_GRT_VALUE_TYPE myx_grt_struct_member_get_content_type(MYX_GR
 MYX_PUBLIC_FUNC const char * myx_grt_struct_member_get_struct_name(MYX_GRT_STRUCT_MEMBER *member);
 MYX_PUBLIC_FUNC const char * myx_grt_struct_member_get_content_struct_name(MYX_GRT_STRUCT_MEMBER *member);
 MYX_PUBLIC_FUNC unsigned int myx_grt_struct_member_get_is_ref(MYX_GRT_STRUCT_MEMBER *member);
+MYX_PUBLIC_FUNC const char * myx_grt_struct_member_get_content_struct_name_overridden(MYX_GRT_STRUCT_MEMBER *member);
 
 // Internal
 MYX_PUBLIC_FUNC int myx_grt_scan_for_structs(MYX_GRT *grt, const char *directory, MYX_GRT_ERROR *error);
@@ -554,12 +609,25 @@ MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_value_retain(MYX_GRT_VALUE *value);
 MYX_PUBLIC_FUNC int myx_grt_value_release(MYX_GRT_VALUE *value);
 MYX_PUBLIC_FUNC int myx_grt_value_get_current_reference_count(MYX_GRT_VALUE *value);
 MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_value_dup(MYX_GRT_VALUE *value);
-  
-MYX_PUBLIC_FUNC char * myx_grt_value_to_xml(MYX_GRT_VALUE *value);
+
+MYX_PUBLIC_FUNC void myx_grt_value_listener_add(MYX_GRT *grt, MYX_GRT_VALUE *value, void *user_data, 
+                                                int (*listener_callback)(MYX_GRT *grt, MYX_GRT_VALUE *value, MYX_GRT_VALUE_CALLBACK_REASON reason, void *user_data));
+MYX_PUBLIC_FUNC void myx_grt_value_listener_remove(MYX_GRT_VALUE *value, void *user_data, 
+                                                int (*listener_callback_remove)(MYX_GRT *grt, MYX_GRT_VALUE *value, MYX_GRT_VALUE_CALLBACK_REASON reason, void *user_data));
+MYX_PUBLIC_FUNC int myx_grt_value_listener_get(MYX_GRT *grt, MYX_GRT_VALUE *value, void *user_data, 
+                                                int (*listener_callback_get)(MYX_GRT *grt, MYX_GRT_VALUE *value, MYX_GRT_VALUE_CALLBACK_REASON reason, void *user_data));
+MYX_PUBLIC_FUNC int myx_grt_value_listener_call(MYX_GRT_VALUE *value, MYX_GRT_VALUE_CALLBACK_REASON reason);
+
+
+MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_value_diff_make(MYX_GRT *grt, MYX_GRT_VALUE *source, MYX_GRT_VALUE *target);
+MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_value_diff_apply(MYX_GRT *grt, MYX_GRT_VALUE *value, MYX_GRT_VALUE *diff);
+//MYX_GRT_VALUE * myx_grt_value_change_tree_make(MYX_GRT *grt, MYX_GRT_VALUE *original, MYX_GRT_VALUE *modified);
+
+MYX_PUBLIC_FUNC char * myx_grt_value_to_xml(MYX_GRT *grt, MYX_GRT_VALUE *value);
 MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_value_from_xml(MYX_GRT *grt, const char *str, size_t size);
 
 MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_retrieve_from_file(MYX_GRT *grt, const char *filename);
-MYX_PUBLIC_FUNC MYX_GRT_ERROR myx_grt_store_to_file(MYX_GRT_VALUE *value, const char *filename);
+MYX_PUBLIC_FUNC MYX_GRT_ERROR myx_grt_store_to_file(MYX_GRT *grt, MYX_GRT_VALUE *value, const char *filename);
 
 MYX_PUBLIC_FUNC char * myx_grt_value_to_xml_global_object(const char *objectPath, MYX_GRT_VALUE *value);
 MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_value_from_xml_global_object(MYX_GRT *grt, const char *str, size_t size);
@@ -568,7 +636,9 @@ MYX_PUBLIC_FUNC MYX_GRT_VALUE_TYPE myx_grt_value_get_type(MYX_GRT_VALUE *value);
 MYX_PUBLIC_FUNC const char * myx_get_value_type_as_string(MYX_GRT_VALUE_TYPE value_type);
 MYX_PUBLIC_FUNC MYX_GRT_VALUE_TYPE myx_get_value_type_from_string(const char *value_type_name, MYX_GRT_ERROR *error);
 MYX_PUBLIC_FUNC int myx_grt_value_is_simple_type(MYX_GRT_VALUE *value);
+MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_value_extend(MYX_GRT_VALUE *value);
 MYX_PUBLIC_FUNC int myx_grt_value_is_bridged(MYX_GRT_VALUE *value);
+MYX_PUBLIC_FUNC int myx_grt_value_has_listeners(MYX_GRT_VALUE *value);
 
 MYX_PUBLIC_FUNC int myx_grt_value_as_int(MYX_GRT_VALUE *value);
 MYX_PUBLIC_FUNC double myx_grt_value_as_real(MYX_GRT_VALUE *value);
@@ -604,6 +674,7 @@ MYX_PUBLIC_FUNC int myx_grt_list_item_add(MYX_GRT_VALUE *list, MYX_GRT_VALUE *va
 MYX_PUBLIC_FUNC int myx_grt_list_item_add_as_string(MYX_GRT_VALUE *list, const char *s);
 MYX_PUBLIC_FUNC int myx_grt_list_item_del(MYX_GRT_VALUE *list, int index);
 MYX_PUBLIC_FUNC int myx_grt_list_item_del_as_string(MYX_GRT_VALUE *list, const char *s);
+MYX_PUBLIC_FUNC int myx_grt_list_item_del_value(MYX_GRT_VALUE *list, MYX_GRT_VALUE *value);
 
 // Items, index based access
 MYX_PUBLIC_FUNC unsigned int myx_grt_list_item_count(MYX_GRT_VALUE *list);
@@ -611,6 +682,9 @@ MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_list_item_get(MYX_GRT_VALUE *list, unsig
 MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_list_item_get_reference_value(MYX_GRT *grt, MYX_GRT_VALUE *list, unsigned int index);
 MYX_PUBLIC_FUNC const char * myx_grt_list_item_get_as_string(MYX_GRT_VALUE *list, unsigned int index);
 MYX_PUBLIC_FUNC int myx_grt_list_item_set(MYX_GRT_VALUE *list, unsigned int index, MYX_GRT_VALUE *value);
+MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_list_item_get_by_object_name(MYX_GRT_VALUE *list, const char *name);
+MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_list_item_get_reference_value_by_object_name(MYX_GRT *grt, MYX_GRT_VALUE *list, const char *name);
+MYX_PUBLIC_FUNC int myx_grt_list_del_by_object_name(MYX_GRT_VALUE *list, const char *name);
 
 // Struct/content handling
 MYX_PUBLIC_FUNC MYX_GRT_VALUE_TYPE myx_grt_list_content_get_type(MYX_GRT_VALUE *list);
@@ -657,7 +731,7 @@ MYX_PUBLIC_FUNC int myx_grt_dict_item_by_index(MYX_GRT_VALUE *dict, unsigned int
 
 // Items, by path
 MYX_PUBLIC_FUNC char * myx_get_parent_path(const char *path);
-MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_dict_item_get_by_path(MYX_GRT_VALUE *dict, const char *path);
+MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_dict_item_get_by_path(MYX_GRT *grt, MYX_GRT_VALUE *dict, const char *path);
 MYX_PUBLIC_FUNC int myx_grt_dict_item_set_by_path(MYX_GRT_VALUE *dict, const char *path, MYX_GRT_VALUE *new_value);
 
 
@@ -679,6 +753,7 @@ MYX_PUBLIC_FUNC int myx_grt_dict_struct_set_name_no_cache_register(MYX_GRT *grt,
 MYX_PUBLIC_FUNC int myx_grt_dict_struct_validate(MYX_GRT *grt, MYX_GRT_VALUE *value, const char *struct_name, int strict);
 
 MYX_PUBLIC_FUNC int myx_grt_dict_struct_inherits_from(MYX_GRT *grt, MYX_GRT_VALUE *dict, const char *parent_struct_name);
+MYX_PUBLIC_FUNC int myx_grt_dict_struct_is_or_inherits_from(MYX_GRT *grt, MYX_GRT_VALUE *dict, const char *parent_struct_name);
 
 MYX_PUBLIC_FUNC MYX_GRT_VALUE_TYPE myx_grt_dict_content_get_type(MYX_GRT_VALUE *dict);
 MYX_PUBLIC_FUNC const char * myx_grt_dict_content_get_struct_name(MYX_GRT_VALUE *dict);
@@ -715,6 +790,13 @@ MYX_PUBLIC_FUNC int myx_grt_bridge_dict_item_set_value_from_string(MYX_GRT_VALUE
 MYX_PUBLIC_FUNC int myx_grt_bridge_dict_item_set_value_from_int(MYX_GRT_VALUE *dict, const char *key, int i, int do_bridge_callback);
 MYX_PUBLIC_FUNC int myx_grt_bridge_dict_item_set_value_from_real(MYX_GRT_VALUE *dict, const char *key, double d, int do_bridge_callback);
 
+MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_bridge_dict_item_get_value(MYX_GRT_VALUE *dict, const char *key, int do_bridge_callback);
+
+MYX_PUBLIC_FUNC unsigned int myx_grt_bridge_dict_item_count(MYX_GRT_VALUE *dict, int do_bridge_callback);
+MYX_PUBLIC_FUNC const char * myx_grt_bridge_dict_item_key_by_index(MYX_GRT_VALUE *dict, unsigned int index, int do_bridge_callback);
+MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_bridge_dict_item_value_by_index(MYX_GRT_VALUE *dict, unsigned int index, int do_bridge_callback);
+MYX_PUBLIC_FUNC int myx_grt_bridge_dict_item_del(MYX_GRT_VALUE *dict, const char *key, int do_bridge_callback);
+
 MYX_PUBLIC_FUNC int myx_grt_bridge_value_as_int(MYX_GRT_VALUE *value);
 MYX_PUBLIC_FUNC double myx_grt_bridge_value_as_real(MYX_GRT_VALUE *value);
 MYX_PUBLIC_FUNC const char * myx_grt_bridge_value_as_string(MYX_GRT_VALUE *value);
@@ -732,7 +814,7 @@ MYX_PUBLIC_FUNC int myx_grt_bridge_dict_struct_set_name(MYX_GRT *grt, MYX_GRT_VA
 // Java module
 
 #ifdef ENABLE_JAVA_MODULES
-MYX_PUBLIC_FUNC MYX_GRT_MODULE_LOADER * myx_java_init_loader(MYX_GRT *grt, const char *class_path, MYX_GRT_ERROR *error, char *jvm_library);
+MYX_PUBLIC_FUNC MYX_GRT_MODULE_LOADER * myx_java_init_loader(MYX_GRT *grt, const char *class_path, MYX_GRT_ERROR *error, const char *jvm_library, const char *grt_class_path_prefix);
 #endif
 
 // ----------------------------------------------------------------------------------------------------------------------
@@ -746,29 +828,32 @@ MYX_PUBLIC_FUNC MYX_GRT_MODULE_LOADER * myx_php_init_loader(MYX_GRT *grt, MYX_GR
 // Lua module
 
 #ifdef ENABLE_LUA_MODULES
-MYX_PUBLIC_FUNC MYX_GRT_MODULE_LOADER * myx_lua_init_loader(MYX_GRT *grt, MYX_GRT_ERROR *error);
-
-// Lua Shell
-MYX_PUBLIC_FUNC MYX_GRT_ERROR myx_grt_init_lua_shell(MYX_GRT *grt);
-MYX_PUBLIC_FUNC char * myx_grt_lua_shell_get_prompt(MYX_GRT *grt);
-MYX_PUBLIC_FUNC MYX_GRT_SHELL_COMMAND myx_grt_lua_shell_execute(MYX_GRT *grt, const char *linebuf);
-MYX_PUBLIC_FUNC int myx_grt_lua_shell_run_file(MYX_GRT *grt, const char *file_name, int interactive);
-MYX_PUBLIC_FUNC void myx_grt_shell_print_welcome(MYX_GRT *grt);
-MYX_PUBLIC_FUNC lua_State * myx_grt_lua_shell_get_lua(MYX_GRT *grt);
-MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_lua_shell_get_global_var(MYX_GRT *grt, const char *var_name);
-MYX_PUBLIC_FUNC void myx_grt_lua_shell_set_global_var(MYX_GRT *grt, const char *var_name, MYX_GRT_VALUE *value);
-
+MYX_PUBLIC_FUNC MYX_GRT_MODULE_LOADER * myx_lua_init_loader(MYX_GRT *grt, MYX_GRT_ERROR *error, const char *lua_module_path);
 #endif
 
-// Module messaging (with sockets)
-MYX_PUBLIC_FUNC int myx_grt_setup_module_messaging(MYX_GRT *grt); 
-MYX_PUBLIC_FUNC int myx_grt_cleanup_module_messaging(MYX_GRT *grt);
-MYX_PUBLIC_FUNC MYX_SOCKET myx_grt_get_messaging_fd(MYX_GRT *grt);
-MYX_PUBLIC_FUNC MYX_SOCKET myx_grt_get_messaging_module_fd(MYX_GRT *grt);
-MYX_PUBLIC_FUNC int myx_grt_check_module_connected(MYX_GRT *grt);
-MYX_PUBLIC_FUNC int myx_grt_check_module_message(MYX_GRT *grt, char **message, int timeout);
-MYX_PUBLIC_FUNC int myx_grt_send_module_message(MYX_GRT *grt, const char *message);
-  
+// ----------------------------------------------------------------------------------------------------------------------
+// Python module
+
+#ifdef ENABLE_PYTHON_MODULES
+MYX_PUBLIC_FUNC MYX_GRT_MODULE_LOADER * myx_python_init_loader(MYX_GRT *grt, MYX_GRT_ERROR *error, const char *python_module_path);
+#endif
+
+// ----------------------------------------------------------------------------------------------------------------------
+// Shell Support  
+
+MYX_PUBLIC_FUNC MYX_GRT_ERROR myx_grt_setup_shell(MYX_GRT *grt, MYX_GRT_SHELL_INTERFACE flavour);
+
+MYX_PUBLIC_FUNC int myx_grt_shell_init(MYX_GRT *grt);
+MYX_PUBLIC_FUNC char * myx_grt_shell_get_prompt(MYX_GRT *grt);
+MYX_PUBLIC_FUNC MYX_GRT_SHELL_COMMAND myx_grt_shell_execute(MYX_GRT *grt, const char *linebuf);
+MYX_PUBLIC_FUNC int myx_grt_shell_run_file(MYX_GRT *grt, const char *file_name, int interactive);
+MYX_PUBLIC_FUNC void myx_grt_shell_print_welcome(MYX_GRT *grt);
+MYX_PUBLIC_FUNC void * myx_grt_shell_get_interpreter_data(MYX_GRT *grt);
+MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_shell_get_global_var(MYX_GRT *grt, const char *var_name);
+MYX_PUBLIC_FUNC int myx_grt_shell_set_global_var(MYX_GRT *grt, const char *var_name, MYX_GRT_VALUE *value);
+
+
+
 // Aux funcs
 MYX_PUBLIC_FUNC int _myx_grt_get_refcount(MYX_GRT_VALUE *value);
 MYX_PUBLIC_FUNC MYX_GRT_VALUE *make_return_value(MYX_GRT_VALUE *value);
@@ -776,6 +861,11 @@ MYX_PUBLIC_FUNC MYX_GRT_VALUE *make_return_value_error(const char *message, cons
 
 MYX_PUBLIC_FUNC char * myx_grt_get_guid();
 
+#ifdef ENABLE_LUA_MODULES
+#include <lua.h>
+// Text User Interface stuff
+MYX_PUBLIC_FUNC void myx_grt_init_textui(lua_State *l);
+#endif
 
 
 #ifdef __cplusplus
