@@ -1,18 +1,20 @@
-/* Copyright (C) 2004 MySQL AB
-
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-
-   This program is distributed in the hope that it will be useful,
+/* Copyright (c) 2004, 2005 MySQL AB
+  
+   This library is free software; you can redistribute it and/or
+   modify it under the terms of the GNU Lesser General Public
+   License as published by the Free Software Foundation; either
+   version 2 of the License, or (at your option) any later version.
+  
+   This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
-
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+   Lesser General Public License for more details.
+  
+   You should have received a copy of the GNU Lesser General Public
+   License along with this library; if not, write to the
+   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+   Boston, MA 02111-1307, USA.
+ */
 
 /**
  * @file myx_grt.c 
@@ -26,19 +28,26 @@
 #include <libxml/xmlmemory.h>
 #include <libxml/parser.h>
 
-#include "myx_xml_aux_functions.h"
+#include "myx_xml_util_functions.h"
 #include "myx_grt_private.h"
+#include "myx_shared_util_functions.h"
 
 #if defined(__WIN__) || defined(_WIN32) || defined(_WIN64)
 #include <objbase.h>
+#elif defined(__APPLE__)
+#include <CoreFoundation/CoreFoundation.h>
 #else
 // Linux specific
 #include <uuid/uuid.h>
 #endif
 
 
-static MYX_GRT_VALUE *unserialize_from_xml(xmlNodePtr node, GHashTable *objects_by_id);
-static xmlNodePtr serialize_to_xml(xmlNodePtr parent, MYX_GRT_VALUE *value, GHashTable *saved_ids);
+static MYX_GRT_VALUE *unserialize_from_xml(MYX_GRT *grt, xmlNodePtr node, GHashTable *objects_by_id);
+static MYX_GRT_VALUE *unserialize_from_xml_global_object(MYX_GRT *grt, xmlNodePtr node, GHashTable *objects_by_id);
+static xmlNodePtr serialize_to_xml(MYX_GRT *grt, xmlNodePtr parent, MYX_GRT_VALUE *value, GHashTable *saved_ids);
+
+static int refcache_callback(MYX_GRT* grt, MYX_GRT_VALUE* value, MYX_GRT_VALUE_CALLBACK_REASON reason, void* data);
+static void refcache_callback2(gpointer key, gpointer value, gpointer data);
 
 
 static void default_print(const char *msg, void *data)
@@ -52,6 +61,22 @@ static void default_log(MYX_GRT *grt, int code, const char *msg, const char *det
   printf("%i: %s: %s\n", code, msg, detail?detail:"");
 }
 
+void myx_grt_init_threads()
+{
+  g_thread_init(NULL);
+}
+
+/**
+ **************************************************************************** 
+ * @brief Returns the version of the GRT environment
+ *
+ * @return a const char with the version information
+ ****************************************************************************/
+const char * myx_grt_version()
+{
+  return GRT_VERSION;
+}
+
 
 /**
  **************************************************************************** 
@@ -80,12 +105,15 @@ MYX_GRT *myx_grt_initialize(int options)
   
   grt->logfunc= default_log;
   
-  grt->root= myx_grt_dict_new(NULL);
+  grt->root= myx_grt_dict_new(NULL, NULL);
 
   grt->struct_icon_cache= g_hash_table_new_full(g_str_hash, g_str_equal,
                                                 g_free, g_free);
 
-  grt->reference_cache= g_hash_table_new_full(g_str_hash, g_str_equal,
+  grt->global_reference_cache= g_hash_table_new_full(g_str_hash, g_str_equal,
+                                                g_free, NULL);
+
+  grt->direct_reference_cache= g_hash_table_new_full(g_str_hash, g_str_equal,
                                                 g_free, NULL);
   
   myx_grt_register_module_loader(grt, myx_builtin_init_loader(grt));
@@ -112,9 +140,42 @@ MYX_GRT *myx_grt_initialize(int options)
 void myx_grt_finalize(MYX_GRT *grt)
 {
   //XXX TODO
+
+  // Remove all value listener registrations.
+  g_hash_table_foreach(grt->global_reference_cache, refcache_callback2, NULL); 
+  g_hash_table_destroy(grt->global_reference_cache);
+
+  // Remove all value listener registrations.
+  g_hash_table_foreach(grt->direct_reference_cache, refcache_callback2, NULL); 
+  g_hash_table_destroy(grt->direct_reference_cache);
 }
 
 
+void glib_message_handler(const gchar *log_domain, GLogLevelFlags log_level, 
+                          const gchar *message, gpointer user_data)
+{
+  myx_grt_messages_stack_add_message((MYX_GRT *) user_data, message, NULL, 0);
+}
+
+/**
+ ****************************************************************************
+ * @brief Activates or deactivates the catching of glib messages
+ * 
+ *  Sets the function that is used when GRT needs to print messages.
+ *
+ * @param grt      the GRT environment
+ * @param user_data a pointer to be passed to the callback function. 
+ * @param process_output_func the callback function which should take 
+ *        the text and the userdata pointer as arguments
+ ****************************************************************************/
+void myx_grt_catch_glib_messages(MYX_GRT *grt, int do_catch)
+{
+  if (do_catch)
+    g_log_set_handler(NULL, G_LOG_LEVEL_MASK, glib_message_handler, grt);
+  else
+    g_log_default_handler(NULL, G_LOG_LEVEL_MASK, "", NULL);
+}
+
 /**
  ****************************************************************************
  * @brief Sets the function to be used to print values
@@ -158,6 +219,39 @@ void myx_grt_set_message_callback(MYX_GRT *grt, void *user_data,
   GRT_LEAVE(grt);
 }
 
+/**
+ ****************************************************************************
+ * @brief Sets the function to be used to process messages
+ * 
+ *  Sets the function that is used when GRT needs to process messages.
+ *
+ * @param grt      the GRT environment
+ * @param user_data a pointer to be passed to the callback function. 
+ * @param process_output_func the callback function which should take 
+ *        the text and the userdata pointer as arguments
+ ****************************************************************************/
+void myx_grt_set_input_callback(MYX_GRT *grt, void *user_data,
+                                MYX_GRT_INPUT_CALLBACK process_input_func)
+{
+  GRT_ENTER(grt);
+
+  grt->process_input= process_input_func;
+  grt->process_input_data= user_data;
+  
+  GRT_LEAVE(grt);
+}
+
+void myx_grt_set_status_query_callback(MYX_GRT *grt, void *user_data,
+                                 int (*process_status_query_func)(void *user_data))
+{
+  GRT_ENTER(grt);
+
+  grt->process_status_query= process_status_query_func;
+  grt->process_status_query_data= user_data;
+  
+  GRT_LEAVE(grt);
+}
+
 
 /**
  ****************************************************************************
@@ -555,7 +649,7 @@ MYX_GRT_ERROR myx_grt_set_root(MYX_GRT *grt, MYX_GRT_VALUE *new_root)
  *
  * @return NULL on error and the value if the file could be correctly loaded.
  ****************************************************************************/
-MYX_GRT_VALUE *myx_grt_retrieve_from_file(const char *filename)
+MYX_GRT_VALUE *myx_grt_retrieve_from_file(MYX_GRT *grt, const char *filename)
 {
   xmlDocPtr doc;
   xmlNodePtr root;
@@ -568,11 +662,11 @@ MYX_GRT_VALUE *myx_grt_retrieve_from_file(const char *filename)
   if (root)
   {
     root= root->children;
-    while (root && xmlStrcmp(root->name, "value")!=0) root= root->next;
+    while (root && xmlStrcmp(root->name, (xmlChar*)"value")!=0) root= root->next;
     if (root)
     {
       GHashTable *objects_by_id= g_hash_table_new(g_str_hash, g_str_equal);
-      value= unserialize_from_xml(root, objects_by_id);
+      value= unserialize_from_xml(grt, root, objects_by_id);
       g_hash_table_destroy(objects_by_id);
     }
   }
@@ -595,18 +689,18 @@ MYX_GRT_VALUE *myx_grt_retrieve_from_file(const char *filename)
  *
  * @return  MYX_GRT_NO_ERROR if there were no errors.
  ****************************************************************************/
-MYX_GRT_ERROR myx_grt_store_to_file(MYX_GRT_VALUE *value, const char *filename)
+MYX_GRT_ERROR myx_grt_store_to_file(MYX_GRT *grt, MYX_GRT_VALUE *value, const char *filename)
 {
   xmlDocPtr doc;
   int res;
   GHashTable *saved_ids;
   
-  doc= xmlNewDoc("1.0");
-  doc->children= xmlNewDocRawNode(doc, NULL, "data", NULL);
+  doc= xmlNewDoc((xmlChar*)"1.0");
+  doc->children= xmlNewDocRawNode(doc, NULL, (xmlChar*)"data", NULL);
   
   saved_ids= g_hash_table_new(g_str_hash, g_str_equal);
   
-  serialize_to_xml(doc->children, value, saved_ids);
+  serialize_to_xml(grt, doc->children, value, saved_ids);
 
   g_hash_table_destroy(saved_ids);
   
@@ -660,67 +754,72 @@ int myx_grt_printf(MYX_GRT *grt, const char *fmt, ...)
 
 
 
-static xmlNodePtr serialize_to_xml(xmlNodePtr parent, MYX_GRT_VALUE *value,
+static xmlNodePtr serialize_to_xml(MYX_GRT *grt, xmlNodePtr parent, MYX_GRT_VALUE *value,
                                    GHashTable *saved_ids)
 {
+  MYX_GRT_STRUCT *gstruct;
   unsigned int i;
   char buffer[100];
   const char *id;
-  xmlNodePtr node;
+  xmlNodePtr node= NULL;
 
   switch (value->type)
   {
   case MYX_INT_VALUE:
     g_snprintf(buffer, sizeof(buffer), "%i", value->value.i);
-    node= xmlNewTextChild(parent, NULL, "value", buffer);
+    node= xmlNewTextChild(parent, NULL, (xmlChar*)"value", (xmlChar*)buffer);
 
-    xmlNewProp(node, "type", "int");
+    xmlNewProp(node, (xmlChar*)"type", (xmlChar*)"int");
     break;
   case MYX_REAL_VALUE:
     g_snprintf(buffer, sizeof(buffer), "%f", value->value.r);
-    node= xmlNewTextChild(parent, NULL, "value", buffer);
+    node= xmlNewTextChild(parent, NULL, (xmlChar*)"value", (xmlChar*)buffer);
 
-    xmlNewProp(node, "type", "real");
+    xmlNewProp(node, (xmlChar*)"type", (xmlChar*)"real");
     break;
   case MYX_STRING_VALUE:
-    node= xmlNewTextChild(parent, NULL, "value", value->value.s);
+    node= xmlNewTextChild(parent, NULL, (xmlChar*)"value", (xmlChar*)value->value.s);
 
-    xmlNewProp(node, "type", "string");
+    xmlNewProp(node, (xmlChar*)"type", (xmlChar*)"string");
     break;
   case MYX_LIST_VALUE:
-    node= xmlNewTextChild(parent, NULL, "value", NULL);
-    xmlNewProp(node, "type", "list");
-    xmlNewProp(node, "content-type", myx_get_value_type_as_string(value->value.l->content_type));
+    node= xmlNewTextChild(parent, NULL, (xmlChar*)"value", NULL);
+    xmlNewProp(node, (xmlChar*)"type", (xmlChar*)"list");
+    xmlNewProp(node, (xmlChar*)"content-type", (xmlChar*)myx_get_value_type_as_string(value->value.l->content_type));
 
     if (value->value.l->content_struct_name)
-      xmlNewProp(node, "content-struct-name", value->value.l->content_struct_name);
+      xmlNewProp(node, (xmlChar*)"content-struct-name", (xmlChar*)value->value.l->content_struct_name);
 
     for (i= 0; i < value->value.l->items_num; i++)
     {
       g_return_val_if_fail((value->value.l->items[i]->type == value->value.l->content_type) ||
          (value->value.l->content_type == MYX_ANY_VALUE), NULL);
-      serialize_to_xml(node, value->value.l->items[i], saved_ids);
+      serialize_to_xml(grt, node, value->value.l->items[i], saved_ids);
     }
     break;
   case MYX_DICT_VALUE:
-    node= xmlNewTextChild(parent, NULL, "value", NULL);
-    xmlNewProp(node, "type", "dict");
+    gstruct= NULL;
+    node= xmlNewTextChild(parent, NULL, (xmlChar*)"value", NULL);
+    xmlNewProp(node, (xmlChar*)"type", (xmlChar*)"dict");
 
     if ((value->value.d->struct_name) && (value->value.d->struct_name[0]))
-      xmlNewProp(node, "struct-name", value->value.d->struct_name);
+    {
+      xmlNewProp(node, (xmlChar*)"struct-name", (xmlChar*)value->value.d->struct_name);
+      gstruct= myx_grt_struct_get(grt, value->value.d->struct_name);
+    }
 
     if (value->value.d->content_type != MYX_ANY_VALUE)
-      xmlNewProp(node, "content-type", myx_get_value_type_as_string(value->value.d->content_type));
+      xmlNewProp(node, (xmlChar*)"content-type", (xmlChar*)myx_get_value_type_as_string(value->value.d->content_type));
 
     if ((value->value.d->content_struct_name) && (value->value.d->content_struct_name[0]))
-      xmlNewProp(node, "content-struct-name", value->value.d->content_struct_name);
+      xmlNewProp(node, (xmlChar*)"content-struct-name", (xmlChar*)value->value.d->content_struct_name);
 
     if ((id= myx_grt_dict_id_item_as_string(value)))
     {
       if (g_hash_table_lookup(saved_ids, id))
       {
         // the object is already in the XML tree, just leave a link to it
-        xmlNewProp(node, "link", id);
+        xmlNewProp(node, (xmlChar*)"link", (xmlChar*)id);
 
         return node;
       }
@@ -735,8 +834,16 @@ static xmlNodePtr serialize_to_xml(xmlNodePtr parent, MYX_GRT_VALUE *value,
       
       if (value->value.d->items[i].value)
       {
-        child= serialize_to_xml(node, value->value.d->items[i].value, saved_ids);
-        xmlNewProp(child, "key", value->value.d->items[i].key);
+        child= serialize_to_xml(grt, node, value->value.d->items[i].value, saved_ids);
+        xmlNewProp(child, (xmlChar*)"key", (xmlChar*)value->value.d->items[i].key);
+        
+        if (gstruct)
+        {
+          MYX_GRT_STRUCT_MEMBER *member= myx_grt_struct_get_member_by_name(grt, gstruct, value->value.d->items[i].key, 1);
+
+          if (member && member->is_ref)
+            xmlNewProp(child, (xmlChar*)"option", (xmlChar*)"ref");
+        }
       }
     }
     break;
@@ -747,10 +854,10 @@ static xmlNodePtr serialize_to_xml(xmlNodePtr parent, MYX_GRT_VALUE *value,
 }
 
 
-static MYX_GRT_VALUE *unserialize_from_xml(xmlNodePtr node, GHashTable *objects_by_id)
+static MYX_GRT_VALUE *unserialize_from_xml(MYX_GRT *grt, xmlNodePtr node, GHashTable *objects_by_id)
 {
   xmlChar *str;
-  xmlChar *node_type= xmlGetProp(node, "type");
+  xmlChar *node_type= xmlGetProp(node, (xmlChar*)"type");
   MYX_GRT_VALUE *value= NULL;
   
   if (!node_type)
@@ -759,29 +866,37 @@ static MYX_GRT_VALUE *unserialize_from_xml(xmlNodePtr node, GHashTable *objects_
     return NULL;
   }
 
-  if (strcmp(node_type, "int")==0)
+  if (strcmp((char*)node_type, "int")==0)
   {
     str= xmlNodeGetContent(node);
-    value= myx_grt_value_from_int(strtol(str, NULL, 0));
+    value= myx_grt_value_from_int(strtol((char*)str, NULL, 0));
     xmlFree(str);
   }
-  else if (strcmp(node_type, "real")==0)
+  else if (strcmp((char*)node_type, "real")==0)
   {
     str= xmlNodeGetContent(node);
-    value= myx_grt_value_from_real(strtod(str, NULL));
+    value= myx_grt_value_from_real(strtod((char*)str, NULL));
     xmlFree(str);
   }
-  else if (strcmp(node_type, "string")==0)
+  else if (strcmp((char*)node_type, "string")==0)
   {
     str= xmlNodeGetContent(node);
-    value= myx_grt_value_from_string(str);
+    value= myx_grt_value_from_string((char*)str);
     xmlFree(str);
   }
-  else if (strcmp(node_type, "dict")==0)
+  else if (strcmp((char*)node_type, "dict")==0)
   {
-    xmlChar *link_id= xmlGetProp(node, "link");
+    xmlChar *global_object_path= xmlGetProp(node, (xmlChar*)"global-object-path");
+    xmlChar *link_id= xmlGetProp(node, (xmlChar*)"link");
     xmlNodePtr child;
 
+    if (global_object_path)
+    {
+      xmlFree(link_id);
+      return unserialize_from_xml_global_object(grt, node, objects_by_id);
+    }
+    xmlFree(global_object_path);
+
     if (link_id)
     {
       MYX_GRT_VALUE *linked_object;
@@ -802,16 +917,16 @@ static MYX_GRT_VALUE *unserialize_from_xml(xmlNodePtr node, GHashTable *objects_
     }
     else
     {
-      xmlChar *prop= xmlGetProp(node, "content-type");
+      xmlChar *prop= xmlGetProp(node, (xmlChar*)"content-type");
       MYX_GRT_VALUE_TYPE content_type= MYX_ANY_VALUE;
       MYX_GRT_ERROR error;
 
-      content_type= myx_get_value_type_from_string(prop, &error);
+      content_type= myx_get_value_type_from_string((char*)prop, &error);
       xmlFree(prop);
 
       if (content_type != MYX_ANY_VALUE)
       {
-        char *content_struct_name= xmlGetProp(node, "content-struct-name");
+        char *content_struct_name= (char*)xmlGetProp(node, (xmlChar*)"content-struct-name");
 
         value= myx_grt_dict_new_typed(content_type, content_struct_name);
 
@@ -819,9 +934,14 @@ static MYX_GRT_VALUE *unserialize_from_xml(xmlNodePtr node, GHashTable *objects_
       }
       else
       {
-        xmlChar *struct_name= xmlGetProp(node, "struct-name");
+        xmlChar *struct_name= xmlGetProp(node, (xmlChar*)"struct-name");
 
-        value= myx_grt_dict_new(struct_name);
+        // this does the work of myx_grt_dict_new()
+        value= g_new0(MYX_GRT_VALUE, 1);
+        value->type= MYX_DICT_VALUE;
+        value->value.d= g_new0(MYX_GRT_DICT, 1);
+        value->refcount= 1;
+        value->value.d->struct_name= g_strdup((char*)struct_name);
 
         xmlFree(struct_name);
       }
@@ -831,14 +951,14 @@ static MYX_GRT_VALUE *unserialize_from_xml(xmlNodePtr node, GHashTable *objects_
       {
         MYX_GRT_VALUE *sub_value;
         
-        if (strcmp(child->name, "value")==0)
+        if (strcmp((char*)child->name, "value")==0)
         {
-          xmlChar *key= xmlGetProp(child, "key");
+          xmlChar *key= xmlGetProp(child, (xmlChar*)"key");
 
-          sub_value= unserialize_from_xml(child, objects_by_id);
+          sub_value= unserialize_from_xml(grt, child, objects_by_id);
           if (sub_value)
           {
-            myx_grt_dict_item_set_value(value, key, sub_value);
+            myx_grt_dict_item_set_value(value, (char*)key, sub_value);
             myx_grt_value_release(sub_value);
           }
           else
@@ -848,7 +968,7 @@ static MYX_GRT_VALUE *unserialize_from_xml(xmlNodePtr node, GHashTable *objects_
             break;
           }
 
-          if (xmlStrcmp(key, "_id")==0 && sub_value)
+          if (xmlStrcmp(key, (xmlChar*)"_id")==0 && sub_value)
           {
             if (sub_value->type == MYX_STRING_VALUE)
               g_hash_table_insert(objects_by_id, (gpointer)myx_grt_value_as_string(sub_value),
@@ -862,13 +982,13 @@ static MYX_GRT_VALUE *unserialize_from_xml(xmlNodePtr node, GHashTable *objects_
       }
     }
   }
-  else if (strcmp(node_type, "list")==0)
+  else if (strcmp((char*)node_type, "list")==0)
   {
-    xmlChar *ctype= xmlGetProp(node, "content-type");
-    xmlChar *cstruct_name= xmlGetProp(node, "content-struct-name");
+    xmlChar *ctype= xmlGetProp(node, (xmlChar*)"content-type");
+    xmlChar *cstruct_name= xmlGetProp(node, (xmlChar*)"content-struct-name");
     xmlNodePtr child;
     MYX_GRT_ERROR error;
-    MYX_GRT_VALUE_TYPE content_type= myx_get_value_type_from_string(ctype, &error);
+    MYX_GRT_VALUE_TYPE content_type= myx_get_value_type_from_string((char*)ctype, &error);
 
     /*for (i= 0; sizeof(value_type_strings)/sizeof(char*); i++)
       if (strcmp2(value_type_strings[i], ctype)==0)
@@ -879,18 +999,22 @@ static MYX_GRT_VALUE *unserialize_from_xml(xmlNodePtr node, GHashTable *objects_
 
     if (error == MYX_GRT_NO_ERROR)
     {
-      value= myx_grt_list_new(content_type, NULL);
-
-      value->value.l->content_struct_name= g_strdup(cstruct_name);
+      // this does the work of myx_grt_list_new()
+      value= g_new0(MYX_GRT_VALUE, 1);
+      value->type= MYX_LIST_VALUE;
+      value->value.l= g_new0(MYX_GRT_LIST, 1);
+      value->value.l->content_type= content_type;
+      value->refcount= 1;
+      value->value.l->content_struct_name= g_strdup((char*)cstruct_name);
 
       child= node->children;
       while (child)
       {
         MYX_GRT_VALUE *sub_value;
 
-        if (child->type == XML_ELEMENT_NODE && strcmp(child->name, "value")==0)
+        if (child->type == XML_ELEMENT_NODE && strcmp((char*)child->name, "value")==0)
         {
-          sub_value= unserialize_from_xml(child, objects_by_id);
+          sub_value= unserialize_from_xml(grt, child, objects_by_id);
           if (sub_value)
           {
             myx_grt_list_item_add(value, sub_value);
@@ -930,32 +1054,32 @@ static xmlNodePtr serialize_to_xml_global_object(const char *objectPath, xmlNode
   {
   case MYX_INT_VALUE:
     g_snprintf(buffer, sizeof(buffer), "%i", value->value.i);
-    node= xmlNewTextChild(parent, NULL, "value", buffer);
+    node= xmlNewTextChild(parent, NULL, (xmlChar*)"value", (xmlChar*)buffer);
     
-    xmlNewProp(node, "type", "int");
-    xmlNewProp(node, "globalObjectPath", objectPath);
+    xmlNewProp(node, (xmlChar*)"type", (xmlChar*)"int");
+    xmlNewProp(node, (xmlChar*)"globalObjectPath", (xmlChar*)objectPath);
     break;
   case MYX_REAL_VALUE:
     g_snprintf(buffer, sizeof(buffer), "%f", value->value.r);
-    node= xmlNewTextChild(parent, NULL, "value", buffer);
+    node= xmlNewTextChild(parent, NULL, (xmlChar*)"value", (xmlChar*)buffer);
 
-    xmlNewProp(node, "type", "real");
+    xmlNewProp(node, (xmlChar*)"type", (xmlChar*)"real");
 
-    xmlNewProp(node, "global-object-path", objectPath);
+    xmlNewProp(node, (xmlChar*)"global-object-path", (xmlChar*)objectPath);
     break;
   case MYX_STRING_VALUE:
-    node= xmlNewTextChild(parent, NULL, "value", value->value.s);
+    node= xmlNewTextChild(parent, NULL, (xmlChar*)"value", (xmlChar*)value->value.s);
 
-    xmlNewProp(node, "type", "string");
-    xmlNewProp(node, "global-object-path", objectPath);
+    xmlNewProp(node, (xmlChar*)"type", (xmlChar*)"string");
+    xmlNewProp(node, (xmlChar*)"global-object-path", (xmlChar*)objectPath);
     break;
   case MYX_LIST_VALUE:
-    node= xmlNewTextChild(parent, NULL, "value", NULL);
-    xmlNewProp(node, "type", "list");
-    xmlNewProp(node, "content-type", myx_get_value_type_as_string(value->value.l->content_type));
+    node= xmlNewTextChild(parent, NULL, (xmlChar*)"value", NULL);
+    xmlNewProp(node, (xmlChar*)"type", (xmlChar*)"list");
+    xmlNewProp(node, (xmlChar*)"content-type", (xmlChar*)myx_get_value_type_as_string(value->value.l->content_type));
 
     if (value->value.l->content_struct_name)
-      xmlNewProp(node, "content-struct-name", value->value.l->content_struct_name);
+      xmlNewProp(node, (xmlChar*)"content-struct-name", (xmlChar*)value->value.l->content_struct_name);
 
     /*for (i= 0; i < value->value.l->items_num; i++)
     {
@@ -963,20 +1087,20 @@ static xmlNodePtr serialize_to_xml_global_object(const char *objectPath, xmlNode
          (value->value.l->content_type == MYX_ANY_VALUE), NULL);
       serialize_to_xml(node, value->value.l->items[i], saved_ids);
     }*/
-    xmlNewProp(node, "global-object-path", objectPath);
+    xmlNewProp(node, (xmlChar*)"global-object-path", (xmlChar*)objectPath);
     break;
   case MYX_DICT_VALUE:
-    node= xmlNewTextChild(parent, NULL, "value", NULL);
-    xmlNewProp(node, "type", "dict");
+    node= xmlNewTextChild(parent, NULL, (xmlChar*)"value", NULL);
+    xmlNewProp(node, (xmlChar*)"type", (xmlChar*)"dict");
 
     if ((value->value.d->struct_name) && (value->value.d->struct_name[0]))
-      xmlNewProp(node, "struct-name", value->value.d->struct_name);
+      xmlNewProp(node, (xmlChar*)"struct-name", (xmlChar*)value->value.d->struct_name);
 
     if (value->value.d->content_type != MYX_ANY_VALUE)
-      xmlNewProp(node, "content-type", myx_get_value_type_as_string(value->value.d->content_type));
+      xmlNewProp(node, (xmlChar*)"content-type", (xmlChar*)myx_get_value_type_as_string(value->value.d->content_type));
 
     if ((value->value.d->content_struct_name) && (value->value.d->content_struct_name[0]))
-      xmlNewProp(node, "content-struct-name", value->value.d->content_struct_name);
+      xmlNewProp(node, (xmlChar*)"content-struct-name", (xmlChar*)value->value.d->content_struct_name);
 
     /*if ((id= myx_grt_dict_id_item_as_string(value)))
     {
@@ -999,7 +1123,7 @@ static xmlNodePtr serialize_to_xml_global_object(const char *objectPath, xmlNode
       child= serialize_to_xml(node, value->value.d->items[i].value, saved_ids);
       xmlNewProp(child, "key", value->value.d->items[i].key);
     }*/
-    xmlNewProp(node, "global-object-path", objectPath);
+    xmlNewProp(node, (xmlChar*)"global-object-path", (xmlChar*)objectPath);
     break;
   case MYX_ANY_VALUE:
     break;
@@ -1009,7 +1133,7 @@ static xmlNodePtr serialize_to_xml_global_object(const char *objectPath, xmlNode
 
 static MYX_GRT_VALUE *unserialize_from_xml_global_object(MYX_GRT *grt, xmlNodePtr node, GHashTable *objects_by_id)
 {
-  xmlChar *node_type= xmlGetProp(node, "type");
+  char *node_type= (char*)xmlGetProp(node, (xmlChar*)"type");
   MYX_GRT_VALUE *value= NULL;
 
   if (!node_type)
@@ -1023,17 +1147,17 @@ static MYX_GRT_VALUE *unserialize_from_xml_global_object(MYX_GRT *grt, xmlNodePt
   // check if we have a global object
   if ((strcmp(node_type, "dict")==0) || (strcmp(node_type, "list")==0))
   {
-    xmlChar *global_object_path= xmlGetProp(node, "global-object-path");
+    xmlChar *global_object_path= xmlGetProp(node, (xmlChar*)"global-object-path");
 
     if (global_object_path)
     {
-      value= myx_grt_dict_item_get_by_path(myx_grt_get_root(grt), global_object_path);
+      value= myx_grt_dict_item_get_by_path(grt, myx_grt_get_root(grt), (char*)global_object_path);
       xmlFree(global_object_path);
       GRT_RETURN(grt, value, MYX_GRT_VALUE*);
     }
   }
 
-  GRT_RETURN(grt, unserialize_from_xml(node, objects_by_id), MYX_GRT_VALUE*);
+  GRT_RETURN(grt, unserialize_from_xml(grt, node, objects_by_id), MYX_GRT_VALUE*);
 }
 
 /**
@@ -1048,7 +1172,7 @@ static MYX_GRT_VALUE *unserialize_from_xml_global_object(MYX_GRT *grt, xmlNodePt
  * 
  * @see myx_grt_value_from_xml
  *****************************************************************************/
-char *myx_grt_value_to_xml(MYX_GRT_VALUE *value)
+char *myx_grt_value_to_xml(MYX_GRT *grt, MYX_GRT_VALUE *value)
 {
   xmlDocPtr doc;
   xmlNodePtr root;
@@ -1058,12 +1182,12 @@ char *myx_grt_value_to_xml(MYX_GRT_VALUE *value)
   
   if (value)
   {
-    doc= xmlNewDoc("1.0");
-    doc->children= root= xmlNewDocRawNode(doc, NULL, "data", NULL);
+    doc= xmlNewDoc((xmlChar*)"1.0");
+    doc->children= root= xmlNewDocRawNode(doc, NULL, (xmlChar*)"data", NULL);
     
     saved_ids= g_hash_table_new(g_str_hash, g_str_equal);
     
-    serialize_to_xml(root, value, saved_ids);
+    serialize_to_xml(grt, root, value, saved_ids);
     
     g_hash_table_destroy(saved_ids);
 
@@ -1092,7 +1216,7 @@ char *myx_grt_value_to_xml(MYX_GRT_VALUE *value)
  * 
  * @see myx_grt_value_to_xml
  *****************************************************************************/
-MYX_GRT_VALUE *myx_grt_value_from_xml(const char *str, size_t size)
+MYX_GRT_VALUE *myx_grt_value_from_xml(MYX_GRT *grt, const char *str, size_t size)
 {
   xmlDocPtr doc= xmlParseMemory(str, (int)size);
   xmlNodePtr root;
@@ -1100,24 +1224,34 @@ MYX_GRT_VALUE *myx_grt_value_from_xml(const char *str, size_t size)
 
   if (!doc)
   {
-    g_warning("Could not parse XML data");
+    xmlErrorPtr error= xmlGetLastError();
+
+    if (error) 
+    {
+      myx_grt_messages_stack_add_error(grt, "Could not parse XML data. Line %d, %s", NULL, 0, 
+        error->line, error->message);
+    }
+    else
+    {
+      g_warning("Could not parse XML data");
+    }
     return NULL;
   }
 
   root= xmlDocGetRootElement(doc);
 
   //check if we have a <data> rootnode
-  if (root && (xmlStrcmp(root->name, "data")==0))
+  if (root && (xmlStrcmp(root->name, (xmlChar*)"data")==0))
     root= root->children;
 
   //skip all nodes that are no <value> nodes
-  while (root && xmlStrcmp(root->name, "value")!=0) 
+  while (root && xmlStrcmp(root->name, (xmlChar*)"value")!=0) 
     root= root->next;
 
   if (root)
   {
     GHashTable *objects_by_id= g_hash_table_new(g_str_hash, g_str_equal);
-    value= unserialize_from_xml(root, objects_by_id);
+    value= unserialize_from_xml(grt, root, objects_by_id);
     g_hash_table_destroy(objects_by_id);
   }
   else
@@ -1160,11 +1294,11 @@ MYX_GRT_VALUE *myx_grt_value_from_xml_global_object(MYX_GRT *grt, const char *st
   root= xmlDocGetRootElement(doc);
 
   //check if we have a <data> rootnode
-  if (root && (xmlStrcmp(root->name, "data")==0))
+  if (root && (xmlStrcmp(root->name, (xmlChar*)"data")==0))
     root= root->children;
 
   //skip all nodes that are no <value> nodes
-  while (root && xmlStrcmp(root->name, "value")!=0) 
+  while (root && xmlStrcmp(root->name, (xmlChar*)"value")!=0) 
     root= root->next;
 
   if (root)
@@ -1203,8 +1337,8 @@ char *myx_grt_value_to_xml_global_object(const char *objectPath, MYX_GRT_VALUE *
   
   if (value)
   {
-    doc= xmlNewDoc("1.0");
-    doc->children= root= xmlNewDocRawNode(doc, NULL, "data", NULL);
+    doc= xmlNewDoc((xmlChar*)"1.0");
+    doc->children= root= xmlNewDocRawNode(doc, NULL, (xmlChar*)"data", NULL);
     
     saved_ids= g_hash_table_new(g_str_hash, g_str_equal);
     
@@ -1437,6 +1571,60 @@ void myx_grt_messages_stack_flush(MYX_GRT *grt, unsigned int count)
 }
 
 
+/**
+ ****************************************************************************
+ * @brief Adds an object to the direct reference cache
+ *
+ * @param grt MYX_GRT pointer
+ * @param obj The object to add
+ ****************************************************************************
+ */
+void myx_grt_reference_cache_add(MYX_GRT *grt, MYX_GRT_VALUE *obj)
+{
+  // TODO Keep manually added values in a separate list, so they are re-added when the cache is rebuilt.
+  g_hash_table_insert(grt->direct_reference_cache, g_strdup(myx_grt_dict_id_item_as_string(obj)), obj);
+  myx_grt_value_listener_add(grt, obj, NULL, refcache_callback);
+}
+
+
+/**
+ ****************************************************************************
+ * @brief Called by the GRT when certain actions happen to a value, e.g. it is being deleted.
+ *
+ * @param grt The generic runtime that is acting.
+ * @param value The value on which the runtime is acting.
+ * @param reason The reason why this callback was called.
+ * @param data User data given when this callback was registered.
+ * @return 0
+ ****************************************************************************
+ */
+int refcache_callback(MYX_GRT* grt, MYX_GRT_VALUE* value, MYX_GRT_VALUE_CALLBACK_REASON reason, void* data)
+{
+  if (reason == MYX_GVCR_DELETE)
+  {
+    const char *_id= myx_grt_dict_item_get_as_string(value, "_id");
+    g_hash_table_remove(grt->global_reference_cache, _id);
+
+    g_hash_table_remove(grt->direct_reference_cache, _id);
+  };
+
+  return 0;
+}
+
+/**
+ ****************************************************************************
+ * @brief Called by the GRT when the cache is cleared.
+ *
+ * @param key The key of the entry that is currently removed.
+ * @param value The value part of the entry.
+ * @param data User data (NULL).
+ ****************************************************************************
+ */
+void refcache_callback2(gpointer key, gpointer value, gpointer data)
+{
+//  MYX_GRT_VALUE* grtValue = value;
+  myx_grt_value_listener_remove(value, data, refcache_callback);
+}
 
 /** 
  ****************************************************************************
@@ -1448,17 +1636,19 @@ void myx_grt_reference_cache_clear(MYX_GRT *grt)
 {
   GRT_ENTER(grt);
   
+  // Remove all value listener registrations.
+  g_hash_table_foreach(grt->global_reference_cache, refcache_callback2, NULL); 
+
   // free current cache
-  g_hash_table_destroy(grt->reference_cache);
+  g_hash_table_destroy(grt->global_reference_cache);
 
   // reallocate cache
-  grt->reference_cache= g_hash_table_new_full(g_str_hash, g_str_equal,
+  grt->global_reference_cache= g_hash_table_new_full(g_str_hash, g_str_equal,
                                                 g_free, NULL);
   
   GRT_LEAVE(grt);
 }
 
-
 static void grt_rescan_dict(MYX_GRT *grt, MYX_GRT_VALUE *dict, const char *object_path)
 {
   unsigned int i;
@@ -1470,7 +1660,7 @@ static void grt_rescan_dict(MYX_GRT *grt, MYX_GRT_VALUE *dict, const char *objec
   
   // if this dict inherits from GrtObject, cache its _id if it not empty
   // and set the object path
-  if (struct_name && myx_grt_struct_check_inherits_from(grt, struct_name, "GrtObject"))
+  if (struct_name && myx_grt_struct_inherits_from(grt, struct_name, "GrtObject"))
   {
     const char *_id= myx_grt_dict_item_get_as_string(dict, "_id");
 
@@ -1478,7 +1668,8 @@ static void grt_rescan_dict(MYX_GRT *grt, MYX_GRT_VALUE *dict, const char *objec
     {
       dict->value.d->grt= grt;
       dict->value.d->object_path= g_strdup(object_path);
-      g_hash_table_insert(grt->reference_cache, g_strdup(_id), dict);
+      g_hash_table_insert(grt->global_reference_cache, g_strdup(_id), dict);
+      myx_grt_value_listener_add(grt, dict, NULL, refcache_callback);
     }
   }
 
@@ -1544,20 +1735,31 @@ MYX_GRT_VALUE * myx_grt_reference_cache_lookup(MYX_GRT *grt, const char *ref_id)
 {
   MYX_GRT_VALUE *retval= NULL;
   
+  if (!ref_id || !ref_id[0])
+    return NULL;
+
   GRT_ENTER(grt);
   if(ref_id)
   {
     // try to find the value in the cache
-    MYX_GRT_VALUE *value= g_hash_table_lookup(grt->reference_cache, ref_id);
+    MYX_GRT_VALUE *value= g_hash_table_lookup(grt->global_reference_cache, ref_id);
 
     if (value)
       retval= value;
     else
     {
-      // if it is not found, rescan and search again
-      myx_grt_reference_cache_rescan(grt);
+      // try direct cache
+      value= g_hash_table_lookup(grt->direct_reference_cache, ref_id);
 
-      retval= g_hash_table_lookup(grt->reference_cache, ref_id);
+      if (value)
+        retval= value;
+      else
+      {
+        // if it is not found, rescan and search again
+        myx_grt_reference_cache_rescan(grt);
+
+        retval= g_hash_table_lookup(grt->global_reference_cache, ref_id);
+      }
     }
   }
   else
@@ -1569,6 +1771,8 @@ MYX_GRT_VALUE * myx_grt_reference_cache_lookup(MYX_GRT *grt, const char *ref_id)
 
 char * myx_grt_get_guid()
 {
+  /* GUIDs must be no more than 50 chars */
+  
 #if defined(__WIN__) || defined(_WIN32) || defined(_WIN64)
   GUID guid;
   WCHAR guid_wstr[50];
@@ -1584,6 +1788,22 @@ char * myx_grt_get_guid()
     (LPSTR) guid_str, 200, NULL, NULL);
 
   return g_strdup(guid_str);
+#elif defined(__APPLE__)
+  CFUUIDRef uid;
+  CFStringRef str;
+  char *data;
+  
+  uid= CFUUIDCreate(NULL);
+  str= CFUUIDCreateString(NULL, uid);
+  
+  data= g_malloc(sizeof(gchar)*(CFStringGetLength(str)+1));
+  
+  CFStringGetCString(str, data, CFStringGetLength(str)+1, kCFStringEncodingUTF8);
+  
+  CFRelease(uid);
+  CFRelease(str);
+  
+  return data;
 #else
   {
     uuid_t gid;
@@ -1594,3 +1814,61 @@ char * myx_grt_get_guid()
   }
 #endif
 }
+
+
+
+const char *myx_grt_error_string(MYX_GRT_ERROR error)
+{
+  switch (error)
+  {
+  case MYX_GRT_NO_ERROR:
+    return "Success";
+  case MYX_GRT_INTERNAL_ERROR:
+    return "Internal error";
+  case MYX_GRT_BAD_PATH:
+    return "Invalid path";
+  case MYX_GRT_CANT_OPEN_FILE:
+    return "Cannot open file";
+  case MYX_GRT_BAD_FUNCTION:
+    return "Invalid function";
+  case MYX_GRT_DUPLICATE_ENTRY:
+    return "Insertion of duplicate name";
+  case MYX_GRT_BAD_VALUE:
+    return "Invalid value";
+  case MYX_GRT_BAD_DATA:
+    return "Invalid data";
+  case MYX_GRT_VALIDATION_ERROR:
+    return "Validation error";
+  case MYX_GRT_FUNCTION_CALL_ERROR:
+    return "Error calling function";
+  case MYX_GRT_MODULE_INIT_ERROR:
+    return "Error initializing module";
+  case MYX_GRT_BAD_MODULE:
+    return "Invalid module";
+  case MYX_GRT_UNKNOWN_MODULE_TYPE:
+    return "Unknown module type";
+  case MYX_GRT_JAVA_NOT_FOUND:
+    return "Java JRE not found";
+  case MYX_GRT_JAVA_REGISTRY_CORRUPTED:
+    return "Java registry corrupted";
+  case MYX_GRT_JAVA_JRE_CANNOT_BE_LOADED:
+    return "Java JRE cannot be loaded";
+
+  case MYX_GRT_SHELL_UNSUPPORTED:
+    return "Shell type is not supported";
+  case MYX_GRT_SHELL_ALREADY_LOADED:
+    return "Shell was already initialized";
+  case MYX_GRT_SHELL_INIT_ERROR:
+    return "Error initializing shell";
+
+  default:
+    return "Unknown error";
+  }
+}
+
+#if defined(__WIN__) || defined(_WIN32) || defined(_WIN64)
+int myx_free_lib_stringlist(MYX_STRINGLIST *stringlist)
+{
+  return myx_free_stringlist(stringlist);
+}
+#endif
