@@ -1,24 +1,24 @@
-/* Copyright (C) 2004 MySQL AB
+/* Copyright (C) 2005 MySQL AB
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   This library is free software; you can redistribute it and/or
+   modify it under the terms of the GNU Lesser General Public
+   License as published by the Free Software Foundation; either
+   version 2.1 of the License, or (at your option) any later version.
 
-   This program is distributed in the hope that it will be useful,
+   This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+   Lesser General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
+   You should have received a copy of the GNU Lesser General Public
+   License along with this library; if not, write to the Free Software
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */
 
 
 #ifdef ENABLE_JAVA_MODULES
 
 #include "myx_grt_java.h"
-#include "myx_shared_aux_functions.h"
+#include "myx_shared_util_functions.h"
 
 #ifdef __GNUC__
 #define JAVA_PATH_SEPARATOR ":"
@@ -28,7 +28,13 @@
 
 #if defined(__WIN__) || defined(_WIN32) || defined(_WIN64)
 #include <Basetsd.h>
+#define DLOAD_JVM
 #else
+
+#ifdef DLOAD_JVM
+#include <dlfcn.h>
+#endif
+
 #define LongToPtr(l) ((void*)(long)(l))
 #define PtrToLong(p) ((long)(p))
 #endif
@@ -39,34 +45,37 @@
 static MYX_GRT_ERROR java_call_function(MYX_GRT_FUNCTION *function, MYX_GRT_VALUE *value, MYX_GRT_VALUE **retval);
 static MYX_GRT_ERROR java_init_module(MYX_GRT_MODULE_LOADER *loader, const char *file, MYX_GRT_MODULE **retmodule);
 
-static jobject j_xml_object_from_grt_value(JNIEnv *env, MYX_GRT_VALUE *value);
+static jobject j_xml_object_from_grt_value(MYX_GRT *grt, JNIEnv *env, MYX_GRT_VALUE *value);
 static MYX_GRT_VALUE *j_grt_value_from_java_xml_object(MYX_GRT *grt, JNIEnv *env, jobject object);
 static jobject j_call_method(JNIEnv *env, jclass theclass, jobject object, const char *name, const char *signature, jvalue *args);
 static jobject j_call_static_method(JNIEnv *env, jclass theclass, const char *name, const char *signature, jvalue *args);
 static char *j_get_string(JNIEnv *env, jobject strobj);
-static MYX_GRT_LIST *j_get_string_list(MYX_JAVA_LOADER *jloader, jobject lobj);
+//static MYX_GRT_LIST *j_get_string_list(MYX_JAVA_LOADER *jloader, jobject lobj);
 
 static MYX_GRT_ERROR java_cache_java_objects_and_methods(MYX_GRT *grt, MYX_JAVA_LOADER *priv);
 
-#if defined(__WIN__) || defined(_WIN32) || defined(_WIN64)
+#ifdef DLOAD_JVM
 typedef jint (JNICALL *CreateJavaVM_t)(JavaVM **pvm, void **env, void *args);
 
 static int get_java_runtimelib_filename(const char *version, char **runtimelib_filename);
 #endif
 
-MYX_GRT_MODULE_LOADER *myx_java_init_loader(MYX_GRT *grt, const char *class_path, MYX_GRT_ERROR *error, char *jvm_library)
+static JavaVM *global_jvm= NULL;
+
+MYX_GRT_MODULE_LOADER *myx_java_init_loader(MYX_GRT *grt, const char *class_path, MYX_GRT_ERROR *error, const char *jvm_library,
+                                            const char *grt_class_path_prefix)
 {
   MYX_GRT_MODULE_LOADER *loader= g_new0(MYX_GRT_MODULE_LOADER, 1);
   MYX_JAVA_LOADER *priv= g_new0(MYX_JAVA_LOADER, 1);
   JavaVMInitArgs vm_init_args;
-  JavaVMOption jvm_options[6];
+  JavaVMOption jvm_options[7];
   jclass grt_class;
   static char *file_extensions[]= {
     ".class"
   };
   int res;
-  const char *grt_class_path= "java";
-  const char *grt_class_path_libraries_dir= "java/lib";
+  char *grt_class_path= g_build_filename(grt_class_path_prefix, "java", NULL);
+  char *grt_class_path_libraries_dir= g_build_filename(grt_class_path_prefix, "java/lib", NULL);
   char *grt_class_path_libraries= NULL;
   GDir *dir;
   const char *entry;
@@ -76,8 +85,12 @@ MYX_GRT_MODULE_LOADER *myx_java_init_loader(MYX_GRT *grt, const char *class_path
   jobject grt_object;
   jvalue *args= NULL;
   char *runtimelib_filename;
-#if defined(__WIN__) || defined(_WIN32) || defined(_WIN64)
+#ifdef DLOAD_JVM
+# if defined(__WIN__) || defined(_WIN32) || defined(_WIN64)
   HINSTANCE handle;
+# else
+  void *handle;
+# endif
   CreateJavaVM_t create_java_vm;
 #endif
 
@@ -92,38 +105,48 @@ MYX_GRT_MODULE_LOADER *myx_java_init_loader(MYX_GRT *grt, const char *class_path
   loader->call_function= java_call_function;
   loader->extensions_num= 1;
   loader->extensions= file_extensions;
-
+  
+  memset(&vm_init_args, 0, sizeof(vm_init_args));
   vm_init_args.version = JNI_VERSION_1_4;
+#ifndef DLOAD_JVM
+  JNI_GetDefaultJavaVMInitArgs(&vm_init_args);
+#endif
 
   if (grt->options & MYX_GRT_REMOTE_DEBUG)
-    vm_init_args.nOptions = 5;
+    vm_init_args.nOptions = 6;
   else
-    vm_init_args.nOptions = 1;
+    vm_init_args.nOptions = 2;
 
   if (jvm_library && jvm_library[0]) 
   {
     runtimelib_filename= g_strdup(jvm_library);
   }
-#if defined(__WIN__) || defined(_WIN32) || defined(_WIN64)
+#ifdef DLOAD_JVM
   else
   {
     //try to locate the JVM .dll
-    *error= get_java_runtimelib_filename("1.4", &runtimelib_filename);
+    *error= get_java_runtimelib_filename("1.5", &runtimelib_filename);
     if (*error == MYX_GRT_JAVA_NOT_FOUND)
-      *error= get_java_runtimelib_filename("1.5", &runtimelib_filename);
+      *error= get_java_runtimelib_filename("1.4", &runtimelib_filename);
 
     if (*error != MYX_GRT_NO_ERROR)
     {
+#if defined(__WIN__) || defined(_WIN32) || defined(_WIN64)
       myx_grt_messages_stack_add_error(grt, "The JRE 1.4 cannot be found." _br
         "Please make sure the Sun JRE 1.4 is installed correctly." _br
         "Verify the following registry key:" _br
         "HKEY_LOCAL_MACHINE\\SOFTWARE\\JavaSoft\\Java Runtime Environment\\1.4\\RuntimeLib",
                                        NULL, 0, 0);
-
+#else
+      myx_grt_messages_stack_add_error(grt, "The JRE 1.4 cannot be found." _br
+        "Please make sure the Sun JRE 1.4 is installed correctly." _br
+        "Verify the GRT_JVM_PATH is set to the directory libjvm.so is located.",
+                                       NULL, 0, 0);
+#endif
       goto errorfree;
     }
   }
-#endif
+#endif // DLOAD_JVM
 
   //scan for .jars in the lib directory
   dir= g_dir_open(grt_class_path_libraries_dir, 0, NULL);
@@ -164,8 +187,14 @@ MYX_GRT_MODULE_LOADER *myx_java_init_loader(MYX_GRT *grt, const char *class_path
     jvm_options[2].optionString= strdup("-Xrunjdwp:transport=dt_socket,server=y,address=8000,suspend=n");
     jvm_options[3].optionString= "-Xnoagent";
     jvm_options[4].optionString= "-Djava.compiler=NONE";
-    jvm_options[5].optionString= "-Xss200m";
+    jvm_options[5].optionString= "-Xmx256m";
+    //jvm_options[6].optionString= "-Xms256m";
+
+    // do not modify the stack size
+    //jvm_options[5].optionString= "-Xss64m";
   }
+  else
+    jvm_options[1].optionString= "-Xmx256m";
 
   vm_init_args.options = jvm_options;
   vm_init_args.ignoreUnrecognized = JNI_FALSE;
@@ -200,10 +229,44 @@ MYX_GRT_MODULE_LOADER *myx_java_init_loader(MYX_GRT *grt, const char *class_path
 
   // Call JNI_CreateJavaVM to get the Java environment
   res= create_java_vm(&priv->jvm, (void **)&priv->env, &vm_init_args);
-#else
-  res= JNI_CreateJavaVM(&priv->jvm, (void **)&priv->env, &vm_init_args);
-#endif
 
+#elif defined(DLOAD_JVM)
+
+  if (!(handle= dlopen(runtimelib_filename, RTLD_LAZY)))
+  {
+    myx_grt_messages_stack_add_error(grt, "The JRE library cannot be found at %s.", NULL, 0, runtimelib_filename);
+    
+    *error= MYX_GRT_JAVA_JRE_CANNOT_BE_LOADED;
+    GRT_RETURN(grt, NULL, MYX_GRT_MODULE_LOADER*);
+  }
+
+  // Locate the JNI_CreateJavaVM function
+  create_java_vm= dlsym(handle, "JNI_CreateJavaVM");
+  if (create_java_vm == 0)
+  {
+    myx_grt_messages_stack_add_error(grt, "The JRE cannot be loaded.", NULL, 0);
+
+    *error= MYX_GRT_JAVA_JRE_CANNOT_BE_LOADED;
+    GRT_RETURN(grt, NULL, MYX_GRT_MODULE_LOADER*);
+  }    
+    
+  // Call JNI_CreateJavaVM to get the Java environment
+  res= create_java_vm(&priv->jvm, (void **)&priv->env, &vm_init_args);
+
+#else
+  if (!global_jvm)
+  {
+    res= JNI_CreateJavaVM(&priv->jvm, (void **)&priv->env, &vm_init_args);
+    global_jvm= priv->jvm;
+  }
+  else
+  {
+    vm_init_args.nOptions= 0;
+    jvm_options[0].optionString= 0;
+    
+    res= (*global_jvm)->AttachCurrentThread(global_jvm, (void **)&priv->env, &vm_init_args);
+  }
+#endif  
   g_free(jvm_options[0].optionString);
 
   if (res < 0)
@@ -280,9 +343,13 @@ MYX_GRT_MODULE_LOADER *myx_java_init_loader(MYX_GRT *grt, const char *class_path
   args= g_new0(jvalue, 2);
   args[0].l= (*priv->env)->NewStringUTF(priv->env, "GrtCallbackNative");
   {
+#ifdef FROM_XCODE
+    args[1].l= (*priv->env)->NewStringUTF(priv->env, grt_class_path_prefix);
+#else
     char *path= str_g_append(g_get_current_dir(), "/");
     args[1].l= (*priv->env)->NewStringUTF(priv->env, path);
     g_free(path);
+#endif
   }
 
   (*priv->env)->CallVoidMethodA(priv->env, grt_object, setCallback_methodID, args);
@@ -308,6 +375,8 @@ MYX_GRT_MODULE_LOADER *myx_java_init_loader(MYX_GRT *grt, const char *class_path
 error:
   *error= MYX_GRT_MODULE_INIT_ERROR;
 errorfree:
+  g_free(grt_class_path_libraries_dir);
+  g_free(grt_class_path);
   g_free(priv);
   g_free(loader);
   g_free(args);
@@ -722,12 +791,12 @@ static MYX_GRT_ERROR java_call_function(MYX_GRT_FUNCTION *function, MYX_GRT_VALU
   args[0].l= jmodule->class_ref;
   args[1].l= (*env)->NewStringUTF(env, function->name);
   args[2].l= (*env)->NewStringUTF(env, jfunc->java_signature);
-  args[3].l= j_xml_object_from_grt_value(env, value);
+  args[3].l= j_xml_object_from_grt_value(function->module->loader->grt, env, value);
   
   res= (*env)->CallStaticObjectMethodA(env, jloader->grt_class, jloader->grt_call_func, args);
   if (res)
   {
-    *retval= j_grt_value_from_java_xml_object(NULL, env, res);
+    *retval= j_grt_value_from_java_xml_object(function->module->loader->grt, env, res);
     (*env)->DeleteLocalRef(env, res);
   }
   else
@@ -809,12 +878,12 @@ static char *j_get_string(JNIEnv *env, jobject strobj)
 }
 
 
-static jobject j_xml_object_from_grt_value(JNIEnv *env, MYX_GRT_VALUE *value)
+static jobject j_xml_object_from_grt_value(MYX_GRT *grt, JNIEnv *env, MYX_GRT_VALUE *value)
 {
   jobject jstr= NULL;
   char *data;
 
-  data= myx_grt_value_to_xml(value);
+  data= myx_grt_value_to_xml(grt, value);
   if (data)
   {
     jstr= (*env)->NewStringUTF(env, data);
@@ -1182,9 +1251,9 @@ static MYX_GRT_VALUE *java_object_to_global_grt_value(MYX_GRT *grt, JNIEnv *env,
       }
       else if (myx_grt_struct_member_get_type(member) == MYX_LIST_VALUE)
       {
-        if (myx_grt_struct_member_get_content_struct_name(member) != NULL)
+        if (myx_grt_struct_member_get_content_struct_name_overridden(member) != NULL)
         {
-          char *class_name= str_g_replace(g_strdup(myx_grt_struct_member_get_content_struct_name(member)), ".", "/");
+          char *class_name= str_g_replace(g_strdup(myx_grt_struct_member_get_content_struct_name_overridden(member)), ".", "/");
           method_sig= str_g_append(method_sig, "L" GRTCLASSPREFIX);
           method_sig= str_g_append_and_free(method_sig, class_name);
           method_sig= str_g_append(method_sig, "List;");
@@ -1196,7 +1265,14 @@ static MYX_GRT_VALUE *java_object_to_global_grt_value(MYX_GRT *grt, JNIEnv *env,
       }
       else if (myx_grt_struct_member_get_type(member) == MYX_DICT_VALUE)
       {
-        if (myx_grt_struct_member_get_struct_name(member) != NULL)
+        if (myx_grt_struct_member_get_content_struct_name_overridden(member) != NULL)
+        {
+          char *class_name= str_g_replace(g_strdup(myx_grt_struct_member_get_content_struct_name_overridden(member)), ".", "/");
+          method_sig= str_g_append(method_sig, "L" GRTCLASSPREFIX);
+          method_sig= str_g_append_and_free(method_sig, class_name);
+          method_sig= str_g_append(method_sig, ";");
+        }
+        else if (myx_grt_struct_member_get_struct_name(member) != NULL)
         {
           char *class_name= str_g_replace(g_strdup(myx_grt_struct_member_get_struct_name(member)), ".", "/");
           method_sig= str_g_append(method_sig, "L" GRTCLASSPREFIX); 
@@ -1208,9 +1284,9 @@ static MYX_GRT_VALUE *java_object_to_global_grt_value(MYX_GRT *grt, JNIEnv *env,
           if (myx_grt_struct_member_get_content_type(member) == MYX_STRING_VALUE)
             method_sig= str_g_append(method_sig, "L" GRTCLASSPREFIX "GrtStringHashMap;");
           else if ( (myx_grt_struct_member_get_content_type(member) == MYX_DICT_VALUE) &&
-            (myx_grt_struct_member_get_content_struct_name(member) != NULL) )
+            (myx_grt_struct_member_get_content_struct_name_overridden(member) != NULL) )
           {
-            char *class_name= str_g_replace(g_strdup(myx_grt_struct_member_get_content_struct_name(member)), ".", "/");
+            char *class_name= str_g_replace(g_strdup(myx_grt_struct_member_get_content_struct_name_overridden(member)), ".", "/");
             method_sig= str_g_append(method_sig, "L" GRTCLASSPREFIX);
             method_sig= str_g_append_and_free(method_sig, class_name);
             method_sig= str_g_append(method_sig, "HashMap;");
@@ -1243,13 +1319,13 @@ static MYX_GRT_VALUE *java_object_to_global_grt_value(MYX_GRT *grt, JNIEnv *env,
           jobject j_member= (*env)->CallObjectMethodA(env, value, j_getter, NULL);
           if (j_member)
           {
-            if ( (myx_grt_struct_member_get_content_struct_name(member) != NULL)
+            if ( (myx_grt_struct_member_get_content_struct_name_overridden(member) != NULL)
                 && (myx_grt_struct_member_get_content_type(member) == MYX_STRING_VALUE) )
             {
               //deal with reference lists
               jint item_count= (*env)->CallIntMethodA(env, j_member, jloader->java_grtlist_size, NULL);
               MYX_GRT_VALUE *ref_list= myx_grt_list_new(MYX_STRING_VALUE, 
-                  myx_grt_struct_member_get_content_struct_name(member));
+                  myx_grt_struct_member_get_content_struct_name_overridden(member));
               int j;
 
               for (j= 0; j < item_count; j++)
@@ -1346,7 +1422,7 @@ JNIEXPORT jstring JNICALL Java_com_mysql_grt_GrtCallbackNative_callGrtFunction
   if (c_str_name)
     (*env)->ReleaseStringUTFChars(env, name, c_str_name);
 
-  GRT_RETURN(grt, j_xml_object_from_grt_value(env, result), jstring);
+  GRT_RETURN(grt, j_xml_object_from_grt_value(grt, env, result), jstring);
 }
 
 // ------------------------------------------------------------------------------------------------------------------
@@ -1389,7 +1465,7 @@ JNIEXPORT jobject JNICALL Java_com_mysql_grt_GrtCallbackNative_getGrtGlobalAsObj
   
   GRT_ENTER(grt);
   
-  value= myx_grt_dict_item_get_by_path(myx_grt_get_root(grt), c_str_objectPath);
+  value= myx_grt_dict_item_get_by_path(grt, myx_grt_get_root(grt), c_str_objectPath);
 
   //if the value doesn't exist yet, create it
   if (!value)
@@ -1401,7 +1477,7 @@ JNIEXPORT jobject JNICALL Java_com_mysql_grt_GrtCallbackNative_getGrtGlobalAsObj
     *ptr= 0;
 
     //fetch parent value
-    parent_value= myx_grt_dict_item_get_by_path(myx_grt_get_root(grt), parent_path);
+    parent_value= myx_grt_dict_item_get_by_path(grt, myx_grt_get_root(grt), parent_path);
 
     //if the parent is a dict
     if ((parent_value) && (myx_grt_value_get_type(parent_value) == MYX_DICT_VALUE))
@@ -1447,7 +1523,7 @@ JNIEXPORT jstring JNICALL Java_com_mysql_grt_GrtCallbackNative_getGrtGlobalAsStr
   
   GRT_ENTER(grt);
   
-  value= myx_grt_dict_item_get_by_path(myx_grt_get_root(grt), c_str_objectPath);
+  value= myx_grt_dict_item_get_by_path(grt, myx_grt_get_root(grt), c_str_objectPath);
 
   if (c_str_objectPath)
     (*env)->ReleaseStringUTFChars(env, objectPath, c_str_objectPath);
@@ -1467,7 +1543,7 @@ JNIEXPORT jint JNICALL Java_com_mysql_grt_GrtCallbackNative_getGrtGlobalAsInt
   
   GRT_ENTER(grt);
   
-  value= myx_grt_dict_item_get_by_path(myx_grt_get_root(grt), c_str_objectPath);
+  value= myx_grt_dict_item_get_by_path(grt, myx_grt_get_root(grt), c_str_objectPath);
 
   if (c_str_objectPath)
     (*env)->ReleaseStringUTFChars(env, objectPath, c_str_objectPath);
@@ -1487,7 +1563,7 @@ JNIEXPORT jdouble JNICALL Java_com_mysql_grt_GrtCallbackNative_getGrtGlobalAsRea
   
   GRT_ENTER(grt);
   
-  value= myx_grt_dict_item_get_by_path(myx_grt_get_root(grt), c_str_objectPath);
+  value= myx_grt_dict_item_get_by_path(grt, myx_grt_get_root(grt), c_str_objectPath);
 
   if (c_str_objectPath)
     (*env)->ReleaseStringUTFChars(env, objectPath, c_str_objectPath);
@@ -1594,7 +1670,7 @@ JNIEXPORT jint JNICALL Java_com_mysql_grt_GrtCallbackNative_getGrtGlobalListSize
   
   GRT_ENTER(grt);
   
-  value= myx_grt_dict_item_get_by_path(myx_grt_get_root(grt), c_str_objectPath);
+  value= myx_grt_dict_item_get_by_path(grt, myx_grt_get_root(grt), c_str_objectPath);
 
   if (c_str_objectPath)
     (*env)->ReleaseStringUTFChars(env, objectPath, c_str_objectPath);
@@ -1616,12 +1692,19 @@ JNIEXPORT jobject JNICALL Java_com_mysql_grt_GrtCallbackNative_getGrtGlobalListI
   
   GRT_ENTER(grt);
   
-  list= myx_grt_dict_item_get_by_path(myx_grt_get_root(grt), c_str_objectPath);
+  list= myx_grt_dict_item_get_by_path(grt, myx_grt_get_root(grt), c_str_objectPath);
 
   if (myx_grt_value_get_type(list) == MYX_LIST_VALUE)
   {
-    MYX_GRT_VALUE *list_item= myx_grt_list_item_get(list, index);
-    char *item_path= g_strdup_printf("%s/%d", c_str_objectPath, index);
+    MYX_GRT_VALUE *list_item;
+    char *item_path= g_strdup_printf("%s/%d", c_str_objectPath, (int)index);
+
+    /*// if this is a reference list, add _id of list_item instead
+    if ((myx_grt_list_content_get_type(list) == MYX_STRING_VALUE) && 
+      (myx_grt_list_content_get_struct_name(list) != NULL))
+      list_item= myx_grt_list_item_get_reference_value(grt, list, index);
+    else*/
+    list_item= myx_grt_list_item_get(list, index);
 
     jobj= java_global_object_from_grt_value(grt, env, item_path, list_item);
     g_free(item_path);
@@ -1643,11 +1726,18 @@ JNIEXPORT void JNICALL Java_com_mysql_grt_GrtCallbackNative_addGrtGlobalListItem
 
   GRT_ENTER(grt);
   
-  list= myx_grt_dict_item_get_by_path(myx_grt_get_root(grt), c_str_objectPath);
+  list= myx_grt_dict_item_get_by_path(grt, myx_grt_get_root(grt), c_str_objectPath);
   list_item= java_object_to_global_grt_value(grt, env, value);
   
   if ( (list) && (list_item) )
-    myx_grt_list_item_add(list, list_item);
+  {
+    // if this is a reference list, add _id of list_item instead
+    if ((myx_grt_list_content_get_type(list) == MYX_STRING_VALUE) && 
+      (myx_grt_list_content_get_struct_name(list) != NULL))
+      myx_grt_list_item_add(list, myx_grt_dict_item_get_value(list_item, "_id"));
+    else
+      myx_grt_list_item_add(list, list_item);
+  }
 
   if (c_str_objectPath)
     (*env)->ReleaseStringUTFChars(env, objectPath, c_str_objectPath);
@@ -1677,7 +1767,7 @@ JNIEXPORT jobject JNICALL Java_com_mysql_grt_GrtCallbackNative_getGrtGlobalDictI
   
   GRT_ENTER(grt);
   
-  dict= myx_grt_dict_item_get_by_path(myx_grt_get_root(grt), c_str_objectPath);
+  dict= myx_grt_dict_item_get_by_path(grt, myx_grt_get_root(grt), c_str_objectPath);
 
   if (myx_grt_value_get_type(dict) == MYX_DICT_VALUE)
   {
@@ -1703,11 +1793,11 @@ JNIEXPORT void JNICALL Java_com_mysql_grt_GrtCallbackNative_addGrtGlobalDictItem
   const char *c_str_objectPath= (*env)->GetStringUTFChars(env, objectPath, 0);
   const char *c_str_key= (*env)->GetStringUTFChars(env, key, 0);
   MYX_GRT_VALUE *dict;
-  MYX_GRT_VALUE *result= NULL;
+//  MYX_GRT_VALUE *result= NULL;
 
   GRT_ENTER(grt);
   
-  dict= myx_grt_dict_item_get_by_path(myx_grt_get_root(grt), c_str_objectPath);
+  dict= myx_grt_dict_item_get_by_path(grt, myx_grt_get_root(grt), c_str_objectPath);
   
   if (c_str_objectPath)
     (*env)->ReleaseStringUTFChars(env, objectPath, c_str_objectPath);
@@ -1727,11 +1817,50 @@ JNIEXPORT void JNICALL Java_com_mysql_grt_GrtCallbackNative_removeGrtGlobalDictI
 
 }
 
-JNIEXPORT jint JNICALL Java_com_mysql_grt_GrtCallbackNative_processMessages
+JNIEXPORT jint JNICALL Java_com_mysql_grt_GrtCallbackNative_getGrtGlobalDictKeyCount
+  (JNIEnv *env, jobject obj, jlong grtMyxGrtPointer, jstring objectPath)
+{
+  MYX_GRT *grt= LongToPtr(grtMyxGrtPointer);
+  const char *c_str_objectPath= (*env)->GetStringUTFChars(env, objectPath, 0);
+  MYX_GRT_VALUE *value;
+
+  GRT_ENTER(grt);
+
+  value= myx_grt_dict_item_get_by_path(grt, myx_grt_get_root(grt), c_str_objectPath);
+
+  if (c_str_objectPath)
+    (*env)->ReleaseStringUTFChars(env, objectPath, c_str_objectPath);
+
+  if (value && (myx_grt_value_get_type(value) == MYX_DICT_VALUE))
+    GRT_RETURN(grt, myx_grt_dict_item_count(value), jint);
+  else
+    GRT_RETURN(grt, -1, jint);
+}
+
+JNIEXPORT jstring JNICALL Java_com_mysql_grt_GrtCallbackNative_getGrtGlobalDictKey
+  (JNIEnv *env, jobject obj, jlong grtMyxGrtPointer, jstring objectPath, jint index)
+{
+  MYX_GRT *grt= LongToPtr(grtMyxGrtPointer);
+  const char *c_str_objectPath= (*env)->GetStringUTFChars(env, objectPath, 0);
+  MYX_GRT_VALUE *value;
+  
+  GRT_ENTER(grt);
+  
+  value= myx_grt_dict_item_get_by_path(grt, myx_grt_get_root(grt), c_str_objectPath);
+
+  if (c_str_objectPath)
+    (*env)->ReleaseStringUTFChars(env, objectPath, c_str_objectPath);
+
+  if (value && (myx_grt_value_get_type(value) == MYX_DICT_VALUE))
+    GRT_RETURN(grt, (*env)->NewStringUTF(env, myx_grt_dict_item_key_by_index(value, index)), jstring);
+  else
+    GRT_RETURN(grt, NULL, jstring);
+}
+
+JNIEXPORT void JNICALL Java_com_mysql_grt_GrtCallbackNative_processMessages
   (JNIEnv *env, jobject obj, jlong grtMyxGrtPointer, jstring msgs)
 {
   MYX_GRT *grt= LongToPtr(grtMyxGrtPointer);
-  jint res;
   
   if (grt->process_messages)
   {
@@ -1741,142 +1870,220 @@ JNIEXPORT jint JNICALL Java_com_mysql_grt_GrtCallbackNative_processMessages
     MYX_GRT_MSGS *msgs= myx_grt_messages_convert(value);
 
     // call the callback function to process the messages
-    res= grt->process_messages(msgs, grt->process_messages_data);
+    grt->process_messages(msgs, grt->process_messages_data);
 
     myx_grt_messages_free(msgs);
 
     myx_grt_value_release(value);
   }
+}
+
+JNIEXPORT jint JNICALL Java_com_mysql_grt_GrtCallbackNative_processStatusQuery
+  (JNIEnv *env, jobject obj, jlong grtMyxGrtPointer)
+{
+  MYX_GRT *grt= LongToPtr(grtMyxGrtPointer);
+  jint res= 0;
+  
+  if (grt->process_status_query)
+  {
+    // call the callback function to query the status, if returned a int <> 0 the execution will be aborted
+    res= grt->process_status_query(grt->process_status_query_data);
+  }
 
   return res;
 }
 
+JNIEXPORT void JNICALL Java_com_mysql_grt_GrtCallbackNative_setBridgeDataObject
+  (JNIEnv *env, jobject obj, jlong grtMyxGrtPointer, jstring objectPath, jobject dataObj)
+{
+  MYX_GRT *grt= LongToPtr(grtMyxGrtPointer);
+  const char *c_str_objectPath= (*env)->GetStringUTFChars(env, objectPath, 0);
+  MYX_GRT_VALUE *value;
+  
+  GRT_ENTER(grt);
+  
+  value= myx_grt_dict_item_get_by_path(grt, myx_grt_get_root(grt), c_str_objectPath);
+
+  if (dataObj == NULL)
+  {
+    dataObj= myx_grt_value_bridge_data_object_get(value);
+    (*env)->DeleteGlobalRef(env, dataObj);
+    dataObj= NULL;
+  }
+  else
+    dataObj= (*env)->NewGlobalRef(env, dataObj);
+
+  myx_grt_value_bridge_data_object_set(value, dataObj);
+
+  if (c_str_objectPath)
+    (*env)->ReleaseStringUTFChars(env, objectPath, c_str_objectPath);
+
+  GRT_LEAVE(grt);
+}
+
+JNIEXPORT jobject JNICALL Java_com_mysql_grt_GrtCallbackNative_getBridgeDataObject
+  (JNIEnv *env, jobject obj, jlong grtMyxGrtPointer, jstring objectPath)
+{
+  MYX_GRT *grt= LongToPtr(grtMyxGrtPointer);
+  const char *c_str_objectPath= (*env)->GetStringUTFChars(env, objectPath, 0);
+  MYX_GRT_VALUE *value;
+  jobject dataObj;
+  
+  GRT_ENTER(grt);
+  
+  value= myx_grt_dict_item_get_by_path(grt, myx_grt_get_root(grt), c_str_objectPath);
+
+  dataObj= myx_grt_value_bridge_data_object_get(value);
+
+  if (c_str_objectPath)
+    (*env)->ReleaseStringUTFChars(env, objectPath, c_str_objectPath);
+
+  GRT_RETURN(grt, dataObj, jobject);
+}
+
 // -------------------------------------------------------------------------------------------------
 
 #if defined(__WIN__) || defined(_WIN32) || defined(_WIN64)
 
-static int registry_get_value_as_string(HKEY key, const char *name, char *buffer, int buffer_size)
-{
-    DWORD reg_value_type;
-    DWORD reg_value_size;
-
-    if (((RegQueryValueEx(key, name, 0, &reg_value_type, 0, &reg_value_size) == 0) && 
-        (reg_value_type == REG_SZ) && (reg_value_size < (unsigned int)buffer_size))) 
-    {
-	    if (RegQueryValueEx(key, name, 0, 0, buffer, &reg_value_size) == 0) 
-	      return 1;
-    }
-
-    return 0;
-}
-
 #define MAX_KEY_LENGTH 255
 #define MAX_VALUE_NAME 16383
 
 static int get_java_runtimelib_filename(const char *version, char **runtimelib_filename)
 {
   HKEY java_key;
+  HKEY java_subkey;
   TCHAR subkey_name[MAX_KEY_LENGTH];   // buffer for subkey name
   DWORD subkey_length;
   DWORD cSubKeys=0;               // number of subkeys 
-  DWORD cbMaxSubKey;              // longest subkey size 
-  FILETIME ftLastWriteTime;      // last write time 
   DWORD i, retCode;
   char *best_hit= NULL;
-  int best_update= 0;
-  char buffer[MAX_VALUE_NAME];
+  int usedSubversion = -1;
+  int usedBuild = 0;
 
   // Find the Java key
-  if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, "Software\\JavaSoft\\Java Runtime Environment", 
-    0, KEY_READ, &java_key) != 0)
+  if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, "Software\\JavaSoft\\Java Runtime Environment", 0, KEY_READ, &java_key) != 0)
 	  return MYX_GRT_JAVA_NOT_FOUND;
 
   // Get the class name and the value count. 
-  retCode = RegQueryInfoKey(
-    java_key,                    // key handle 
-    NULL,                // buffer for class name 
-    NULL,           // size of class string 
-    NULL,                    // reserved 
-    &cSubKeys,               // number of subkeys 
-    &cbMaxSubKey,            // longest subkey size 
-    NULL,            // longest class string 
-    NULL,                // number of values for this key 
-    NULL,            // longest value name 
-    NULL,         // longest value data 
-    NULL,   // security descriptor 
-    &ftLastWriteTime);       // last write time
+  RegQueryInfoKey(java_key, NULL, NULL, NULL, &cSubKeys, NULL, NULL, NULL, NULL, NULL, NULL, NULL); 
 
   for (i= 0; i < cSubKeys; i++) 
   {
     subkey_length= MAX_KEY_LENGTH;
-
-    retCode = RegEnumKeyEx(java_key, i,
-      subkey_name, 
-      &subkey_length, 
-      NULL, 
-      NULL, 
-      NULL, 
-      &ftLastWriteTime);
-
+    retCode = RegEnumKeyEx(java_key, i, subkey_name, &subkey_length, NULL, NULL, NULL, NULL);
     if (retCode == ERROR_SUCCESS) 
     {
       char *better_hit= NULL;
 
       if (str_beginswith(subkey_name, version))
       {
-        if (subkey_length > 3)
-        {
-          int better_update= atoi(subkey_name + 6);
+        DWORD reg_value_type;
+        BOOL fileExists = TRUE;
+        char buffer[MAX_VALUE_NAME];
+        int bufferSize = MAX_VALUE_NAME;
 
-          if (better_update > best_update)
+        // Check if the jvm files exists and keep its name if it does, otherwise skip the entry.
+        RegOpenKeyEx(java_key, subkey_name, 0, KEY_READ, &java_subkey);
+        retCode = RegQueryValueEx(java_subkey, "RuntimeLib", NULL, &reg_value_type, buffer, &bufferSize) == 0;
+        RegCloseKey(java_subkey);
+        if (retCode == ERROR_SUCCESS);
+        {
+          // We got the RuntimeLib entry. Check whether it is a string value and points to an exisiting file.
+          if (reg_value_type == REG_SZ)
           {
-            best_update= better_update;
-            better_hit= g_strdup_printf("Software\\JavaSoft\\Java Runtime Environment\\%s", subkey_name);
-          }
-        }
-        else if (best_update == 0)
-        {
-          better_hit= g_strdup_printf("Software\\JavaSoft\\Java Runtime Environment\\%s", subkey_name);
-        }
-      }
+            TCHAR filename[MAX_PATH];
+            FILE* file;
 
-      if (better_hit)
-      {
-        g_free(best_hit);
-        best_hit= better_hit;
-      }
-    }
-  }
+            ExpandEnvironmentStrings(buffer, filename, MAX_PATH);
+            file = fopen(filename, "r");
+            if (file != NULL)
+            {
+              fileExists = TRUE;
+              fclose(file);
+              better_hit = g_strdup(filename);
+            };
+          };
 
+          if (fileExists)
+          {
+            // Now check the version info to find the highest installed version.
+            // Since the major and minor version numbers are given as parameters we only need 
+            // to check the sub version and the build numbers (e.g. as in 1.5.0_02).
+            gchar** parts = g_strsplit(subkey_name, ".", 3);
+            
+            // No need to process subversion or build if they aren't given (e.g. as in 1.5).
+            if (parts[1] != NULL && parts[2] != NULL)
+            {
+              int subVersion, build;
+              gchar** restParts = g_strsplit(parts[2], "_", 2);
+              subVersion = atoi(restParts[0]);
+              if (restParts[1] != NULL)
+                build = atoi(restParts[1]);
+              else
+                build = 0;
+              g_strfreev(restParts);
+              if (usedSubversion < 0)
+              {
+                // Nothing found yet, so use this entry.
+                best_hit = better_hit;
+                usedSubversion = subVersion;
+                usedBuild = build;
+              }
+              else
+                if ((usedSubversion < subVersion) || (usedSubversion == subVersion && usedBuild < build))
+                {
+                  usedSubversion = subVersion;
+                  usedBuild = build;
+                  g_free(best_hit);
+                  best_hit = better_hit;
+                };
+            }
+            else
+              if (usedSubversion < 0)
+              {
+                // Nothing found yet, so use this entry.
+                best_hit = better_hit;
+                usedSubversion = 0;
+                usedBuild = 0;
+              };
+
+            g_strfreev(parts);
+          };
+        };
+      };
+    };
+  };
   RegCloseKey(java_key);
 
-  // if a 1.4 hit was found, get the RuntimeLib value
   if (best_hit)
   {
-    if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, best_hit, 
-      0, KEY_READ, &java_key) != 0)
-    {
-      g_free(best_hit);
-	    return MYX_GRT_JAVA_NOT_FOUND;
-    }
-
-    g_free(best_hit);
-
-    if (!registry_get_value_as_string(java_key, "RuntimeLib", buffer, sizeof(buffer))) 
-	    return MYX_GRT_JAVA_REGISTRY_CORRUPTED;
-
-    *runtimelib_filename= g_strdup(buffer);
-
-    RegCloseKey(java_key);
-
+    *runtimelib_filename= best_hit;
     return MYX_GRT_NO_ERROR;
   }
   else
+    return MYX_GRT_JAVA_NOT_FOUND;
+}
+
+#else // !windows
+
+#ifdef DLOAD_JVM
+static int get_java_runtimelib_filename(const char *version, char **runtimelib_filename)
+{
+  char *path= getenv("GRT_JVM_PATH");
+  if (!path)
   {
     return MYX_GRT_JAVA_NOT_FOUND;
   }
+  
+  if (access(path, F_OK)!=0)
+    return MYX_GRT_JAVA_NOT_FOUND;
+  
+  *runtimelib_filename= g_strdup(path);
+
+  return MYX_GRT_NO_ERROR;
 }
+#endif // DLOAD_JVM
 
-#endif
+#endif // !windows
 
-#endif
+#endif // ENABLE_JAVA_MODULES
