@@ -106,7 +106,7 @@ bool MGConnectDialog::init(int flags, const std::string &header_image_file)
   Gtk::Image *prog;
 
   try {
-    _xml= new MGGladeXML(myg_datadir+"/"GLADE_FILE);
+    _xml= new MGGladeXML(myg_datadir+"/"GLADE_FILE, "", "mysql-gui-common");
   } catch (...) {
     g_warning("could not load glade xml file '%s'", 
               std::string(myg_datadir+"/"GLADE_FILE).c_str());
@@ -118,11 +118,11 @@ bool MGConnectDialog::init(int flags, const std::string &header_image_file)
 
   if (flags & MYG_FLAG_ALLOW_SKIP)
   {
-    ((Gtk::Window*)_xml->get_widget("connect_dialog"))->signal_key_press_event().connect(SigC::slot(*this,&MGConnectDialog::on_key_pressed));
-    ((Gtk::Window*)_xml->get_widget("connect_dialog"))->signal_key_release_event().connect(SigC::slot(*this,&MGConnectDialog::on_key_released));
+    ((Gtk::Window*)_xml->get_widget("connect_dialog"))->signal_key_press_event().connect(sigc::mem_fun(*this,&MGConnectDialog::on_key_pressed));
+    ((Gtk::Window*)_xml->get_widget("connect_dialog"))->signal_key_release_event().connect(sigc::mem_fun(*this,&MGConnectDialog::on_key_released));
   }
 
-  ((Gtk::Window*)_xml->get_widget("connect_dialog"))->signal_delete_event().connect(SigC::slot(*this,&MGConnectDialog::on_window_close));
+  ((Gtk::Window*)_xml->get_widget("connect_dialog"))->signal_delete_event().connect(sigc::mem_fun(*this,&MGConnectDialog::on_window_close));
 
   _pick_schema= (flags&MYG_FLAG_PICK_SCHEMA)!=0;
 
@@ -141,34 +141,34 @@ bool MGConnectDialog::init(int flags, const std::string &header_image_file)
     Gtk::Button *btn;
 
     btn= (Gtk::Button*)_xml->get_widget("connect_button");
-    btn->signal_clicked().connect(SigC::slot(*this, &MGConnectDialog::on_connect_button_clicked));
+    btn->signal_clicked().connect(sigc::mem_fun(*this, &MGConnectDialog::on_connect_button_clicked));
 
     btn= (Gtk::Button*)_xml->get_widget("clear_button");
-    btn->signal_clicked().connect(SigC::slot(*this, &MGConnectDialog::on_clear_button_clicked));
+    btn->signal_clicked().connect(sigc::mem_fun(*this, &MGConnectDialog::on_clear_button_clicked));
 
     btn= (Gtk::Button*)_xml->get_widget("cancel_button");
-    btn->signal_clicked().connect(SigC::slot(*this, &MGConnectDialog::on_cancel_button_clicked));
+    btn->signal_clicked().connect(sigc::mem_fun(*this, &MGConnectDialog::on_cancel_button_clicked));
 
     btn= (Gtk::Button*)_xml->get_widget("details_button");
-    btn->signal_clicked().connect(SigC::slot(*this, &MGConnectDialog::on_details_button_clicked));
+    btn->signal_clicked().connect(sigc::mem_fun(*this, &MGConnectDialog::on_details_button_clicked));
   }
   {
     Gtk::Entry *entry;
 
     entry= (Gtk::Entry*)_xml->get_widget("username_entry");
-    entry->signal_changed().connect(SigC::slot(*this, &MGConnectDialog::on_check_entry));
+    entry->signal_changed().connect(sigc::mem_fun(*this, &MGConnectDialog::on_check_entry));
 
     entry= (Gtk::Entry*)_xml->get_widget("password_entry");
-//    entry->signal_changed().connect(SigC::slot(*this, &MGConnectDialog::on_check_entry));
+//    entry->signal_changed().connect(sigc::mem_fun(*this, &MGConnectDialog::on_check_entry));
 
     entry= (Gtk::Entry*)_xml->get_widget("hostname_entry");
-    entry->signal_changed().connect(SigC::slot(*this, &MGConnectDialog::on_check_entry));
+    entry->signal_changed().connect(sigc::mem_fun(*this, &MGConnectDialog::on_check_entry));
 
     entry= (Gtk::Entry*)_xml->get_widget("port_spin");
-    entry->signal_changed().connect(SigC::slot(*this, &MGConnectDialog::on_check_entry));
+    entry->signal_changed().connect(sigc::mem_fun(*this, &MGConnectDialog::on_check_entry));
 
     entry= (Gtk::Entry*)_xml->get_widget("schema_entry");
-    entry->signal_changed().connect(SigC::slot(*this, &MGConnectDialog::on_check_entry));
+    entry->signal_changed().connect(sigc::mem_fun(*this, &MGConnectDialog::on_check_entry));
 
     std::list<std::string> dummy;
     myg_show_advanced_options(_xml, option_list, dummy);
@@ -177,7 +177,7 @@ bool MGConnectDialog::init(int flags, const std::string &header_image_file)
     Gtk::OptionMenu *menu;
 
     menu= (Gtk::OptionMenu*)_xml->get_widget("connection_list");
-    menu->signal_changed().connect(SigC::slot(*this,
+    menu->signal_changed().connect(sigc::mem_fun(*this,
                                               &MGConnectDialog::on_connection_list_changed));
   }
 
@@ -186,7 +186,7 @@ bool MGConnectDialog::init(int flags, const std::string &header_image_file)
   prog= (Gtk::Image*)_xml->get_widget("progress_image");
 
   anim= PIXCACHE->load_anim(PROGRESS_INDICATOR_FILE);
-  if (anim.is_null())
+  if (!anim)
   {
 //    if (error)
 //    {
@@ -236,6 +236,7 @@ bool MGConnectDialog::run_with_defaults(const Glib::ustring &title,
 
       dlg= new Gtk::MessageDialog(_("Can't show connection dialog.\n"
                                     "Software installation might be incorrect or corrupted."),
+                                  false,
                                   Gtk::MESSAGE_ERROR,
                                   Gtk::BUTTONS_OK,
                                   true);
@@ -245,6 +246,9 @@ bool MGConnectDialog::run_with_defaults(const Glib::ustring &title,
       return false;
     }
   }
+  
+  _cancel_clicked= false;
+  
 
   if (_connections)
   {
@@ -316,7 +320,7 @@ bool MGConnectDialog::run_with_defaults(const Glib::ustring &title,
     return true;
   }
 
-  return _skip_connect ? true : false;
+  return _skip_connect && !_cancel_clicked ? true : false;
 }
 
 
@@ -423,10 +427,10 @@ void MGConnectDialog::load_connection_list(const std::string &clpath)
     /* add extras */
     menu->append(*Gtk::manage(myg_make_separator_item()));
     item= Gtk::manage(new Gtk::MenuItem(_("Save This Connection...")));
-    item->signal_activate().connect(SigC::slot(*this,&MGConnectDialog::save_connection));
+    item->signal_activate().connect(sigc::mem_fun(*this,&MGConnectDialog::save_connection));
     menu->append(*item);
     item= Gtk::manage(new Gtk::MenuItem(_("Open Connection Editor")));
-    item->signal_activate().connect(SigC::slot(*this,&MGConnectDialog::open_connection_editor));
+    item->signal_activate().connect(sigc::mem_fun(*this,&MGConnectDialog::open_connection_editor));
     menu->append(*item);
     menu->show_all();
 
@@ -482,12 +486,16 @@ void *MGConnectDialog::connection_thread(void *data)
   /* check if we got a cancelation before setting it async */
   pthread_testcancel();
 
+  mysql_thread_init();
+
   me->_cur_conn.fill(&conn);
   
   me->_conn_result= -1;
   me->_conn_result= myx_connect_to_instance(&conn, me->_mysql);
   me->_connecting= false;
 
+  mysql_thread_end();
+
   return NULL;
 }
 
@@ -615,7 +623,12 @@ void MGConnectDialog::preferences_closed()
 
 bool MGConnectDialog::on_window_close(GdkEventAny *ev)
 {
+  bool cancel= _cancel_clicked;
+  
   on_cancel_button_clicked();
+  
+  _cancel_clicked= cancel;
+  
   return true;
 }
 
@@ -625,9 +638,9 @@ void MGConnectDialog::open_connection_editor()
   // select back the original item
   ((Gtk::OptionMenu*)_xml->get_widget("connection_list"))->set_history(_current_selected_item);
 
-  SigC::Connection c1= MGPreferencesEditor::instance()->signal_closed().connect(SigC::slot(*this,
+  SigC::Connection c1= MGPreferencesEditor::instance()->signal_closed().connect(sigc::mem_fun(*this,
                                                                                            &MGConnectDialog::preferences_closed));
-  SigC::Connection c2= MGPreferencesEditor::instance()->signal_changed().connect(SigC::slot(*this,
+  SigC::Connection c2= MGPreferencesEditor::instance()->signal_changed().connect(sigc::mem_fun(*this,
                                                                                             &MGConnectDialog::preferences_changed));
   MGPreferencesEditor::instance()->show(true);
   MGPreferencesEditor::instance()->set_modal(true);
@@ -663,10 +676,10 @@ void MGConnectDialog::show_connection_failed()
                                             _cur_conn.hostname.c_str(),
                                             (myerror=myx_mysql_errno(_mysql)),
                                             (tmp=myx_mysql_error(_mysql))),
+                                       false,
                                        Gtk::MESSAGE_ERROR,
                                        Gtk::BUTTONS_OK,
-                                       true,
-                                       false);
+                                       true);
   g_free(tmp);
 
   btn= _ping_dialog->add_button(_("_Ping Host"), 1);
@@ -715,7 +728,7 @@ void MGConnectDialog::show_connection_failed()
       }
       else
       {
-        input_handler= Glib::signal_io().connect(SigC::slot(*this, &MGConnectDialog::process_ping_output),
+        input_handler= Glib::signal_io().connect(sigc::mem_fun(*this, &MGConnectDialog::process_ping_output),
                                                  fileno(_ping_output),
                                                  Glib::IO_IN|Glib::IO_HUP);
         pinging= true;
@@ -794,7 +807,7 @@ void MGConnectDialog::start_connection()
     /* start connection thread */
     pthread_create(&_conn_thread, NULL, connection_thread, this);
 
-    Glib::signal_idle().connect(SigC::slot(*this, &MGConnectDialog::monitor_connection_thread));
+    Glib::signal_idle().connect(sigc::mem_fun(*this, &MGConnectDialog::monitor_connection_thread));
   }
 }
 
@@ -942,6 +955,7 @@ void MGConnectDialog::on_clear_button_clicked()
 
 void MGConnectDialog::on_cancel_button_clicked()
 {
+  _cancel_clicked= true;
   if (_conn_thread)
     stop_connection();
   else
