@@ -44,16 +44,47 @@ static struct OptionsByEngine {
 
 
 static struct EngineOptions {
-  char *option;
+  const char *label;
+  const char *name;
   MYX_DBM_TABLE_ENGINE type;
+  const char *description;
 } EngineOptionNameList[] = {
-  {"myisam_radio", MYX_DBM_TE_MYISAM},
-  {"innodb_radio", MYX_DBM_TE_INNODB},
-  {"memory_radio", MYX_DBM_TE_MEMORY},
-  {"merge_radio", MYX_DBM_TE_MERGE},
-  {"bdb_radio", MYX_DBM_TE_BDB},
-  {"isam_radio", MYX_DBM_TE_ISAM},
-  {"ndb_radio", MYX_DBM_TE_NDB}
+  {"MyISAM", "MyISAM", MYX_DBM_TE_MYISAM, 
+      "Very fast, disk based storage engine without support for transactions.\n"
+      "Offers fulltext search, packed keys, and is the default storage engine."},
+  {"InnoDB", "InnoDB", MYX_DBM_TE_INNODB,
+      "Transaction safe, disk based storage engine with row locking.\n"
+      "Recommended engine for tables that need support for transactions."},
+  {"Falcon", "jstar", MYX_DBM_TE_FALCON,
+      "Transactional engine for modern scale-out applications.\n"
+      "Supports MVCC (multiversion concurrency control)."},
+  {"NDB", "NDB", MYX_DBM_TE_NDB,
+      "MySQL Cluster storage engine. Transaction safe, memory based with row locking.\n"
+      "Recommended storage engine for high performance, real time critical applications."},
+  {"Archive", "ARCHIVE", MYX_DBM_TE_ARCHIVE,
+      "Archive engine storing data in a compressed form to save storage space.\n"
+      "Note that SELECTs require a full table scan."},
+  {"CSV", "CSV", MYX_DBM_TE_CSV,
+      "Engine that stores data as a comma separated values list, suited for data export.\n"
+      "Note that this engine does not support indices."},
+  {"Federated", "Federated", MYX_DBM_TE_FEDERATED,
+    "The FEDERATED storage engine accesses data in tables of remote databases rather\n"
+    "than in local tables."},
+  {"BLACKHOLE", "BLACKHOLE", MYX_DBM_TE_BLACKHOLE,
+      "The BLACKHOLE storage engine acts as a \"black hole\" that accepts data but\n"
+      "throws it away and does not store it."},
+  {"BerkeleyDB", "BerkeleyDB", MYX_DBM_TE_BDB,
+      "Transaction safe storage engine with page locking.  Also called Berkeley DB."},
+  {"MEMORY (HEAP)", "MEMORY", MYX_DBM_TE_MEMORY,
+      "Extremly fast memory based storage engine that uses hash indices.\n"
+      "Recommended storage engine for temporary data that can be lost in case\n"
+      "of a server shutdown."},
+  {"MERGE", "MERGE", MYX_DBM_TE_MERGE,
+      "Collection of MyISAM tables with identical column and index information.\n"
+      "Recommended storage engine for log tables or tables with archived data.\n"
+      "Note that the list of union tables and insert method has to be defined."},
+  {"ISAM (deprecated)", "ISAM", MYX_DBM_TE_ISAM,
+      "Deprecated. This storage engine was replaced by the MyISAM storage engine."},
 };
 
 
@@ -216,10 +247,15 @@ MGTableEditor::MGTableEditor(bool windowed)
   ((Gtk::CellRendererToggle*)tree->get_column(3)->get_first_cell_renderer())->signal_toggled().connect(sigc::bind<int>(sigc::mem_fun(*this,&MGTableEditor::column_toggled), 3));
   ((Gtk::CellRendererText*)tree->get_column(5)->get_first_cell_renderer())->signal_edited().connect(sigc::bind<int>(sigc::mem_fun(*this,&MGTableEditor::column_edited), 5));
   ((Gtk::CellRendererText*)tree->get_column(6)->get_first_cell_renderer())->signal_edited().connect(sigc::bind<int>(sigc::mem_fun(*this,&MGTableEditor::column_edited), 6));
-  
-  for (unsigned i= 0; i < sizeof(EngineOptionNameList)/sizeof(EngineOptions); i++)
+
   {
-    _xml->get_toggle(EngineOptionNameList[i].option)->signal_toggled().connect(sigc::mem_fun(*this,&MGTableEditor::update_for_selected_engine));
+    Gtk::ComboBoxText *combo= (Gtk::ComboBoxText*)_xml->get_combo("engine_combo");
+    Glib::RefPtr<Gtk::ListStore>::cast_dynamic(combo->get_model())->clear();
+    for (unsigned i= 0; i < sizeof(EngineOptionNameList)/sizeof(EngineOptions); i++)
+    {
+      combo->append_text(EngineOptionNameList[i].label);
+    }
+    combo->signal_changed().connect(sigc::mem_fun(*this,&MGTableEditor::update_for_selected_engine));
   }
 
   // handle change events for column options
@@ -314,8 +350,10 @@ void MGTableEditor::show_table_info(MYX_DBM_TABLE_DATA *data)
 {
   _xml->get_entry("name_entry")->set_text(data->name?:"");
   
+  /*
   ((Gtk::Entry*)_xml->get_combo_entry("database_combo")->get_child())->set_text(data->schema?:"");
   _xml->get_combo_entry("database_combo")->set_sensitive(false);
+   */
 
   _xml->get_entry("comment_entry")->set_text(data->comment?:"");
 }
@@ -340,10 +378,10 @@ void MGTableEditor::show_table_options(MYX_DBM_TABLE_DATA *data)
 {
   // general options
   for (unsigned i= 0; i < sizeof(EngineOptionNameList)/sizeof(EngineOptions); i++)
-  {
+  {    
     if (EngineOptionNameList[i].type == data->table_engine)
     {
-      _xml->get_toggle(EngineOptionNameList[i].option)->set_active();
+      _xml->get_combo("engine_combo")->set_active(i);
       break;
     }
   }
@@ -742,6 +780,15 @@ void MGTableEditor::update_for_selected_engine()
 {
   MYX_DBM_TABLE_ENGINE type= get_selected_engine();
 
+  for (unsigned i= 0; i < sizeof(EngineOptionNameList)/sizeof(*EngineOptionNameList); i++)
+  {
+    if (type == EngineOptionNameList[i].type)
+    {
+      _xml->get_label("engine_label")->set_text(EngineOptionNameList[i].description);
+      break;
+    }
+  }
+
   for (unsigned i= 0; i < sizeof(EngineOptionList)/sizeof(OptionsByEngine); i++)
   {
     if (EngineOptionList[i].engines & (1<<type))
@@ -787,7 +834,7 @@ void MGTableEditor::fill_aux_data()
   MYX_CATALOGS *cats= _catalog_data->ptr();
   MYX_CATALOG *my_catalog= NULL;
   MYX_SCHEMA_TABLES *my_schema_tables= NULL;
-  Glib::RefPtr<Gtk::ListStore> dblist= Gtk::ListStore::create(_columns);
+  //Glib::RefPtr<Gtk::ListStore> dblist= Gtk::ListStore::create(_columns);
   Glib::RefPtr<Gtk::ListStore> tblist= Gtk::ListStore::create(_columns);
   Glib::RefPtr<Gtk::ListStore> typelist= Gtk::ListStore::create(_columns);
   Glib::RefPtr<Gtk::ListStore> cslist1= Gtk::ListStore::create(_columns);
@@ -805,11 +852,11 @@ void MGTableEditor::fill_aux_data()
       
       for (unsigned int j= 0; j < my_catalog->schemata_num; j++)
       {
-        iter= dblist->append();
-        row= *iter;
+        //iter= dblist->append();
+        //row= *iter;
         
-        row[_columns.name]= my_catalog->schemata[j].schema_name;
-        row[_columns.object]= my_catalog->schemata+j;
+        //row[_columns.name]= my_catalog->schemata[j].schema_name;
+        //row[_columns.object]= my_catalog->schemata+j;
 
         if (_schema.compare(my_catalog->schemata[j].schema_name)==0)
           my_schema_tables= my_catalog->schemata[j].schema_tables;
@@ -817,9 +864,10 @@ void MGTableEditor::fill_aux_data()
       break;
     }
   }
-
+/*
   _xml->get_combo_entry("database_combo")->set_model(dblist);
   _xml->get_combo_entry("database_combo")->set_text_column(0);
+*/
 
   // fill table list in referenced table combo
   for (unsigned int i= 0; i < my_schema_tables->schema_tables_num; i++)
@@ -1018,10 +1066,8 @@ Glib::ustring MGTableEditor::description_for_charset(const Glib::ustring &s)
 
 MYX_DBM_TABLE_ENGINE MGTableEditor::get_selected_engine()
 {
-  for (unsigned int i= 0; i < sizeof(EngineOptionNameList)/sizeof(EngineOptions); i++)
-    if (_xml->get_toggle(EngineOptionNameList[i].option)->get_active())
-      return EngineOptionNameList[i].type;
-  return MYX_DBM_TE_MYISAM;
+  int row= _xml->get_combo("engine_combo")->get_active_row_number();
+  return EngineOptionNameList[row].type;
 }
 
 
@@ -1182,22 +1228,27 @@ void MGTableEditor::column_edited(const Glib::ustring &path, const Glib::ustring
 
   if (_showing_column_info)
     return;
-  
+
   switch (column)
   {
   case 0: // name
     if (row[_tcolumns.name] != new_text)
     {
+      g_message("NAME %s", new_text.c_str());
       row[_tcolumns.name]= new_text; // we need to manually set the value here
       _xml->get_entry("column_name_entry")->set_text(new_text);
       changed= true;
     }
     break;
   case 1: // type
-    if (row[_tcolumns.type] != new_text)
+    //if (row[_tcolumns.type] != new_text)
     {
+      bool ok= false;
       if (check_datatype(new_text))
+      {
         ((Gtk::Entry*)_xml->get_combo_entry("datatype_combo")->get_child())->set_text(new_text);
+        ok= true;
+      }
       else
         row[_tcolumns.type]= ((Gtk::Entry*)_xml->get_combo_entry("datatype_combo")->get_child())->get_text();
       
@@ -1208,6 +1259,8 @@ void MGTableEditor::column_edited(const Glib::ustring &path, const Glib::ustring
         row[_tcolumns.icon]= get_icon_for_column(find_datatype(_data_types, a),
                                                  row[_tcolumns.pk]);
       }
+      if (ok)
+        column_option_changed("type");
     }
     break;
   case 5: // default
@@ -1320,9 +1373,8 @@ void MGTableEditor::column_option_changed(const char *option)
       split_datatype((Glib::ustring)row[_tcolumns.type], otype, param);
 
       row[_tcolumns.type]= ((Gtk::Entry*)_xml->get_combo_entry("datatype_combo")->get_child())->get_text();
-      
+
       split_datatype((Glib::ustring)row[_tcolumns.type], type, param);
-      
       if (type != otype)
       {
         _flag_store->clear();
@@ -1828,18 +1880,11 @@ void MGTableEditor::commit_changes()
   
   // table options
   ASSIGN(_data->name, _xml->get_entry("name_entry")->get_text());
-  ASSIGN(_data->schema, ((Gtk::Entry*)_xml->get_combo_entry("database_combo")->get_child())->get_text());
+//  ASSIGN(_data->schema, ((Gtk::Entry*)_xml->get_combo_entry("database_combo")->get_child())->get_text());
   ASSIGN(_data->comment, _xml->get_entry("comment_entry")->get_text());
   
   // more table options
-  for (unsigned i= 0; i < sizeof(EngineOptionNameList)/sizeof(EngineOptions); i++)
-  {
-    if (_xml->get_toggle(EngineOptionNameList[i].option)->get_active())
-    {
-      _data->table_engine= EngineOptionNameList[i].type;
-      break;
-    }
-  }
+  _data->table_engine= EngineOptionNameList[_xml->get_combo("engine_combo")->get_active_row_number()].type;
 
   ASSIGN(_data->merge_union, _xml->get_entry("merge_tables_entry")->get_text());
   
