@@ -25,52 +25,6 @@
  * @brief
  */
 
-
-
-class BlockingRequest : public MGRT::Request {
-    Glib::ustring _module, _function;
-    MGRTValue _arguments;
-    Glib::Cond _finished;
-    Glib::Mutex _finished_mx;
-    bool _ready;
-  public:
-    BlockingRequest(const Glib::ustring &module,
-                    const Glib::ustring &function,
-                    const MGRTValue &arguments)
-      : _module(module), _function(function), _arguments(arguments)
-    {
-      _ready= false;
-    }
-
-    virtual void finish()
-    {
-      _finished_mx.lock();
-      _ready= true;
-      _finished.signal();
-      _finished_mx.unlock();
-    }
-    
-    virtual void wait()
-    {
-      _finished_mx.lock();
-      while (!_ready)
-        _finished.wait(_finished_mx);
-      _finished_mx.unlock();
-    }
-
-    virtual void execute(MGRT *grt)
-    {
-      _result= MGRTValue(myx_grt_function_get_and_call(grt->grt(),
-                                                       _module.c_str(),
-                                                       _function.c_str(),
-                                                       0,
-                                                       _arguments.grtValue(),
-                                                       &_error));
-      finish();
-    }
-};
-
-
 static Glib::ustring make_error_message_from_result(const MGRTValue &result)
 {
   if (result.isValid())
@@ -86,31 +40,179 @@ static Glib::ustring make_error_message_from_result(const MGRTValue &result)
 }
 
 
+void MGRT::Request::set_output_handler(const OutputHandlerSlot &handler)
+{
+  _outputHandler= handler;
+}
+
+    
+void MGRT::Request::set_message_handler(const MessageHandlerSlot &handler)
+{
+  _messageHandler= handler;
+}
+
+
+void MGRT::Request::set_input_handler(const InputHandlerSlot &handler)
+{
+ _inputHandler= handler;
+}
+
+
+
+class BlockingRequest : public MGRT::Request {
+    Glib::Cond _finished;
+    Glib::Mutex _finished_mx;
+    bool _ready;
+  public:
+    BlockingRequest(const Glib::ustring &module,
+                    const Glib::ustring &function,
+                    const MGRTValue &arguments)
+    {
+      _module= module;
+      _function= function;
+      _arguments= arguments;
+      _ready= false;
+    }
+
+    virtual bool is_async()
+    {
+      return false;
+    }
+    
+    virtual void finish(MGRT *grt)
+    {
+      _finished_mx.lock();
+      _ready= true;
+      _finished.signal();
+      _finished_mx.unlock();
+    }
+    
+    virtual bool wait()
+    {
+      _finished_mx.lock();
+      while (!_ready)
+        _finished.wait(_finished_mx);
+      _finished_mx.unlock();
+      
+      return true;
+    }
+
+    virtual void execute(MGRT *grt)
+    {
+      _result= MGRTValue(myx_grt_function_get_and_call(grt->grt(),
+                                                       _module.c_str(),
+                                                       _function.c_str(),
+                                                       0,
+                                                       _arguments.grtValue(),
+                                                       &_error));
+    }
+};
+
+
+
+MGRT::AsyncRequest::AsyncRequest(const Glib::ustring &module,
+                                 const Glib::ustring &function,
+                                 const MGRTValue &arguments,
+                                 void *user_data,
+                                 sigc::slot<void,const MGRTValue&,bool,void*> finished,
+                                 bool null_ok)
+  : _data(user_data), _finished(finished)
+{
+  _module= module;
+  _function= function;
+  _arguments= arguments;
+  _null_ok= null_ok;
+}
+
+
+bool MGRT::AsyncRequest::is_async()
+{
+  return true;
+}
+
+
+void MGRT::AsyncRequest::finish(MGRT *grt)
+{
+  bool error= false;
+  
+  if ((!_result.isValid() || !grt->result_is_error(_result)) && _error == MYX_GRT_NO_ERROR)
+  {
+    if (!_result.isValid() && !_null_ok)
+    {
+      grt->out_text(ufmt("Error: function %s.%s returned unexpected NULL result",
+                          _module.c_str(), _function.c_str()));
+      error= true;
+    }
+  }
+  else
+  {    
+    if (_result.isValid())
+      grt->out_text(ufmt("%s calling %s.%s: %s",
+                          error == MYX_GRT_NO_ERROR ? "Error" : grt->error_text(_error).c_str(),
+                          _module.c_str(), _function.c_str(),
+                          myx_grt_dict_item_get_as_string(_result, "error")?:""));
+    else
+      grt->out_text(ufmt("%s calling %s.%s",
+                          error == MYX_GRT_NO_ERROR ? "Error" : grt->error_text(_error).c_str(),
+                          _module.c_str(), _function.c_str()));
+    error= true;
+  }
+
+  if (_finished)
+    _finished(error || !_result.isValid() ? _result : _result["value"], error, _data);
+}
+
+
+void MGRT::AsyncRequest::execute(MGRT *grt)
+{
+  _result= MGRTValue(myx_grt_function_get_and_call(grt->grt(),
+                                                   _module.c_str(),
+                                                   _function.c_str(),
+                                                   0,
+                                                   _arguments.grtValue(),
+                                                   &_error));
+}
+
+
+bool MGRT::AsyncRequest::wait()
+{
+  return false;
+}
+
+
+
 MGRT::MGRT()
 {
   _grt= myx_grt_initialize(0);
+  
+  _finishedRequestSignal.connect(sigc::mem_fun(*this,&MGRT::handle_finished_async_requests));
 }
   
   
-void MGRT::initialize_grt_thread(const Glib::ustring &resourcePath)
+void MGRT::initialize_grt_thread(const Glib::ustring &resourcePath, const Glib::ustring &binResourcePath)
 { 
   _thread_ready.lock();
   
-  Glib::Thread::create(sigc::bind<std::string>(sigc::mem_fun(*this, &MGRT::request_thread), resourcePath),
+  Glib::Thread::create(sigc::bind<std::string,std::string>(sigc::mem_fun(*this, &MGRT::request_thread), resourcePath, binResourcePath),
                        false);
 
   // wait until thread ready
   _thread_ready.lock();
+
+  // cleanup
+  _thread_ready.unlock();
 }
 
 
-void MGRT::init_thread(const std::string &resourcePath)
+void MGRT::init_thread(const std::string &resourcePath, const std::string &binResourcePath)
 {
   MYX_GRT_MODULE_LOADER *loader;
   MYX_GRT_ERROR error;
   
+  myx_grt_set_input_callback(_grt, this, process_grt_input);
   myx_grt_set_output_callback(_grt, this, process_grt_output);
   myx_grt_set_message_callback(_grt, this, process_grt_message);
+  myx_grt_set_status_query_callback(_grt, this, process_grt_status_query);
 
   if (myx_grt_setup_shell(_grt, MYX_GRT_SHELL_LUA) != MYX_GRT_NO_ERROR)
   {
@@ -128,18 +230,20 @@ void MGRT::init_thread(const std::string &resourcePath)
   myx_register_builtin_grt_module_reverse_engineer_mysql(_grt);
   myx_register_builtin_grt_module_transformation_mysql(_grt);
 
+
 #ifdef ENABLE_JAVA_MODULES
   // java modules
   out_text("Loading Java modules...");
-  loader= myx_java_init_loader(_grt, NULL, &error, NULL,
-                               std::string(resourcePath+"/").c_str());
+  loader= myx_java_init_loader(_grt, 
+                               binResourcePath.c_str(), &error, NULL,
+                               resourcePath.c_str());
   if (loader)
   {
     if (myx_grt_register_module_loader(_grt, loader) < 0)
       out_text("Error registering Java module loader\n");
     else
     {
-      int i= myx_grt_scan_for_modules(_grt, std::string(resourcePath+"/java/com/mysql/grt/modules").c_str(), &error);
+      int i= myx_grt_scan_for_modules(_grt, std::string(resourcePath+"/java").c_str(), &error);
       if (error != MYX_GRT_NO_ERROR)
         out_text(ufmt("Error scanning Java modules (%i)\n", error));
       else
@@ -147,7 +251,8 @@ void MGRT::init_thread(const std::string &resourcePath)
     }
   }
   else
-    out_text(ufmt("Error initializing Java module loader (%i)\n",error));
+    out_text(ufmt("Could not initialize Java module loader (%i)\n",error));
+  flush_messages();
 #endif
 
 #ifdef ENABLE_PHP_MODULES
@@ -170,6 +275,7 @@ void MGRT::init_thread(const std::string &resourcePath)
   }
   else
     out_text("Error initializing PHP module loader (%i)\n",error);
+  flush_messages();
 #endif
 
   // lua modules
@@ -190,10 +296,12 @@ void MGRT::init_thread(const std::string &resourcePath)
   }
   else
     out_text(ufmt("Error initializing Lua module loader (%i)\n",error));
+  flush_messages();
 
   // builtin c++
   out_text("Initializing C++ loader...");
   setup_loader();
+  flush_messages();
 
   // this will "signal" the main thread that init is done
   _thread_ready.unlock();
@@ -203,9 +311,17 @@ void MGRT::init_thread(const std::string &resourcePath)
 
 
 
+void MGRT::flush_messages()
+{
+  myx_grt_messages_stack_flush(_grt, 0);
+}
+
+
 
 MGRT::~MGRT()
 {
+  queue_request(0);
+
   myx_grt_finalize(_grt);
 }
     
@@ -218,6 +334,21 @@ void MGRT::set_console(MGShellView *shell)
 }
 
 
+
+int MGRT::process_grt_input(const char *caption, MYX_GRT_INPUT_OPTIONS options, const char **text, void *user_data)
+{
+  MGRT *me= (MGRT*)user_data;
+  Glib::ustring out;
+  
+  if (!me->_inputHandler(caption, options & MYX_GR_IO_PASSWORD, &out))
+    return -1;
+  
+  *text= g_strdup(out.c_str());
+    
+  return 0;
+}
+
+
 void MGRT::process_grt_output(const char *text, void *data)
 {
   MGRT *me= (MGRT*)data;
@@ -256,18 +387,33 @@ void MGRT::process_grt_message(MYX_GRT_MSGS *msgs, void *data)
 }
 
 
+int MGRT::process_grt_status_query(void *data)
+{
+  MGRT *me= (MGRT*)data;
+
+  if (me->_statusQueryHandler)
+    return me->_statusQueryHandler();
+  return 0;
+}
+
+
 void MGRT::out_text(const Glib::ustring &text)
 {
   if (!_outputHandler.empty())
     _outputHandler(text);
   else if (_console)
     _console->out_text(text);
+  if (getenv("DEBUG"))
+    g_message("%s", text.c_str());
 }
 
 
 void MGRT::out_message(const Glib::ustring &text,
                       const Glib::ustring &type)
 {
+  if (getenv("DEBUG"))
+    g_message("%s: %s", type.c_str(), text.c_str());
+
   if (!_messageHandler.empty())
     _messageHandler(text, type);
   else if (_console)
@@ -292,33 +438,35 @@ Glib::ustring MGRT::shell_prompt()
 
 
 
-bool MGRT::save_app_dict()
+bool MGRT::save_subtree(const char *subtree_path, const std::string &path)
 {
+  MGRTValue subtree(global_value(subtree_path));
+  
+  if (!subtree.isValid())
+    return true;
+  
   if (myx_grt_store_to_file(_grt,
-                            global_value("/app").grtValue(),
-                            _appDictPath.c_str()) == MYX_GRT_NO_ERROR)
+                            subtree.grtValue(),
+                            path.c_str()) == MYX_GRT_NO_ERROR)
     return true;
   return false;
 }
 
 
-bool MGRT::load_app_dict_from_path(const std::string &path)
+bool MGRT::load_subtree(const char *subtree_path, const std::string &path)
 {
-  MYX_GRT_VALUE *appDict= 0;
-  
-  _appDictPath= path;
+  MYX_GRT_VALUE *dict= 0;
 
   if (access(path.c_str(), F_OK)==0)
-    appDict= myx_grt_retrieve_from_file(_grt, _appDictPath.c_str());
+    dict= myx_grt_retrieve_from_file(_grt, path.c_str());
 
-  if (appDict)
+  if (dict)
   {
-    set_global_value("/app", appDict);
-    myx_grt_value_release(appDict);
+    set_global_value(subtree_path, dict);
+    myx_grt_value_release(dict);
     return true;
   }
   return false;
-
 }
 
 
@@ -378,13 +526,16 @@ bool MGRT::scan_modules_in_path(const std::string &path)
 
 
 
-void MGRT::request_thread(std::string resourcePath)
+void MGRT::request_thread(const std::string &resourcePath, const std::string &binResourcePath)
 {
-  init_thread(resourcePath);
+  init_thread(resourcePath, binResourcePath);
   
   for (;;)
   {
     Request *req;
+    OutputHandlerSlot outputSlot;
+    InputHandlerSlot inputSlot;
+    MessageHandlerSlot messageSlot;
     
     _requestQueueLock.lock();
     while (_requestQueue.empty())
@@ -393,8 +544,35 @@ void MGRT::request_thread(std::string resourcePath)
     req= *_requestQueue.begin();
     _requestQueue.erase(_requestQueue.begin());
     _requestQueueLock.unlock();
+    
+    if (!req)
+      break;
+
+    if (req->_inputHandler)
+      inputSlot= set_input_handler(req->_inputHandler);
+    if (req->_outputHandler)
+      outputSlot= set_output_handler(req->_outputHandler);
+    if (req->_messageHandler)
+      messageSlot= set_message_handler(req->_messageHandler);
 
     req->execute(this);
+
+    if (outputSlot)
+      set_output_handler(outputSlot);
+    if (inputSlot)
+      set_input_handler(inputSlot);
+    if (messageSlot)
+      set_message_handler(messageSlot);
+
+    if (req->is_async())
+    {
+      _finishedRequestLock.lock();
+      _finishedRequestQueue.push_back(req);
+      _finishedRequestLock.unlock();
+      _finishedRequestSignal.emit();
+    }
+    else
+      req->finish(this);
   }
 }
 
@@ -408,6 +586,62 @@ void MGRT::queue_request(Request *req)
 }
 
 
+
+void MGRT::handle_finished_async_requests()
+{
+  Request *req;
+  
+  _finishedRequestLock.lock();
+  req= *_finishedRequestQueue.begin();
+  _finishedRequestQueue.erase(_finishedRequestQueue.begin());
+  _finishedRequestLock.unlock();
+
+  req->finish(this);
+
+  delete req;
+}
+
+
+MGRT::AsyncRequest *MGRT::prepare_async_call(const std::string &module,
+                                             const std::string &function,
+                                             const MGRTValue &arguments,
+                                             void *data,
+                                             sigc::slot<void,const MGRTValue&,bool,void*> finished,
+                                             bool null_ok)
+{
+  AsyncRequest *request;
+
+  _lastError= MYX_GRT_NO_ERROR;
+  _lastErrorMessage= "";
+
+  request= new AsyncRequest(module, function, arguments, data, finished, null_ok);
+  
+  return request;
+}
+
+
+
+bool MGRT::call_async_function(const std::string &module,
+                               const std::string &function,
+                               const MGRTValue &arguments,
+                               void *data,
+                               sigc::slot<void,const MGRTValue&,bool,void*> finished,
+                               bool null_ok)
+{
+  AsyncRequest *request;
+
+  _lastError= MYX_GRT_NO_ERROR;
+  _lastErrorMessage= "";
+
+  request= new AsyncRequest(module, function, arguments, data, finished, null_ok);
+  
+  queue_request(request);
+
+  return true;
+}
+
+
+
 bool MGRT::call_procedure(const std::string &module,
                          const std::string &procedure,
                          const MGRTValue &arguments)
@@ -430,7 +664,7 @@ MGRTValue MGRT::call_function(const std::string &module,
   request= new BlockingRequest(module, function, arguments);
   
   queue_request(request);
-  
+
   request->wait();
   
   MGRTValue result(request->result());
@@ -462,7 +696,7 @@ MGRTValue MGRT::call_function(const std::string &module,
                     error == MYX_GRT_NO_ERROR ? "Error" : error_text(error).c_str(),
                     module.c_str(), function.c_str()));
     return MGRTValue();
-  }  
+  }
 }
 
     
@@ -488,40 +722,67 @@ Glib::ustring MGRT::call_string_function(const std::string &module,
     return result.asString();
   return "";
 }
-    
-void MGRT::set_output_handler(sigc::slot<void,Glib::ustring> handler)
+
+
+MGRT::OutputHandlerSlot MGRT::set_output_handler(const OutputHandlerSlot &handler)
 {
+  _handlerMutex.lock();
+  OutputHandlerSlot old= _outputHandler;
   _outputHandler= handler;
+  _handlerMutex.unlock();
+  return old;
+}
+
+
+MGRT::InputHandlerSlot MGRT::set_input_handler(const InputHandlerSlot &handler)
+{
+  _handlerMutex.lock();
+  InputHandlerSlot old= _inputHandler;
+  _inputHandler= handler;
+  _handlerMutex.unlock();
+  return old;
 }
 
 
 void MGRT::reset_output_handler()
 {
+  _handlerMutex.lock();
   _outputHandler.disconnect();
+  _handlerMutex.unlock();
 }
 
 
-
-void MGRT::set_message_handler(sigc::slot<void,Glib::ustring,Glib::ustring> handler)
+void MGRT::set_status_query_handler(sigc::slot<int> handler)
 {
+  _statusQueryHandler= handler;
+}
+
+
+MGRT::MessageHandlerSlot MGRT::set_message_handler(const MessageHandlerSlot &handler)
+{
+  _handlerMutex.lock();
+  MessageHandlerSlot old= _messageHandler;
   _messageHandler= handler;
+  _handlerMutex.unlock();
+  return old;
 }
 
 
 void MGRT::reset_message_handler()
 {
+  _handlerMutex.lock();
   _messageHandler.disconnect();
+  _handlerMutex.unlock();
 }
 
 
-    
+
 void MGRT::report_error(MYX_GRT_ERROR error)
 {
   out_text(error_text(error));
 }
 
 
-    
 MYX_GRT_ERROR MGRT::last_error()
 {
   return _lastError;
@@ -543,6 +804,12 @@ bool MGRT::result_is_error(const MGRTValue &value)
 }
 
 
+MGRTValue MGRT::result_value(const MGRTValue &value)
+{
+  return value["value"];
+}
+
+
 void MGRT::report_error_result(const MGRTValue &result)
 {
   if (result.isValid() && result["error"].isValid())
@@ -599,6 +866,17 @@ void MGRT::set_global_value(const char *path, const MGRTValue &value)
 }
 
 
+void MGRT::unset_global_value(const char *path)
+{
+  char *end= strrchr(path, '/');
+  std::string subpath(path, end-path);
+  MYX_GRT_VALUE *dict;
+  
+  dict= myx_grt_dict_item_get_by_path(_grt, myx_grt_get_root(_grt), subpath.c_str());
+  myx_grt_dict_item_del(dict, end+1);
+}
+
+
 //----------------------------------------------------------------------
 
 class MGRTModuleFunctionWrapper {    
@@ -675,7 +953,7 @@ void MGRT::register_builtin_module(const Glib::ustring &module_name,
   module->extends= NULL;
     
   error= myx_grt_add_module(_grt, module);
-  if (error != MYX_GRT_NO_ERROR)
+  if (error == MYX_GRT_NO_ERROR)
     out_text("Registration ok");
   else
     out_text(ufmt("C++ loader registration failed (%i)", error));
