@@ -11,9 +11,12 @@
 
 @interface MAccessoryScrollView : NSScrollView 
 {
-  NSMutableArray *_accessoryViews;
+  NSMutableArray *_vAccessoryViews;  
+  NSMutableArray *_hAccessoryViews;
 }
 
-- (void)addAccessoryView:(NSView*)view;
+- (void)addVerticalAccessoryView:(NSView*)view;
+- (void)addHorizontalAccessoryView:(NSView*)view;
+- (void)removeAccessoryViews;
 
 @end
