@@ -7,7 +7,7 @@
 //
 
 #import "MDialogs.h"
-
+ 
 @implementation MStringRequestSheet
 
 + (MStringRequestSheet*)sheetWithTitle:(NSString*)title labels:(NSArray*)labels;
@@ -16,7 +16,19 @@
 }
 
 
++ (MStringRequestSheet*)sheetWithTitle:(NSString*)title label:(NSString*)label
+{
+  return [[[MStringRequestSheet alloc] initWithTitle:title labels:[NSArray arrayWithObject:label]] autorelease];
+}
+
+
 - (id)initWithTitle:(NSString*)title labels:(NSArray*)labels
+{
+  return [self initWithTitle:title labels:labels buttons:nil];
+}
+
+
+- (id)initWithTitle:(NSString*)title labels:(NSArray*)labels buttons:(NSArray*)buttons
 {
   NSRect contentRect;
   
@@ -55,37 +67,68 @@
 	{
 	  [form addEntry:[labels objectAtIndex:i]];
 	}
-    w= [[NSButton alloc] initWithFrame:NSMakeRect(316.0, 11.0, 90.0, 32.0)];
-    [contentView addSubview:w];
-    [w setTitle:@"OK"];
-    [w release];
-    [w setAction:@selector(ok:)];
-    [w setTarget:self];
-    [[w cell] setBezelStyle:NSRoundedBezelStyle];
-    [self setDefaultButtonCell:[w cell]];
     
-    w= [[NSButton alloc] initWithFrame:NSMakeRect(224.0, 11.0, 90.0, 32.0)];
-    [contentView addSubview:w];
-    [w setTitle:@"Cancel"];
-    [w release];
-    [w setAction:@selector(cancel:)];
-    [w setTarget:self];
-    [[w cell] setBezelStyle:NSRoundedBezelStyle];
+    if (!buttons)
+    {
+      w= [[NSButton alloc] initWithFrame:NSMakeRect(316.0, 11.0, 90.0, 32.0)];
+      [w setTag:0];
+      [contentView addSubview:w];
+      [w setTitle:@"OK"];
+      [w release];
+      [w setAction:@selector(buttonClicked:)];
+      [w setTarget:self];
+      [[w cell] setBezelStyle:NSRoundedBezelStyle];
+      [self setDefaultButtonCell:[w cell]];
+    
+      w= [[NSButton alloc] initWithFrame:NSMakeRect(224.0, 11.0, 90.0, 32.0)];
+      [w setTag:-1];
+      [contentView addSubview:w];
+      [w setTitle:@"Cancel"];
+      [w setKeyEquivalent:@"\E"];
+      [w release];
+      [w setAction:@selector(buttonClicked:)];
+      [w setTarget:self];
+      [[w cell] setBezelStyle:NSRoundedBezelStyle];
+    }
+    else
+    {
+      unsigned int i;
+      float x= 420-14;
+      NSFont *font= [NSFont systemFontOfSize:[NSFont systemFontSize]];
+      
+      for (i= 0; i < [buttons count]; i++)
+      {
+        float size;
+        
+        size= [font widthOfString:[buttons objectAtIndex:i]];
+        if (size < 90.0-20.0)
+          size= 90.0;
+        else
+          size+= 35.0;
+        
+        w= [[NSButton alloc] initWithFrame:NSMakeRect(x-size, 11.0, size, 32.0)];
+        [w setTag:i];
+        [contentView addSubview:w];
+        [w setTitle:[buttons objectAtIndex:i]];
+        [w release];
+        [w setAction:@selector(buttonClicked:)];
+        [w setTarget:self];
+        [[w cell] setBezelStyle:NSRoundedBezelStyle];
+        
+        x-= size+2;
+      }
+      // Last is supposed to be Escape
+      [w setKeyEquivalent:@"\E"];
+    }
   }
   return self;
 }
 
 
-- (void)cancel:(id)sender
+- (void)buttonClicked:(id)sender
 {
   [NSApp endSheet:self];
-  canceled= YES;
-}
-
-- (void)ok:(id)sender
-{
-  [NSApp endSheet:self];
-  canceled= NO;
+  button= [sender tag];
 }
 
 - (void)sheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo
@@ -111,7 +154,7 @@
 
   [NSApp runModalForWindow:self];
 
-  if (canceled)
+  if (button == -1)
     return nil;
   else
   {
@@ -132,4 +175,10 @@
 	[[form cellAtIndex:i] setStringValue:[array objectAtIndex:i]];
 }
 
+
+- (int)clickedButton
+{
+  return button;
+}
+
 @end
