@@ -13,12 +13,29 @@
 - (void)displayMenu:(NSTimer *)theTimer
 {
   NSEvent *theEvent=[timer userInfo];
-  NSEvent *click = [NSEvent mouseEventWithType:[theEvent type] location:[self frame].origin modifierFlags:[theEvent modifierFlags] timestamp:[theEvent timestamp] windowNumber:[theEvent windowNumber] context:[theEvent context] eventNumber:[theEvent eventNumber] clickCount:[theEvent clickCount] pressure:[theEvent pressure]]; 
+  NSEvent *click = [NSEvent mouseEventWithType:[theEvent type]
+                                      location:[self convertPoint:NSMakePoint(0, NSMaxY([self frame]))
+                                                           toView:nil]
+                                 modifierFlags:[theEvent modifierFlags] 
+                                     timestamp:[theEvent timestamp] 
+                                  windowNumber:[theEvent windowNumber] 
+                                       context:[theEvent context] 
+                                   eventNumber:[theEvent eventNumber] 
+                                    clickCount:[theEvent clickCount] 
+                                      pressure:[theEvent pressure]]; 
   [NSMenu popUpContextMenu:[self menu] withEvent:click forView:self];
   [timer invalidate];
+  timer= nil;
   [self highlight:NO];
 }   
 
+
+/*- (void)drawRect:(NSRect)aRect
+{
+  [super drawRect:aRect];
+}
+*/
+
 - (void)mouseDown:(NSEvent *)theEvent 
 { 
   if ([self isEnabled])
@@ -35,6 +52,7 @@
   {
     [self highlight:NO];
     [timer invalidate];
+    timer= nil;
     [self sendAction:[self action] to:[self target]];
   }
 }
