@@ -3,6 +3,9 @@
 #import <Cocoa/Cocoa.h>
 #include "myx_public_interface.h"
 
+
+extern NSString *MTableEditorTableDidChange;
+
 @class MAccessoryScrollView;
 
 @interface MTableEditor : NSWindowController
@@ -11,16 +14,19 @@
     IBOutlet NSPopUpButton *colCollationPop;
     IBOutlet NSTextView *colCommentText;
     IBOutlet NSTextField *colDefaultText;
+    IBOutlet NSButton *colDefaultNull;
     IBOutlet NSTableView *colFlagsList;
     IBOutlet NSView *colGroup;
     IBOutlet NSTextField *colNameText;
     IBOutlet NSButtonCell *colAIncCheck;
+    IBOutlet NSButtonCell *colUniqueCheck;
     IBOutlet NSButtonCell *colNNullCheck;
     IBOutlet NSButtonCell *colPKCheck;
     IBOutlet NSComboBox *colTypeCombo;
     IBOutlet NSTableView *columnList;
     IBOutlet NSTextField *commentText;
-    IBOutlet NSMatrix *engineMatrix;
+    IBOutlet NSPopUpButton *enginePop;
+    IBOutlet NSTextField *engineText;
     IBOutlet NSView *fkContainer;
     IBOutlet NSBox *fkBox;
     IBOutlet NSTableView *fkColumnList;
@@ -44,6 +50,7 @@
     IBOutlet NSView *stoOpView;
     IBOutlet NSView *mergeOpView;
     IBOutlet NSView *raidOpView;
+    IBOutlet NSView *federOpView;
     
     IBOutlet NSPanel *confirmPanel;
     IBOutlet NSTextView *confirmText;
@@ -62,11 +69,15 @@
     NSImage *_spatialIcon;
     NSImage *_stringIcon;
     NSImage *_userdefIcon;
+  
+    NSImage *_nullIcon;
     
     MYX_DBM_DATATYPES *_data_types;
     MYX_DBM_CHARSETS *_charsets;
     MYX_DBM_TABLE_DATA *_table;
     
+    int _draggedRow;
+    
     BOOL _releaseOnClose;
 }
 - (IBAction)applyChanges:(id)sender;
@@ -93,6 +104,6 @@
           catalog:(NSString*)catalog
            schema:(NSString*)schema;
 
-- (BOOL)confirmScript:(NSString*)script;
+- (NSString*)confirmScript:(NSString*)script;
 
 @end
