@@ -6,6 +6,67 @@
 
 #define COLUMN_TYPE(c) ((c).datatype_pointer ?  (c).datatype_pointer->group : 0)
 
+
+static struct EngineOptions {
+  const char *label;
+  const char *name;
+  MYX_DBM_TABLE_ENGINE type;
+  const char *description;
+} EngineOptionNameList[] = {
+  {"MyISAM", "MyISAM", MYX_DBM_TE_MYISAM,
+    "Very fast, disk based storage engine without support for transactions.\n"
+    "Offers fulltext search, packed keys, and is the default storage engine."},
+  {"InnoDB", "InnoDB", MYX_DBM_TE_INNODB,
+    "Transaction safe, disk based storage engine with row locking.\n"
+    "Recommended engine for tables that need support for transactions."},
+  {"Falcon", "jstar", MYX_DBM_TE_FALCON,
+    "Transactional engine for modern scale-out applications.\n"
+    "Supports MVCC (multiversion concurrency control)."},
+  {"NDB", "NDB", MYX_DBM_TE_NDB,
+    "MySQL Cluster storage engine. Transaction safe, memory based with row locking.\n"
+    "Recommended storage engine for high performance, real time critical applications."},
+  {"Archive", "ARCHIVE", MYX_DBM_TE_ARCHIVE,
+    "Archive engine storing data in a compressed form to save storage space.\n"
+    "Note that SELECTs require a full table scan."},
+  {"CSV", "CSV", MYX_DBM_TE_CSV,
+    "Engine that stores data as a comma separated values list, suited for data export.\n"
+    "Note that this engine does not support indices."},
+  {"BLACKHOLE", "BLACKHOLE", MYX_DBM_TE_BLACKHOLE,
+    "The BLACKHOLE storage engine acts as a \"black hole\" that accepts data but\n"
+    "throws it away and does not store it."},
+  {"BerkeleyDB", "BerkeleyDB", MYX_DBM_TE_BDB,
+    "Transaction safe storage engine with page locking.  Also called BDB."},
+  {"Federated", "Federated", MYX_DBM_TE_FEDERATED,
+    "The FEDERATED storage engine accesses data in tables of remote databases rather\n"
+    "than in local tables."},
+  {"MEMORY (HEAP)", "MEMORY", MYX_DBM_TE_MEMORY,
+    "Extremly fast memory based storage engine that uses hash indices.\n"
+    "Recommended storage engine for temporary data that can be lost in case\n"
+    "of a server shutdown."},
+  {"MERGE", "MERGE", MYX_DBM_TE_MERGE,
+    "Collection of MyISAM tables with identical column and index information.\n"
+    "Recommended storage engine for log tables or tables with archived data.\n"
+    "Note that the list of union tables and insert method has to be defined."},
+  {"ISAM (deprecated)", "ISAM", MYX_DBM_TE_ISAM,
+    "Deprecated. This storage engine was replaced by the MyISAM storage engine."},
+};
+
+
+static int indexOfEngine(MYX_DBM_TABLE_ENGINE engine)
+{
+  int i;
+  for (i= 0; i < sizeof(EngineOptionNameList)/sizeof(struct EngineOptions); i++)
+  {
+    if (EngineOptionNameList[i].type == engine)
+      return i;
+  }
+  return 0;
+}
+
+
+NSString *MTableEditorTableDidChange= @"MTableEditorTableDidChange";
+NSString *MTableRowPboardType= @"MTableRowPboardType";
+
 @interface MTableEditor(Private)
 - (void)addColumn:(id)sender;
 - (void)deleteColumn:(id)sender;
@@ -25,10 +86,6 @@
                     popup:(id)popup;
 - (void)handleTextEdited:(NSNotification*)notif;
 
-- (int)numberOfRowsInTableView:(NSTableView *)aTableView;
-- (id)tableView:(NSTableView *)aTableView objectValueForTableColumn:(NSTableColumn *)aTableColumn row:(int)rowIndex;
-- (void)tableView:(NSTableView *)aTableView setObjectValue:(id)anObject forTableColumn:(NSTableColumn *)aTableColumn row:(int)rowIndex;
-
 - (void)tableViewSelectionDidChange:(NSNotification *)aNotification;
 - (BOOL)commitChanges;
 - (void)revertChanges;
@@ -58,9 +115,7 @@ static MYX_SCHEMA_TABLE *find_table(MYX_CATALOGS *cats, const char *catalog,
           {
             MYX_SCHEMA_TABLE *tab= sch->schema_tables->schema_tables+t;
             if (strcmp2(tab->table_name, table)==0)
-            {
               return tab;
-            }
           }
           break;
         }
@@ -81,6 +136,8 @@ static MYX_SCHEMA_TABLE *find_table(MYX_CATALOGS *cats, const char *catalog,
   _table->columns[i].name= g_strdup("newcolumn");
   _table->columns[i].datatype_name= g_strdup("INT");
   _table->columns[i].datatype_pointer= myx_dbm_get_datatype(_data_types, &_table->columns[i].datatype_name);
+  _table->columns[i].default_value= g_strdup("");
+  _table->columns[i].default_value_is_null= 0;
   [columnList reloadData];
   [columnList selectRow:i byExtendingSelection:NO];
   [columnList editColumn:0 row:i withEvent:nil select:YES];
@@ -224,6 +281,7 @@ static MYX_SCHEMA_TABLE *find_table(MYX_CATALOGS *cats, const char *catalog,
   [fkColumnList reloadData];
 }
 
+
 - (void)addIndex:(id)sender
 {
   int i;
@@ -235,6 +293,7 @@ static MYX_SCHEMA_TABLE *find_table(MYX_CATALOGS *cats, const char *catalog,
   [indexList selectRow:i byExtendingSelection:NO];
 }
 
+
 - (void)deleteIndex:(id)sender
 {
   int i= [indexList selectedRow];
@@ -242,10 +301,10 @@ static MYX_SCHEMA_TABLE *find_table(MYX_CATALOGS *cats, const char *catalog,
   {
     MYX_DBM_INDEX_DATA *index= _table->indices+i;
     int j;
-    
+
     g_free(index->name);
     g_free(index->original_name);
-    
+
     for (j= 0; j < index->columns_num; j++)
     {
       g_free(index->columns[j].name);
@@ -253,15 +312,49 @@ static MYX_SCHEMA_TABLE *find_table(MYX_CATALOGS *cats, const char *catalog,
       g_free(index->columns[j].value_order);
     }
     g_free(index->columns);
-    
+
     memmove(_table->indices+i, _table->indices+i+1,
             sizeof(MYX_DBM_INDEX_DATA)*(_table->indices_num-i-1));
     _table->indices_num--;
-    
+
     [indexList reloadData];
   }
 }
 
+
+static int addColumnToIndex(MYX_DBM_INDEX_DATA *idata, const char *colname)
+{
+  int i= idata->columns_num++;
+  idata->columns= g_realloc(idata->columns, sizeof(MYX_DBM_INDEX_COLUMN_DATA)*idata->columns_num);
+  memset(&idata->columns[i], 0, sizeof(MYX_DBM_INDEX_COLUMN_DATA));
+  idata->columns[i].name= g_strdup(colname);  
+  
+  return idata->columns_num-1;
+}
+
+
+static void removeColumnFromIndex(MYX_DBM_INDEX_DATA *idata, const char *colname)
+{
+  int i;
+  
+  for (i= 0; i < idata->columns_num; i++)
+  {
+    if (strcmp2(idata->columns[i].name, colname)==0)
+      break;
+  }
+  
+  if (idata->columns_num == i)
+    return;
+  
+  g_free(idata->columns[i].name);
+  g_free(idata->columns[i].len);
+  g_free(idata->columns[i].value_order);
+  memmove(idata->columns+i, idata->columns+i+1,
+          sizeof(MYX_DBM_INDEX_COLUMN_DATA)*(idata->columns_num-i-1));
+  idata->columns_num--;
+}
+
+
 - (void)addIndexColumn:(id)sender
 {
   int index= [indexList selectedRow];
@@ -273,17 +366,25 @@ static MYX_SCHEMA_TABLE *find_table(MYX_CATALOGS *cats, const char *catalog,
   if (index < 0)
     return;
   
+  idata= _table->indices+index;
+  
   if (column < 0)
     colname= "";
   else
+  {
     colname= _table->columns[column].name;
-  
-  idata= _table->indices+index;
+    
+    // if this is the PRIMARY index, then set the column's PRIMARY flag
+    if (strcmp2(idata->name, "PRIMARY")==0)
+    {
+      _table->columns[column].primary_key= 1;
+      
+      [self showColumn:_table->columns+column];
+    }
+  }
 
-  i= idata->columns_num++;
-  idata->columns= g_realloc(idata->columns, sizeof(MYX_DBM_INDEX_COLUMN_DATA)*idata->columns_num);
-  memset(&idata->columns[i], 0, sizeof(MYX_DBM_INDEX_COLUMN_DATA));
-  idata->columns[i].name= g_strdup(colname);
+  i= addColumnToIndex(idata, colname);
+  
   [indColumnList reloadData];
   [indColumnList selectRow:i byExtendingSelection:NO];  
   [indColumnList editColumn:0 row:i withEvent:nil select:YES];
@@ -300,12 +401,21 @@ static MYX_SCHEMA_TABLE *find_table(MYX_CATALOGS *cats, const char *catalog,
 
   idata= _table->indices+index;
 
-  g_free(idata->columns[icolumn].name);
-  g_free(idata->columns[icolumn].len);
-  g_free(idata->columns[icolumn].value_order);
-  memmove(idata->columns+icolumn, idata->columns+icolumn+1,
-          sizeof(MYX_DBM_INDEX_COLUMN_DATA)*(idata->columns_num-icolumn-1));
-  idata->columns_num--;
+  // if this is the PRIMARY index, then set the column's PRIMARY flag
+  if (strcmp2(idata->name, "PRIMARY")==0)
+  {
+    int column;
+    for (column= 0; column < _table->columns_num; column++)
+    {
+      if (strcmp2(_table->columns[column].name, idata->columns[icolumn].name)==0)
+        break;
+    }
+    _table->columns[column].primary_key= 0;
+    
+    [self showColumn:_table->columns+column];
+  }
+  
+  removeColumnFromIndex(idata, idata->columns[icolumn].name);
 
   [indColumnList reloadData];
 }
@@ -320,6 +430,8 @@ static MYX_SCHEMA_TABLE *find_table(MYX_CATALOGS *cats, const char *catalog,
   {
     g_free(column->name);
     column->name= g_strdup([str UTF8String]);
+    
+    [self columnListChanged];
   }
   else if (obj == colTypeCombo)
   {
@@ -354,7 +466,17 @@ static MYX_SCHEMA_TABLE *find_table(MYX_CATALOGS *cats, const char *catalog,
     [colNameText setStringValue:NSStr(column->name)];
     
     [colTypeCombo setStringValue:[NSString stringWithFormat:@"%s%s",column->datatype_name,column->datatype_params?:""]];
-    [colDefaultText setStringValue:NSStr(column->default_value)];
+    [colDefaultNull setState:column->default_value_is_null ? NSOnState : NSOffState];
+    if (column->default_value_is_null)
+    {
+      [colDefaultText setStringValue:@""];
+      [colDefaultText setEnabled:NO];
+    }
+    else
+    {
+      [colDefaultText setStringValue:NSStr(column->default_value)];
+      [colDefaultText setEnabled:YES];
+    }
     [colFlagsList reloadData];
         
     [colCharsetPop selectItemAtIndex:[[colCharsetPop menu] indexOfItemWithRepresentedObject:NSStr(column->charset)]];
@@ -376,6 +498,7 @@ static MYX_SCHEMA_TABLE *find_table(MYX_CATALOGS *cats, const char *catalog,
     
     [colTypeCombo setStringValue:@""];
     [colDefaultText setStringValue:@""];
+    [colDefaultNull setState:NSOffState];
     [colFlagsList reloadData];
     
     [colCharsetPop selectItemWithTitle:@""];
@@ -393,10 +516,13 @@ static MYX_SCHEMA_TABLE *find_table(MYX_CATALOGS *cats, const char *catalog,
 - (void)showIndex:(MYX_DBM_INDEX_DATA*)index
 {
   if (index)
-  {
+  {      
     [indNameText setStringValue:NSStr(index->name)];
     [indKindPop selectItemAtIndex:index->index_kind];
     [indTypePop selectItemAtIndex:index->index_type];
+
+    if (strcmp(index->name, "PRIMARY")==0)
+      [indBox setEnabledRecursive:NO];
   }
   else
   {
@@ -416,14 +542,28 @@ static MYX_SCHEMA_TABLE *find_table(MYX_CATALOGS *cats, const char *catalog,
     [fkOnDeletePop selectItemAtIndex:fk->on_delete];
     [fkOnUpdatePop selectItemAtIndex:fk->on_update];
     [fkRefTablePop selectItemWithTitle:NSStr(fk->reference_table_name)];
+    
+    // setup the menu in the column dataCell
+    id cell= [[fkColumnList tableColumnWithIdentifier:@"fcolumn"] dataCell];
+    [cell removeAllItems];
+    // fetch the column names for the table
+    MYX_SCHEMA_TABLE *refTable= 
+      find_table(_catalogs, [_catalogName UTF8String], [_schemaName UTF8String],
+                 fk->reference_table_name);
+    if (refTable)
+    {
+      unsigned int i;
+      for (i= 0; i < refTable->columns_num; i++)
+        [cell addItemWithTitle:NSStr(refTable->columns[i].column_name)];
+    }
   }
   else
   {
-    [indNameText setStringValue:@""];
+    [fkNameText setStringValue:@""];
     [fkOnDeletePop selectItemAtIndex:0];
     [fkOnUpdatePop selectItemAtIndex:0];
-	if ([fkRefTablePop numberOfItems]>0)
-	  [fkRefTablePop selectItemAtIndex:0];
+    if ([fkRefTablePop numberOfItems]>0)
+      [fkRefTablePop selectItemAtIndex:0];
   }  
   [fkColumnList reloadData];  
 }
@@ -464,7 +604,7 @@ static MYX_SCHEMA_TABLE *find_table(MYX_CATALOGS *cats, const char *catalog,
   else if (aTableView == fkColumnList)
   {
     int index= [fkList selectedRow];
-    if (index < 0)
+    if (index < 0 || !_table->fks)
       return 0;
     return _table->fks[index].column_mapping_num;
   }
@@ -511,11 +651,19 @@ static MYX_SCHEMA_TABLE *find_table(MYX_CATALOGS *cats, const char *catalog,
       }      
       [aCell setImage:img];
     }
+    else if ([[aTableColumn identifier] isEqualTo:@"default"])
+    {
+      if (_table->columns[rowIndex].default_value_is_null)
+        [aCell setImage:_nullIcon];
+      else
+        [aCell setImage:nil];
+    }
   }
   else if (aTableView == colFlagsList)
   {
     int r= [columnList selectedRow];
-    [aCell setTitle:[NSString stringWithUTF8String:_table->columns[r].datatype_pointer->flags[rowIndex]]];
+    if (rowIndex < _table->columns[r].datatype_pointer->flags_num)
+      [aCell setTitle:[NSString stringWithUTF8String:_table->columns[r].datatype_pointer->flags[rowIndex]]];
   }
 }
 
@@ -558,7 +706,12 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
       return ms;
     }
     else if ([[aTableColumn identifier] isEqualTo:@"default"])
-      return NSStr(_table->columns[rowIndex].default_value);
+    {
+      if (_table->columns[rowIndex].default_value_is_null)
+        return @"";
+      else
+        return NSStr(_table->columns[rowIndex].default_value);
+    }
     else if ([[aTableColumn identifier] isEqualTo:@"comments"])
       return NSStr(_table->columns[rowIndex].comment);
   }
@@ -589,11 +742,16 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
   }
   else if (aTableView == indColumnList)
   {
-    int index= [indexList selectedRow];
+    int index= [indexList selectedRow];        
     if (index < 0)
-      return @"";
+      return nil;
     if ([[aTableColumn identifier] isEqualTo:@"column"])
-      return NSStr(_table->indices[index].columns[rowIndex].name);
+    {
+      int i= [[aTableColumn dataCell] indexOfItemWithTitle:NSStr(_table->indices[index].columns[rowIndex].name)];
+      if (i == NSNotFound)
+        return nil;
+      return [NSNumber numberWithInt:i];
+    }
     else
       return NSStr(_table->indices[index].columns[rowIndex].len);
   }
@@ -605,11 +763,21 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
   {
     int index= [fkList selectedRow];
     if (index < 0)
-      return 0;
+      return nil;
     if ([[aTableColumn identifier] isEqualTo:@"column"])
-      return NSStr(_table->fks[index].column_mapping[rowIndex].name);
+    {
+      int i= [[aTableColumn dataCell] indexOfItemWithTitle:NSStr(_table->fks[index].column_mapping[rowIndex].name)];
+      if (i < 0)
+        return nil;
+      return [NSNumber numberWithInt:i];
+    }
     else
-      return NSStr(_table->fks[index].column_mapping[rowIndex].value);
+    {
+      int i= [[aTableColumn dataCell] indexOfItemWithTitle:NSStr(_table->fks[index].column_mapping[rowIndex].value)];
+      if (i < 0)
+        return nil;
+      return [NSNumber numberWithInt:i];
+    }
   }
   return @"";
 }
@@ -627,6 +795,7 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
     {
       g_free(_table->columns[rowIndex].name);
       _table->columns[rowIndex].name= g_strdup([anObject UTF8String]);
+      [self columnListChanged];
     }
     else if ([[aTableColumn identifier] isEqualTo:@"type"])
     {
@@ -658,6 +827,9 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
     {
       g_free(_table->columns[rowIndex].default_value);
       _table->columns[rowIndex].default_value= g_strdup([anObject UTF8String]);
+      if ([colDefaultNull state] == NSOnState)
+        [colDefaultNull performClick:nil];
+      _table->columns[rowIndex].default_value_is_null= 0;
     }
     else if ([[aTableColumn identifier] isEqualTo:@"comments"])
     {
@@ -686,8 +858,9 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
     {
       _table->columns[r].datatype_flags_num++;
       _table->columns[r].datatype_flags= g_realloc(_table->columns[r].datatype_flags, 
-                                          sizeof(char*)*_table->columns[r].datatype_flags_num);
+                                          sizeof(char*)*(_table->columns[r].datatype_flags_num+1));
       _table->columns[r].datatype_flags[_table->columns[r].datatype_flags_num-1]= g_strdup(flag);
+      _table->columns[r].datatype_flags[_table->columns[r].datatype_flags_num]= NULL;
     }
     else if (index >= 0 && [anObject intValue]==NSOffState)
     {
@@ -703,6 +876,32 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
     }
     [columnList reloadData];
   }
+  // FK Columns
+  else if (aTableView == fkColumnList)
+  {
+    int index= [fkList selectedRow];
+    int cindex= [fkColumnList selectedRow];
+    MYX_NAME_VALUE_PAIR *cdata= _table->fks[index].column_mapping+cindex;
+    int i= [anObject intValue];
+    NSString *name;
+    
+    if (i >= 0)
+    {
+      name= [[aTableColumn dataCell] itemTitleAtIndex:i];
+      
+      if ([[aTableColumn identifier] isEqualTo:@"column"])
+      {
+        g_free(cdata->name);
+        cdata->name= g_strdup([name UTF8String]);
+      }
+      else
+      {
+        g_free(cdata->value);
+        cdata->value= g_strdup([name UTF8String]);
+      }
+      [fkColumnList reloadData];
+    }
+  }  
   // Index Columns
   else if (aTableView == indColumnList)
   {
@@ -726,57 +925,168 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
       else
         cdata->len= g_strdup([anObject UTF8String]);
     }
+    [indColumnList reloadData];
   }
 }
 
+- (BOOL)tableView:(NSTableView *)tableView
+        writeRows:(NSArray *)rows
+     toPasteboard:(NSPasteboard *)pboard
+{
+  if (tableView == columnList)
+  {
+    _draggedRow= [[rows objectAtIndex:0] intValue];
+    
+    [pboard declareTypes:[NSArray arrayWithObject:MTableRowPboardType] owner:nil];
+    [pboard setData:[NSData data] forType:MTableRowPboardType];
+    
+    return YES;
+  }
+  else
+    return NO;
+}
+
+- (NSDragOperation)tableView:(NSTableView *)tableView
+                validateDrop:(id <NSDraggingInfo>)info
+                 proposedRow:(int)row
+       proposedDropOperation:(NSTableViewDropOperation)operation
+{
+  NSDragOperation op;
+  if (tableView == columnList)
+  {
+    if ([info draggingSource] == tableView && row != _draggedRow && row >= 0)
+      op= NSDragOperationPrivate;
+    else
+      op= NSDragOperationNone;
+  }
+  else if (tableView == fkColumnList && [fkColumnList isEnabled] && [fkList selectedRow]>=0)
+    op= NSDragOperationPrivate;
+  else if (tableView == indColumnList && [indColumnList isEnabled] && [indexList selectedRow]>=0)
+    op= NSDragOperationPrivate;
+  else 
+    op= NSDragOperationNone;
+  return op;
+}
+
+- (BOOL)tableView:(NSTableView *)tableView 
+       acceptDrop:(id <NSDraggingInfo>)info
+              row:(int)row 
+    dropOperation:(NSTableViewDropOperation)operation
+{
+  if (tableView == columnList)
+  {
+    MYX_DBM_COLUMN_DATA column;
+    
+    if (row >= _table->columns_num)
+      row= _table->columns_num-1;
+    
+    memcpy(&column, _table->columns+_draggedRow, sizeof(column));
+    
+    if (_draggedRow < row)
+    {
+      // I1
+      // I2 ---
+      // I3   |
+      // I4 <--
+      // I5
+      memmove(_table->columns+_draggedRow, _table->columns+_draggedRow+1,
+              (row-_draggedRow)*sizeof(column));
+      memcpy(_table->columns+row, &column, sizeof(column));
+    }
+    else
+    {
+      // I1
+      // I2 <--
+      // I3   |
+      // I4 ---
+      // I5    
+      memmove(_table->columns+row+1, _table->columns+row,
+              (_draggedRow-row)*sizeof(column));
+      memcpy(_table->columns+row, &column, sizeof(column));    
+    }
+    [tableView reloadData];
+    return YES;
+  }
+  else if (tableView == fkColumnList)
+  {
+    int index= [fkList selectedRow];
+    MYX_NAME_VALUE_PAIR *fdata;
+    int findex;
+    
+    [self addFKColumn:nil];
+    findex= [fkColumnList selectedRow];
+    fdata= _table->fks[index].column_mapping+findex;
+    
+    g_free(fdata->name);
+    fdata->name= g_strdup(_table->columns[_draggedRow].name);
+    fdata->value= NULL;
+    
+    [fkColumnList reloadData];
+    return YES;    
+  }
+  else if (tableView == indColumnList)
+  {
+    int cindex;
+    int index= [indexList selectedRow];
+    MYX_DBM_INDEX_COLUMN_DATA *cdata;
+    
+    [self addIndexColumn:nil];
+    cindex= [indColumnList selectedRow];
+    cdata= _table->indices[index].columns+cindex;
+    
+    g_free(cdata->name);
+    cdata->name= g_strdup(_table->columns[_draggedRow].name);
+    
+    [fkColumnList reloadData];
+    return YES;
+  }
+  else
+    return NO;
+}
 
 - (void)tableViewSelectionDidChange:(NSNotification *)aNotification
 {
   NSTableView *table= [aNotification object];
   int row= [table selectedRow];
-  BOOL enabled= NO;
 
   if (table == columnList)
   {  
     if (row < 0)
     {
-      enabled= NO;
+      [colGroup setEnabledRecursive:NO];
       [self showColumn:0];
     }
     else
     {
-      enabled= YES;
+      [colGroup setEnabledRecursive:YES];
       [self showColumn:_table->columns+row];
     }
-	[colGroup setEnabledRecursive:enabled];
   }
   else if (table == indexList)
   {
     if (row < 0)
     {
-      enabled= NO;
+      [indBox setEnabledRecursive:NO];
       [self showIndex:0];
     }
     else
     {
-      enabled= YES;
+      [indBox setEnabledRecursive:YES];
       [self showIndex:_table->indices+row];
     }
-	[indBox setEnabledRecursive:enabled];
   }
   else if (table == fkList)
   {
     if (row < 0)
     {
-      enabled= NO;
+      [fkBox setEnabledRecursive:NO];
       [self showFK:0];
     }
     else
     {
-      enabled= YES;
+      [fkBox setEnabledRecursive:YES];
       [self showFK:_table->fks+row];
     }
-	[fkBox setEnabledRecursive:enabled];
   }  
 }
 
@@ -834,7 +1144,8 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
   else
   {
     _table= g_new0(MYX_DBM_TABLE_DATA, 1);
-    
+
+    _table->schema= g_strdup([_schemaName UTF8String]);
     _table->charset= g_strdup("utf8");
   }
 
@@ -861,7 +1172,7 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
   [fkList reloadData];
   
   // Engine Stuff
-  [[engineMatrix cellWithTag:10+_table->table_engine] setState:NSOnState];
+  [enginePop selectItemAtIndex:indexOfEngine(_table->table_engine)];
   [self refreshForTableType];
   
   // Charset Stuff
@@ -894,6 +1205,9 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
   [[raidOpView viewWithTag:1] selectItemAtIndex:(int)_table->raid_type];
   [[raidOpView viewWithTag:2] setStringValue:NSStr(_table->raid_chunks)];
   [[raidOpView viewWithTag:3] setStringValue:NSStr(_table->raid_chunk_size)];
+  
+  // Federated
+  [[federOpView viewWithTag:1] setStringValue:NSStr(_table->federated_connection)];
 }
 
 
@@ -928,6 +1242,9 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
   _table->raid_type= [[raidOpView viewWithTag:1] indexOfSelectedItem];
   ASSIGNSTR(_table->raid_chunks, [[raidOpView viewWithTag:2] stringValue]);
   ASSIGNSTR(_table->raid_chunk_size, [[raidOpView viewWithTag:3] stringValue]);
+  
+  // Federated stuff
+  ASSIGNSTR(_table->federated_connection, [[federOpView viewWithTag:1] stringValue]);
 
   return YES;
 }
@@ -936,13 +1253,14 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
 - (void)refreshForTableType
 {
   BOOL fkEnabled= NO;
+  
+  [engineText setStringValue:NSStr(EngineOptionNameList[[enginePop indexOfSelectedItem]].description)];
 
   switch (_table->table_engine)
   {
     case MYX_DBM_TE_MYISAM:
       break;
     case MYX_DBM_TE_INNODB:
-      NSLog(@"!!!");
       fkEnabled= YES;
       break;
     case MYX_DBM_TE_MEMORY:
@@ -955,9 +1273,21 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
       break;
     case MYX_DBM_TE_NDB:
       break;
+    case MYX_DBM_TE_FALCON:
+      break;
+    case MYX_DBM_TE_FEDERATED:
+      break;
+    case MYX_DBM_TE_ARCHIVE:
+      break;
+    case MYX_DBM_TE_CSV:
+      break;
+    case MYX_DBM_TE_BLACKHOLE:
+      break;
   }
   
   [fkContainer setEnabledRecursive:fkEnabled];
+  if (fkEnabled)
+    [fkBox setEnabledRecursive:NO];
 }
 
 @end
@@ -1030,7 +1360,7 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
     {
       NSString *tmp= [NSString stringWithUTF8String:script];
       
-      if ([self confirmScript:tmp])
+      if ((tmp= [self confirmScript:tmp]))
       {
         MYX_LIB_ERROR error;
         NSString *msg;
@@ -1040,7 +1370,7 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
         else
           msg= @"Error executing SQL commands to update table.";
         
-        myx_query_execute_direct(_mysql, script, &error);
+        myx_query_execute_direct(_mysql, [tmp UTF8String], &error);
         
         if (error == MYX_SQL_ERROR)
           MXRunAlertPanelWithMySQLError(@"Error", msg, _mysql);
@@ -1055,10 +1385,13 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
             NSRunAlertPanel(@"Edit Table", @"Modifications to Table '%@' Were Saved.",
                             @"OK", nil, nil, [NSString stringWithUTF8String:_table->name]);
           
+          _tableName= [NSString stringWithUTF8String:_table->name];
+
           // reload newly saved data
           [self revertChanges];
-          
-          _tableName= [NSString stringWithUTF8String:_table->name];
+		  
+          [[NSNotificationCenter defaultCenter] postNotificationName:MTableEditorTableDidChange
+                                                              object:self];
         }
       }
     }
@@ -1094,9 +1427,9 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
     g_free(_table->collation);
     _table->collation= g_strdup(collation);
   }
-  else if (sender == engineMatrix)
+  else if (sender == enginePop)
   {
-    _table->table_engine= [[engineMatrix selectedCell] tag] - 10;
+    _table->table_engine= [[[enginePop selectedItem] representedObject] intValue];
     [self refreshForTableType];
   }
 }
@@ -1105,7 +1438,25 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
 {
   int column= [columnList selectedRow];
 
-  if (sender == colCharsetPop)
+  if (sender == colTypeCombo)
+  {
+    NSString *type= [colTypeCombo stringValue];
+    if (g_strncasecmp(_table->columns[column].datatype_name,[type UTF8String],[type length])!=0)
+    {
+      char *dt= g_strdup([type UTF8String]);
+      MYX_DBM_DATATYPE *dtype= myx_dbm_get_datatype(_data_types, &dt);
+      if (dtype)
+      {
+        _table->columns[column].datatype_pointer= dtype;
+        _table->columns[column].datatype_name= myx_dbm_get_datatype_name(dt);
+        _table->columns[column].datatype_params= myx_dbm_get_datatype_params(_table->columns[column].datatype_name,
+                                                                               dt);
+      }
+      g_free(dt);
+      [columnList reloadData];
+    }
+  }
+  else if (sender == colCharsetPop)
   {
     const char *charset= [[[sender selectedItem] representedObject] UTF8String];
     
@@ -1124,22 +1475,70 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
     g_free(_table->columns[column].collation);
     _table->columns[column].collation= g_strdup(collation);    
   }
+  else if (sender == colDefaultNull)
+  {
+    _table->columns[column].default_value_is_null= [sender state] == NSOnState;
+    if (_table->columns[column].default_value_is_null)
+    {
+      [colDefaultText setEnabled:NO];
+    }
+    else
+    {
+      g_free(_table->columns[column].default_value);
+      _table->columns[column].default_value= g_strdup("");
+      [colDefaultText setEnabled:YES];
+    }
+    [columnList reloadData];
+  }
   else if ([sender isMemberOfClass:[NSMatrix class]])
   {
+    int i;
+    MYX_DBM_INDEX_DATA *idata= NULL;
+    
     _table->columns[column].primary_key= [colPKCheck state] == NSOnState ? 1 : 0;
     if (COLUMN_TYPE(_table->columns[column]) != MYX_DBM_DTG_NUMERIC)
       _table->columns[column].auto_inc= 0;
     else
       _table->columns[column].auto_inc= [colAIncCheck state] == NSOnState ? 1 : 0;
 
+    // find the PRIMARY index
+    for (i= 0; i < _table->indices_num; i++)
+    {
+      if (strcmp2(_table->indices[i].name, "PRIMARY")==0)
+      {
+        idata= _table->indices+i;
+        break;
+      }
+    }
+    
+    // no PRIMARY index yet, create one
+    if (!idata)
+    {
+      int i= _table->indices_num++;
+      _table->indices= g_realloc(_table->indices, sizeof(MYX_DBM_INDEX_DATA)*_table->indices_num);
+      memset(&_table->indices[i], 0, sizeof(MYX_DBM_INDEX_DATA));
+      _table->indices[i].name= g_strdup("PRIMARY");
+      _table->indices[i].index_kind= MYX_DBM_IK_PRIMARY;
+      _table->indices[i].index_type= MYX_DBM_IT_DEFAULT;
+      idata= _table->indices + i;
+    }
+
     if (_table->columns[column].primary_key)
     {
-      _table->columns[column].not_null= 1;
+      addColumnToIndex(idata, _table->columns[column].name);
+      [indColumnList reloadData];
+      
       [colNNullCheck setState:NSOnState];
+      
+      _table->columns[column].not_null= 1;
     }
     else
+    {
+      removeColumnFromIndex(idata, _table->columns[column].name);
+      [indColumnList reloadData];
+      
       _table->columns[column].not_null= [colNNullCheck state] == NSOnState ? 1 : 0;
-    
+    }
     [colNNullCheck setEnabled:_table->columns[column].primary_key?NO:YES];
     
     [columnList reloadData];
@@ -1150,8 +1549,14 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
 {
   int index= [indexList selectedRow];
   MYX_DBM_INDEX_DATA *idata= _table->indices+index;
-  
-  if (sender == indKindPop)
+
+  if (sender == indNameText)
+  {
+    g_free(idata->name);
+    idata->name= g_strdup([[sender stringValue] UTF8String]);
+    [indexList reloadData];
+  }
+  else if (sender == indKindPop)
   {
     idata->index_kind= [sender indexOfSelectedItem];
   }
@@ -1237,11 +1642,12 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
   self= [super initWithWindowNibName: @"TableEditor"];
   if (self)
   {
+    [self setShouldCascadeWindows:NO];
     [self loadWindow];
     
     [[NSNotificationCenter defaultCenter] addObserver:self
                                              selector:@selector(handleTextEdited:)
-                                                 name:NSControlTextDidEndEditingNotification
+                                                 name:NSControlTextDidChangeNotification
                                                object:nil];
     [[self window] setDelegate:self];
   }
@@ -1257,7 +1663,7 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
     myx_free_charsets(_charsets);
 
   [[NSNotificationCenter defaultCenter] removeObserver:self];
-  
+
   [_pkIcon release];
   [_columnIcon release];
   [_blobIcon release];
@@ -1266,7 +1672,7 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
   [_spatialIcon release];
   [_stringIcon release];
   [_userdefIcon release];
-  
+
   [super dealloc];
 }
 
@@ -1292,7 +1698,7 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
   [button setTarget:self];
   [button setAction:addSel];
   [sv addSubview:button];
-  [sv addAccessoryView:button];
+  [sv addHorizontalAccessoryView:button];
   
   button= [[[NSButton alloc] initWithFrame:NSMakeRect(0,0,size,size)] autorelease];
   [button setButtonType: NSMomentaryChangeButton];
@@ -1303,12 +1709,19 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
   [button setTarget:self];
   [button setAction:delSel];
   [sv addSubview:button];
-  [sv addAccessoryView:button];
+  [sv addHorizontalAccessoryView:button];
 }
 
 
 - (void)awakeFromNib
 {
+  int i;
+  NSBundle *bundle= [NSBundle bundleForClass:[self class]];
+  
+  [self setWindowFrameAutosaveName:@"TableEditor"];
+  [[self window] setFrameUsingName:@"TableEditor"];
+  [[self window] setFrameAutosaveName:@"TableEditor"];
+  
   [self setupButtonsForTableView:columnList 
                        addAction:@selector(addColumn:)
                     deleteAction:@selector(deleteColumn:)];
@@ -1326,18 +1739,32 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
                     deleteAction:@selector(deleteIndexColumn:)];
 
   [[columnList tableColumnWithIdentifier:@"nnull"] setHeaderCell:
-    [[NSTableHeaderCell alloc] initImageCell:[NSImage imageNamed:@"editor_table_not_null.png"]]];
+    [[NSTableHeaderCell alloc] initImageCell:MXGetImageFromBundle(bundle,@"editor_table_not_null.png")]];
   [[columnList tableColumnWithIdentifier:@"autoinc"] setHeaderCell:
-    [[NSTableHeaderCell alloc] initImageCell:[NSImage imageNamed:@"editor_table_auto_inc.png"]]];
+    [[NSTableHeaderCell alloc] initImageCell:MXGetImageFromBundle(bundle,@"editor_table_auto_inc.png")]];
 
-  _pkIcon= [[NSImage imageNamed:@"column_pk.png"] retain];
-  _columnIcon= [[NSImage imageNamed:@"column.png"] retain];
-  _blobIcon= [[NSImage imageNamed:@"datatype_blob.png"] retain];
-  _dateIcon= [[NSImage imageNamed:@"datatype_datetime.png"] retain];
-  _numericIcon= [[NSImage imageNamed:@"datatype_numeric.png"] retain];
-  _spatialIcon= [[NSImage imageNamed:@"datatype_spatial.png"] retain];
-  _stringIcon= [[NSImage imageNamed:@"datatype_string.png"] retain];
-  _userdefIcon= [[NSImage imageNamed:@"datatype_userdefined.png"] retain];
+  _pkIcon= [MXGetImageFromBundle(bundle,@"column_pk.png") retain];
+  _columnIcon= [MXGetImageFromBundle(bundle,@"column.png") retain];
+  _blobIcon= [MXGetImageFromBundle(bundle,@"datatype_blob.png") retain];
+  _dateIcon= [MXGetImageFromBundle(bundle,@"datatype_datetime.png") retain];
+  _numericIcon= [MXGetImageFromBundle(bundle,@"datatype_numeric.png") retain];
+  _spatialIcon= [MXGetImageFromBundle(bundle,@"datatype_spatial.png") retain];
+  _stringIcon= [MXGetImageFromBundle(bundle,@"datatype_string.png") retain];
+  _userdefIcon= [MXGetImageFromBundle(bundle,@"datatype_userdefined.png") retain];
+  
+  _nullIcon= [MXGetImageFromBundle(bundle,@"field_overlay_null.png") retain];
+  
+  [columnList registerForDraggedTypes:[NSArray arrayWithObject:MTableRowPboardType]];
+  [fkColumnList registerForDraggedTypes:[NSArray arrayWithObject:MTableRowPboardType]];
+  [indColumnList registerForDraggedTypes:[NSArray arrayWithObject:MTableRowPboardType]];  
+  
+  [enginePop removeAllItems];
+  for (i= 0; i < sizeof(EngineOptionNameList)/sizeof(struct EngineOptions); i++)
+  {
+    [enginePop addItemWithTitle:NSStr(EngineOptionNameList[i].label)];
+    [[enginePop lastItem] setRepresentedObject:[NSNumber numberWithInt:EngineOptionNameList[i].type]];
+  }
+  [enginePop selectItemAtIndex:1]; //innodb
 }
 
 
@@ -1387,9 +1814,9 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
     _tableName= nil;
   _catalogName= [catalog retain];
   _schemaName= [schema retain];
-
   // set up the Schema selection popup button
   [schemaPop removeAllItems];
+  
   for (c= 0; c < _catalogs->catalogs_num; c++)
   {
     MYX_CATALOG *cat= _catalogs->catalogs+c;
@@ -1403,10 +1830,11 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
       [schemaPop addItemWithTitle:NSStr(sch->schema_name)];
       if (strcmp(sch->schema_name, [schema UTF8String])==0)
       {
-        if (!sch->schema_tables)
-        { //XXX make threaded
-          sch->schema_tables= myx_get_schema_tables(_mysql, [catalog UTF8String], [schema UTF8String]);
-        }
+        if (sch->schema_tables)
+          myx_free_schema_tables(sch->schema_tables);
+
+        sch->schema_tables= myx_get_schema_tables(_mysql, [catalog UTF8String], [schema UTF8String]);
+
         my_schema= sch->schema_tables;
         [schemaPop selectItem:[schemaPop lastItem]];
       }
@@ -1466,9 +1894,10 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
 }
 
 
-- (BOOL)confirmScript:(NSString*)script
+- (NSString*)confirmScript:(NSString*)script
 {
   int rc;
+  NSString *final;
   
   [confirmText setString:script];
   
@@ -1480,9 +1909,14 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
 
   rc= [NSApp runModalForWindow:confirmPanel];
   
+  if (rc == 1)
+	final= [[[confirmText string] retain] autorelease];
+  else
+	final= nil;
+  
   [confirmPanel close];
   
-  return rc == 1;
+  return final;
 }
 
 
