@@ -44,9 +44,13 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
 - (MYXRSEditStatus)statusOfRow:(int)row
                         column:(int)column;
 
+- (MYXRSCompareStatus)compareStatusOfRow:(int)row
+                                  column:(int)column;
+
 - (BOOL)isEditable;
 - (BOOL)editing;
 - (void)setEditing:(BOOL)flag;
+- (BOOL)hasChanges;
 
 - (void)preCommit;
 - (void)postCommit:(BOOL)failed;
