@@ -64,6 +64,37 @@ NSImage *MXGetImageFromBundle(NSBundle *bundle, NSString *image)
 }
 
 
+@interface mxUtilsClass : NSObject
+{
+}
+@end
+@implementation mxUtilsClass
+@end
+
+
+NSImage *MXGetCommonImage(MXCommonImage img)
+{
+  static NSString *commonImages[]= {
+    @"mini_error.png",
+    @"mini_warning.png",
+    @"mini_notice.png"
+  };
+  static NSMutableDictionary *imageCache= nil;
+  NSImage *image;
+  
+  if (!imageCache)
+    imageCache= [[NSMutableDictionary alloc] init];
+  
+  image= [imageCache objectForKey:commonImages[img]];
+  if (!image)
+  {
+    image= MXGetImageFromBundle([NSBundle bundleForClass:[mxUtilsClass class]], commonImages[img]);
+    [imageCache setObject:image forKey:commonImages[img]];
+  }
+  return image;
+}
+
+
 void MXExpandOutline(NSOutlineView *outline, BOOL expandChildren)
 {
   id ds= [outline dataSource];
@@ -143,9 +174,36 @@ void MXExpandOutlineItems(NSOutlineView *outline, NSDictionary *items, id column
 }
 
 
-
-NSRect MXRectWithSize(NSSize size)
+void MXResizeTableColumnToFitValue(NSTableView *table, NSTableColumn *column, NSString *value)
 {
-  NSRect rect= {{0,0}, size};
-  return rect;
+  NSFont *font= [[column dataCell] font];
+  float width;
+
+  width= [font widthOfString:value]+8;
+  if (width > [column width])
+    [column setWidth:width];
+}
+
+
+void MXResizeTableColumnsToFit(NSTableView *table, NSTableColumn *column)
+{
+  NSFont *font= [[column dataCell] font];
+  float width;
+  unsigned int i, c;
+  id ds= [table dataSource];
+  
+  width= [column width];
+  
+  c= [ds numberOfRowsInTableView:table];  
+  for (i= 0; i < c; i++)
+  {
+    NSString *value= [ds tableView:table objectValueForTableColumn:column row:i];
+    float w;
+    
+    w= [font widthOfString:value];
+    if (w > width)
+      width= w;
+  }
+  if (width > [column width])
+    [column setWidth:width];  
 }
